export let API_CONFIG = {
    CFG_NOME_SISTEMA: 'SIAP2',
    CFG_ENV: 'production',
    CFG_TEMPO_VALIDADE_HASH: 30000, // 5s

    SIAP_SENHA_TOKEN: (process.env.SIAP_SENHA_TOKEN) ? process.env.SIAP_SENHA_TOKEN : 'senhatoken',
    TEMPO_TOKEN: 3600*6, // Em segundos

    IPC_WSDL_URL: "http://200.164.109.39/vsoft.idsps.dataserver/publicsimpleservice.asmx?WSDL",
    SIAP_IPC_CHAVE_ACESSO: (process.env.SIAP_IPC_CHAVE_ACESSO) ? process.env.SIAP_IPC_CHAVE_ACESSO : '',
    IPC_REQUEST_TIMEOUT: 30000,
    IPC_CREATE_CLIENT_TIMEOUT: 5000,

    MP_DITEC_WSDL_URL: "http://10.128.0.23:8080/axis2/services/WSAcesso?wsdl",
    MP_DITEC_REQUEST_TIMEOUT: 5000,

    SIAP_GOOGLE_RECAPTCHA_SECRET_KEY: process.env.SIAP_GOOGLE_RECAPTCHA_SECRET_KEY || '',
    SIAP_GOOGLE_MAPS_API_KEY: process.env.SIAP_GOOGLE_MAPS_API_KEY || '',

    SIAP_MAX_RESULTS: 1000,
    SIAP_CRAWLERS_URL: `http://${process.env.SIAP_CRAWLERS_SERVER}:${process.env.SIAP_CRAWLERS_PORT}`,

    AD_TIMEOUT: 5000,

    AD_MP_URL: '10.128.0.20',
    AD_MP_DOMAIN: '@intranet.mppb',
    AD_MP_BASE_DN: 'dc=mppb,dc=intranet',

    AD_LAB_URL: '10.128.24.47',
    AD_LAB_DOMAIN: '@gaeco.mppb.mp.br',
    AD_LAB_BASE_DN: 'dc=gaeco,dc=intranet',
}

export let API_CODES = {
    CODE_ERRO_500: 'ESERVER',
    CODE_PARAM_INVALIDO: 'EPARAMINVALID',
    CODE_RECURSO_NAO_ENCONTRADO: 'ENOTFOUND',
    CODE_ROTA_NAO_AUTORIZADA: 'ENOTAUTH',
    CODE_TOKEN_INVALIDO: 'ETOKENNOTVALID',
    CODE_TOKEN_NAO_ENCONTRADO: 'ETOKENNOTFOUND',
    CODE_HASH_INVALIDO: 'EHASHNOTVALID',
    CODE_HASH_NAO_ENCONTRADO: 'EHASHNOTFOUND',
    CODE_FALHA_LOGIN:'ELOGINFAILED',
    CODE_PAYLOAD_INVALIDO:'EPAYLOADNOTVALID',
    CODE_PASSWORD_INVALIDO:'EPASSWORDNOTVALID',
    CODE_RECAPTCHA_INVALIDO:'ERECAPTCHANOTVALID',
    CODE_RECAPTCHA_NAO_ENCONTRADO:'ERECAPTCHANOTFOUND',
    CODE_CPF_JA_CADASTRADO:'ECPFNOTVALID',

    CODE_RECURSO_ENCONTRADO: 'OK',
    CODE_SUCESSO: 'OK',
}

export let API_MSGS = {
    MSG_NOME_INSUFICIENTE: 'Parâmetro insuficiente, insira ao menos 2 nomes válidos.'.toUpperCase(),
    MSG_PARAM_INVALIDO: 'Parâmetro inválido.'.toUpperCase(),
    MSG_TOKEN_INVALIDO: 'Token inválido, refaça o login para validar seu acesso.'.toUpperCase(),
    MSG_TOKEN_NAO_ENCONTRADO: 'Token não encontrada, faça o login no sistema'.toUpperCase(),
    MSG_HASH_INVALIDO: 'Hash inválido.'.toUpperCase(),
    MSG_HASH_NAO_ENCONTRADO: 'Hash não encontrado, acesso não autorizado'.toUpperCase(),
    MSG_ACESSO_NAO_AUTORIZADO: 'Acesso não autorizado.'.toUpperCase(),
    MSG_ROTA_NAO_AUTORIZADA: 'Rota não autorizada para o usuário.'.toUpperCase(),
    MSG_FALHA_LOGIN: 'Login e/ou senha incorretos.'.toUpperCase(),
    MSG_LOGIN_SUCESSO: 'Login efetuado.'.toUpperCase(),
    MSG_IPC_TIMEOUT: 'IPC timeout.'.toUpperCase(),
    MSG_LOGIN_TIMEOUT: 'Login timeout.'.toUpperCase(),
    MSG_LOGIN_USUARIO_INATIVO: 'Usuário inativo, peça permissão ao administrador para ter acesso ao sistema.'.toUpperCase(),
    MSG_RECURSO_NAO_ENCONTRADO: 'Registro não encontrado.'.toUpperCase(),
    MSG_PAYLOAD_INVALIDO: 'Payload para acesso externo inválido.'.toUpperCase(),
    MSG_ACESSO_EXTERNO_TOKEN_INVALIDO: 'Falha ao validar token de acesso.'.toUpperCase(),
    MSG_ACESSO_EXTERNO_FALHA_CADASTRO_AUTOMATICO: 'Falha no cadastro automático do usuário'.toUpperCase(),

}

// HASH
// BANCO.TABELA.RECURSO (RECURSO É OPCIONAL, USADO PARA DIFERENCIAR SE A TABELA TIVER MAIS INFORMAÇÃO QUE
// PARA O RECURSO EM QUESTÃO)
export let MODEL_PRIORITY = {
    // PESSOA
    'ipc':                                      {rank: 0, grupo: 'pessoa',                  fonte: 'ipc'},
    'receitanovo.pessoafisica':                 {rank: 1, grupo: 'pessoa',                  fonte: 'receitanovo.pessoafisica'},
    'receita.pf':                               {rank: 2, grupo: 'pessoa',                  fonte: 'receita.pf'},
    'receita_full.pf':                          {rank: 3, grupo: 'pessoa',                  fonte: 'receita_full.pf'},
    'sispesquisa.pessoas':                      {rank: 4, grupo: 'pessoa',                  fonte: 'sispesquisa.pessoas'},
    'sispesquisa.cnh':                          {rank: 5, grupo: 'pessoa',                  fonte: 'sispesquisa.cnh'},
    'sispesquisa.emails.pessoa':                {rank: 6, grupo: 'pessoa',                  fonte: 'sispesquisa.emails'},
    'sispesquisa.prisional.pessoa':             {rank: 6, grupo: 'pessoa',                  fonte: 'sispesquisa.prisional'},
    'sispesquisa.telefones.pessoa':             {rank: 7, grupo: 'pessoa',                  fonte: 'sispesquisa.telefones'},
    'tse.pessoa':                               {rank: 2, grupo: 'pessoa',                  fonte: 'tse'},
    'vep.pessoa':                               {rank: 8, grupo: 'pessoa',                  fonte: 'vep'},
    'renach.pessoa':                            {rank: 2, grupo: 'pessoa',                  fonte: 'renach_2016_08'},

    // FOTO
    'ipc.foto':                                 {rank: 0, grupo: 'foto',                    fonte: 'ipc'},
    'sispesquisa.foto.renach':                  {rank: 0, grupo: 'foto',                    fonte: 'sispesquisa.foto.renach'},
    'sispesquisa.foto':                         {rank: 0, grupo: 'foto',                    fonte: 'sispesquisa.foto'},
    'sisdepen.foto':                            {rank: 0, grupo: 'foto',                    fonte: 'sisdepen.foto'},

    // EMPRESA
    'receitanovo.pessoajuridica':               {rank: 0, grupo: 'empresa',                 fonte: 'receitanovo.pessoajuridica' },
    'sispesquisa.telefones.empresa':            {rank: 1, grupo: 'empresa',                 fonte: 'sispesquisa.telefones' },
    'receita_full.pj':                          {rank: 1, grupo: 'empresa',                 fonte: 'receita_full.pj' },
    'receita.pj':                               {rank: 2, grupo: 'empresa',                 fonte: 'receita.pj' },

    // FILIAL
    'receitanovo.pessoajuridica.filial':        {rank: 0, grupo: 'filial',                 fonte: 'receitanovo.pessoajuridica' },

    // EMPRESA_RESPONSAVEL
    'receita_full.pj.empresa_responsavel':      {rank: 1, grupo: 'empresa_responsavel',     fonte: 'receita_full.pj' },
    'receitanovo.pj.empresa_responsavel':       {rank: 0, grupo: 'empresa_responsavel',     fonte: 'receitanovo.pessoajuridica' },

    // SOCIOS
    'receitanovo.sociopf':                      {rank: 0, grupo: 'socio_pf',                fonte: 'receitanovo.sociopf' },
    'receitanovo.sociopj':                      {rank: 0, grupo: 'socio_pj',                fonte: 'receitanovo.sociopj' },
    'receitanovo.socioestrangeiro':             {rank: 0, grupo: 'socio_estrangeiro',       fonte: 'receitanovo.socioestrangeiro' },
    'receita_full.sociopf':                     {rank: 1, grupo: 'socio_pf',                fonte: 'receita_full.sociopf' },
    'receita_full.sociopj':                     {rank: 1, grupo: 'socio_pj',                fonte: 'receita_full.sociopj' },
    'receita_full.socioestrangeiro':            {rank: 1, grupo: 'socio_estrangeiro',       fonte: 'receita_full.socioestrangeiro' },

    // ENDERECO
    'ipc.endereco':                             {rank: 0, grupo: 'endereco',                fonte: 'ipc' },
    'receitanovo.pessoafisica.endereco':        {rank: 0, grupo: 'endereco',                fonte: 'receitanovo.pessoafisica' },
    'receitanovo.pessoajuridica.endereco':      {rank: 0, grupo: 'endereco',                fonte: 'receitanovo.pessoajuridica' },
    'renach.endereco':                          {rank: 1, grupo: 'endereco',                fonte: 'renach_2016_08' },
    'sispesquisa.enderecos':                    {rank: 1, grupo: 'endereco',                fonte: 'sispesquisa.enderecos' },
    'receita_full.pf.endereco':                 {rank: 2, grupo: 'endereco',                fonte: 'receita_full.pf' },
    'receita_full.pj.endereco':                 {rank: 2, grupo: 'endereco',                fonte: 'receita_full.pj' },
    'receita.pf.endereco':                      {rank: 3, grupo: 'endereco',                fonte: 'receita.pf' },
    'receita.pj.endereco':                      {rank: 3, grupo: 'endereco',                fonte: 'receita.pj' },
    'sispesquisa.veiculos_novo.endereco':       {rank: 3, grupo: 'endereco',                fonte: 'sispesquisa.veiculos_19_07_2017' },
    'vep.endereco':                             {rank: 0, grupo: 'endereco',                fonte: 'vep' },

    // VIZINHO
    'receitanovo.vizinho':                      {rank: 0, grupo: 'vizinho',                 fonte: 'receitanovo' },

    // TELEFONE
    'ipc.telefone':                             {rank: 0, grupo: 'telefone',                fonte: 'ipc' },
    'sispesquisa.telefones':                    {rank: 0, grupo: 'telefone',                fonte: 'sispesquisa.telefones' },
    'receita_full.pf.telefone':                 {rank: 1, grupo: 'telefone',                fonte: 'receita_full.pf' },
    'receita_full.pj.telefone':                 {rank: 1, grupo: 'telefone',                fonte: 'receita_full.pj' },
    'receita.pf.telefone':                      {rank: 2, grupo: 'telefone',                fonte: 'receita.pf' },
    'receita.pj.telefone':                      {rank: 2, grupo: 'telefone',                fonte: 'receita.pj' },
    'vep.telefone':                             {rank: 0, grupo: 'telefone',                fonte: 'vep.telefone' },
    'dna.pj.telefone':                          {rank: 0, grupo: 'telefone',                fonte: 'dna.pj' },

    // PRESO
    'sispesquisa.prisional.preso':              {rank: 0,  grupo: 'preso',                  fonte: 'sispesquisa.prisional' },
    'sispesquisa.dbo.tbl_prontuarios':          {rank: 0,  grupo: 'preso',                  fonte: 'sispesquisa.dbo.tbl_prontuarios' },
    'sisdepen':                                 {rank: 0,  grupo: 'preso',                  fonte: 'sisdepen' },

    // BOLSA FAMILIA
    'filiacao':                                 {rank: 0, grupo: 'filiacao',                fonte: 'filiados_pb' },

    // BOLSA FAMILIA
    'bolsa_familia':                            {rank: 0, grupo: 'bolsa_familia',           fonte: 'bolsafamilia' },

    // EMPREGADO
    'empregador.rais':                          {rank: 0, grupo: 'empregador',              fonte: 'rais' },
    'empregador.caged':                         {rank: 0, grupo: 'empregador',              fonte: 'caged' },

    // CONSELHO
    'vep.advogado':                             {rank: 0, grupo: 'conselho',                fonte: 'vep' },

    // CONDENACOES
    'condenacao':                               {rank: 0, grupo: 'condenacao',              fonte: 'trf5.tre' },

    // PROCESSO
    'tjpb.processo':                            {rank: 0, grupo: 'processo',                fonte: 'tjpb' },
    'vep.processo':                             {rank: 0, grupo: 'processo',                fonte: 'vep' },

    // MANDADO
    'cnj.bnmp.mandado':                         {rank: 0, grupo: 'mandado',                 fonte: 'cnj.bnmp' },

    // RISCO MPPB
    'risco':                                    {rank: 0, grupo: 'risco',                   fonte: 'risco' },

    // CONTADOR
    'receita_full.contador':                    {rank: 0, grupo: 'contador',                fonte: 'receita_full.contador' },

    // TIPOLOGIA
    'tipologias.pf':                            {rank: 0, grupo: 'tipologia_pf',            fonte: 'tipologias.matriz_completa_pf' },
    'tipologias.pj':                            {rank: 0, grupo: 'tipologia_pj',            fonte: 'tipologias.matriz_completa_pj' },
    'tipologias.cacafantasmas':                 {rank: 0, grupo: 'tipologia',               fonte: 'tipologias' },

    // GEOCOORDENADAS
    'nfce_mapa_calor':                          {rank: 0, grupo: 'geocoordenadas',          fonte: 'nfce_mapa_calor.geocoordenadas' },

    // OBITO
    'sisobi.obito':                             {rank: 0, grupo: 'obito',                   fonte: 'sisobi.obito' },

    // QUADRO SOCIETARIO
    'historico_quadro_societario.cne':          {rank: 0, grupo: 'historico_quadro_societario', fonte: 'cne.dbo.procedures' },

    // SERVIDOR
    'sagres_municipal.v_folhapagamento':        {rank: 0, grupo: 'servidor_municipal',      fonte: 'sagres_municipal.v_folhapagamento' },
    'sagres_estadual.vw_folha':                 {rank: 0, grupo: 'servidor_estadual',       fonte: 'sagres_estadual.vw_folha' },

    'sispesquisa.servidores_federais':          {rank: 0, grupo: 'servidor_federal',        fonte: 'sispesquisa.servidores_federais' },
    'sispesquisa.servidores_federais_nordeste': {rank: 0, grupo: 'servidor_federal',        fonte: 'sispesquisa.servidores_federais_nordeste' },

    'sispesquisa.servidores_estaduais':         {rank: 1, grupo: 'servidor_estadual',       fonte: 'sispesquisa.servidores_estaduais' },
    'sispesquisa.servidores_municipais':        {rank: 1, grupo: 'servidor_municipal',      fonte: 'sispesquisa.servidores_municipais' },

    // FOLHA
    'sagres.folha':                             {rank: 0, grupo: 'folhapagamento',          fonte: 'sagres.folha' },

    // LICITACAO
    'sagres_municipal.licitacao':               {rank: 0, grupo: 'licitacao',               fonte: 'sagres_municipal.licitacao' },

    // EMPENHOS
    'sagres_municipal.credores':                {rank: 0, grupo: 'empenho_municipal',       fonte: 'sagres_municipal.credores' },
    'sagres_municipal.empenhos_estado':         {rank: 0, grupo: 'empenho_municipal',       fonte: 'sagres_municipal.empenhos_estado' },

    'sagres_estadual.credores_pb':              {rank: 0, grupo: 'empenho_estadual',        fonte: 'sagres_estadual.credores_pb' },
    'sagres_estadual.empenhos':                 {rank: 0, grupo: 'empenho_estadual',        fonte: 'sagres_estadual.empenhos' },

    'sagres.empenhos':                          {rank: 0, grupo: 'empenhos',                fonte: 'sagres.empenhos' },

    // VEICULO
    'sispesquisa.renavam_2016':                 {rank: 0, grupo: 'veiculo',                 fonte: 'sispesquisa.renavam_2016' },
    'sispesquisa.veiculos_novo':                {rank: 1, grupo: 'veiculo',                 fonte: 'sispesquisa.veiculos_19_07_2017' },
    'sispesquisa.veiculos':                     {rank: 2, grupo: 'veiculo',                 fonte: 'sispesquisa.veiculos' },

    // LOG
    'sispesquisa.logs':                         {rank: 0, grupo: 'log',                     fonte: 'sispesquisa.logs'},

    // USUARIO
    'sispesquisa.usuario':                      {rank: 0, grupo: 'usuario',                 fonte: 'sispesquisa.usuario'},

    // PRE USUARIO
    'sispesquisa.cadastro_usuario.preusuario':  {rank:0, grupo: 'preusuario',               fonte: 'sispesquisa.cadastro_usuario'},

    // REQUISICAO
    'integra.requisicoes':                      {rank:0, grupo: 'requisicao',               fonte: 'integra.requisicoes'},

    // RIF
    'rif':                                      {rank:0, grupo: 'rif',                      fonte: 'sispesquisa.rif'},

    // UTILS
    'utils':                                    {rank:0, grupo: 'utils',                    fonte: 'utils'},
    'geocoordenadas.google':                    {rank:0, grupo: 'geocoordenadas',           fonte: 'google.maps'},

    // OPERACOES
    'operacoes':                                {rank:0, grupo: 'operacao',                 fonte: 'sispesquisa.operacao'},

    // VIRTUAL
    // // EMAIL
    'vep.email':                                {rank:0, grupo: 'virtual',                 fonte: 'vep.email'},
    'sispesquisa.email':                        {rank:0, grupo: 'virtual',                 fonte: 'sispesquisa.email'},
    'dna.pj.email':                             {rank:0, grupo: 'virtual',                 fonte: 'dna.pj'},


    // CRAWLERS
    'crawler.jusbrasil':                        {rank:0, grupo: 'crawler',                 fonte: 'siapcrawlers.jusbrasil'},
    'crawler.googlesearch':                     {rank:0, grupo: 'crawler',                 fonte: 'siapcrawlers.googlesearch'},
    'crawler.facebook':                         {rank:0, grupo: 'crawler',                 fonte: 'siapcrawlers.facebook'},
    'crawler.google.jusbr':                     {rank:0, grupo: 'crawler',                 fonte: 'siapcrawlers.google.jusbr'},
    'crawler.google.mpbr':                      {rank:0, grupo: 'crawler',                 fonte: 'siapcrawlers.google.mpbr'},
    'crawler.google.edubr':                     {rank:0, grupo: 'crawler',                 fonte: 'siapcrawlers.google.edubr'},
    'crawler.google.govbr':                     {rank:0, grupo: 'crawler',                 fonte: 'siapcrawlers.google.govbr'},
}

export let LOG_CODES = {
    TIPO_SISTEMA:                               'SISTEMA',
    TIPO_PESQUISA:                              'PESQUISA',
    TIPO_INSERCAO:                              'INSERCAO',

    TIPO_INTEGRA:                               'INTEGRA',
    TIPO_TIPORANK:                              'TIPORANK',
    TIPO_RELACIONAMENTOS:                       'RELACIONAMENTOS',
    TIPO_CACAFANTASMAS:                         'CACA FANTASMAS',
    TIPO_DNA:                                   'DNA',

    TIPO_REGISTRO_NAO_ENCONTRADO:               'REGISTRO NAO ENCONTRADO',

    TIPO_TENTATIVA_CONSULTA:                    'TENTATIVA DE CONSULTA',
    TIPO_TENTATIVA_CONSULTA_SEM_PERFIL:         'TENTATIVA DE CONSULTA SEM PERFIL DE ACESSO',
    TIPO_TENTATIVA_INTEGRA_SEM_PERFIL:          'TENTATIVA DE USO DO INTEGRA SEM PERFIL DE ACESSO',
    TIPO_TENTATIVA_CONSULTA_ADMIN_SEM_PERFIL:   'TENTATIVA DE CONSULTA ROTA ADMIN SEM PERFIL DE ACESSO',
    TIPO_TENTATIVA_CONSULTA_HASH_INVALIDO:      'TENTATIVA DE CONSULTA - HASH INVALIDO',

    TIPO_LOGIN:                                 'LOGIN',
    TIPO_TENTATIVA_LOGIN:                       'TENTATIVA DE LOGIN',
    TIPO_LOGIN_EXTERNO:                         'LOGIN EXTERNO',
    TIPO_TENTATIVA_LOGIN_EXTERNO:               'TENTATIVA DE LOGIN EXTERNO',

    TIPO_CREATE_CADASTRO_USUARIO_EXTERNO:       'CADASTRO EXTERNO DE CADASTRO_USUÁRIO',
    TIPO_CREATE_CADASTRO_USUARIO:               'CADASTRO DE CADASTRO_USUÁRIO',
    TIPO_TENTATIVA_CREATE_CADASTRO_USUARIO:     'TENTATIVA DE CADASTRO DE CADASTRO_USUÁRIO',
    TIPO_CREATE_USUARIO_EXTERNO:                'CADASTRO EXTERNO DE USUÁRIO',
}

export let LOG_MSGS = {
    USUARIO_LISTA_USUARIOS:                     'USUARIO - LISTA USUÁRIOS',
    USUARIO_GET_USUARIO:                        'USUARIO - PEGA USUÁRIO',
    USUARIO_ATUALIZA_USUARIO:                   'USUARIO - ATUALIZA USUÁRIO',
    USUARIO_REMOVE_USUARIO:                     'USUARIO - REMOVE USUÁRIO',
    USUARIO_ATUALIZA_SENHA:                     'USUARIO - ATUALIZA SENHA',
    USUARIO_REDEFINIR_SENHA:                    'USUARIO - REDEFINIR SENHA',

    CADASTRO_USUARIO_LISTA_USUARIOS:            'CADASTRO_USUARIO - LISTA USUÁRIOS',
    CADASTRO_USUARIO_LISTA_USUARIOS_INATIVOS:   'CADASTRO_USUARIO - LISTA USUÁRIOS INATIVOS',
    CADASTRO_USUARIO_ATIVAR_USUARIO:            'CADASTRO_USUARIO - ATIVAR USUÁRIO',
    CADASTRO_USUARIO_REMOVE_USUARIO:            'CADASTRO_USUARIO - REMOVE USUÁRIO',
    CADASTRO_USUARIO_CADASTRO_USUARIO:          'CADASTRO_USUARIO - CADASTRO USUÁRIO',

    PESSOA_DETALHADO_CPF:                       'PESSOA - BUSCA DETALHADA POR CPF',
    PESSOA_SIMPLIFICADO_CPF:                    'PESSOA - BUSCA SIMPLIFICADA POR CPF',
    PESSOA_SIMPLIFICADO_NOME:                   'PESSOA - BUSCA SIMPLIFICADA POR NOME',
    PESSOA_SIMPLIFICADO_CNH:                    'PESSOA - BUSCA SIMPLIFICADA POR CNH',
    PESSOA_SIMPLIFICADO_TITULO:                 'PESSOA - BUSCA SIMPLIFICADA POR TITULO',
    PESSOA_SIMPLIFICADO_RG:                     'PESSOA - BUSCA SIMPLIFICADA POR RG',
    PESSOA_SIMPLIFICADO_NOME_PAI:               'PESSOA - BUSCA SIMPLIFICADA POR NOME DO PAI',
    PESSOA_SIMPLIFICADO_NOME_MAE:               'PESSOA - BUSCA SIMPLIFICADA POR NOME DA MÃE',
    PESSOA_SIMPLIFICADO_ENDERECO:               'PESSOA - BUSCA SIMPLIFICADA POR ENDERECO',
    PESSOA_SIMPLIFICADO_TELEFONE:               'PESSOA - BUSCA SIMPLIFICADA POR TELEFONE',
    PESSOA_SIMPLIFICADO_EMAIL:                  'PESSOA - BUSCA SIMPLIFICADA POR EMAIL',
    PESSOA_INTEGRADO_CPF:                       'PESSOA - BUSCA INTEGRADA POR CPF',

    EMPRESA_DETALHADO_CNPJ:                     'EMPRESA - BUSCA DETALHADA POR CNPJ',
    EMPRESA_SIMPLIFICADO_CNPJ:                  'EMPRESA - BUSCA SIMPLIFICADA POR CNPJ',
    EMPRESA_SIMPLIFICADO_RAZAO_SOCIAL:          'EMPRESA - BUSCA SIMPLIFICADA POR RAZÃO SOCIAL',
    EMPRESA_SIMPLIFICADO_NOME_FANTASIA:         'EMPRESA - BUSCA SIMPLIFICADA POR NOME FANTASIA',
    EMPRESA_SIMPLIFICADO_EMAIL:                 'EMPRESA - BUSCA SIMPLIFICADA POR EMAIL',
    EMPRESA_SIMPLIFICADO_ENDERECO:              'EMPRESA - BUSCA SIMPLIFICADA POR ENDERECO',
    EMPRESA_SIMPLIFICADO_TELEFONE:              'EMPRESA - BUSCA SIMPLIFICADA POR TELEFONE',
    EMPRESA_SIMPLIFICADO_SOCIOPF_NOME:          'EMPRESA - BUSCA SIMPLIFICADA POR NOME DO SÓCIOPF',
    EMPRESA_SIMPLIFICADO_SOCIOPF_CPF:           'EMPRESA - BUSCA SIMPLIFICADA POR CPF DO SÓCIOPF',
    EMPRESA_SIMPLIFICADO_SOCIOPJ_CNPJ:          'EMPRESA - BUSCA SIMPLIFICADA POR CNPJ DO SÓCIOPJ',
    EMPRESA_INTEGRADO_CNPJ:                     'EMPRESA - BUSCA INTEGRADA POR CNPJ',

    LOG_GET_LOGS:                               'LOG - LISTA LOGS',

    PRESO_DETALHADO_CPF:                        'PRESO - BUSCA DETALHADA POR CPF',
    PRESO_DETALHADO_CNC:                        'PRESO - BUSCA DETALHADA POR CNC',
    PRESO_SIMPLIFICADO_CPF:                     'PRESO - BUSCA SIMPLIFICADA POR CPF',
    PRESO_SIMPLIFICADO_CNC:                     'PRESO - BUSCA SIMPLIFICADA POR CNC',
    PRESO_SIMPLIFICADO_VULGO:                   'PRESO - BUSCA SIMPLIFICADA POR VULGO',
    PRESO_SIMPLIFICADO_NOME:                    'PRESO - BUSCA SIMPLIFICADA POR NOME',
    PRESO_SIMPLIFICADO_NOME_MAE:                'PRESO - BUSCA SIMPLIFICADA POR NOME DA MÃE',

    CACAFANTASMAS_UGESTORA:                     'CACAFANTASMAS - BUSCA POR UNIDADE GESTORA',

    DNA_CNPJ:                                   'DNA - BUSCA POR CNPJ',

    INP_UGESTORA:                               'INP - BUSCA POR UNIDADE GESTORA',
    INP_CPF:                                    'INP - BUSCA POR CPF',

    RELACIONAMENTO_LISTA:                       'RELACIONAMENTOS - BUSCA POR LISTA',
    RELACIONAMENTO_CPF:                         'RELACIONAMENTOS - BUSCA POR CPF',
    RELACIONAMENTO_CNPJ:                        'RELACIONAMENTOS - BUSCA POR CNPJ',
    RELACIONAMENTO_ENDERECO:                    'RELACIONAMENTOS - BUSCA POR ENDERECO',
    RELACIONAMENTO_TELEFONE:                    'RELACIONAMENTOS - BUSCA POR TELEFONE',
    RELACIONAMENTO_CDUGESTORA:                  'RELACIONAMENTOS - BUSCA POR CDUGESTORA',

    SOCIO_SIMPLIFICADO_CNPJ:                    'SOCIO - BUSCA SIMPLIFICADA POR CNPJ',

    MAPACONSUMO_GEOCOORDENADAS_CPF:             'MAPACONSUMO - BUSCA GEOCOORDENADAS POR CPF',

    RISCO_MPPB_QUESTAO:                         'RISCO MPPB - BUSCA QUESTOES POR NUMERO',
    RISCO_MPPB_INSERE_RESPOSTA:                 'RISCO MPPB - CADASTRA RESPOSTA',

    FOLHAPAGAMENTO_MUNICIPAL_CDUGESTORA:        'FOLHA DE PAGAMENTO MUNICIPAL - BUSCA SIMPLIFICADA POR CDUGESTORA',
    FOLHAPAGAMENTO_ESTADUAL_UGESTORA:           'FOLHA DE PAGAMENTO ESTADUAL - BUSCA SIMPLIFICADA POR UGESTORA',

    TELEFONE_SIMPLIFICADO_CPF:                  'TELEFONE - BUSCA SIMPLIFICADA POR CPF',
    TELEFONE_SIMPLIFICADO_CNPJ:                 'TELEFONE - BUSCA SIMPLIFICADA POR CNPJ',
    TELEFONE_SIMPLIFICADO_NOME:                 'TELEFONE - BUSCA SIMPLIFICADA POR NOME',
    TELEFONE_SIMPLIFICADO_RAZAO_SOCIAL:         'TELEFONE - BUSCA SIMPLIFICADA POR RAZÃO SOCIAL',
    TELEFONE_SIMPLIFICADO_TELEFONE:             'TELEFONE - BUSCA SIMPLIFICADA POR TELEFONE',
    TELEFONE_BUSCA_PROFUNDA_TELEFONE:           'TELEFONE - BUSCA PROFUNDA POR TELEFONE',

    VEICULO_DETALHADO_CNPJ:                     'VEICULO - BUSCA DETALHADA POR CNPJ',
    VEICULO_DETALHADO_CPF:                      'VEICULO - BUSCA DETALHADA POR CPF',
    VEICULO_DETALHADO_NOME:                     'VEICULO - BUSCA DETALHADA POR NOME',
    VEICULO_DETALHADO_CHASSI:                   'VEICULO - BUSCA DETALHADA POR CHASSI',
    VEICULO_DETALHADO_PLACA:                    'VEICULO - BUSCA DETALHADA POR PLACA',
    VEICULO_DETALHADO_RENAVAM:                  'VEICULO - BUSCA DETALHADA POR RENAVAM',

    OBITO_DETALHADO_CPF:                        'OBITO - BUSCA DETALHADA POR CPF',
    OBITO_SIMPLIFICADO_NOME:                    'OBITO - BUSCA SIMPLIFICADA POR NOME',
    OBITO_SIMPLIFICADO_CPF:                     'OBITO - BUSCA SIMPLIFICADA POR CPF',

    CONDENACAO_CPF:                             'CONDENAÇÃO - BUSCA SIMPLIFICADA POR CPF',

    EMPENHO_MUNICIPAL_CNPJ:                     'EMPENHO - BUSCA DETALHADA POR CNPJ',

    LICITACAO_MUNICIPAL_CNPJ:                   'LICITAÇÃO - BUSCA DETALHADA POR CNPJ',

    MANDADO_DETALHADO_CPF:                      'MANDADO - BUSCA DETALHADA POR CPF',
    MANDADO_SIMPLIFICADO_CPF:                   'MANDADO - BUSCA SIMPLIFICADA POR CPF',
    MANDADO_SIMPLIFICADO_NOME:                  'MANDADO - BUSCA SIMPLIFICADA POR NOME',
    MANDADO_SIMPLIFICADO_RG:                    'MANDADO - BUSCA SIMPLIFICADA POR RG',

    ENDERECO_SIMPLIFICADO_CNPJ:                 'ENDERECO - BUSCA SIMPLIFICADA POR CNPJ',
    ENDERECO_SIMPLIFICADO_CPF:                  'ENDERECO - BUSCA SIMPLIFICADA POR CPF',
    ENDERECO_SIMPLIFICADO_NOME:                 'ENDERECO - BUSCA SIMPLIFICADA POR NOME',
    ENDERECO_SIMPLIFICADO_RAZAO_SOCIAL:         'ENDERECO - BUSCA SIMPLIFICADA POR RAZÃO SOCIAL',

    TIPOLOGIA_SIMPLIFICADO_CPF:                 'TIPOLOGIA - BUSCA SIMPLIFICADA POR CPF',
    TIPOLOGIA_SIMPLIFICADO_CNPJ:                'TIPOLOGIA - BUSCA SIMPLIFICADA POR CNPJ',
    TIPOLOGIA_SIMPLIFICADO_MUNICIPIO:           'TIPOLOGIA - BUSCA SIMPLIFICADA POR MUNICIPIO',

    INTEGRA_CADASTRA_REQUISICAO:                'INTEGRA - CADASTRA REQUISIÇÃO',
    INTEGRA_LISTA_REQUISICOES:                  'INTEGRA - LISTA REQUISIÇÕES',
    INTEGRA_FINALIZA_REQUISICAO:                'INTEGRA - FINALIZA REQUISIÇÃO',
    INTEGRA_DOWNLOAD_REQUISICAO:                'INTEGRA - DOWNLOAD REQUISIÇÃO',
}