import { Request, Response } from 'express';
import * as moment from 'moment';

import * as logModel from './../../models/log.model';
import { Log } from './../../schemas/log.schema';

import {
    trataErro,
    agrupaEFiltraDuplicados,
    desagrupa,
    getId_Token,
    print,
    limitaNumeroResultados
} from './../../utils';

import {
    API_CONFIG,
    API_CODES,
    API_MSGS,
    LOG_CODES,
    LOG_MSGS
} from './../../config';

import {
    procuraTipologiasAnalise,
    procuraOrgaoSagresMunicipal,
    tipoValido,
} from './cacafantasmas.functions';


export let getTipologiasCacafantasmasUgestora = function (req : Request, res : Response){
    let cdugestora = req.params.cdugestora;
    cdugestora = cdugestora.replace(/[^0-9]/g,'');

    const tipoAnalise = req.query.tipoanalise;
    let processo      = req.query.processo;
    let dtinicio      = req.query.dtinicio;
    let dtfim         = req.query.dtfim;

    if (cdugestora && cdugestora.length === 6 && tipoValido(tipoAnalise)) {
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_CACAFANTASMAS, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.CACAFANTASMAS_UGESTORA}: ${cdugestora} - Análise: ${tipoAnalise} - ${dtinicio} a ${dtfim}`, user_agent: req.headers['user-agent'] }));
        }

        if (!dtinicio) { dtinicio = '20120101'; }
        if (!dtfim) { dtfim = moment().format('YYYYMMDD'); }

        procuraTipologiasAnalise(cdugestora, dtinicio, dtfim, tipoAnalise)

            .then(resultados => agrupaEFiltraDuplicados(resultados))
            .then(tipologias => desagrupa(tipologias))
            .then(pessoas => limitaNumeroResultados(pessoas))

            .then(tipologias => res.status(200).send(tipologias))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));

    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }

};

export let getOrgaoSagresMunicipal = function (req : Request, res : Response){
    let orgao = req.params.orgao;
    let processo = req.query.processo;

    if (orgao && orgao.length > 0) {
        // if (API_CONFIG.CFG_ENV == 'production') {
        //     logModel.salvaLogPorId(new Log({ ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_RELACIONAMENTOS, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.RELACIONAMENTO_CPF}: ${cpf}`, user_agent: req.headers['user-agent'] }));
        // }

        procuraOrgaoSagresMunicipal(orgao)

            .then(resultados => agrupaEFiltraDuplicados(resultados))
            .then(orgaos => desagrupa(orgaos))
            .then(pessoas => limitaNumeroResultados(pessoas))

            .then(orgaos => res.status(200).send(orgaos))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));

    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
};