import { Request, Response } from 'express';

import * as condenacaoModel from '../models/condenacao.model';
import * as logModel from '../models/log.model';
import { Log } from '../schemas/log.schema';

import {
    trataErro,
    filtraNaoEncontrados,
    desagrupa,
    getId_Token,
    agrupaEFiltraDuplicados,
    print,
    limitaNumeroResultados
} from './../utils';

import {
    API_CODES,
    API_MSGS,
    API_CONFIG,
    LOG_CODES,
    LOG_MSGS
} from '../config';

export let getCondenacoesCPF = function (req: Request, res: Response){
    let cpf = req.params.cpf;
    cpf = cpf.replace(/[^0-9]/g,'');
    let processo = req.query.processo;

    if (cpf && cpf.length === 11){
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_PESQUISA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.CONDENACAO_CPF}: ${cpf}`, user_agent: req.headers['user-agent'], processo: processo}));
        }

        Promise.all([condenacaoModel.getCondenacaoTREPBCPF(cpf),
                    condenacaoModel.getCondenacaoTRF5CPF(cpf)])

            .then(condenacoes => filtraNaoEncontrados(condenacoes))
            .then(condenacoes => agrupaEFiltraDuplicados(condenacoes))
            .then(condenacoes => desagrupa(condenacoes))
            .then(condenacoes => limitaNumeroResultados(condenacoes))

            .then(condenacoes => res.status(200).send(condenacoes))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
}

export let getCondenacoesTRF5CPF = function (req: Request, res: Response){
    let cpf = req.params.cpf;
    cpf = cpf.replace(/[^0-9]/g,'');
    let processo = req.query.processo;

    if (cpf && cpf.length === 11){
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_PESQUISA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.CONDENACAO_CPF}: ${cpf}`, user_agent: req.headers['user-agent'], processo: processo}));
        }

        Promise.all([condenacaoModel.getCondenacaoTRF5CPF(cpf)])

            .then(condenacoes => filtraNaoEncontrados(condenacoes))
            .then(condenacoes => agrupaEFiltraDuplicados(condenacoes))
            .then(condenacoes => desagrupa(condenacoes))
            .then(condenacoes => limitaNumeroResultados(condenacoes))

            .then(condenacoes => res.status(200).send(condenacoes))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
}

export let getCondenacoesTREPBCPF = function (req: Request, res: Response){
    let cpf = req.params.cpf;
    cpf = cpf.replace(/[^0-9]/g,'');
    let processo = req.query.processo;

    if (cpf && cpf.length === 11){
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_PESQUISA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.CONDENACAO_CPF}: ${cpf}`, user_agent: req.headers['user-agent'], processo: processo}));
        }

        Promise.all([condenacaoModel.getCondenacaoTREPBCPF(cpf)])

            .then(condenacoes => filtraNaoEncontrados(condenacoes))
            .then(condenacoes => agrupaEFiltraDuplicados(condenacoes))
            .then(condenacoes => desagrupa(condenacoes))
            .then(condenacoes => limitaNumeroResultados(condenacoes))

            .then(condenacoes => res.status(200).send(condenacoes))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
}
