import { Request, Response } from 'express';

import * as logModel from './../../models/log.model';
import { Log } from './../../schemas/log.schema';

import {
    trataErro,
    trataRequisicaNome,
    desagrupa,
    agrupaEFiltraDuplicados,
    getId_Token,
    print,
    filtraLogradouro,
    limitaNumeroResultados
} from './../../utils';

import {
    API_CONFIG,
    API_CODES,
    API_MSGS,
    LOG_CODES,
    LOG_MSGS
} from './../../config';

import {
    procuraEmpresaDetalhado,
    procuraEmpresaSimplificadoCNPJ,
    procuraEmpresaSimplificadoRazaoSocial,
    procuraEmpresaSimplificadoNomeFantasia,
    procuraEmpresaSimplificadoNomeSocioPF,
    procuraEmpresaSimplificadoCPFSocioPF,
    procuraEmpresaSimplificadoCNPJSocioPJ,
    procuraEmpresaIntegradoCNPJ,
    procuraEmpresaSimplificadoLogradouro,
    procuraEmpresaSimplificadoEmail,
    procuraEmpresaSimplificadoTelefone,
} from './empresas.functions';

export let getEmpresaDetalhadoCNPJ = function (req : Request, res : Response){
    let cnpj = req.params.cnpj;
    cnpj = cnpj.replace(/[^0-9]/g,'');
    let processo = req.query.processo;

    if (cnpj && cnpj.length === 14){
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_PESQUISA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.EMPRESA_DETALHADO_CNPJ}: ${cnpj}`, user_agent: req.headers['user-agent'], processo: processo}));
        }

        procuraEmpresaDetalhado(cnpj)

            .then(empresas => agrupaEFiltraDuplicados(empresas))
            .then(empresas => desagrupa(empresas))
            .then(empresas => limitaNumeroResultados(empresas))

            .then(empresas => res.status(200).send(empresas))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
};

export let getEmpresaSimplificadoCNPJ = function (req : Request, res : Response){
    let cnpj = req.params.cnpj;
    cnpj = cnpj.replace(/[^0-9]/g,'');
    let processo = req.query.processo;

    if (cnpj && cnpj.length === 14){
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_PESQUISA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.EMPRESA_SIMPLIFICADO_CNPJ}: ${cnpj}`, user_agent: req.headers['user-agent'], processo: processo}));
        }

        procuraEmpresaSimplificadoCNPJ(cnpj)

            .then(empresas => agrupaEFiltraDuplicados(empresas))
            .then(empresas => desagrupa(empresas))
            .then(empresas => limitaNumeroResultados(empresas))

            .then(empresas => res.status(200).send(empresas))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
};

export let getEmpresaSimplificadoRazaoSocial = function (req : Request, res : Response){
    let razaoSocial = req.params.razaosocial;
    let razaoSocialArray = trataRequisicaNome(razaoSocial);
    let processo = req.query.processo;

    if (razaoSocialArray.length < 2){
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_NOME_INSUFICIENTE));
    }else{
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_PESQUISA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.EMPRESA_SIMPLIFICADO_RAZAO_SOCIAL}: ${razaoSocial}`, user_agent: req.headers['user-agent'], processo: processo}));
        }

        procuraEmpresaSimplificadoRazaoSocial(razaoSocial)

            .then(empresas => agrupaEFiltraDuplicados(empresas))
            .then(empresas => desagrupa(empresas))
            .then(empresas => limitaNumeroResultados(empresas))

            .then(empresas => res.status(200).send(empresas))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
    }
};

export let getEmpresaSimplificadoNomeFantasia = function (req : Request, res : Response){
    let nomeFantasia = req.params.nomefantasia;
    let nomeFantasiaArray = trataRequisicaNome(nomeFantasia);
    let processo = req.query.processo;

    if (nomeFantasiaArray.length < 2){
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_NOME_INSUFICIENTE));
    }else{
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_PESQUISA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.EMPRESA_SIMPLIFICADO_NOME_FANTASIA}: ${nomeFantasia}`, user_agent: req.headers['user-agent'], processo: processo}));
        }

        procuraEmpresaSimplificadoNomeFantasia(nomeFantasia)

            .then(empresas => agrupaEFiltraDuplicados(empresas))
            .then(empresas => desagrupa(empresas))
            .then(empresas => limitaNumeroResultados(empresas))

            .then(empresas => res.status(200).send(empresas))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
    }
};

export let getEmpresaSimplificadoTelefone = function (req: Request, res: Response){
    let telefone = req.params.telefone;
    let processo = req.query.processo;

    if (telefone){
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_PESQUISA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.EMPRESA_SIMPLIFICADO_TELEFONE}: ${telefone}`, user_agent: req.headers['user-agent'], processo: processo}));
        }

        procuraEmpresaSimplificadoTelefone(telefone)

            .then(empresas => agrupaEFiltraDuplicados(empresas))
            .then(empresas => desagrupa(empresas))
            .then(empresas => limitaNumeroResultados(empresas))

            .then(empresas => res.status(200).send(empresas))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
}

export let getEmpresaSimplificadoEmail = function (req : Request, res : Response){
    let email    = req.params.email;
    let processo = req.query.processo;

    if (!email){
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_NOME_INSUFICIENTE));
    }else{
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_PESQUISA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.EMPRESA_SIMPLIFICADO_EMAIL}: ${email}`, user_agent: req.headers['user-agent'], processo: processo}));
        }

        procuraEmpresaSimplificadoEmail(email)

            .then(empresas => agrupaEFiltraDuplicados(empresas))
            .then(empresas => desagrupa(empresas))
            .then(empresas => limitaNumeroResultados(empresas))

            .then(empresas => res.status(200).send(empresas))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
    }
};

export let getEmpresaSimplificadoLogradouro = function (req : Request, res : Response){
    let endereco = req.params.endereco;
    let enderecoArray = filtraLogradouro(endereco);
    let processo = req.query.processo;

    if (enderecoArray.length < 2){
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_NOME_INSUFICIENTE));
    }else{
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_PESQUISA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.EMPRESA_SIMPLIFICADO_ENDERECO}: ${endereco}`, user_agent: req.headers['user-agent'], processo: processo}));
        }

        procuraEmpresaSimplificadoLogradouro(endereco)

            .then(empresas => agrupaEFiltraDuplicados(empresas))
            .then(empresas => desagrupa(empresas))
            .then(empresas => limitaNumeroResultados(empresas))

            .then(empresas => res.status(200).send(empresas))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
    }
};

export let getEmpresaSimplificadoSocioPFNome = function (req : Request, res : Response){
    let nome = req.params.nome;
    let nomeArray = trataRequisicaNome(nome);
    let processo = req.query.processo;

    if (nomeArray.length < 2){
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_NOME_INSUFICIENTE));
    }else{
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_PESQUISA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.EMPRESA_SIMPLIFICADO_SOCIOPF_NOME}: ${nome}`, user_agent: req.headers['user-agent'], processo: processo}));
        }

        procuraEmpresaSimplificadoNomeSocioPF(nome)

            .then(empresas => agrupaEFiltraDuplicados(empresas))
            .then(empresas => desagrupa(empresas))
            .then(empresas => limitaNumeroResultados(empresas))

            .then(empresas => res.status(200).send(empresas))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
    }
};

export let getEmpresaSimplificadoSocioPFCPF = function (req: Request, res: Response){
    let cpf = req.params.cpf;
    cpf = cpf.replace(/[^0-9]/g,'');
    let processo = req.query.processo;

    if (cpf && cpf.length === 11){
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_PESQUISA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.EMPRESA_SIMPLIFICADO_SOCIOPF_CPF}: ${cpf}`, user_agent: req.headers['user-agent'], processo: processo}));
        }

        procuraEmpresaSimplificadoCPFSocioPF(cpf)

            .then(empresas => agrupaEFiltraDuplicados(empresas))
            .then(empresas => desagrupa(empresas))
            .then(empresas => limitaNumeroResultados(empresas))

            .then(empresas => res.status(200).send(empresas))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
}

export let getEmpresaSimplificadoSocioPJCNPJ = function (req: Request, res: Response){
     let cnpj = req.params.cnpj;
    cnpj = cnpj.replace(/[^0-9]/g,'');
    let processo = req.query.processo;

    if (cnpj && cnpj.length === 14){
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_PESQUISA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.EMPRESA_SIMPLIFICADO_SOCIOPJ_CNPJ}: ${cnpj}`, user_agent: req.headers['user-agent'], processo: processo}));
        }

        procuraEmpresaSimplificadoCNPJSocioPJ(cnpj)

            .then(empresas => agrupaEFiltraDuplicados(empresas))
            .then(empresas => desagrupa(empresas))
            .then(empresas => limitaNumeroResultados(empresas))

            .then(empresas => res.status(200).send(empresas))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
}

export let getEmpresaIntegradoCNPJ = function (req: Request, res: Response){
    let cnpj = req.params.cnpj;
    cnpj = cnpj.replace(/[^0-9]/g,'');
    let processo = req.query.processo;

    if (cnpj && cnpj.length === 14){
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_PESQUISA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.EMPRESA_INTEGRADO_CNPJ}: ${cnpj}`, user_agent: req.headers['user-agent'], processo: processo}));
        }

        procuraEmpresaIntegradoCNPJ(cnpj)
            .then(resultados => agrupaEFiltraDuplicados(resultados, ['endereco', 'telefone']))

            .then(empresas => res.status(200).send(empresas))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
}
