import * as empresaModel from './../../models/empresa.model';
import * as filialModel from './../../models/empresa.filial.model';
import * as enderecoModel from './../../models/endereco.model';
import * as telefoneModel from './../../models/telefone.model';
import * as socioModel from './../../models/socio.model';
import * as veiculoModel from './../../models/veiculo.model';
import * as tipologiaModel from './../../models/tipologia.model';
import * as empenhoModel from './../../models/empenho.model';
import * as rifModel from './../../models/rif.model';
import * as operacaoModel from './../../models/operacao.model';
import * as quadroSocietarioModel from './../../models/historico_quadro_societario.model';
import * as empregadorModel from './../../models/empregador.model';
import * as virtualModel from './../../models/virtual.model';
import * as contadorModel from './../../models/contador.model';
import * as crawlersModel from './../../models/crawler.model';

import {
    filtraNaoEncontrados,
    desagrupa,
    agrupaEFiltraDuplicados,
    trataRequisicaNome,
    toTextSearch,
    registraNaoEncontrados,
    print,
    filtraLogradouro,
} from './../../utils';


export let procuraEmpresaDetalhado = function (cnpj) {
    cnpj = cnpj.replace(/[^0-9]/g, '');

    return Promise.all([
        empresaModel.getEmpresaDetalhadoCNPJ_ReceitaNovo_PessoaJuridica(cnpj),
        empresaModel.getEmpresaDetalhadoCNPJ_Receita_PJ(cnpj),
        empresaModel.getEmpresaDetalhadoCNPJ_ReceitaFull_PJ(cnpj)
    ])
        .then(empresas => filtraNaoEncontrados(empresas))
        .then(empresas => registraNaoEncontrados(empresas, 'CNPJ', cnpj))
}

export let procuraEmpresaSimplificadoCNPJ = function (cnpj) {
    cnpj = cnpj.replace(/[^0-9]/g, '');

    return Promise.all([
        empresaModel.getEmpresaSimplificadoCNPJ_ReceitaNovo_PessoaJuridica(cnpj),
        empresaModel.getEmpresaSimplificadoCNPJ_Receita_PJ(cnpj),
        empresaModel.getEmpresaSimplificadoCNPJ_ReceitaFull_PJ(cnpj)
    ])
        .then(empresas => filtraNaoEncontrados(empresas))
        .then(empresas => registraNaoEncontrados(empresas, 'CNPJ', cnpj))
}

export let procuraEmpresaSimplificadoRazaoSocial = function (razaoSocial) {
    let razaoSocialTextSearch = toTextSearch(trataRequisicaNome(razaoSocial));

    return Promise.all([
        empresaModel.getEmpresaSimplificadoRazaoSocial_ReceitaNovo_PessoaJuridica(razaoSocialTextSearch),
        empresaModel.getEmpresaSimplificadoRazaoSocial_Receita_PJ(razaoSocialTextSearch),
        empresaModel.getEmpresaSimplificadoRazaoSocial_ReceitaFull_PJ(razaoSocialTextSearch)
    ])
        .then(empresas => filtraNaoEncontrados(empresas))
}

export let procuraEmpresaSimplificadoNomeFantasia = function (nomeFantasia) {
    let nomeFantasiaTextSearch = toTextSearch(trataRequisicaNome(nomeFantasia));

    return Promise.all([
        empresaModel.getEmpresaSimplificadoNomeFantasia_ReceitaNovo_PessoaJuridica(nomeFantasiaTextSearch),
        empresaModel.getEmpresaSimplificadoNomeFantasia_Receita_PJ(nomeFantasiaTextSearch),
        empresaModel.getEmpresaSimplificadoNomeFantasia_ReceitaFull_PJ(nomeFantasiaTextSearch)
    ])
        .then(empresas => filtraNaoEncontrados(empresas))
}

export let procuraEmpresaSimplificadoTelefone = function (telefone) {
    telefone = telefone.replace(/[^0-9]/g, '');

    return Promise.all([
        empresaModel.getEmpresaTelefone_Sispesquisa_Telefones(telefone)
    ])
        .then(pessoas => filtraNaoEncontrados(pessoas))
}

export let procuraEmpresaSimplificadoEmail = function (email) {

    return Promise.all([
        empresaModel.getEmpresaEmail_Sispesquisa_Emails(email)
    ])
        .then(pessoas => filtraNaoEncontrados(pessoas))
}

export let procuraEmpresaSimplificadoLogradouro = function (logradouro) {
    let logradouroTextSearch = toTextSearch(filtraLogradouro(logradouro));

    return Promise.all([
        empresaModel.getEmpresaSimplificadoLogradouro_ReceitaNovo_PessoaJuridica(logradouroTextSearch),
    ])
        .then(empresas => filtraNaoEncontrados(empresas))
}

export let procuraEmpresaSimplificadoNomeSocioPF = function (nome) {
    let nomeTextSearch = toTextSearch(trataRequisicaNome(nome));

    return Promise.all([
        empresaModel.getEmpresaSimplificadoSocioPFNome_ReceitaFull(nomeTextSearch),
        empresaModel.getEmpresaSimplificadoSocioPFNome_ReceitaNovo(nomeTextSearch),
    ])
        .then(empresas => filtraNaoEncontrados(empresas))
}

export let procuraEmpresaSimplificadoCPFSocioPF = function (cpf) {
    cpf = cpf.replace(/[^0-9]/g, '');

    return Promise.all([
        empresaModel.getEmpresaSimplificadoSocioPFCPF_ReceitaFull(cpf),
        empresaModel.getEmpresaSimplificadoSocioPFCPF_ReceitaNovo(cpf),
    ])
        .then(empresas => filtraNaoEncontrados(empresas))
}

export let procuraEmpresaSimplificadoCNPJSocioPJ = function (cnpj) {
    cnpj = cnpj.replace(/[^0-9]/g, '');

    return Promise.all([
        empresaModel.getEmpresaSimplificadoSocioPJCNPJ_ReceitaFull(cnpj),
        empresaModel.getEmpresaSimplificadoSocioPJCNPJ_ReceitaNovo(cnpj),

    ])
        .then(empresas => filtraNaoEncontrados(empresas))
}


export let procuraEmpresaIntegradoCNPJ = function (cnpj) {
    cnpj = cnpj.replace(/[^0-9]/g, '');

    return Promise.all([
        //empresaModel.getEmpresaDetalhadoCNPJ_Receita_PJ(cnpj),
        empresaModel.getEmpresaDetalhadoCNPJ_ReceitaFull_PJ(cnpj),
        empresaModel.getEmpresaDetalhadoCNPJ_ReceitaNovo_PessoaJuridica(cnpj),

        // FILIAIS
        filialModel.getFilialEmpresaDetalhadoCNPJ_ReceitaNovo_PessoaJuridica(cnpj),

        // CONTADOR
        contadorModel.getContadorCNPJ_Receita_Full(cnpj),

        // TELEFONE
        telefoneModel.getTelefoneCNPJ_ReceitaFull_PJ(cnpj),
        telefoneModel.getTelefoneCNPJ_Sispesquisa_Telefones(cnpj),
        telefoneModel.getTelefoneCNPJ_DNA_PJ(cnpj),

        // ENDERECO
        enderecoModel.getEnderecoCNPJ_Receita_PJ(cnpj),
        enderecoModel.getEnderecoCNPJ_ReceitaFull_PJ(cnpj),
        enderecoModel.getEnderecoCNPJ_ReceitaNovo_PessoaJuridica(cnpj),
        enderecoModel.getEnderecoCNPJ_Sispesquisa_VeiculosNovo(cnpj),
        enderecoModel.getEnderecoCNPJ_Sispesquisa_Enderecos(cnpj),

        // SOCIOS
        socioModel.getSocioPFEmpresaSimplificadoCNPJ_ReceitaNovo(cnpj),
        socioModel.getSocioPJEmpresaSimplificadoCNPJ_ReceitaNovo(cnpj),
        socioModel.getSocioEstrangeiroEmpresaSimplificadoCNPJ_ReceitaNovo(cnpj),

        //socioModel.getSocioPFEmpresaSimplificadoCNPJ_ReceitaFull(cnpj),
        //socioModel.getSocioPJEmpresaSimplificadoCNPJ_ReceitaFull(cnpj),
        //socioModel.getSocioEstrangeiroEmpresaSimplificadoCNPJ_ReceitaFull(cnpj),

        // EMPENHO
        empenhoModel.getEmpenhoSimplificadoCNPJ_SAGRES_MUNICIPAL_Credores(cnpj),
        empenhoModel.getEmpenhoSimplificadoCNPJ_SAGRES_ESTADUAL_Credores_PB(cnpj),

        // VEICULO
        veiculoModel.getVeiculoDetalhadoCNPJ_Sispesquisa_Veiculos(cnpj),
        veiculoModel.getVeiculoDetalhadoCNPJ_Sispesquisa_VeiculosNovo(cnpj),
        veiculoModel.getVeiculoDetalhadoCNPJ_Renavam_2016(cnpj),

        // HISTORICO QUADRO SOCIETARIO
        quadroSocietarioModel.getHistoricoQuadroPFDetalhadoCNPJ_CNE(cnpj),
        quadroSocietarioModel.getHistoricoQuadroPJDetalhadoCNPJ_CNE(cnpj),

        //EMPREGADOR ()
        empregadorModel.getEstatisticasEmpregadoresCNPJ_Caged(cnpj),
        empregadorModel.getEstatisticasEmpregadoresCNPJ_RAIS(cnpj),

        // TIPOLOGIA
        tipologiaModel.getTipologiaSimplificadoCNPJ_Tipologias(cnpj),

        // RIF
        rifModel.existeRIFCNPJ_Sispesquisa_RIF(cnpj),

        // OPERACOES
        operacaoModel.getOperacoesCNPJ_Sispesquisa_Operacoes(cnpj),

        // VIRTUAL
        virtualModel.getEmailCNPJ_DNA_PJ(cnpj),


        // CRAWLERS
        //crawlersModel.getGoogleEduBRCNPJ_SIAP_CRAWLERS(cnpj),
        //crawlersModel.getGoogleGovBRCNPJ_SIAP_CRAWLERS(cnpj),
        //crawlersModel.getGoogleJusBRCNPJ_SIAP_CRAWLERS(cnpj),
        //crawlersModel.getGoogleMPBRCNPJ_SIAP_CRAWLERS(cnpj),

        //crawlersModel.getGoogleCNPJ_SIAP_CRAWLERS(cnpj),
        //crawlersModel.getJusBrasilCNPJ_SIAP_CRAWLERS(cnpj),

    ])
        .then(empresas => filtraNaoEncontrados(empresas))
        .then(empresas => registraNaoEncontrados(empresas, 'CNPJ', cnpj))
}

