import { Request, Response } from 'express';

import * as enderecoModel from '../models/endereco.model';
import * as logModel from '../models/log.model';
import { Log } from '../schemas/log.schema';

import {
    trataErro,
    filtraNaoEncontrados,
    desagrupa,
    agrupaEFiltraDuplicados,
    getId_Token,
    print,
    limitaNumeroResultados
} from './../utils';

import {
    API_CONFIG,
    API_CODES,
    API_MSGS,
    LOG_CODES,
    LOG_MSGS
} from '../config';

export let getEnderecoSimplificadoCNPJ = function (req : Request, res : Response){
    let cnpj = req.params.cnpj;
    cnpj = cnpj.replace(/[^0-9]/g,'');
    let processo = req.query.processo;

    if (cnpj && cnpj.length === 14){
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_PESQUISA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.ENDERECO_SIMPLIFICADO_CNPJ}: ${cnpj}`, user_agent: req.headers['user-agent'], processo: processo}));
        }

        Promise.all([
            enderecoModel.getEnderecoCNPJ_Receita_PJ(cnpj),
            enderecoModel.getEnderecoCNPJ_ReceitaFull_PJ(cnpj),
            enderecoModel.getEnderecoCNPJ_Sispesquisa_VeiculosNovo(cnpj)
        ])

            .then(enderecos => filtraNaoEncontrados(enderecos))
            .then(enderecos => agrupaEFiltraDuplicados(enderecos))
            .then(enderecos => desagrupa(enderecos))
            .then(enderecos => limitaNumeroResultados(enderecos))

            .then(enderecos => res.status(200).send(enderecos))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
};

export let getEnderecoSimplificadoCPF = function (req : Request, res : Response){
    let cpf = req.params.cpf;
    cpf = cpf.replace(/[^0-9]/g,'');
    let processo = req.query.processo;

    if (cpf && cpf.length === 11){
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_PESQUISA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.ENDERECO_SIMPLIFICADO_CPF}: ${cpf}`, user_agent: req.headers['user-agent'], processo: processo}));
        }

        Promise.all([
            enderecoModel.getEnderecoCPF_ReceitaFull_PF(cpf),
            enderecoModel.getEnderecoCPF_Receita_PF(cpf),
            enderecoModel.getEnderecoCPF_ReceitaNovo_PessoaFisica(cpf),
            enderecoModel.getEnderecoCPF_Sispesquisa_VeiculosNovo(cpf),
            enderecoModel.getEnderecoCPF_Sispesquisa_Enderecos(cpf),
            enderecoModel.getEnderecoCPF_Renach_2016_08(cpf),
            enderecoModel.getEnderecoCPF_VEP(cpf),
        ])
            .then(enderecos => filtraNaoEncontrados(enderecos))
            .then(enderecos => agrupaEFiltraDuplicados(enderecos, []))
            .then(enderecos => desagrupa(enderecos))
            .then(enderecos => limitaNumeroResultados(enderecos))

            .then(enderecos => res.status(200).send(enderecos))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
};

// export let getEnderecoSimplificadoNome = function (req : Request, res : Response){

//     let nome = req.params.nome;
//     let nomeArray = trataRequisicaNome(nome);
//     let processo = req.query.processo;

//     if (nomeArray.length < 2){
//         res.status(200).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_NOME_INSUFICIENTE));
//     }else{
//         if (API_CONFIG.CFG_ENV == 'production') {
//             logModel.salvaLogPorId(new Log({ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_PESQUISA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.PESSOA_SIMPLIFICADO_NOME}: ${nome}`, user_agent: req.headers['user-agent'], processo: processo}));
//         }

//         let nomeTextSearch = toTextSearch(nomeArray);

//         Promise.all([enderecoModel.getPessoaSimplificadoNome_Sispesquisa_Pessoas(nomeTextSearch),
//                      ])

//             .then(enderecos => filtraNaoEncontrados(enderecos))
//             .then(enderecos => agrupaEFiltraDuplicados(enderecos))
//             .then(enderecos => desagrupa(enderecos))

//             .then(enderecos => res.status(200).send(enderecos))
//             .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)))
//     }

// };

export let insertEnderecoSimplificadoCPF = function (req : Request, res : Response){
    let form = req.body;
    let cpf = form.cpf;
    let cep = form.cep;
    cpf = cpf.replace(/[^0-9]/g,'');
    cep = cep.replace(/[^0-9]/g,'');

    if (cpf && cpf.length === 11){
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_INSERCAO, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ENDEREÇO: ${cpf}`, user_agent: req.headers['user-agent'], processo: null}));
        }
        const fonte = `OP${(new Date()).getFullYear()}`;

        enderecoModel.insertEnderecoCPF_Sispesquisa_Enderecos({
            cpf: cpf,
            logradouro: form.logradouro,
            numeroLogradouro: form.numeroLogradouro,
            complemento: form.complemento,
            bairro: form.bairro,
            cep: cep,
            municipio: form.municipio,
            uf: form.uf,
            fonte: fonte
        })

            .then(resultado => res.status(200).send(resultado))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
};
