import { Request, Response } from 'express';

import * as folhapagamentoModel from '../models/folhapagamento.model';
import * as logModel from '../models/log.model';
import { Log } from '../schemas/log.schema';

import {
    trataErro,
    filtraNaoEncontrados,
    desagrupa,
    agrupaEFiltraDuplicados,
    getId_Token,
    print,
    limitaNumeroResultados
} from './../utils';

import {
    API_CODES,
    API_MSGS,
    API_CONFIG,
    LOG_CODES,
    LOG_MSGS
} from '../config';

export let getFolhaMunicipalCdugestoraMesAno = function (req: Request, res: Response){
    let processo   = req.query.processo;
    let cdugestora = req.query.cdugestora;
    let mes        = req.query.mes;
    let ano        = req.query.ano;

    if (cdugestora && mes && ano){
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_PESQUISA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.FOLHAPAGAMENTO_MUNICIPAL_CDUGESTORA}: ${cdugestora} de ${mes + ano}`, user_agent: req.headers['user-agent'], processo: processo}));
        }

        Promise.all([folhapagamentoModel.getFolhaMunicipalMesAnoCdugestora(cdugestora, mes, ano)])

            .then(folha => filtraNaoEncontrados(folha))
            .then(folha => agrupaEFiltraDuplicados(folha))
            .then(folha => desagrupa(folha))
            .then(folha => limitaNumeroResultados(folha))

            .then(folha => res.status(200).send(folha))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
}

export let getFolhaEstadualUgestoraMesAno = function (req: Request, res: Response){
    let processo   = req.query.processo;
    let ugestora = req.query.ugestora;
    let mes        = req.query.mes;
    let ano        = req.query.ano;

    if (ugestora && mes && ano){
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_PESQUISA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.FOLHAPAGAMENTO_ESTADUAL_UGESTORA}: ${ugestora} de ${mes + ano}`, user_agent: req.headers['user-agent'], processo: processo}));
        }

        Promise.all([folhapagamentoModel.getFolhaEstadualMesAnoUgestora(ugestora, mes, ano)])

            .then(folha => filtraNaoEncontrados(folha))
            .then(folha => agrupaEFiltraDuplicados(folha))
            .then(folha => desagrupa(folha))
            .then(folha => limitaNumeroResultados(folha))

            .then(folha => res.status(200).send(folha))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
}