import { Request, Response } from 'express';

import * as geocoordenadasModel from '../models/geocoordenadas.model';
import * as logModel from '../models/log.model';
import { Log } from '../schemas/log.schema';

import {
    trataErro,
    filtraNaoEncontrados,
    desagrupa,
    agrupaEFiltraDuplicados,
    getId_Token,
    print,
    limitaNumeroResultados
} from './../utils';

import {
    API_CODES,
    API_MSGS,
    API_CONFIG,
    LOG_CODES,
    LOG_MSGS
} from '../config';

export let getGeoCoordenadasEmpresasCPF = function (req: Request, res: Response){
    let cpf        = req.query.cpf;
    let datainicio = req.query.dtinicio;
    let datafim    = req.query.dtfim;

    if (cpf){
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_PESQUISA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.MAPACONSUMO_GEOCOORDENADAS_CPF}: ${cpf} `, user_agent: req.headers['user-agent'], processo: null}));
        }

        Promise.all([geocoordenadasModel.getGeocoordenadasMapaCalorCPFPeriodo(cpf, datainicio, datafim)])

            .then(geo => filtraNaoEncontrados(geo))
            .then(geo => agrupaEFiltraDuplicados(geo))
            .then(geo => desagrupa(geo))
            .then(geo => limitaNumeroResultados(geo))

            .then(geo => res.status(200).send(geo))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
}