import { Request, Response } from 'express';

import { mailer }  from '../services/mail.service';
import { Log } from '../schemas/log.schema';
import * as logModel from '../models/log.model';
import * as integraModel from '../models/integra.model';
import * as utilsModel from '../models/utils/utils.model';

import {
    trataErro,
    agrupaEFiltraDuplicados,
    filtraNaoEncontrados,
    desagrupa,
    getId_Token,
    print,
} from '../utils';

import {
    API_CODES,
    API_MSGS,
    API_CONFIG,
    LOG_CODES,
    LOG_MSGS
} from '../config';

export let cadastraRequisicaoIntegra = function (req: any, res: Response){
    let requisicao = req.body;
    let listaAnexos = req.files;

    if (requisicao){
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_INTEGRA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.INTEGRA_CADASTRA_REQUISICAO}`, user_agent: req.headers['user-agent']}));
        }

        Promise.all([integraModel.insereRequisicao(requisicao)])
            .then(idRequisicao => {
                JSON.parse(requisicao.listaImovel).forEach(imovel => integraModel.insereImovel(imovel, idRequisicao));
                return idRequisicao;
            })
            .then(idRequisicao => {
                JSON.parse(requisicao.listaVeiculo).forEach(veiculo => integraModel.insereVeiculo(veiculo, idRequisicao));
                return idRequisicao;
            })
            .then(idRequisicao => {
                JSON.parse(requisicao.listaPF).forEach(pf => integraModel.inserePF(pf, idRequisicao));
                return idRequisicao;
            })
            .then(idRequisicao => {
                JSON.parse(requisicao.listaPJ).forEach(pj => integraModel.inserePJ(pj, idRequisicao));
                return idRequisicao;
            })
            .then(idRequisicao => {
                listaAnexos.forEach(anexo => integraModel.insereAnexo(anexo, idRequisicao));
                return idRequisicao;
            })
            // .then(() => mailer.send({
            //     template: 'services/mail',
            //     message: {
            //         to: 'augustovanucci@gmail.com'
            //     },
            //     locals: requisicao
            // }))
            // .then()
            .then(() => res.status(200).send({status: API_CODES.CODE_SUCESSO,  msg: 'Requisição cadastrada com sucesso.'}))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
}

export let listaRequisicoesIntegra = function (req: Request, res: Response){

    if (API_CONFIG.CFG_ENV == 'production') {
        logModel.salvaLogPorId(new Log({ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_INTEGRA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.INTEGRA_LISTA_REQUISICOES}`, user_agent: req.headers['user-agent']}));
    }

    Promise.all([integraModel.getRequisicoes()])

        .then(requisicoes => filtraNaoEncontrados(requisicoes))
        .then(requisicoes => agrupaEFiltraDuplicados(requisicoes))
        .then(requisicoes => desagrupa(requisicoes))

        .then(requisicoes => res.status(200).send(requisicoes))
        .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
}

export let finalizaRequisicoesIntegra = function (req: Request, res: Response){
    let id = req.params.id;

    if (id) {
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_INTEGRA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.INTEGRA_FINALIZA_REQUISICAO}: ${id}`, user_agent: req.headers['user-agent']}));
        }

        integraModel.finalizaRequisicao(id)

            .then(resultado => res.status(200).send(resultado))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));

    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
}

export let downloadAnexoRequisicoesIntegra = function (req: Request, res: Response){
    let idAnexo = req.params.id;

    if (idAnexo) {
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_INTEGRA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.INTEGRA_DOWNLOAD_REQUISICAO}: ${idAnexo}`, user_agent: req.headers['user-agent']}));
        }

        integraModel.getAnexo(idAnexo)
            .then(anexo => {
                res.setHeader('Content-Length', anexo.tamanho);
                res.setHeader('Content-Type',   anexo.mime);
                res.setHeader('Content-Disposition', `'attachment; filename=${anexo.nomeArquivo}`);
                res.write(anexo.arquivo, 'binary');
                res.end();
            })
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
}

export let getPromotoriasMPPB = function (req: Request, res: Response){
    let promotoria = req.params.promotoria;

    if (promotoria) {
        // if (API_CONFIG.CFG_ENV == 'production') {
        //     logModel.salvaLogPorId(new Log({ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_INTEGRA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.INTEGRA_DOWNLOAD_REQUISICAO}: ${idAnexo}`, user_agent: req.headers['user-agent']}));
        // }

        Promise.all([
            utilsModel.getPromotoriasMPPB_Integra_Promotorias(promotoria)
        ])
            .then(promotorias => filtraNaoEncontrados(promotorias))
            .then(promotorias => agrupaEFiltraDuplicados(promotorias))
            .then(promotorias => desagrupa(promotorias))

            .then(promotorias => res.status(200).send(promotorias))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
}
