import { Request, Response } from 'express';

import * as logModel from '../models/log.model';
import { Log } from '../schemas/log.schema';

import {
    trataErro,
    desagrupa,
    print,
    getId_Token,
    filtraNaoEncontrados,
    agrupaEFiltraDuplicados
} from './../utils';

import {
    API_CODES,
    API_MSGS,
    API_CONFIG,
    LOG_CODES,
    LOG_MSGS
} from '../config';

export let getLogs = function (req : Request, res : Response){

    if (API_CONFIG.CFG_ENV == 'production') {
        logModel.salvaLogPorId(new Log({ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_SISTEMA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.LOG_GET_LOGS}: ${null}`, user_agent: req.headers['user-agent']}));
    }

    Promise.all([logModel.getLogs()]) 
       
        .then(logs => filtraNaoEncontrados(logs))
        .then(logs => agrupaEFiltraDuplicados(logs))
        .then(logs => desagrupa(logs))

        .then(logs => res.status(200).send(logs))
        .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
};

export let getRankings = function (req : Request, res : Response){
    let sistema   = req.query.sistema;
    let ranking   = req.query.ranking;
    let parametro = req.query.parametro;
    let duracao   = req.query.duracao;
    let top       = req.query.top;

    if (API_CONFIG.CFG_ENV == 'production') {
        //logModel.salvaLogPorId(new Log({ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_SISTEMA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.LOG_GET_LOGS}: ${null}`, user_agent: req.headers['user-agent']}));
    }

    let resposta;
    if (sistema === 'siapv2'){
        if (ranking === 'pesquisa'){
            if(parametro === 'cpf') {
                resposta = Promise.all([logModel.getCPFsMaisPesquisados_Pesquisa(duracao, top)]);
            } else if (parametro === 'cnpj') {
                resposta = Promise.all([logModel.getCNPJsMaisPesquisados_Pesquisa(duracao, top)]);
            } else if (parametro === 'geral') {
                resposta = Promise.all([logModel.getValoresMaisPesquisados_Pesquisa(duracao, top)]);
            }
        } else if (ranking === 'relacionamentos') {
            resposta = Promise.all([logModel.getLogs()]);
        } else if (ranking === 'uso'){
            resposta = Promise.all([logModel.getRankingUsoSIAP(duracao, top, 'siapv2')]);
        }
    } else if (sistema === 'siapv1'){
        if (ranking === 'pesquisa'){
            resposta = Promise.all([logModel.getLogs()]);
        } else if (ranking === 'relacionamentos') {
            resposta = Promise.all([logModel.getLogs()]);
        } else if (ranking === 'uso'){
            resposta = Promise.all([logModel.getRankingUsoSIAP(duracao, top, 'siapv1')]);
        }
    }

    resposta

        .then(logs => filtraNaoEncontrados(logs))
        .then(logs => agrupaEFiltraDuplicados(logs))
        .then(logs => desagrupa(logs))

        .then(logs => res.status(200).send(logs))
        .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
};

export let getRecursosMaisUtilizados = function (req : Request, res : Response){
    let sistema = req.query.sistema;
    let duracao = req.query.duracao;
    let chave = req.query.chave;

    if (API_CONFIG.CFG_ENV == 'production') {
        //logModel.salvaLogPorId(new Log({ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_SISTEMA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.LOG_GET_LOGS}: ${null}`, user_agent: req.headers['user-agent']}));
    }

    let resposta;
    if (sistema === 'siapv2'){
        if(chave) {
            resposta = Promise.all([logModel.getRecursosEChavesMaisPesquisados_Pesquisa(duracao)]);
        } else {
            resposta = Promise.all([logModel.getRecursosMaisPesquisados_Pesquisa(duracao)]);
        }
    } else if (sistema === 'siapv1'){
        resposta = Promise.all([logModel.getRecursosMaisPesquisados_Pesquisa(duracao)]);
    }

    resposta

        .then(logs => filtraNaoEncontrados(logs))
        .then(logs => agrupaEFiltraDuplicados(logs))
        .then(logs => desagrupa(logs))

        .then(logs => res.status(200).send(logs))
        .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
};

export let getUsuariosQuePesquisaramValores = function (req : Request, res : Response){
    let sistema = req.query.sistema;
    let duracao = req.query.duracao;
    let chave   = req.query.chave;
    let valor   = req.query.valor;

    if (API_CONFIG.CFG_ENV == 'production') {
        //logModel.salvaLogPorId(new Log({ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_SISTEMA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.LOG_GET_LOGS}: ${null}`, user_agent: req.headers['user-agent']}));
    }

    let resposta;
    if (sistema === 'siapv2'){
        resposta = Promise.all([logModel.getUsuariosQuePesquisaram_Pesquisa(duracao, {chave: chave, valor: valor})]);
    } else if (sistema === 'siapv1'){
        resposta = Promise.all([logModel.getRecursosMaisPesquisados_Pesquisa(duracao)]);
    }

    resposta

        .then(logs => filtraNaoEncontrados(logs))
        .then(logs => agrupaEFiltraDuplicados(logs))
        .then(logs => desagrupa(logs))

        .then(logs => res.status(200).send(logs))
        .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
};

export let getTokensValidos = function (req : Request, res : Response){

    if (API_CONFIG.CFG_ENV == 'production') {
        //logModel.salvaLogPorId(new Log({ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_SISTEMA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.LOG_GET_LOGS}: ${null}`, user_agent: req.headers['user-agent']}));
    }

    Promise.all([logModel.getTokensValidos()])

        .then(logs => filtraNaoEncontrados(logs))
        .then(logs => agrupaEFiltraDuplicados(logs))
        .then(logs => desagrupa(logs))

        .then(logs => res.status(200).send(logs))
        .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
};

export let getEstatisticasUso = function (req : Request, res : Response){
    let categoria = req.query.categoria;
    let duracao   = req.query.duracao;

    if (API_CONFIG.CFG_ENV == 'production') {
        //logModel.salvaLogPorId(new Log({ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_SISTEMA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.LOG_GET_LOGS}: ${null}`, user_agent: req.headers['user-agent']}));
    }

    let resposta;
    if (categoria === 'pesquisa') {
        resposta = Promise.all([logModel.getQuantidadePesquisas(duracao)]);
    } else if (categoria === 'login') {
        resposta = Promise.all([logModel.getQuantidadeLogins(duracao)]);
    }

    resposta

        .then(logs => filtraNaoEncontrados(logs))
        .then(logs => agrupaEFiltraDuplicados(logs))
        .then(logs => desagrupa(logs))

        .then(logs => res.status(200).send(logs))
        .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
};

export let getRegistrosNaoEncontrados = function (req : Request, res : Response){

    if (API_CONFIG.CFG_ENV == 'production') {
        //logModel.salvaLogPorId(new Log({ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_SISTEMA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.LOG_GET_LOGS}: ${null}`, user_agent: req.headers['user-agent']}));
    }

    Promise.all([logModel.getCNPJsNaoEncontrados(),
                logModel.getCPFsNaoEncontrados()]) 

        .then(logs => filtraNaoEncontrados(logs))
        .then(logs => agrupaEFiltraDuplicados(logs))
        .then(logs => desagrupa(logs))

        .then(logs => res.status(200).send(logs))
        .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
};