import { Request, Response } from 'express';

import * as mandadoModel from '../models/mandado.model';
import * as logModel from '../models/log.model';
import { Log } from '../schemas/log.schema';

import {
    trataErro,
    trataRequisicaNome,
    toTextSearch,
    filtraNaoEncontrados,
    desagrupa,
    getId_Token,
    agrupaEFiltraDuplicados,
    print,
    limitaNumeroResultados
} from './../utils';

import {
    API_CODES,
    API_MSGS,
    API_CONFIG,
    LOG_CODES,
    LOG_MSGS
} from '../config';

export let getMandadoDetalhadoCPF = function (req: Request, res: Response){
    let cpf = req.params.cpf;
    cpf = cpf.replace(/[^0-9]/g,'');
    let processo = req.query.processo;

    if(cpf && cpf.length === 11){
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_PESQUISA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.MANDADO_DETALHADO_CPF}: ${cpf}`, user_agent: req.headers['user-agent'], processo: processo}));
        }

        Promise.all([mandadoModel.getMandadoDetalhadoCPF_BNMP(cpf)])

            //.then(mandados => print(mandados))
            .then(mandados => filtraNaoEncontrados(mandados))
            .then(mandados => agrupaEFiltraDuplicados(mandados))
            .then(mandados => desagrupa(mandados))
            .then(mandados => limitaNumeroResultados(mandados))

            .then(mandados => res.status(200).send(mandados))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
}

export let getMandadoSimplificadoCPF = function (req: Request, res: Response){
    let cpf = req.params.cpf;
    cpf = cpf.replace(/[^0-9]/g,'');
    let processo = req.query.processo;

    if(cpf && cpf.length === 11){
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_PESQUISA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.MANDADO_SIMPLIFICADO_CPF}: ${cpf}`, user_agent: req.headers['user-agent'], processo: processo}));
        }

        Promise.all([mandadoModel.getMandadoSimplificadoCPF_BNMP(cpf)])

            .then(mandados => filtraNaoEncontrados(mandados))
            .then(mandados => agrupaEFiltraDuplicados(mandados))
            .then(mandados => desagrupa(mandados))
            .then(mandados => limitaNumeroResultados(mandados))

            .then(mandados => res.status(200).send(mandados))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
}

export let getMandadoSimplificadoRG = function (req: Request, res: Response){
    let rg = req.params.rg;
    rg = rg.replace(/[^0-9]/g,'');
    let processo = req.query.processo;

    if(rg && rg.length >= 1){
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_PESQUISA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.MANDADO_SIMPLIFICADO_RG}: ${rg}`, user_agent: req.headers['user-agent'], processo: processo}));
        }

        Promise.all([mandadoModel.getMandadoSimplificadoRG_BNMP(rg)])

            .then(mandados => filtraNaoEncontrados(mandados))
            .then(mandados => agrupaEFiltraDuplicados(mandados))
            .then(mandados => desagrupa(mandados))
            .then(mandados => limitaNumeroResultados(mandados))

            .then(mandados => res.status(200).send(mandados))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
}

export let getMandadoSimplificadoNome = function (req: Request, res: Response){
    let nome = req.params.nome;
    let nomeArray = trataRequisicaNome(nome);
    let processo = req.query.processo;

    if (nomeArray.length < 2){
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_NOME_INSUFICIENTE));
    }else{
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_PESQUISA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.MANDADO_SIMPLIFICADO_NOME}: ${nome}`, user_agent: req.headers['user-agent'], processo: processo}));
        }

        let nomeTextSearch = toTextSearch(nomeArray);

        Promise.all([mandadoModel.getMandadoSimplificadoNome_BNMP(nome)])

            .then(mandados => filtraNaoEncontrados(mandados))
            .then(mandados => agrupaEFiltraDuplicados(mandados))
            .then(mandados => desagrupa(mandados))
            .then(mandados => limitaNumeroResultados(mandados))

            .then(mandados => res.status(200).send(mandados))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)))
    }
}