import { Request, Response } from 'express';
import * as _ from 'underscore';

import * as logModel from './../../models/log.model';
import { Log } from './../../schemas/log.schema';

import {
    trataErro,
    agrupaEFiltraDuplicados,
    desagrupa,
    getId_Token,
    print,
    limitaNumeroResultados,
} from './../../utils';

import {
    procuraOrgaoSagresMunicipalEstadual, procuraNepotismoUgestora, procuraNepotismoCPF
} from './nepotismo.functions';

import {
    API_CODES,
    API_MSGS,
    API_CONFIG,
    LOG_CODES,
    LOG_MSGS
} from './../../config';


export let getOrgaoSagresMunicipalEstadual = function (req: Request, res: Response) {
    let orgao    = req.params.orgao;
    let processo = req.query.processo;

    if (orgao && orgao.length > 0) {
        // if (API_CONFIG.CFG_ENV == 'production') {
        //     logModel.salvaLogPorId(new Log({ ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_RELACIONAMENTOS, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.RELACIONAMENTO_CPF}: ${cpf}`, user_agent: req.headers['user-agent'] }));
        // }

        procuraOrgaoSagresMunicipalEstadual(orgao)

            .then(orgaos => agrupaEFiltraDuplicados(orgaos))
            .then(orgaos => desagrupa(orgaos))
            .then(orgaos => limitaNumeroResultados(orgaos))

            .then(orgaos => res.status(200).send(orgaos))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));

    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
};

export let getNepotismoUgestora = function (req: Request, res: Response){
    let lotacao = req.query.lotacao;
    let ano     = req.query.ano;
    let esfera  = req.query.esfera;

    if (lotacao) {
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_RELACIONAMENTOS, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.INP_UGESTORA}: ${lotacao}`, user_agent: req.headers['user-agent']}));
        }

        procuraNepotismoUgestora(lotacao, esfera, ano)

            .then(resultados => agrupaEFiltraDuplicados(resultados))
            .then(orgaos => desagrupa(orgaos))
            .then(orgaos => limitaNumeroResultados(orgaos))

            .then(orgaos => res.status(200).send(orgaos))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
}

export let getNepotismoCPF = function (req: Request, res: Response){
    let ano     = req.query.ano;
    let cpf     = req.query.cpf;
    cpf = cpf.replace(/[^0-9]/g, '');

    if (cpf && ano) {
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_RELACIONAMENTOS, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.INP_CPF}: ${cpf}`, user_agent: req.headers['user-agent']}));
        }

        procuraNepotismoCPF(cpf, ano)

            .then(resultados => agrupaEFiltraDuplicados(resultados))
            .then(orgaos => desagrupa(orgaos))
            .then(orgaos => limitaNumeroResultados(orgaos))

            .then(orgaos => res.status(200).send(orgaos))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
}
