import { Request, Response } from 'express';

import * as obitoModel from '../models/obito.model';
import * as logModel from '../models/log.model';
import { Log } from '../schemas/log.schema';

import {
    trataErro,
    trataRequisicaNome,
    toTextSearch,
    filtraNaoEncontrados,
    desagrupa,
    agrupaEFiltraDuplicados,
    getId_Token,
    print,
    limitaNumeroResultados
} from '../utils';

import {
    API_CODES,
    API_MSGS,
    API_CONFIG,
    LOG_CODES,
    LOG_MSGS
} from '../config';

export let getObitoDetalhadoCPF = function (req: Request, res: Response){
    let cpf = req.params.cpf;
    cpf = cpf.replace(/[^0-9]/g,'');
    let processo = req.query.processo;

    if(cpf && cpf.length === 11){
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_PESQUISA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.OBITO_DETALHADO_CPF}: ${cpf}`, user_agent: req.headers['user-agent'], processo: processo}));
        }

        Promise.all([obitoModel.getObitoDetalhadoCPF_Sisobi_Obito(cpf)])

            .then(obitos => filtraNaoEncontrados(obitos))
            .then(obitos => agrupaEFiltraDuplicados(obitos))
            .then(obitos => desagrupa(obitos))
            .then(obitos => limitaNumeroResultados(obitos))

            .then(obitos => res.status(200).send(obitos))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
}

export let getObitoSimplificadoCPF = function (req: Request, res: Response){
    let cpf = req.params.cpf;
    cpf = cpf.replace(/[^0-9]/g,'');
    let processo = req.query.processo;

    if(cpf && cpf.length === 11){
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_PESQUISA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.OBITO_SIMPLIFICADO_CPF}: ${cpf}`, user_agent: req.headers['user-agent'], processo: processo}));
        }

        Promise.all([obitoModel.getObitoSimplificadoCPF_Sisobi_Obito(cpf)])

            .then(obitos => filtraNaoEncontrados(obitos))
            .then(obitos => agrupaEFiltraDuplicados(obitos))
            .then(obitos => desagrupa(obitos))
            .then(obitos => limitaNumeroResultados(obitos))

            .then(obitos => res.status(200).send(obitos))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
}

export let getObitoSimplificadoNome = function (req: Request, res: Response){
    let nome = req.params.nome;
    let nomeArray = trataRequisicaNome(nome);
    let processo = req.query.processo;

    if (nomeArray.length < 2){
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_NOME_INSUFICIENTE));
    }else{
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_PESQUISA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.OBITO_SIMPLIFICADO_NOME}: ${nome}`, user_agent: req.headers['user-agent'], processo: processo}));
        }

        let nomeTextSearch = toTextSearch(nomeArray);

        Promise.all([obitoModel.getObitoSimplificadoNome_Sisobi_Obito(nomeTextSearch)])

            .then(obitos => filtraNaoEncontrados(obitos))
            .then(obitos => agrupaEFiltraDuplicados(obitos))
            .then(obitos => desagrupa(obitos))
            .then(obitos => limitaNumeroResultados(obitos))

            .then(obitos => res.status(200).send(obitos))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)))
    }
}