import { Request, Response } from 'express';

import * as pessoaModel from './../../models/pessoa.model';
import * as enderecoModel from './../../models/endereco.model';
import * as empresaModel from './../../models/empresa.model';
import * as veiculoModel from './../../models/veiculo.model';
import * as servidorModel from './../../models/servidor.model';
import * as mandadoModel from './../../models/mandado.model';
import * as telefoneModel from './../../models/telefone.model';
import * as presoModel from './../../models/preso.model';
import * as fotoModel from './../../models/foto.model';
import * as logModel from './../../models/log.model';
import * as obitoModel from './../../models/obito.model';
import * as tipologiaModel from './../../models/tipologia.model';
import * as empenhoModel from './../../models/empenho.model';
import * as rifModel from './../../models/rif.model';
import * as operacaoModel from './../../models/operacao.model';
import * as bolsaFamiliaModel from './../../models/bolsa_familia.model';
import * as empregadorModel from './../../models/empregador.model';
import * as quadroSocietarioModel from './../../models/historico_quadro_societario.model';
import * as crawlersModel from './../../models/crawler.model';
import * as conselhoModel from './../../models/conselho.model';
import * as filiacaoModel from './../../models/filiacao.model';
import * as virtualModel from './../../models/virtual.model';
import * as processoModel from './../../models/processo.model';
import * as relacionamentoModel from './../../models/relacionamento.model';
import * as condenacaoModel from './../../models/condenacao.model';
import * as vizinhoModel from '../../models/vizinho.model';

import { Log } from './../../schemas/log.schema';
import { Pessoa } from './../../models/schemas';
import {
    trataRequisicaNome,
    toTextSearch,
    agrupaEFiltraDuplicados,
    filtraNaoEncontrados,
    desagrupa,
    print,
    flat,
    getId_Token,
    registraNaoEncontrados,
    filtraLogradouro
} from './../../utils';

import {
    API_CODES,
    API_CONFIG,
    LOG_CODES,
    LOG_MSGS
} from './../../config';


export let procuraPessoaDetalhadoCPF = function (cpf) {
    cpf = cpf.replace(/[^0-9]/g, '');

    return Promise.all([
        pessoaModel.getPessoaSimplificadoCPF_IPC(cpf),
        pessoaModel.getPessoaDetalhadoCPF_Sispesquisa_Pessoas(cpf),
        pessoaModel.getPessoaDetalhadoCPF_ReceitaNovo_PessoaFisica(cpf),
        pessoaModel.getPessoaDetalhadoCPF_ReceitaFull_PF(cpf)
    ])
        .then(pessoas => filtraNaoEncontrados(pessoas))
        .then(pessoas => registraNaoEncontrados(pessoas, 'CPF', cpf))
}

export let procuraPessoaSimplificadoCPF = function (cpf) {
    cpf = cpf.replace(/[^0-9]/g, '');

    return Promise.all([
        pessoaModel.getPessoaSimplificadoCPF_Sispesquisa_Pessoas(cpf),
        pessoaModel.getPessoaSimplificadoCPF_ReceitaNovo_PessoaFisica(cpf),
        pessoaModel.getPessoaSimplificadoCPF_IPC(cpf)
    ])
        .then(pessoas => filtraNaoEncontrados(pessoas))
        .then(pessoas => registraNaoEncontrados(pessoas, 'CPF', cpf))
}

export let procuraPessoaSimplificadoCNH = function (cnh) {
    cnh = cnh.replace(/[^0-9]/g, '');

    return Promise.all([
        pessoaModel.getPessoaSimplificadoCNH_Sispesquisa_CNH(cnh)
    ])
        .then(pessoas => filtraNaoEncontrados(pessoas))
}

export let procuraPessoaSimplificadoTitulo = function (titulo) {
    titulo = titulo.replace(/[^0-9]/g, '');

    return Promise.all([
        pessoaModel.getPessoaSimplificadoTitulo_Sispesquisa_Pessoas(titulo),
        //pessoaModel.getPessoaSimplificadoTitulo_ReceitaFull_PF(titulo),
        pessoaModel.getPessoaSimplificadoTitulo_TSE_Eleitor(titulo)
    ])
        .then(pessoas => filtraNaoEncontrados(pessoas))
}

export let procuraPessoaSimplificadoRG = function (rg) {
    rg = rg.replace(/[^0-9]/g, '');

    return Promise.all([
        pessoaModel.getPessoaSimplificadoRG_Sispesquisa_CNH(rg),
        pessoaModel.getPessoaSimplificadoRG_IPC(rg)
    ])
        .then(pessoas => filtraNaoEncontrados(pessoas))
}

export let procuraPessoaSimplificadoNome = function (nome) {
    const nomeTextSearch = toTextSearch(trataRequisicaNome(nome));

    return Promise.all([
        pessoaModel.getPessoaSimplificadoNome_Sispesquisa_Pessoas(nomeTextSearch),
        pessoaModel.getPessoaSimplificadoNome_ReceitaNovo_PessoaFisica(nomeTextSearch),
        pessoaModel.getPessoaSimplificadoNome_IPC(nome)
    ])
        .then(pessoas => filtraNaoEncontrados(pessoas))
}

export let procuraPessoaSimplificadoNomePai = function (nomePai) {
    const nomePaiTextSearch = toTextSearch(trataRequisicaNome(nomePai));

    return Promise.all([
        pessoaModel.getPessoaSimplificadoNomePai_IPC(nomePai),
        pessoaModel.getPessoaNomePai_Sispesquisa_CNH(nomePaiTextSearch),
        pessoaModel.getPessoaSimplificadoNomePai_TSE_Eleitor(nomePaiTextSearch),
        pessoaModel.getPessoaNomePai_Sispesquisa_Prisional(nomePaiTextSearch),
    ])
        .then(pessoas => filtraNaoEncontrados(pessoas))
}

export let procuraPessoaSimplificadoNomeMae = function (nomeMae) {
    const nomeMaeTextSearch = toTextSearch(trataRequisicaNome(nomeMae));

    return Promise.all([
        pessoaModel.getPessoaSimplificadoNomeMae_IPC(nomeMae),
        pessoaModel.getPessoaNomeMae_Sispesquisa_Pessoas(nomeMaeTextSearch),
        pessoaModel.getPessoaNomeMae_ReceitaNovo_PessoaFisica(nomeMaeTextSearch),
        pessoaModel.getPessoaSimplificadoNomeMae_TSE_Eleitor(nomeMaeTextSearch),
    ])
        .then(pessoas => filtraNaoEncontrados(pessoas))
}

export let procuraPessoaSimplificadoEndereco = function (endereco) {
    const enderecoTextSearch = toTextSearch(filtraLogradouro(endereco));

    return Promise.all([
        pessoaModel.getPessoaSimplificadoLogradouro_ReceitaNovo_PessoaFisica(enderecoTextSearch),
    ])
        .then(pessoas => filtraNaoEncontrados(pessoas))
}

export let procuraPessoaSimplificadoTelefone = function (telefone) {
    telefone = telefone.replace(/[^0-9]/g, '');

    return Promise.all([
        pessoaModel.getPessoaTelefone_Sispesquisa_Telefones(telefone)
    ])
        .then(pessoas => filtraNaoEncontrados(pessoas))
}

export let procuraPessoaSimplificadoEmail = function (email) {

    return Promise.all([
        pessoaModel.getPessoaEmail_Sispesquisa_Emails(email)
    ])
        .then(pessoas => filtraNaoEncontrados(pessoas))
}

export let procuraVizinhosPessoaCPF = function (cpf) {
    cpf = cpf.replace(/[^0-9]/g, '');

    return Promise.all([
        vizinhoModel.getVizinhosReceitaNovoCPF(cpf)
    ])
        .then(pessoas => filtraNaoEncontrados(pessoas))
}

export let procuraPessoaIntegradoCPF = function (cpf) {
    cpf = cpf.replace(/[^0-9]/g, '');

    return Promise.all([
        // PESSOA
        pessoaModel.getPessoaDetalhadoCPF_Receita_PF(cpf),
        pessoaModel.getPessoaDetalhadoCPF_ReceitaFull_PF(cpf),
        pessoaModel.getPessoaDetalhadoCPF_ReceitaNovo_PessoaFisica(cpf),
        pessoaModel.getPessoaDetalhadoCPF_Sispesquisa_Pessoas(cpf),
        pessoaModel.getPessoaDetalhadoCPF_IPC(cpf),
        //pessoaModel.getPessoaDetalhadoCPF_Sispesquisa_Prisional(cpf),
        pessoaModel.getPessoaDetalhadoCPF_Sispesquisa_CNH(cpf),
        pessoaModel.getPessoaDetalhadoCPF_TSE_Eleitor(cpf),
        pessoaModel.getPessoaDetalhadoCPF_Renach_2016_08(cpf),
        pessoaModel.getPessoaDetalhadoCPF_VEP(cpf),

        // PARENTESCOS
        relacionamentoModel.getParentescosCPF(cpf),

        // FOTO
        fotoModel.getFotosCPF_IPC(cpf),
        fotoModel.getFotoCPF_FileSystem_Renach(cpf),
        fotoModel.getFotoCPF_FileSystem_Fotos(cpf),
        fotoModel.getFotoCPF_SISDEPEN(cpf),

        // VIZINHOS
        vizinhoModel.getVizinhosReceitaNovoCPF(cpf),

        // ENDERECO
        enderecoModel.getEnderecoCPF_Sispesquisa_Enderecos(cpf),
        enderecoModel.getEnderecoCPF_IPC(cpf),
        enderecoModel.getEnderecoCPF_ReceitaNovo_PessoaFisica(cpf),
        enderecoModel.getEnderecoCPF_ReceitaFull_PF(cpf),
        enderecoModel.getEnderecoCPF_Receita_PF(cpf),
        enderecoModel.getEnderecoCPF_Sispesquisa_VeiculosNovo(cpf),
        enderecoModel.getEnderecoCPF_Renach_2016_08(cpf),
        enderecoModel.getEnderecoCPF_VEP(cpf),

        // TELEFONE
        telefoneModel.getTelefoneCPF_ReceitaFull_PF(cpf),
        telefoneModel.getTelefoneCPF_Sispesquisa_Telefones(cpf),
        telefoneModel.getTelefoneSimplificadoCPF_IPC(cpf),
        telefoneModel.getTelefoneCPF_VEP(cpf),

        // EMPRESAS QUE É SÓCIO
        //empresaModel.getEmpresaSimplificadoSocioPFCPF_ReceitaFull(cpf),
        empresaModel.getEmpresaSimplificadoSocioPFCPF_ReceitaNovo(cpf),

        // EMPRESAS QUE É RESPONSÁVEL
        //empresaModel.getEmpresaSimplificadoResponsavelCPF_ReceitaFull(cpf),
        empresaModel.getEmpresaSimplificadoResponsavelCPF_ReceitaNovo(cpf),

        // VEICULO
        veiculoModel.getVeiculoDetalhadoCPF_Sispesquisa_Veiculos(cpf),
        veiculoModel.getVeiculoDetalhadoCPF_Sispesquisa_VeiculosNovo(cpf),
        veiculoModel.getVeiculoDetalhadoCPF_Renavam_2016(cpf),

        // SERVIDOR
        servidorModel.getServidorFederalCPF_Sispesquisa_Servidores_Federais(cpf),
        servidorModel.getServidorFederalCPF_Sispesquisa_Servidores_Federais_Nordeste(cpf),

        servidorModel.getServidorEstadualSimplificadoCPF_SAGRES_ESTADUAL_VW_FOLHA(cpf),
        servidorModel.getServidorMunicipalSimplificadoCPF_SAGRES_MUNICIPAL_FolhaPagamento(cpf),

        //servidorModel.getServidorMunicipalSimplificadoCPF_Sispesquisa_Servidores_Municipais(cpf),
        //servidorModel.getServidorEstadualSimplificadoCPF_Sispesquisa_Servidores_Estaduais(cpf),

        // EMPENHO
        empenhoModel.getEmpenhoSimplificadoCPF_SAGRES_MUNICIPAL_Credores(cpf),
        empenhoModel.getEmpenhoSimplificadoCPF_SAGRES_ESTADUAL_Credores_PB(cpf),

        // MANDADO
        // mandadoModel.getMandadoSimplificadoCPF_BNMP(cpf),

        // PRESO
        presoModel.getPresoDetalhadoCPF_Sispesquisa_Prisional(cpf),
        presoModel.getPresoDetalhadoCPF_SDS_Prisional(cpf),
        presoModel.getPresoDetalhadoCPF_SISDEPEN(cpf),

        // FILIACAO
        condenacaoModel.getCondenacaoTREPBCPF(cpf),
        condenacaoModel.getCondenacaoTRF5CPF(cpf),

        // FILIACAO
        filiacaoModel.getFiliacaoPartidariaDetalhadoCPF_Filiados(cpf),

        // CONSELHO
        // ADVOGADO
        conselhoModel.getAdvogadoDetalhadoCPF_VEP(cpf),

        // OBITO
        obitoModel.getObitoDetalhadoCPF_Sisobi_Obito(cpf),

        // TIPOLOGIA
        tipologiaModel.getTipologiaSimplificadoCPF_Tipologias(cpf),

        // RIF
        rifModel.existeRIFCPF_Sispesquisa_RIF(cpf),

        // EMPREGADOR
        //empregadorModel.getEmpregadoresDetalhadoCPF_Caged(cpf),
        empregadorModel.getEmpregadoresDetalhadoCPF_RAIS(cpf),

        // BOLSA FAMILIA
        bolsaFamiliaModel.getBolsaFamiliaCPF_BolsaFamilia(cpf),

        // HISTORICO QUADRO SOCIETARIO
        quadroSocietarioModel.getHistoricoQuadroPFDetalhadoCPF_CNE(cpf),

        // OPERACOES
        operacaoModel.getOperacoesCPF_Sispesquisa_Operacoes(cpf),

        // VIRTUAL
        virtualModel.getEmailCPF_VEP(cpf),
        virtualModel.getEmailCPF_SispesquisaEmail(cpf),

        // PROCESSO
        processoModel.getProcessoCPF_TJPB(cpf),
        processoModel.getPenaProcessoCPF_VEP(cpf),
        processoModel.getProcessoCPF_VEP(cpf),

        // CRAWLERS
        crawlersModel.getJusBrasilCPF_SIAP_CRAWLERS(cpf),
        crawlersModel.getGoogleCPF_SIAP_CRAWLERS(cpf),

        crawlersModel.getGoogleMPBRCPF_SIAP_CRAWLERS(cpf),
        crawlersModel.getGoogleJusBRCPF_SIAP_CRAWLERS(cpf),
        crawlersModel.getGoogleEduBRCPF_SIAP_CRAWLERS(cpf),
        crawlersModel.getGoogleGovBRCPF_SIAP_CRAWLERS(cpf),
        crawlersModel.getFacebookCPF_SIAP_CRAWLERS(cpf),
    ])
        .then(pessoas => filtraNaoEncontrados(pessoas))
        .then(pessoas => registraNaoEncontrados(pessoas, 'CPF', cpf))
}


export let getPessoasListaCPFs = function (listaCPFs: Array<string>) {
    return Promise.all(listaCPFs.map(cpf => {
        // Realiza a busca de cada um dos CPFs nas nossas bases
        return Promise.all([pessoaModel.getPessoaSimplificadoCPF_Sispesquisa_Pessoas(cpf),
                            pessoaModel.getPessoaSimplificadoCPF_Receita_PF(cpf)])
            .then(pessoas => filtraNaoEncontrados(pessoas))
            .then(pessoas => (pessoas[0].status === API_CODES.CODE_RECURSO_NAO_ENCONTRADO) ? pessoaModel.getPessoaDetalhadoCPF_ReceitaFull_PF(cpf) : pessoas)
    }))

        .then(pessoas => flat(pessoas))
        .then(pessoas => filtraNaoEncontrados(pessoas))
        .then(pessoas => agrupaEFiltraDuplicados(pessoas))
        .then(pessoas => desagrupa(pessoas))

        .catch(error => console.error('ERROR: getPessoasListaCPFs - ', error))
}