import { Request, Response } from 'express';
import * as md5 from 'md5';

import * as preUsuarioModel from '../models/preusuario.model';
import * as usuarioModel from '../models/usuario.model';
import * as logModel from '../models/log.model';
import { Log } from '../schemas/log.schema';
import { Usuario } from '../models/schemas';

import {
    trataErro,
    agrupaEFiltraDuplicados,
    filtraNaoEncontrados,
    desagrupa,
    getId_Token,
    validaRecaptcha,
    print
} from './../utils';

import {
    API_CODES,
    API_MSGS,
    API_CONFIG,
    LOG_CODES,
    LOG_MSGS
} from '../config';


export let getPreUsuariosInativos = function (req : Request, res : Response){

    if (API_CONFIG.CFG_ENV == 'production') {
        logModel.salvaLogPorId(new Log({ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_SISTEMA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.CADASTRO_USUARIO_LISTA_USUARIOS_INATIVOS}: ${null}`, user_agent: req.headers['user-agent']}));
    }

    Promise.all([preUsuarioModel.getPreUsuariosInativos_Sispesquisa_CadastroUsuario()])

        .then(preusuarios => filtraNaoEncontrados(preusuarios))
        .then(preusuarios => agrupaEFiltraDuplicados(preusuarios))
        .then(preusuarios => desagrupa(preusuarios))

        .then(preusuarios => res.status(200).send(preusuarios))
        .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
};

export let insertPreUsuarios = async function (req : Request, res : Response){
    const formData = req.body;

    if (formData.recaptcha) {
        const recaptchaResponse = formData.recaptcha;
        try {
            let valido = await validaRecaptcha(recaptchaResponse);
            if (valido) {
                try {
                    let usuarioExiste = await preUsuarioModel.checkCPFPreUsuarios_Sispesquisa_CadastroUsuario(formData);
                    if (!usuarioExiste) {
                        try {
                            if (API_CONFIG.CFG_ENV == 'production') {
                                logModel.salvaLog(new Log({ip: req.headers['x-real-ip'], usuario: formData.nome, tipo: LOG_CODES.TIPO_CREATE_CADASTRO_USUARIO, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.CADASTRO_USUARIO_CADASTRO_USUARIO}: ${formData.nome}`, user_agent: req.headers['user-agent']}));
                            }

                            let criouUsuario = await preUsuarioModel.insertPreUsuarios_Sispesquisa_CadastroUsuario(formData);
                            res.status(200).send(criouUsuario);
                        } catch (errInsertUsuario) {
                            res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, errInsertUsuario));
                        }
                    } else {
                        if (API_CONFIG.CFG_ENV == 'production') {
                            logModel.salvaLog(new Log({ ip: req.headers['x-real-ip'], usuario: 'null', tipo: LOG_CODES.TIPO_TENTATIVA_CREATE_CADASTRO_USUARIO, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${API_CODES.CODE_CPF_JA_CADASTRADO}: ${formData.cpf} - ${formData.nome} - 'CPF já cadastrado no sistema.'`, user_agent: req.headers['user-agent'] }));
                        }

                        res.status(400).send(trataErro(API_CODES.CODE_CPF_JA_CADASTRADO, 'CPF já cadastrado no sistema.'));
                    }
                } catch (errCheckCPF){
                    if (API_CONFIG.CFG_ENV == 'production') {
                        logModel.salvaLog(new Log({ ip: req.headers['x-real-ip'], usuario: 'null', tipo: LOG_CODES.TIPO_TENTATIVA_CREATE_CADASTRO_USUARIO, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${API_CODES.CODE_ERRO_500}: ${formData.cpf} - ${formData.nome} - 'Falha em checkCPFPreUsuarios_Sispesquisa_CadastroUsuario.'`, user_agent: req.headers['user-agent'] }));
                    }

                    res.status(401).send(trataErro(API_CODES.CODE_ERRO_500, errCheckCPF));
                }
            } else {
                if (API_CONFIG.CFG_ENV == 'production') {
                    logModel.salvaLog(new Log({ ip: req.headers['x-real-ip'], usuario: 'null', tipo: LOG_CODES.TIPO_TENTATIVA_CREATE_CADASTRO_USUARIO, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${API_CODES.CODE_RECAPTCHA_INVALIDO}: ${formData.cpf} - ${formData.nome} - 'Não foi possível validar o recaptcha.'`, user_agent: req.headers['user-agent'] }));
                }

                res.status(401).send(trataErro(API_CODES.CODE_RECAPTCHA_INVALIDO, 'Não foi possível validar o recaptcha.'))
            }
        } catch (errValidaRecaptcha){
            if (API_CONFIG.CFG_ENV == 'production') {
                logModel.salvaLog(new Log({ ip: req.headers['x-real-ip'], usuario: 'null', tipo: LOG_CODES.TIPO_TENTATIVA_CREATE_CADASTRO_USUARIO, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${API_CODES.CODE_RECAPTCHA_INVALIDO}: ${formData.cpf} - ${formData.nome} - 'Erro na validação do recaptcha.'`, user_agent: req.headers['user-agent'] }));
            }

            res.status(401).send(trataErro(API_CODES.CODE_RECAPTCHA_INVALIDO, 'Erro na validação do recaptcha.'))
        }
    } else {
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLog(new Log({ ip: req.headers['x-real-ip'], usuario: 'null', tipo: LOG_CODES.TIPO_TENTATIVA_CREATE_CADASTRO_USUARIO, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${API_CODES.CODE_RECAPTCHA_NAO_ENCONTRADO}: ${formData.cpf} - ${formData.nome} - 'Valor para validação do recaptcha não encontrado.'`, user_agent: req.headers['user-agent'] }));
        }

        res.status(401).send(trataErro(API_CODES.CODE_RECAPTCHA_NAO_ENCONTRADO, 'Valor para validação do recaptcha não encontrado.'))
    }
};

export let deletePreUsuario = function (req: Request, res: Response){
    let id = req.params.id;

    if (id) {
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_SISTEMA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.CADASTRO_USUARIO_REMOVE_USUARIO}: ${id}`, user_agent: req.headers['user-agent']}));
        }

        preUsuarioModel.deletePreUsuario_Sispesquisa_CadastroUsuario(id)

            .then(resultado => res.status(200).send(resultado))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
}

export let ativarPreUsuario = function (req: Request, res: Response){
    let id = req.params.id;
    let dados = req.body;

    if (id) {
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_SISTEMA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.CADASTRO_USUARIO_ATIVAR_USUARIO}: ${id}`, user_agent: req.headers['user-agent']}));
        }

        let novoUsuario           = new Usuario(dados);
        novoUsuario.administrador = (dados.perfil === 'admin')   ? 'S':              'N';
        novoUsuario.perfil        = (dados.perfil === 'publico') ? 'publico':      'privado';
        novoUsuario.ad            = dados.acesso;
        novoUsuario.proximoLogon  = (dados.acesso === 'LOCAL')   ? 'S':              'N';
        novoUsuario.processo      = 'S';
        novoUsuario.senha         = (dados.acesso === 'LOCAL')   ? md5(dados.senha): null;
        novoUsuario.idCadastrador = dados.idCadastrador;
        novoUsuario.membro        = 'N';
        novoUsuario.operacoes     = 0;

        let erroSalvo;
        usuarioModel.insertUsuario_Sispesquisa_Usuario(novoUsuario)
            .then(() => usuarioModel.insertControleAcessoUsuario_Sispesquisa_Usuario(novoUsuario))
            .then(() => preUsuarioModel.ativarPreUsuario_Sispesquisa_CadastroUsuario(novoUsuario.id.toString()))

            .then(resultado => res.status(200).send(resultado))
            .catch(error => { erroSalvo = error; })
                .then(() => preUsuarioModel.desativarPreUsuario_Sispesquisa_CadastroUsuario(novoUsuario.id.toString()))
                .then(() => usuarioModel.removeControleAcessoUsuarioLogin_Sispesquisa_Usuario(novoUsuario))
                .then(() => usuarioModel.removeUsuarioLogin_Sispesquisa_Usuario(novoUsuario))
                .then(() => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, erroSalvo)))
    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
}