import {
    Request,
    Response
} from 'express';

import * as riscoModel from '../models/risco.model';
import * as logModel from '../models/log.model';
import { Log } from '../schemas/log.schema';

import {
    trataErro,
    filtraNaoEncontrados,
    desagrupa,
    agrupaEFiltraDuplicados,
    getId_Token,
    print 
} from '../utils';

import { 
    API_CODES,
    API_MSGS,
    API_CONFIG,
    LOG_CODES,
    LOG_MSGS 
} from '../config';

export let getQuestao = function (req: Request, res: Response){
    let numquestao = req.query.numquestao;

    if (numquestao){
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_PESQUISA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.RISCO_MPPB_QUESTAO}: ${numquestao} `, user_agent: req.headers['user-agent'], processo: null}));
        }

        Promise.all([riscoModel.getQuestao_Sispesquisa_PerguntaResposta(numquestao)])

            .then(risco => filtraNaoEncontrados(risco))
            .then(risco => agrupaEFiltraDuplicados(risco)) 
            .then(risco => desagrupa(risco))

            .then(risco => res.status(200).send(risco))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
}

export let insertRespostasQuestionario = function (req: Request, res: Response){
    let respostas  = req.body;

    if (respostas){
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_PESQUISA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.RISCO_MPPB_INSERE_RESPOSTA} `, user_agent: req.headers['user-agent'], processo: null}));
        }

        const idUsuario = getId_Token(req.headers['authorization']);
        const listaRespostas = respostas;

        console.log(idUsuario, listaRespostas);

        // Promise.all([riscoModel.getQuestao_Sispesquisa_PerguntaResposta('123')])

        //     .then(risco => filtraNaoEncontrados(risco))
        //     .then(risco => agrupaEFiltraDuplicados(risco)) 
        //     .then(risco => desagrupa(risco))

        //     .then(risco => res.status(200).send(risco))
        //     .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
}