import { Request, Response } from 'express';

import * as socioModel from '../models/socio.model';
import * as logModel from '../models/log.model';
import { Log } from '../schemas/log.schema';

import {
    trataErro,
    filtraNaoEncontrados,
    desagrupa,
    agrupaEFiltraDuplicados,
    getId_Token,
    print
} from './../utils';

import {
    API_CODES,
    API_MSGS,
    API_CONFIG,
    LOG_CODES,
    LOG_MSGS
} from '../config';

export let getSocioEmpresaSimplificadoCNPJ = function (req: Request, res: Response){
    let cnpj = req.params.cnpj;
    cnpj = cnpj.replace(/[^0-9]/g,'');
    let processo = req.query.processo;

    if(cnpj && cnpj.length === 14){
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_PESQUISA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.SOCIO_SIMPLIFICADO_CNPJ}: ${cnpj}`, user_agent: req.headers['user-agent'], processo: processo}));
        }

        Promise.all([socioModel.getSocioPFEmpresaSimplificadoCNPJ_ReceitaFull(cnpj),
                    socioModel.getSocioPJEmpresaSimplificadoCNPJ_ReceitaFull(cnpj),
                    socioModel.getSocioEstrangeiroEmpresaSimplificadoCNPJ_ReceitaFull(cnpj)])

            .then(socios => filtraNaoEncontrados(socios))
            .then(socios => agrupaEFiltraDuplicados(socios))
            // .then(socios => desagrupa(socios))

            .then(socios => res.status(200).send(socios))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
}