import * as telefoneModel from './../../models/telefone.model';

import { 
    filtraNaoEncontrados,
    trataRequisicaNome,
    toTextSearch,
    print
} from './../../utils';

export let procuraTelefoneSimplificadoCPF = function (cpf) {
    cpf = cpf.replace(/[^0-9]/g, '');

    return Promise.all([
        telefoneModel.getTelefoneCPF_ReceitaFull_PF(cpf),
        telefoneModel.getTelefoneCPF_Sispesquisa_Telefones(cpf)
    ])
        .then(telefones => filtraNaoEncontrados(telefones))
}

export let procuraTelefoneSimplificadoCNPJ = function (cnpj) {
    cnpj = cnpj.replace(/[^0-9]/g, '');

    return Promise.all([
        telefoneModel.getTelefoneCNPJ_ReceitaFull_PJ(cnpj)
    ])
        .then(telefones => filtraNaoEncontrados(telefones))
}

export let procuraTelefoneSimplificadoTelefone = function (telefone) {
    telefone = telefone.replace(/[^0-9]/g, '');

    return Promise.all([
        telefoneModel.getTelefonePFTelefone_Sispesquisa_Telefones(telefone),
        telefoneModel.getTelefoneTelefone_ReceitaFull_PF(telefone),
        //telefoneModel.getTelefoneTelefone_ReceitaFull_PJ(telefone),
    ])
        .then(telefones => filtraNaoEncontrados(telefones))
}

export let procuraTelefoneBuscaProfundaTelefone = function (telefone) {
    telefone = telefone.replace(/[^0-9]/g, '');

    return Promise.all([
        telefoneModel.getTelefoneBuscaProfundaTelefone_Sispesquisa_Telefones(telefone),
    ])
        .then(telefones => filtraNaoEncontrados(telefones))
}

export let procuraTelefoneSimplificadoNome = function (nome) {
    let nomeTextSearch = toTextSearch(trataRequisicaNome(nome));

    return Promise.all([
        telefoneModel.getTelefoneNome_Sispesquisa_Telefones(nomeTextSearch),
        telefoneModel.getTelefoneNome_ReceitaFull_PF(nomeTextSearch)
    ])
        .then(telefones => filtraNaoEncontrados(telefones))
}

export let procuraTelefoneSimplificadoRazaoSocial = function (razaoSocial) {
    let razaoSocialTextSearch = toTextSearch(trataRequisicaNome(razaoSocial));

    return Promise.all([
        telefoneModel.getTelefoneRazaoSocial_ReceitaFull_PJ(razaoSocialTextSearch)
    ])
        .then(telefones => filtraNaoEncontrados(telefones))
}
