import { Request, Response } from 'express';

import * as tipologiaModel from '../models/tipologia.model';
import * as logModel from '../models/log.model';
import { Log } from '../schemas/log.schema';

import {
    trataErro,
    agrupaEFiltraDuplicados,
    filtraNaoEncontrados,
    desagrupa,
    getId_Token,
    print,
    limitaNumeroResultados
} from './../utils';

import {
    API_CODES,
    API_MSGS,
    API_CONFIG,
    LOG_CODES,
    LOG_MSGS
} from '../config';


export let getTipologiaSimplificadoCPF = function (req : Request, res : Response){
    let cpf = req.params.cpf;
    cpf = cpf.replace(/[^0-9]/g,'');
    let processo = req.query.processo;

    if (cpf && cpf.length === 11){
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_PESQUISA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.TIPOLOGIA_SIMPLIFICADO_CPF}: ${cpf}`, user_agent: req.headers['user-agent'], processo: processo}));
        }

        Promise.all([tipologiaModel.getTipologiaSimplificadoCPF_Tipologias(cpf)])

            .then(tipologias => filtraNaoEncontrados(tipologias))
            .then(tipologias => agrupaEFiltraDuplicados(tipologias))
            .then(tipologias => desagrupa(tipologias))
            .then(tipologias => limitaNumeroResultados(tipologias))

            .then(tipologias => res.status(200).send(tipologias))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
};

export let getTipologiaSimplificadoCNPJ = function (req : Request, res : Response){
	let cnpj = req.params.cnpj;
    cnpj = cnpj.replace(/[^0-9]/g,'');
    let processo = req.query.processo;

    if (cnpj && cnpj.length === 14){
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_PESQUISA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.TIPOLOGIA_SIMPLIFICADO_CNPJ}: ${cnpj}`, user_agent: req.headers['user-agent'], processo: processo}));
        }

        Promise.all([tipologiaModel.getTipologiaSimplificadoCNPJ_Tipologias(cnpj)])

            .then(tipologias => filtraNaoEncontrados(tipologias))
            .then(tipologias => agrupaEFiltraDuplicados(tipologias))
            .then(tipologias => desagrupa(tipologias))
            .then(tipologias => limitaNumeroResultados(tipologias))

            .then(tipologias => res.status(200).send(tipologias))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
};
