import { Request, Response } from 'express';

import * as logModel from './../../models/log.model';
import { Log } from './../../schemas/log.schema';

import {
    trataErro,
    agrupaEFiltraDuplicados,
    desagrupa,
    getId_Token,
    print,
} from './../../utils';

import {
    API_CONFIG,
    API_CODES,
    API_MSGS,
    LOG_CODES,
    LOG_MSGS
} from './../../config';

import {
    procuraTipologiasMunicipioUF,
    procuraMunicioUF,
} from './tiporank.functions';

export let getTipologiaSimplificadoUFMunicipio = function (req: Request, res: Response) {
    let uf = req.params.uf;
    let municipio = req.params.municipio;

    let processo = req.query.processo;

    if (uf && municipio) {
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_TIPORANK, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.TIPOLOGIA_SIMPLIFICADO_MUNICIPIO}: ${municipio}/${uf}`, user_agent: req.headers['user-agent'], processo: processo }));
        }

        procuraTipologiasMunicipioUF(uf, municipio)
            .then(tipologias => agrupaEFiltraDuplicados(tipologias))

            .then(tipologias => res.status(200).send(tipologias))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
};

export let getMunicioUF = function (req: Request, res: Response) {
    let uf = req.params.uf;
    let municipio = req.query.municipio;

    if (uf && municipio) {
        procuraMunicioUF(uf, municipio)
            .then(municipios => agrupaEFiltraDuplicados(municipios, []))
            .then(municipios => desagrupa(municipios))

            .then(municipios => res.status(200).send(municipios))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
}