import { Request, Response } from 'express';

import * as logModel from './../../models/log.model';
import { Log } from './../../schemas/log.schema';

import {
    trataErro,
    agrupaEFiltraDuplicados,
    desagrupa,
    trataRequisicaNome,
    getId_Token,
    print,
    limitaNumeroResultados
} from './../../utils';

import {
    API_CODES,
    API_MSGS,
    API_CONFIG,
    LOG_CODES,
    LOG_MSGS
} from './../../config';

import {
    procuraVeiculoDetalhadoCNPJ,
    procuraVeiculoDetalhadoCPF,
    procuraVeiculoDetalhadoNome,
    procuraVeiculoDetalhadoChassi,
    procuraVeiculoDetalhadoPlaca,
    procuraVeiculoDetalhadoRenavam,

} from './veiculos.functions';

export let getVeiculoDetalhadoCNPJ = function (req: Request, res: Response){
    let cnpj = req.params.cnpj;
    cnpj = cnpj.replace(/[^0-9]/g,'');
    let processo = req.query.processo;

    if(cnpj && cnpj.length === 14){
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_PESQUISA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.VEICULO_DETALHADO_CNPJ}: ${cnpj}`, user_agent: req.headers['user-agent'], processo: processo}));
        }

        procuraVeiculoDetalhadoCNPJ(cnpj)

            .then(veiculos => agrupaEFiltraDuplicados(veiculos))
            .then(veiculos => desagrupa(veiculos))
            .then(veiculos => limitaNumeroResultados(veiculos))

            .then(veiculos => res.status(200).send(veiculos))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
}

export let getVeiculoDetalhadoCPF = function (req: Request, res: Response){
    let cpf = req.params.cpf;
    cpf = cpf.replace(/[^0-9]/g,'');
    let processo = req.query.processo;

    if(cpf && cpf.length === 11){
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_PESQUISA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.VEICULO_DETALHADO_CPF}: ${cpf}`, user_agent: req.headers['user-agent'], processo: processo}));
        }

        procuraVeiculoDetalhadoCPF(cpf)

            .then(veiculos => agrupaEFiltraDuplicados(veiculos))
            .then(veiculos => desagrupa(veiculos))
            .then(veiculos => limitaNumeroResultados(veiculos))

            .then(veiculos => res.status(200).send(veiculos))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
}

export let getVeiculoDetalhadoNome = function (req: Request, res: Response){
    let nome = req.params.nome;
    let nomeArray = trataRequisicaNome(nome);
    let processo = req.query.processo;

    if (nomeArray.length < 2){
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_NOME_INSUFICIENTE));
    }else{
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_PESQUISA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.VEICULO_DETALHADO_NOME}: ${nome}`, user_agent: req.headers['user-agent'], processo: processo}));
        }

        procuraVeiculoDetalhadoNome(nome)

            .then(veiculos => agrupaEFiltraDuplicados(veiculos))
            .then(veiculos => desagrupa(veiculos))
            .then(veiculos => limitaNumeroResultados(veiculos))

            .then(veiculos => res.status(200).send(veiculos))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)))
    }
}

export let getVeiculoDetalhadoChassi = function (req: Request, res: Response){
    let chassi = req.params.chassi;
    chassi = chassi.replace(/[^0-9a-zA-Z]/g,'');
    let processo = req.query.processo;

    if(chassi && chassi.length > 1){
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_PESQUISA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.VEICULO_DETALHADO_CHASSI}: ${chassi}`, user_agent: req.headers['user-agent'], processo: processo}));
        }

        procuraVeiculoDetalhadoChassi(chassi)

            .then(veiculos => agrupaEFiltraDuplicados(veiculos))
            .then(veiculos => desagrupa(veiculos))
            .then(veiculos => limitaNumeroResultados(veiculos))

            .then(veiculos => res.status(200).send(veiculos))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
}

export let getVeiculoDetalhadoPlaca = function (req: Request, res: Response){
    let placa = req.params.placa;
    placa = placa.replace(/[^0-9a-zA-Z]/g,'');
    let processo = req.query.processo;

    if(placa && placa.length > 1){
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_PESQUISA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.VEICULO_DETALHADO_PLACA}: ${placa}`, user_agent: req.headers['user-agent'], processo: processo}));
        }

        procuraVeiculoDetalhadoPlaca(placa)

            .then(veiculos => agrupaEFiltraDuplicados(veiculos))
            .then(veiculos => desagrupa(veiculos))
            .then(veiculos => limitaNumeroResultados(veiculos))

            .then(veiculos => res.status(200).send(veiculos))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
}

export let getVeiculoDetalhadoRenavam = function (req: Request, res: Response){
    let renavam = req.params.renavam;
    renavam = renavam.replace(/[^0-9]/g,'');
    let processo = req.query.processo;

    if(renavam && renavam.length === 11){
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_PESQUISA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.VEICULO_DETALHADO_RENAVAM}: ${renavam}`, user_agent: req.headers['user-agent'], processo: processo}));
        }

        procuraVeiculoDetalhadoRenavam(renavam)

            .then(veiculos => agrupaEFiltraDuplicados(veiculos))
            .then(veiculos => desagrupa(veiculos))
            .then(veiculos => limitaNumeroResultados(veiculos))

            .then(veiculos => res.status(200).send(veiculos))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
}
