import * as veiculoModel from './../../models/veiculo.model';

import {
    filtraNaoEncontrados,
    trataRequisicaNome,
    toTextSearch,
    print
} from './../../utils';

export let procuraVeiculoDetalhadoCNPJ = function (cnpj: string){
    cnpj = cnpj.replace(/[^0-9]/g, '');
 
    return Promise.all([
        veiculoModel.getVeiculoDetalhadoCNPJ_Renavam_2016(cnpj),
        veiculoModel.getVeiculoDetalhadoCNPJ_Sispesquisa_Veiculos(cnpj),
        veiculoModel.getVeiculoDetalhadoCNPJ_Sispesquisa_VeiculosNovo(cnpj)
    ])

        .then(veiculos => filtraNaoEncontrados(veiculos))
}

export let procuraVeiculoDetalhadoCPF = function (cpf: string){
    cpf = cpf.replace(/[^0-9]/g, '');
 
    return Promise.all([
        veiculoModel.getVeiculoDetalhadoCPF_Sispesquisa_Veiculos(cpf),
        veiculoModel.getVeiculoDetalhadoCPF_Sispesquisa_VeiculosNovo(cpf),
        veiculoModel.getVeiculoDetalhadoCPF_Renavam_2016(cpf)
    ])

        .then(veiculos => filtraNaoEncontrados(veiculos))
}

export let procuraVeiculoDetalhadoNome = function (nome: string){
    const nomeTextSearch = toTextSearch(trataRequisicaNome(nome));
 
    return Promise.all([
        veiculoModel.getVeiculoDetalhadoNome_Sispesquisa_Veiculos(nomeTextSearch),
        veiculoModel.getVeiculoDetalhadoNome_Sispesquisa_VeiculosNovo(nomeTextSearch),
        veiculoModel.getVeiculoDetalhadoNome_Renavam_2016(nomeTextSearch)
    ])

        .then(veiculos => filtraNaoEncontrados(veiculos))
}

export let procuraVeiculoDetalhadoChassi = function (chassi: string){
    chassi = chassi.replace(/[^0-9a-zA-Z]/g, '');
    
    return Promise.all([
        veiculoModel.getVeiculoDetalhadoChassi_Sispesquisa_Veiculos(chassi),
        veiculoModel.getVeiculoDetalhadoChassi_Sispesquisa_VeiculosNovo(chassi),
        veiculoModel.getVeiculoDetalhadoChassi_Renavam_2016(chassi)
    ])

        .then(veiculos => filtraNaoEncontrados(veiculos))
}

export let procuraVeiculoDetalhadoPlaca = function (placa: string){
    placa = placa.replace(/[^0-9a-zA-Z]/g, '');

    return Promise.all([
        veiculoModel.getVeiculoDetalhadoPlaca_LIKE_Sispesquisa_Veiculos(placa),
        veiculoModel.getVeiculoDetalhadoPlaca_LIKE_Sispesquisa_VeiculosNovo(placa),
        veiculoModel.getVeiculoDetalhadoPlaca_Renavam_2016(placa)
    ])

        .then(veiculos => filtraNaoEncontrados(veiculos))
}

export let procuraVeiculoDetalhadoRenavam = function (renavam: string){
    renavam = renavam.replace(/[^0-9a-zA-Z]/g, '');

    return Promise.all([
        veiculoModel.getVeiculoDetalhadoRenavam_Sispesquisa_Veiculos(renavam),
        veiculoModel.getVeiculoDetalhadoRenavam_Sispesquisa_VeiculosNovo(renavam),
        veiculoModel.getVeiculoDetalhadoRenavam_Renavam_2016(renavam)
    ])

        .then(veiculos => filtraNaoEncontrados(veiculos))
}
