import * as mssql from 'mssql';

import { connection } from './../services/db.service';
import { MODEL_PRIORITY, API_CONFIG } from './../config';
import {
    trataResultado,
    resultNotFound,
    resultFound,
    logErroBuscaBD
} from './../utils';

const fonte = MODEL_PRIORITY['bolsa_familia'].fonte;
const rank  = MODEL_PRIORITY['bolsa_familia'].rank;
const grupo = MODEL_PRIORITY['bolsa_familia'].grupo;

export let getBolsaFamiliaCPF_BolsaFamilia = function (cpf: string) {

    return connection.getConn().request()
        .input('cpf', mssql.VarChar, cpf)
        .query(`
             SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                BF.Municipio as agenciaMunicipio,
                BF.UF as agenciaUf,
                BF.NomeTitular as nome,
                CAD.CPF as cpf,
                BF.NisTitular as nis,
                BF.CompetFolha as competenciaFolha,
                BF.VlrTotal as vlrTotal,
                BF.Endereco as logradouro,
                BF.Localidade as municipio,

                BF.NISDEPENDEN as nisDependente,
                BF.Dependente as nomeDependente,
                BF.Idade as idadeDependente,
                BF.RendaPerc as rendaPerCapita,
                BF.Renda_com_PBF as rendaComPBF,
                BF.QTDE_MEMBROS as qtdMembros,

                CAD.COD_FAMILIAR as codFamiliar,
                CAD.NUM_MEMBROS_FAM as numMembros,
                CAD.VLR_RENDA_FAM as vlrRendaFamilia
            FROM (
                SELECT * FROM BolsaFamilia.dbo.VW_BolsaFamilia_PB_2014
                UNION ALL
                SELECT * FROM BolsaFamilia.dbo.VW_BolsaFamilia_PB_2015
                UNION ALL
                SELECT * FROM BolsaFamilia.dbo.VW_BolsaFamilia_PB_2016
            ) AS BF
            INNER JOIN Cadastro_Unico.dbo.bloco_00 AS CAD ON (BF.NISTitular = CAD.NIS)
            WHERE CAD.CPF=@cpf
        `)
        .then(result => {
            if (result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar registro de Bolsa Familia de ${cpf}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelo registro de Bolsa Familia de ${cpf}.`, 'getBolsaFamiliaCPF_BolsaFamilia'); });
};

export let getBolsaFamiliaNIS_BolsaFamilia = function (nis: string) {

    return connection.getConn().request()
        .input('nis', mssql.VarChar, nis)
        .query(`
            SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                BF.Municipio as agenciaMunicipio,
                BF.UF as agenciaUf,
                BF.NomeTitular as nome,
                CAD.CPF as cpf,
                BF.NisTitular as nis,
                BF.CompetFolha as competenciaFolha,
                BF.VlrTotal as vlrTotal,
                BF.Endereco as logradouro,
                BF.Localidade as municipio,

                BF.NISDEPENDEN as nisDependente,
                BF.Dependente as nomeDependente,
                BF.Idade as idadeDependente,
                BF.RendaPerc as rendaPerCapita,
                BF.Renda_com_PBF as rendaComPBF,
                BF.QTDE_MEMBROS as qtdMembros,

                CAD.COD_FAMILIAR as codFamiliar,
                CAD.NUM_MEMBROS_FAM as numMembros,
                CAD.VLR_RENDA_FAM as vlrRendaFamilia
            FROM (
                SELECT * FROM BolsaFamilia.dbo.VW_BolsaFamilia_PB_2014
                UNION ALL
                SELECT * FROM BolsaFamilia.dbo.VW_BolsaFamilia_PB_2015
                UNION ALL
                SELECT * FROM BolsaFamilia.dbo.VW_BolsaFamilia_PB_2016
            ) AS BF
            INNER JOIN Cadastro_Unico.dbo.bloco_00 AS CAD ON (BF.NISTitular = CAD.NIS)
            WHERE CAD.NIS=@nis
        `)
        .then(result => {
            if (result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar registro de Bolsa Familia de ${nis}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelo registro de Bolsa Familia de ${nis}.`, 'getBolsaFamiliaNIS_BolsaFamilia'); });
};
