import * as mssql from 'mssql';

import { connection } from '../services/db.service';
import { MODEL_PRIORITY, API_CONFIG } from '../config';
import { BenchmarkService } from '../services/benchmark.service';
import {
    resultNotFound,
    resultFound,
    logErroBuscaBD,
    printTempoExecucao
} from '../utils';

const fonte = MODEL_PRIORITY['condenacao'].fonte;
const rank  = MODEL_PRIORITY['condenacao'].rank;
const grupo = MODEL_PRIORITY['condenacao'].grupo;

export let getCondenacaoTRF5CPF = function (cpf: string) {

    const tempoInicial = new BenchmarkService();

    return connection.getConn().request()
        .input('CPF', mssql.Char(11), cpf)
        .query(`
            SELECT DISTINCT
                NUMERO_PROCESSO as processo, DATA_PROCESSO_DT as dataProcesso,
                CPF as cpf, DATA_JULGAMENTO_DT as dataJulgamento,
                CASE
                    WHEN OBSERVACOES NOT LIKE 'O campo%' THEN OBSERVACOES
                    ELSE NULL
                END as resultado,
                CASE
                    WHEN ISNULL(ASSUNTO_RESUMO_DELIBERACAO, '-1') <> '-1' THEN UPPER(ASSUNTO_RESUMO_DELIBERACAO)
                    WHEN ISNULL(OBSERVACOES, '-1') <> '-1' AND ISNULL(LIVRE_1, '-1') <> '-1' AND OBSERVACOES NOT LIKE 'O campo%' THEN LIVRE_1 + ', ' + ISNULL(LIVRE_2,'') + ', ' + LIVRE_3
                    ELSE NULL
                END AS decisao,
                'trf5' as fonte

            FROM SISPESQUISA.dbo.condenacoes_TRF
            WHERE CPF=@CPF
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getCondenacaoTRF5CPF');

            if (result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar condenacoes de ${cpf}.`, fonte);
            else {
                return resultFound(result, fonte, rank, grupo);
            }
        }).catch(error => logErroBuscaBD(error, `Falha na busca pelas condenacoes de ${cpf}.`, 'getCondenacaoTRF5CPF'));
};

export let getCondenacaoTREPBCPF = function (cpf: string) {

    const tempoInicial = new BenchmarkService();

    return connection.getConn().request()
        .input('CPF', mssql.Char(11), cpf)
        .query(`
            SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                Processo as processo,
                UPPER(Jurisdicionado) AS jurisdicionado,
                CPF as cpf,
                Decisao as decisao,
                Data_Julgamento as dataJulgamento,
                Res_Decisao_Poder_Legislativo as decisaoLegislativo,
                'trepb' as fonte

            FROM SISPESQUISA.dbo.Contas_Reprovadas_TCEPB_TRE2018
            WHERE CPF=@CPF
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getCondenacaoTREPBCPF');

            if (result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar condenacoes de ${cpf}.`, fonte);
            else {
                return resultFound(result, fonte, rank, grupo);
            }
        }).catch(error => logErroBuscaBD(error, `Falha na busca pelas condenacoes de ${cpf}.`, 'getCondenacaoTREPBCPF'));
};