import * as mssql from 'mssql';

import { connection }  from './../../services/db.service';
import { MODEL_PRIORITY, API_CONFIG } from './../../config';
import {
    trataResultado,
    resultNotFound,
    resultFound,
    logErroBuscaBD
} from './../../utils';

import {SIGLA_FONTE} from './../_fontes';

const fonte_dado = SIGLA_FONTE['vep'];

const fonte = MODEL_PRIORITY['vep.advogado'].fonte;
const rank  = MODEL_PRIORITY['vep.advogado'].rank;
const grupo = MODEL_PRIORITY['vep.advogado'].grupo;

export let getAdvogadoDetalhadoCPF_VEP = function (cpf: string){

    return connection.getConn().request()
        .input('cpf_parameter', mssql.VarChar(11), cpf)
        .query(`
            SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                PF.CPF as cpf
                ,UPPER(P.DS_PESSOA) as nome
                ,ADV.STATUS as status
                ,ADV.MATRICULA as matricula
                ,ADV.DEFENSOR as defensor
                ,ADV.NUMEROOAB as numeroOAB
                ,UF_OAB.DS_SIGLA AS ufOAB
                ,'advogado' as tipo
                ,'${fonte_dado}' as fonte

            FROM VEP.dbo.VEP_CNJ_TB_ADVOGADO ADV
                INNER JOIN VEP.dbo.VEP_CNJ_TB_PESSOAFISICA PF ON (PF.ID_PESSOAFISICA = ADV.FK_PESSOAFISICA)
                LEFT OUTER JOIN VEP.dbo.VEP_CNJ_TB_UF UF_OAB ON (UF_OAB.ID_UF = ADV.FK_OAB_UF)
                LEFT OUTER JOIN VEP.dbo.VEP_CNJ_TB_PESSOA P ON (P.ID_PESSOA = PF.FK_PESSOA)
            WHERE CPF=@cpf_parameter
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o advogado ${cpf}.`, fonte);
            else {
                return resultFound(result, fonte, rank, grupo);
            }
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelo advogado ${cpf}.`, 'getAdvogadoDetalhadoCPF_VEP'); });
};