import * as mssql from 'mssql';

import { connection }  from './../../services/db.service';

import {
    MODEL_PRIORITY,
    API_CONFIG
} from './../../config';

import {
    resultNotFound,
    resultFound,
    logErroBuscaBD
} from './../../utils';

import {SIGLA_FONTE} from './../_fontes';

const fonte_dado = SIGLA_FONTE['receita_full'];

const fonte = MODEL_PRIORITY['receita_full.contador'].fonte;
const rank  = MODEL_PRIORITY['receita_full.contador'].rank;
const grupo = MODEL_PRIORITY['receita_full.contador'].grupo;

export let getContadorCNPJ_Receita_Full = function (cnpj: string){

    return connection.getConn().request()
        .input('CNPJ', mssql.VarChar(14), cnpj)
        .query(`
            SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                CNPJ as cnpj,
                CPFContador AS cpf,
                PF.Nome as nome,
                CRCContadorPF as crc,
                UFCRCPF as ufCRC,
                '${fonte_dado}' as fonte

            FROM Receita_Full.Receita.Contador C
                INNER JOIN ReceitaNovo.dbo.PessoaFisica PF ON (C.CPFContador = PF.CPF)
            WHERE C.CNPJ = @CNPJ
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o contador de ${cnpj}.`, fonte);
            else {
                return resultFound(result, fonte, rank, grupo);
            }
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelo advogado ${cnpj}.`, 'getContadorCNPJ_Receita_Full'); });
};