import * as rp from 'request-promise-native';

import * as pessoaModel from '../pessoa/pessoa.model.receitanovo';
import { MODEL_PRIORITY, API_CONFIG } from './../../config';
import {
	resultNotFound,
	logErroBuscaBD,
	resultFoundRaw,
	print,
	agrupaEFiltraDuplicados,
	filtraNaoEncontrados,
	desagrupa,
	geraListaNomes,
	trataResultadoFacebook,
} from './../../utils';

const fonte = MODEL_PRIORITY['crawler.facebook'].fonte;
const rank  = MODEL_PRIORITY['crawler.facebook'].rank;
const grupo = MODEL_PRIORITY['crawler.facebook'].grupo;

export let getFacebookCPF_SIAP_CRAWLERS = function (cpf: string){

	let nomeReal;
	return pessoaModel.getPessoaDetalhadoCPF_ReceitaNovo_PessoaFisica(cpf)
	.then((r: any) => {
		if (r.status === 'OK'){
			nomeReal = r.resultado.dados[0].nome;

			return r.resultado.dados[0].nome;
		} else {
			throw r.msg;
		}
	})
	.then(nome => geraListaPromises(geraListaNomes(nome)))
	.then(registros => filtraNaoEncontrados(registros))
	.then(registros => agrupaEFiltraDuplicados(registros))
	.then(registros => desagrupa(registros))
	.then(resultado => {
		if (resultado.status === 'OK' && resultado.dados.length > 0){
			resultado.dados = trataResultadoFacebook(resultado.dados, nomeReal);
			return resultFoundRaw(resultado.dados, fonte, rank, grupo);
		} else {
			return resultado;
		}
	})
	//.then(r => print(r))

	.catch(error => {return error;})
}

let geraPromiseBusca = function (nome : string) {
	const nomeFormatado  = nome.split(' ').join('+');
	const urlBase        = 'https://www.google.com.br';
	const CRAWLER_SERVER = API_CONFIG.SIAP_CRAWLERS_URL;

	let options = {
		method: 'GET',
		uri: `${CRAWLER_SERVER}/crawl.json?spider_name=facebook&url=${urlBase}/search?q=site%3Ahttps%3A%2F%2Fpt-br.facebook.com+%22${nomeFormatado}%22`,
		timeout: 5000,
		json: true
	};

	return rp(options)
	.then(data => {
		if (data.status === 'ok' && data.items.length > 0) {
			data.items = data.items.map(item => {
				item.fonte = 'facebook';
				return item;
			});

			if (data.items.length > 0) {
        return resultFoundRaw(data.items, fonte, rank, grupo);
			}
		}
		return resultNotFound(`Nao foi possivel encontrar registros no google para ${nome}.`, fonte);
	})
	.catch(error => { return logErroBuscaBD(error, `Falha na busca pelos registros no google de ${nome}.`, 'getFacebookCPF_SIAP_CRAWLERS') });
}

let geraListaPromises = function (listaNomes: Array<string>) {
	return Promise.all(listaNomes.map(nome => geraPromiseBusca(nome)));
}
