import * as rp from 'request-promise-native';

import * as pessoaModel from '../pessoa/pessoa.model.receitanovo';
import * as empresaModel from '../empresa/empresa.model.receitanovo';

import { MODEL_PRIORITY, API_CONFIG } from './../../config';
import {
  resultNotFound,
  logErroBuscaBD,
  resultFoundRaw,
  print,
} from './../../utils';

const fonte = MODEL_PRIORITY['crawler.jusbrasil'].fonte; // = 'bnmp.mandado';
const rank  = MODEL_PRIORITY['crawler.jusbrasil'].rank;  // 1;
const grupo = MODEL_PRIORITY['crawler.jusbrasil'].grupo;  // 1;

export let getJusBrasilCPF_SIAP_CRAWLERS = function (cpf: string) {
  return pessoaModel.getPessoaDetalhadoCPF_ReceitaNovo_PessoaFisica(cpf)
  .then((r: any) => {
    if (r.status === 'OK') {
      return r.resultado.dados[0].nome;
    } else {
      throw r.msg;
    }
  })
  .then(Request_JusBrasil)
  .catch(error => {return error; })
};

export let getJusBrasilCNPJ_SIAP_CRAWLERS = function (cnpj: string) {
  return empresaModel.getEmpresaDetalhadoCNPJ_ReceitaNovo_PessoaJuridica(cnpj)
  .then((r: any) => {
    if (r.status === 'OK') {
      return r.resultado.dados[0].razaoSocial;
    } else {
      throw r.msg;
    }
  })
  .then(Request_JusBrasil)
  .catch(error => { return error; })
};

let Request_JusBrasil = function (nome) {

  const CRAWLER_SERVER = API_CONFIG.SIAP_CRAWLERS_URL;
  const urlBase        = 'https://www.jusbrasil.com.br';
  const nomeFormatado  = nome.split(' ').join('+');

  let options = {
    method: 'GET',
    uri: `${CRAWLER_SERVER}/crawl.json?spider_name=jusbrasil&url=${urlBase}/busca?q=%22${nomeFormatado}%22`,
    timeout: 5000,
    json: true
  };

  return rp(options)
  .then(data => {
    if (data.status === 'ok' && data.items.length > 0) {
      data.items = data.items.map(item => {
        if (!item || !item.url) {
          return null;
        }

        item.fonte = 'jusbrasil';
        item.url   = urlBase + item.url;

        return item;
      }).filter(i => i !== null);

      if (data.items.length > 0) {
        return resultFoundRaw(data.items, fonte, rank, grupo);
      }
    }

    return resultNotFound(`Nao foi possivel encontrar registros no jusbrasil para ${nome}.`, fonte);
  })
  .catch(error => { throw logErroBuscaBD(error, `Falha na busca pelos registros no jusbrasil de ${nome}.`, 'Request_JusBrasil') });
}