import * as rp from 'request-promise-native';

import * as pessoaModel from '../pessoa/pessoa.model.receitanovo';
import * as empresaModel from '../empresa/empresa.model.receitanovo';

import { MODEL_PRIORITY, API_CONFIG } from './../../config';
import {
  resultNotFound,
  logErroBuscaBD,
  resultFoundRaw,
  print,
} from './../../utils';

const fonte = MODEL_PRIORITY['crawler.google.mpbr'].fonte;
const rank  = MODEL_PRIORITY['crawler.google.mpbr'].rank;
const grupo = MODEL_PRIORITY['crawler.google.mpbr'].grupo;

export let getGoogleMPBRCPF_SIAP_CRAWLERS = function (cpf: string) {
  return pessoaModel.getPessoaDetalhadoCPF_ReceitaNovo_PessoaFisica(cpf)
  .then((r: any) => {
    if (r.status === 'OK') {
      return r.resultado.dados[0].nome;
    } else {
      throw r.msg;
    }
  })
  .then(Request_MPBR)
  .catch(error => { return error; })
};

export let getGoogleMPBRCNPJ_SIAP_CRAWLERS = function (cnpj: string) {
  return empresaModel.getEmpresaDetalhadoCNPJ_ReceitaNovo_PessoaJuridica(cnpj)
  .then((r :any) => {
    if (r.status === 'OK') {
      return r.resultado.dados[0].razaoSocial;
    } else {
      throw r.msg;
    }
  })
  .then(Request_MPBR)
  .catch(error => { return error; })
};

let Request_MPBR = function (nome) {

  const CRAWLER_SERVER = API_CONFIG.SIAP_CRAWLERS_URL;
  const urlBase        = 'https://www.jusbrasil.com.br';
  const nomeFormatado  = nome.split(' ').join('+');

  let options = {
    method: 'GET',
    uri: `${CRAWLER_SERVER}/crawl.json?spider_name=google&url=https://www.google.com.br/search?q=site%3A.mp.br+%22${nomeFormatado}%22`,
    timeout: 5000,
    json: true
  };

  return rp(options)
  .then(data => {
    if (data.status === 'ok' && data.items.length > 0) {
      data.items = data.items.map(item => {
        if (!item || !item.url) {
          return null;
        }

        item.dominio = item.url.split('://')[1].split('/')[0];
        item.fonte = 'google_mpbr';
        return item;
      }).filter(i => i !== null);

      if (data.items.length > 0) {
        return resultFoundRaw(data.items, fonte, rank, grupo);
      }
    }
    return resultNotFound(`Nao foi possivel encontrar registros no google .mp.br para ${nome}.`, fonte);
  })
  .catch(error => { throw logErroBuscaBD(error, `Falha na busca pelos registros no google .mp.br de ${nome}.`, 'Request_MPBR') });
}