import * as mssql from 'mssql';

import { connection } from '../../services/db.service';
import { MODEL_PRIORITY, API_CONFIG } from './../../config';
import {
    resultNotFound,
    resultFound,
    logErroBuscaBD
} from '../../utils';

const fonteEmpenhos   = MODEL_PRIORITY['sagres_estadual.empenhos'].fonte;
const rankEmpenhos    = MODEL_PRIORITY['sagres_estadual.empenhos'].rank;
const grupoEmpenhos   = MODEL_PRIORITY['sagres_estadual.empenhos'].grupo;

export let getEmpenhoDetalhadoCPF_SAGRES_ESTADUAL_Empenhos_Estado = function (cpf: string){

    return connection.getConn().request()
        .input('cpf_parameter', mssql.Char(14), '000' + cpf)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                    TP_LICITACAO.de_TipoLicitacao as tipoLicitacao, nu_Empenho as numEmpenho,
                    dt_Empenho as dataEmpenho, vl_Empenho as vlEmpenho,
                    vl_Pagto as vlPagamento, de_Historico as historico,
                    RIGHT(cd_Credor, 11) as cpf, no_Credor as nome,
                    TP_CREDOR.de_TipoCredor as tipoCredor, CD_UGEST2.de_Ugestora as uGestora

                FROM SAGRES_ESTADUAL.dbo.Empenhos_Estado AS EMPENHO
                    LEFT OUTER JOIN SAGRES_ESTADUAL.dbo.Tipo_Modalidade_Licitacao AS TP_LICITACAO
                        ON (EMPENHO.tp_Licitacao = TP_LICITACAO.tp_Licitacao)
                    LEFT OUTER JOIN SAGRES_ESTADUAL.dbo.Tipo_Credor AS TP_CREDOR
                        ON (EMPENHO.tp_Credor = TP_CREDOR.tp_Credor)
                    LEFT OUTER JOIN SAGRES_ESTADUAL.dbo.Codigo_UGestora AS CD_UGEST2
                        ON (EMPENHO.cd_UGestora_online = CD_UGEST2.cd_Ugestora AND EMPENHO.dt_Ano = CD_UGEST2.dt_Ano)
                WHERE cd_Credor=@cpf_parameter
                ORDER BY dt_Ano DESC, dt_MesAno DESC
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar empenhos de ${cpf}.`, fonteEmpenhos);
            else
                return resultFound(result, fonteEmpenhos, rankEmpenhos, grupoEmpenhos);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pelos empenhos de ${cpf}.`, 'getEmpenhoDetalhadoCPF_SAGRES_ESTADUAL_Empenhos_Estado'); });
};

export let getEmpenhoDetalhadoCNPJ_SAGRES_ESTADUAL_Empenhos_Estado = function (cnpj: string){

    return connection.getConn().request()
        .input('cnpj_parameter', mssql.Char(14), cnpj)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                    TP_LICITACAO.de_TipoLicitacao as tipoLicitacao, nu_Empenho as numEmpenho,
                    dt_Empenho as dataEmpenho, vl_Empenho as vlEmpenho,
                    vl_Pagto as vlPagamento, de_Historico as historico,
                    cd_Credor as cpf, no_Credor as nome,
                    TP_CREDOR.de_TipoCredor as tipoCredor, CD_UGEST2.de_Ugestora as uGestora

                FROM SAGRES_ESTADUAL.dbo.Empenhos_Estado AS EMPENHO
                    LEFT OUTER JOIN SAGRES_ESTADUAL.dbo.Tipo_Modalidade_Licitacao AS TP_LICITACAO
                        ON (EMPENHO.tp_Licitacao = TP_LICITACAO.tp_Licitacao)
                    LEFT OUTER JOIN SAGRES_ESTADUAL.dbo.Tipo_Credor AS TP_CREDOR
                        ON (EMPENHO.tp_Credor = TP_CREDOR.tp_Credor)
                    LEFT OUTER JOIN SAGRES_ESTADUAL.dbo.Codigo_UGestora AS CD_UGEST2
                        ON (EMPENHO.cd_UGestora_online = CD_UGEST2.cd_Ugestora AND EMPENHO.dt_Ano = CD_UGEST2.dt_Ano)
                WHERE cd_Credor=@cnpj_parameter
                ORDER BY dt_Ano DESC, dt_MesAno DESC
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar empenhos de ${cnpj}.`, fonteEmpenhos);
            else
                return resultFound(result, fonteEmpenhos, rankEmpenhos, grupoEmpenhos);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pelos empenhos de ${cnpj}.`, 'getEmpenhoDetalhadoCNPJ_SAGRES_ESTADUAL_Empenhos_Estado'); });
};

const fonteCredores   = MODEL_PRIORITY['sagres_estadual.credores_pb'].fonte;
const rankCredores    = MODEL_PRIORITY['sagres_estadual.credores_pb'].rank;
const grupoCredores   = MODEL_PRIORITY['sagres_estadual.credores_pb'].grupo;

export let getEmpenhoSimplificadoCPF_SAGRES_ESTADUAL_Credores_PB = function (cpf: string){

    return connection.getConn().request()
        .input('cpf_parameter', mssql.Char(14), '000' + cpf)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                    Codigo_Ugestora.de_Ugestora as unidadeGestora,
                    Credores.dt_Ano as dtAno,
                    vl_Empenhos as vEmpenho, vl_Estornos as vEstorno,
                    vl_Pagtos as vPagto

                FROM SAGRES_ESTADUAL.dbo.Credores_PB AS Credores
                    LEFT OUTER JOIN SAGRES_ESTADUAL.dbo.Codigo_UGestora ON (Credores.cd_ugestora = Codigo_Ugestora.cd_Ugestora AND  Credores.dt_Ano = Codigo_Ugestora.dt_Ano)
                WHERE Credores.cd_credor=@cpf_parameter
                ORDER BY Credores.dt_Ano DESC
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar empenhos de ${cpf}.`, fonteCredores);
            else
                return resultFound(result, fonteCredores, rankCredores, grupoCredores);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelos empenhos de ${cpf}.`, 'getEmpenhoSimplificadoCPF_SAGRES_ESTADUAL_Credores_PB'); });
};

export let getEmpenhoSimplificadoCNPJ_SAGRES_ESTADUAL_Credores_PB = function (cnpj: string){

    return connection.getConn().request()
        .input('cnpj_parameter', mssql.Char(14), cnpj)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                    Codigo_Ugestora.de_Ugestora as unidadeGestora, Credores.dt_Ano as dtAno,
                    vl_Empenhos as vEmpenho, vl_Estornos as vEstorno,
                    vl_Pagtos as vPagto

                FROM SAGRES_ESTADUAL.dbo.Credores_PB AS Credores
                    LEFT OUTER JOIN SAGRES_ESTADUAL.dbo.Codigo_UGestora ON (Credores.cd_ugestora = Codigo_Ugestora.cd_Ugestora AND  Credores.dt_Ano = Codigo_Ugestora.dt_Ano)
                WHERE Credores.cd_credor=@cnpj_parameter
                ORDER BY Credores.dt_Ano DESC
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar empenhos de ${cnpj}.`, fonteCredores);
            else
                return resultFound(result, fonteCredores, rankCredores, grupoCredores);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pelos empenhos de ${cnpj}.`, 'getEmpenhoSimplificadoCNPJ_SAGRES_ESTADUAL_Credores_PB'); });
};