import * as mssql from 'mssql';

import { connection } from '../../services/db.service';
import { Pessoa } from '../schemas';
import{
    trataResultado,
    resultNotFound,
    resultFound,
    logErroBuscaBD
} from '../../utils';
import { MODEL_PRIORITY, API_CONFIG } from '../../config';

const fonte = MODEL_PRIORITY['sagres.empenhos'].fonte;
const rank  = MODEL_PRIORITY['sagres.empenhos'].rank;
const grupo = MODEL_PRIORITY['sagres.empenhos'].grupo;

export let getEmpenhosMunicipalEmpresaCNPJ = function (cnpj: string) {

    return connection.getConn().request()
        .input('CNPJ', mssql.Char(14), cnpj)
        .query(`
            EXEC SAGRES_MUNICIPAL.DBO.USP_EMPENHOS_LICITACOES @CNPJ
        `)
        .then((result: any) => {
            result.recordset = result.recordset.map(d => {
                return {
                    ugestora: d.de_Ugestora.toUpperCase(),
                    anoLicitacao: d.Ano_Licitacao,
                    tipoLicitacao: d.de_TipoLicitacao.toUpperCase(),
                    numeroLicitacao: d.nu_Licitacao,
                    vlLicitacao: d.vl_Licitacao,
                    numeroEmpenho: d.nu_Empenho,
                    dataEmpenho: d.dt_Empenho,
                    dataPagamento: d.dt_Pagamento,
                    vlEmpenho: d.vl_Empenho,
                    vlEstornado: d.TotalEstornado,
                    vlPago: d.TotalPago,
                    descricao: d.de_Historico.toUpperCase()
                }
            }).slice(0, API_CONFIG.SIAP_MAX_RESULTS);

            if (result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar empenhos de ${cnpj}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => logErroBuscaBD(error, `Falha na busca pelos empenhos de ${cnpj}.`, 'getEmpenhosEmpresaCNPJ'));
};
