import * as mssql from 'mssql';

import { connection } from './../../services/db.service';
import { Pessoa } from './../schemas';
import { MODEL_PRIORITY, API_CONFIG } from './../../config';
import {
    trataResultado,
    resultNotFound,
    resultFound,
    logErroBuscaBD
} from './../../utils';

const fonte_dado = 'CAGED';

const fonte = MODEL_PRIORITY['empregador.caged'].fonte;
const rank  = MODEL_PRIORITY['empregador.caged'].rank;
const grupo = MODEL_PRIORITY['empregador.caged'].grupo;

export let getEmpregadoresDetalhadoCPF_Caged = function (cpf: string) {

    return connection.getConn().request()
        .input('cpf', mssql.NVarChar, cpf)
        .query(`
             SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                Competencia as competencia
                ,CPF as cpf
                ,NomeTrabalhador as nome
                ,Idade as idade
                ,PIS as pis
                ,DataAdmissao as dataAdmissao
                ,GRAU_INSTR.DS_GRAU_INSTR AS grauInstrucao
                ,QtdHorasContrat as horasSemanais
                ,SalarioMensal as salarioMensal
                ,IndPortadorDefic as indicadorDeficiencia
                ,TP_DEFIC.DS_TP_DEFIC as tipoDeficiencia
                ,CASE
                    WHEN SaldoMov=-1 THEN 'Saída'
                    ELSE 'Entrada'
                END as sentidoMovimentacao
                ,TempoEmprego as mesesEmprego
                ,TP_MOV_DESAG.DS_TP_MOV_DESAG AS tipoMovimentacao
                ,CNPJCEI as cnpj
                ,PJ.RazaoSocial as razaoSocial
                ,PJ.NomeFantasia as nomeFantasia
                ,PJ.Municipio as municipio
                ,PJ.Uf as uf
                ,FX_TAMANHO.DS_TAM_ESTAB AS numeroEmpregados
                ,'${fonte_dado}' as fonte

            FROM CAGED.dbo.Empregados AS E
                INNER JOIN [CAGED].[dbo].[FX_EMP_JAN] AS FX_TAMANHO ON (E.FaixaEmpInicioJan = FX_TAMANHO.CD_TAM_ESTAB)
                INNER JOIN CAGED.dbo.GRAU_INSTR ON (E.GrauInstrucao = GRAU_INSTR.CD_GRAU_INSTR)
                INNER JOIN CAGED.dbo.TP_DEFIC ON (E.TipoDefic = TP_DEFIC.CD_TP_DEFIC)
                INNER JOIN CAGED.dbo.TP_MOV_DESAG ON (E.TipoMovDesagregado = TP_MOV_DESAG.CD_TP_MOV_DESAG)
                LEFT OUTER JOIN ReceitaNovo.dbo.PessoaJuridica AS PJ ON (E.CNPJCEI = PJ.CNPJ)
            WHERE E.CPF=@cpf
            ORDER BY Competencia
        `)
        .then(result => {
            if (result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar registro de empregador para ${cpf}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelo registro de emprego de ${cpf}.`, 'getEmpregadoresCPF_Caged'); });
};

export let getEstatisticasEmpregadoresCNPJ_Caged = function (cnpj: string) {

    return connection.getConn().request()
        .input('cnpj', mssql.NVarChar, cnpj)
        .query(`
             SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                LEFT(Competencia,4) as ano,
                SUM(NULLIF(CAST(SaldoMov as int),-1)) as numeroEntradas,
                SUM(NULLIF(CAST(SaldoMov as int), 1)) as numeroSaidas,
                FX_TAMANHO.DS_TAM_ESTAB AS numeroEmpregados,
                AVG(NULLIF(salarioMensal, 0)) as mediaSalarial,
                MIN(NULLIF(salarioMensal, 0)) as minSalarial,
                MAX(NULLIF(salarioMensal, 0)) as maxSalarial,
                '${fonte_dado}' as fonte

            FROM CAGED.dbo.Empregados AS E
                INNER JOIN [CAGED].[dbo].[FX_EMP_JAN] AS FX_TAMANHO ON (E.FaixaEmpInicioJan = FX_TAMANHO.CD_TAM_ESTAB)
                INNER JOIN CAGED.dbo.GRAU_INSTR ON (E.GrauInstrucao = GRAU_INSTR.CD_GRAU_INSTR)
                INNER JOIN CAGED.dbo.TP_DEFIC ON (E.TipoDefic = TP_DEFIC.CD_TP_DEFIC)
                INNER JOIN CAGED.dbo.TP_MOV_DESAG ON (E.TipoMovDesagregado = TP_MOV_DESAG.CD_TP_MOV_DESAG)
                LEFT OUTER JOIN ReceitaNovo.dbo.PessoaJuridica AS PJ ON (E.CNPJCEI = PJ.CNPJ)
            WHERE E.CNPJCEI=@cnpj
            GROUP BY LEFT(Competencia,4), FX_TAMANHO.DS_TAM_ESTAB
            ORDER BY LEFT(Competencia,4)
        `)
        .then(result => {
            if (result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar registro de empregador para ${cnpj}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelo registro de emprego de ${cnpj}.`, 'getEstatisticasEmpregadoresCNPJ_Caged'); });
};


// SELECT
// [Competencia]
//     , [CPF] as cpf
//     , [NomeTrabalhador] as nome
//     , [PIS] as pis
//     , [DataAdmissao]
//     , FX_ETARIA.DS_FAIXA_ETARIA AS faixaEtaria
//         , FX_HORARIA.DS_FAIXA_HORA_CONTRATUAL AS faixaHoraria
//             , FX_REMUNERACAO.DS_FAIXA_REMUNERAÇAO_MEDIA_DEZEMBRO AS faixaRemuneracao
//                 , FX_TEMPO_EMPREGADO.DS_FX_TEMPO_EMPREGO AS faixaTempoEmprego
//                     , GRAU_INSTR.DS_GRAU_INSTR AS grauInstrucao
//                         , [QtdHorasContrat] as qtdHoras
//                         , [Idade]
//                         , [SalarioMensal]
//                         , [IndPortadorDefic]
//                         , TP_DEFIC.DS_TP_DEFIC
//                         , CASE
// WHEN SaldoMov = -1 THEN 'Saída'
// ELSE 'Entrada'
// END as sentidoMovimentacao
//     , [TempoEmprego] as mesesEmprego
//     , TP_MOV_DESAG.DS_TP_MOV_DESAG AS tipoMovimentacao

//         , [CNPJCEI] as cnpj
//         , PJ.RazaoSocial as razaoSocial
//         , PJ.NomeFantasia as nomeFantasia
//         , PJ.Municipio as municipio
//         , PJ.Uf as uf
//         , FX_TAMANHO.DS_TAM_ESTAB AS numeroEmpregados

// FROM[CAGED].[dbo].[Empregados] AS E
// INNER JOIN[CAGED].[dbo].[FAIXA_ETARIA] AS FX_ETARIA ON(E.FaixaEtaria = FX_ETARIA.CD_FAIXA_ETARIA)
// INNER JOIN[CAGED].[dbo].[FAIXA_HORA_CONTRATUAL] AS FX_HORARIA ON(E.FaixaHoraContratada = FX_HORARIA.CD_FAIXA_HORA_CONTRATUAL)
// INNER JOIN[CAGED].[dbo].[FAIXA_REMUNERAÇAO_MENSAL] AS FX_REMUNERACAO ON(E.FaixaSalarialMensal = FX_REMUNERACAO.CD_FAIXA_REMUNERAÇAO_MEDIA_DEZEMBRO)
// INNER JOIN[CAGED].[dbo].[FX_EMP_JAN] AS FX_TAMANHO ON(E.FaixaEmpInicioJan = FX_TAMANHO.CD_TAM_ESTAB)
// INNER JOIN[CAGED].[dbo].[FX_TEMPO_EMP] AS FX_TEMPO_EMPREGADO ON(E.FaixaTempoEmprego = FX_TEMPO_EMPREGADO.CD_FX_TEMPO_EMPREGO)
// INNER JOIN[CAGED].[dbo].[GRAU_INSTR] ON(E.GrauInstrucao = GRAU_INSTR.CD_GRAU_INSTR)
// INNER JOIN[CAGED].[dbo].[TP_DEFIC] ON(E.TipoDefic = TP_DEFIC.CD_TP_DEFIC)
// INNER JOIN[CAGED].[dbo].[TP_MOV_DESAG] ON(E.TipoMovDesagregado = TP_MOV_DESAG.CD_TP_MOV_DESAG)
// LEFT OUTER JOIN[ReceitaNovo].dbo.PessoaJuridica AS PJ ON(E.CNPJCEI = PJ.CNPJ)
// WHERE CPF = '09498728909'
// ORDER BY Competencia