import * as mssql from 'mssql';

import { connection } from './../../services/db.service';
import { MODEL_PRIORITY, API_CONFIG } from './../../config';
import {
    resultNotFound,
    resultFound,
    logErroBuscaBD
} from './../../utils';

const fonte_dado = 'RAIS';

const fonte = MODEL_PRIORITY['empregador.rais'].fonte;
const rank  = MODEL_PRIORITY['empregador.rais'].rank;
const grupo = MODEL_PRIORITY['empregador.rais'].grupo;

export let getEmpregadoresDetalhadoCPF_RAIS = function (cpf: string) {

    return connection.getConn().request()
        .input('cpf', mssql.VarChar, cpf)
        .query(`
             SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                    ano, tipoAdmissao, dataAdmissao, mediaSalarial as salarioMensal, salarioContratual,
                    horasSemanais, mesesEmprego, motivoDesligamento, pis, ctps, cpf,
                    nome, cei, cnpj, razaoSocial, nomeFantasia, tamanhoEstabelecimento,
                    municipio, uf, '${fonte_dado}' as fonte

            FROM RAIS.dbo.VW_RAIS
            WHERE CPF=@cpf
            ORDER BY ano
        `)
        .then(result => {
            if (result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar registro de empregador para ${cpf}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelo registro de empregador para ${cpf}.`, 'getEmpregadoresCPF_RAIS'); });
};

export let getEstatisticasEmpregadoresCNPJ_RAIS = function (cnpj: string) {

    return connection.getConn().request()
        .input('cnpj', mssql.VarChar, cnpj)
        .query(`
            SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                ano,
                COUNT(DISTINCT CPF) as numeroFuncionariosDistintos,
                SUM(CASE
                        WHEN tipoAdmissao = 'NAO ADM ANO' THEN 0
                        ELSE 1
                    END) as numeroEntradas,
                SUM(CASE
                        WHEN motivoDesligamento = 'NAO DESL ANO' THEN 0
                        ELSE -1
                    END) as numeroSaidas,
                SUM(CASE
                        WHEN tipoAdmissao = 'NAO ADM ANO' AND motivoDesligamento = 'NAO DESL ANO' THEN 1
                        ELSE 0
                    END) as numeroManteve,
                tamanhoEstabelecimento,
                AVG(NULLIF(mediaSalarial, 0)) as mediaSalarial,
                MIN(NULLIF(mediaSalarial, 0)) as minSalarial,
                MAX(NULLIF(mediaSalarial, 0)) as maxSalarial,
                '${fonte_dado}' as fonte

            FROM RAIS.dbo.VW_RAIS
            WHERE cnpj=@cnpj
            GROUP BY ano, tamanhoEstabelecimento
            ORDER BY ano
        `)
        .then(result => {
            if (result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar registro de empregador para ${cnpj}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelo registro de empregador para ${cnpj}.`, 'getEstatisticasEmpregadoresCNPJ_RAIS'); });
};