import * as mssql from 'mssql';

import { connection } from './../services/db.service';
import { MODEL_PRIORITY, API_CONFIG } from './../config';
import {
    trataResultado,
    resultNotFound,
    resultFound,
    logErroBuscaBD
} from './../utils';

const fonte = MODEL_PRIORITY['receitanovo.pessoajuridica.filial'].fonte;
const rank  = MODEL_PRIORITY['receitanovo.pessoajuridica.filial'].rank;
const grupo = MODEL_PRIORITY['receitanovo.pessoajuridica.filial'].grupo;

export let getFilialEmpresaDetalhadoCNPJ_ReceitaNovo_PessoaJuridica = function (cnpj: string) {


    return connection.getConn().request()
        .input('cnpj', mssql.VarChar, cnpj)
        .query(`SELECT
                    CNPJ as cnpj, IdentificadorMatrizFilial as matriz

                FROM ReceitaNovo.dbo.PessoaJuridica
                WHERE CNPJ=@cnpj AND IdentificadorMatrizFilial = 1
        `)
        .then(result => {
            if (result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar a filial de ${cnpj}.`, fonte);
            else
                return getFilialEmpresa_ReceitaNovo(cnpj);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pela filial de ${cnpj}.`, 'getFilialEmpresaDetalhadoCNPJ_ReceitaNovo_PessoaJuridica'); });
};


let getFilialEmpresa_ReceitaNovo = function(cnpj: string) {

    const inicioCnpj = cnpj.substring(0, 10);

    return connection.getConn().request()
        .input('cnpj', mssql.VarChar, inicioCnpj + '%')
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        CNPJ as cnpj, NomeFantasia as nomeFantasia, RazaoSocial as razaoSocial,
                        cast(DataInicioAtividade as date) as dataInicioAtividade,
                        Municipio as municipio,
                        DescricaoTipoLogradouro as tipoLogradouro,
                        Logradouro as logradouro,
                        UF as uf, 'RF4' as fonte

                FROM ReceitaNovo.dbo.PessoaJuridica
                WHERE CNPJ LIKE @cnpj AND IdentificadorMatrizFilial != 1
        `)
        .then(result => {
            if (result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar a filial de ${cnpj}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pela filial de ${cnpj}.`, 'getFilialEmpresa_ReceitaNovo'); });
}
