import * as mssql from 'mssql';

import { connection }  from '../../services/db.service';
import { trataResultado, resultNotFound, resultFound, logErroBuscaBD } from '../../utils';
import { MODEL_PRIORITY, API_CONFIG } from '../../config';

import { SIGLA_FONTE } from './../_fontes';

const fonte_dado = SIGLA_FONTE['receita'];

const fonte  = MODEL_PRIORITY['receita.pj'].fonte;
const rank   = MODEL_PRIORITY['receita.pj'].rank;
const grupo  = MODEL_PRIORITY['receita.pj'].grupo;

export let getEmpresaDetalhadoCNPJ_Receita_PJ = function (cnpj: string){

    return connection.getConn().request()
        .input('cnpj_parameter', mssql.NVarChar, cnpj)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        CNPJ as cnpj, NOME_FANTASIA as nomeFantasia, RAZAO_SOCIAL as razaoSocial,
                        cast(DATA_INICIO_ATIVIDADE as date) as dataInicioAtividade,
                        CPF_RESPONSAVEL as cpfResponsavel, NOME_RESPONSAVEL as nomeResponsavel,
                        DESCRICAO_PF_RESPONSAVEL as descResponsavel, '${fonte_dado}' as fonte

                FROM Receita.dbo.PJ
                WHERE CNPJ=@cnpj_parameter
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar a empresa ${cnpj}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pela empresa ${cnpj}.`, 'getEmpresaDetalhadoCNPJ_Receita_PJ'); });
};

export let getEmpresaSimplificadoCNPJ_Receita_PJ = function (cnpj: string){

    return connection.getConn().request()
        .input('cnpj_parameter', mssql.NVarChar, cnpj)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        CNPJ as cnpj, NOME_FANTASIA as nomeFantasia, RAZAO_SOCIAL as razaoSocial,
                        cast(DATA_INICIO_ATIVIDADE as date) as dataInicioAtividade, '${fonte_dado}' as fonte

                FROM Receita.dbo.PJ
                WHERE CNPJ=@cnpj_parameter
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar a empresa ${cnpj}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pela empresa ${cnpj}.`, 'getEmpresaSimplificadoCNPJ_Receita_PJ'); });
};

export let getEmpresaSimplificadoRazaoSocial_Receita_PJ = function (razaoSocial: string){

    return connection.getConn().request()
        .input('razao_social_parameter', mssql.NVarChar, razaoSocial)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        CNPJ as cnpj, NOME_FANTASIA as nomeFantasia, RAZAO_SOCIAL as razaoSocial,
                        cast(DATA_INICIO_ATIVIDADE as date) as dataInicioAtividade, '${fonte_dado}' as fonte

                FROM Receita.dbo.PJ
                WHERE CONTAINS(RAZAO_SOCIAL, @razao_social_parameter)
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar a empresa ${razaoSocial}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pela empresa ${razaoSocial}.`, 'getEmpresaSimplificadoRazaoSocial_Receita_PJ'); });
};

export let getEmpresaSimplificadoNomeFantasia_Receita_PJ = function (nomeFantasia: string){

    return connection.getConn().request()
        .input('nome_fantasia_parameter', mssql.NVarChar, nomeFantasia)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        CNPJ as cnpj, NOME_FANTASIA as nomeFantasia, RAZAO_SOCIAL as razaoSocial,
                        cast(DATA_INICIO_ATIVIDADE as date) as dataInicioAtividade, '${fonte_dado}' as fonte

                FROM Receita.dbo.PJ
                WHERE CONTAINS(NOME_FANTASIA, @nome_fantasia_parameter)
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar a empresa ${nomeFantasia}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pela empresa ${nomeFantasia}.`, 'getEmpresaSimplificadoNomeFantasia_Receita_PJ'); });
};