import * as mssql from 'mssql';

import { connection }  from '../../services/db.service';
import { trataResultado, resultNotFound, resultFound, logErroBuscaBD } from '../../utils';
import { MODEL_PRIORITY, API_CONFIG } from '../../config';

import { SIGLA_FONTE } from './../_fontes';

const fonte_dado = SIGLA_FONTE['receita_full'];

const fonte  = MODEL_PRIORITY['receita_full.pj'].fonte;
const rank   = MODEL_PRIORITY['receita_full.pj'].rank;
const grupo  = MODEL_PRIORITY['receita_full.pj'].grupo;

export let getEmpresaDetalhadoCNPJ_ReceitaFull_PJ = function (cnpj: string){

    return connection.getConn().request()
        .input('cnpj_parameter', mssql.VarChar, cnpj)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        CNPJ as cnpj, NomeFantasia as nomeFantasia, RazaoSocial as razaoSocial,
                        CASE
                            WHEN isdate(DataInicioAtividade) = 1 THEN cast(DataInicioAtividade as date)
                            ELSE NULL
                        END as dataInicioAtividade,
                        CASE
                            WHEN Matriz = 1 THEN 'S'
                            ELSE 'N'
                        END AS matriz,
                        TipoSituacaoCadastralPJ.Descricao as situacaoCadastral,
                        TipoNaturezaJuridica.Descricao as naturezaJuridica,
                        TipoPorte.DescricaoPorte as porte,
                        CASE
                            WHEN isdate(DataSituacaoCadastral) = 1 THEN cast(DataSituacaoCadastral as date)
                            ELSE NULL
                        END as dataSituacaoCadastral,
                        TipoCNAE.DescricaoSecao as cnaeSecao,
                        TipoCNAE.DescricaoDivisao as cnaeDivisao,
                        TipoCNAE.DescricaoSecao as cnaeGrupo,
                        TipoCNAE.DescricaoClasse as cnaeClasse,
                        TipoCNAE.DescricaoSubclasse as cnaeSubClasse,
                        CPFResponsavel as cpfResponsavel, NomeResponsavel as nomeResponsavel, '${fonte_dado}' as fonte

                FROM Receita_Full.Receita.PJ
                    INNER JOIN Receita_Full.Receita.TipoSituacaoCadastralPJ ON (PJ.SituacaoCadastral = TipoSituacaoCadastralPJ.SituacaoCadastral)
                    INNER JOIN Receita_Full.Receita.TipoPorte ON (PJ.Porte = TipoPorte.Porte)
                    LEFT OUTER JOIN Receita_Full.Receita.TipoNaturezaJuridica ON (PJ.CodigoNaturezaJuridica = TipoNaturezaJuridica.CodigoNaturezaJuridica)
                    LEFT OUTER JOIN Receita_Full.Receita.TipoCNAE ON (PJ.CNAEFiscal = TipoCNAE.CodigoCNAE)
                WHERE CNPJ=@cnpj_parameter
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar a empresa ${cnpj}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pela empresa ${cnpj}.`, 'getEmpresaDetalhadoCNPJ_ReceitaFull_PJ');  });
};

export let getEmpresaSimplificadoCNPJ_ReceitaFull_PJ = function (cnpj: string){

    return connection.getConn().request()
        .input('cnpj_parameter', mssql.VarChar, cnpj)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        CNPJ as cnpj, NomeFantasia as nomeFantasia, RazaoSocial as razaoSocial,
                        CASE
                            WHEN isdate(DataInicioAtividade) = 1 THEN cast(DataInicioAtividade as date)
                            ELSE NULL
                        END as dataInicioAtividade,
                        Municipio as municipio, UF as uf,
                        '${fonte_dado}' as fonte

                FROM Receita_Full.Receita.PJ
                WHERE CNPJ=@cnpj_parameter
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar a empresa ${cnpj}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pela empresa ${cnpj}.`, 'getEmpresaSimplificadoCNPJ_ReceitaFull_PJ'); });
};

export let getEmpresaSimplificadoRazaoSocial_ReceitaFull_PJ = function (razaoSocial: string){

    return connection.getConn().request()
        .input('razao_social_parameter', mssql.VarChar, razaoSocial)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        CNPJ as cnpj, NomeFantasia as nomeFantasia, RazaoSocial as razaoSocial,
                        CASE
                            WHEN isdate(DataInicioAtividade) = 1 THEN cast(DataInicioAtividade as date)
                            ELSE NULL
                        END as dataInicioAtividade,
                        Municipio as municipio, UF as uf,
                        '${fonte_dado}' as fonte

                FROM Receita_Full.Receita.PJ
                WHERE CONTAINS(RazaoSocial, @razao_social_parameter)
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar a empresa ${razaoSocial}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pela empresa ${razaoSocial}.`, 'getEmpresaSimplificadoRazaoSocial_ReceitaFull_PJ'); });
};

export let getEmpresaSimplificadoNomeFantasia_ReceitaFull_PJ = function (nomeFantasia: string){

    return connection.getConn().request()
        .input('nome_fantasia_parameter', mssql.VarChar, nomeFantasia)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        CNPJ as cnpj, NomeFantasia as nomeFantasia, RazaoSocial as razaoSocial,
                        CASE
                            WHEN isdate(DataInicioAtividade) = 1 THEN cast(DataInicioAtividade as date)
                            ELSE NULL
                        END as dataInicioAtividade,
                        Municipio as municipio, UF as uf,
                        '${fonte_dado}' as fonte

                FROM Receita_Full.Receita.PJ
                WHERE CONTAINS(NomeFantasia, @nome_fantasia_parameter)
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar a empresa ${nomeFantasia}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pela empresa ${nomeFantasia}.`, 'getEmpresaSimplificadoNomeFantasia_ReceitaFull_PJ'); });
};

// Retorna todas as empresas que o CPF é sócio
export let getEmpresaSimplificadoSocioPFCPF_ReceitaFull = function (cpf: string){
    return connection.getConn().request()
        .input('cpf_parameter', mssql.VarChar, cpf)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        S.Nome as socio_nome, S.CPF as socio_cpf, S.PercentualCapitalSocial as socio_percCapital,

                        S.CNPJ as cnpj, PJ.RazaoSocial as razaoSocial, PJ.NomeFantasia as nomeFantasia,
                        PJ.CPFResponsavel as cpfResponsavel, PJ.NomeResponsavel as nomeResponsavel,
                        CASE
                            WHEN isdate(PJ.DataInicioAtividade) = 1 THEN cast(PJ.DataInicioAtividade as date)
                            ELSE NULL
                        END as dataInicioAtividade,
                        PJ.Municipio as municipio, PJ.UF as uf,
                        '${fonte_dado}' as fonte

                FROM Receita_Full.Receita.SocioPF AS S
                    INNER JOIN Receita_Full.Receita.PJ ON (S.CNPJ = PJ.CNPJ)
                WHERE S.CPF=@cpf_parameter
                `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar as empresas as quais ${cpf} é sócio.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pelas empresas de ${cpf}.`, 'getEmpresaSocioSimplificadoCPF_ReceitaFull'); });
};

export let getEmpresaSimplificadoSocioPFNome_ReceitaFull = function (nome: string){
    return connection.getConn().request()
        .input('nome_parameter', mssql.VarChar, nome)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        S.Nome as socio_nome, S.CPF as socio_cpf, S.PercentualCapitalSocial as socio_percCapital,

                        S.CNPJ as cnpj, PJ.RazaoSocial as razaoSocial, PJ.NomeFantasia as nomeFantasia,
                        PJ.CPFResponsavel as cpfResponsavel, PJ.NomeResponsavel as nomeResponsavel,
                        CASE
                            WHEN isdate(PJ.DataInicioAtividade) = 1 THEN cast(PJ.DataInicioAtividade as date)
                            ELSE NULL
                        END as dataInicioAtividade,
                        PJ.Municipio as municipio, PJ.UF as uf,
                        '${fonte_dado}' as fonte

                FROM Receita_Full.Receita.SocioPF AS S
                    INNER JOIN Receita_Full.Receita.PJ ON (S.CNPJ = PJ.CNPJ)
                WHERE CONTAINS(S.Nome, @nome_parameter)
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar as empresas as quais ${nome} é sócio.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pelas empresas de ${nome}.`, 'getEmpresaSimplificadoSocioNome_ReceitaFull'); });
};

export let getEmpresaSimplificadoSocioPJCNPJ_ReceitaFull = function (cnpj: string){
    return connection.getConn().request()
        .input('cnpj_parameter', mssql.VarChar, cnpj)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        S.RazaoSocial as socio_razaoSocial, S.CNPJSocio as socio_cnpj, S.PercentualCapitalSocial as socio_percCapital,

                        PJ.CNPJ as cnpj, PJ.RazaoSocial as razaoSocial, PJ.NomeFantasia as nomeFantasia,
                        PJ.CPFResponsavel as cpfResponsavel, PJ.NomeResponsavel as nomeResponsavel,
                        CASE
                            WHEN isdate(PJ.DataInicioAtividade) = 1 THEN cast(PJ.DataInicioAtividade as date)
                            ELSE NULL
                        END as dataInicioAtividade,
                        PJ.Municipio as municipio, PJ.UF as uf,
                        '${fonte_dado}' as fonte

                FROM Receita_Full.Receita.SocioPJ AS S
                    INNER JOIN Receita_Full.Receita.PJ ON (S.CNPJ = PJ.CNPJ)
                WHERE S.CNPJSocio=@cnpj_parameter
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar as empresas as quais ${cnpj} é sócio.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pelas empresas de ${cnpj}.`, 'getEmpresaSimplificadoSocioPJCNPJ_ReceitaFull'); });
};
