import * as mssql from 'mssql';

import { connection }  from '../../services/db.service';
import { MODEL_PRIORITY, API_CONFIG } from '../../config';
import {
    trataResultado,
    resultNotFound,
    resultFound,
    logErroBuscaBD
} from '../../utils';

import { SIGLA_FONTE } from './../_fontes';

const fonte_dado = SIGLA_FONTE['receitanovo'];

const fonte  = MODEL_PRIORITY['receitanovo.pessoajuridica'].fonte;
const rank   = MODEL_PRIORITY['receitanovo.pessoajuridica'].rank;
const grupo  = MODEL_PRIORITY['receitanovo.pessoajuridica'].grupo;

export let getEmpresaDetalhadoCNPJ_ReceitaNovo_PessoaJuridica = function (cnpj: string){

    return connection.getConn().request()
        .input('cnpj_parameter', mssql.Char(14), cnpj)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        CNPJ as cnpj, NomeFantasia as nomeFantasia, RazaoSocial as razaoSocial,
                        cast(DataInicioAtividade as date) as dataInicioAtividade,
                        CASE
                            WHEN IdentificadorMatrizFilial = 1 THEN 'S'
                            ELSE 'N'
                        END AS matriz,
                        Municipio as municipio,
                        UF as uf,
                        TipoCNAE.DescricaoSecao as cnaeSecao,
                        CPFResponsavel as cpfResponsavel, NomeResponsavel as nomeResponsavel, '${fonte_dado}' as fonte

                FROM ReceitaNovo.dbo.PessoaJuridica
                    LEFT OUTER JOIN Receita_Full.Receita.TipoCNAE ON (PessoaJuridica.CNAEFiscal = TipoCNAE.CodigoCNAE)
                WHERE CNPJ=@cnpj_parameter
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar a empresa ${cnpj}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pela empresa ${cnpj}.`, 'getEmpresaDetalhadoCNPJ_ReceitaNovo_PessoaJuridica');  });
};

export let getEmpresaDetalhadoCPFResponsavel_ReceitaNovo_PessoaJuridica = function (cpf: string){

    return connection.getConn().request()
        .input('cpf', mssql.Char(11), cpf)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        CNPJ as cnpj, NomeFantasia as nomeFantasia, RazaoSocial as razaoSocial,
                        cast(DataInicioAtividade as date) as dataInicioAtividade,
                        CASE
                            WHEN IdentificadorMatrizFilial = 1 THEN 'S'
                            ELSE 'N'
                        END AS matriz,
                        Municipio as municipio,
                        UF as uf,
                        TipoCNAE.DescricaoSecao as cnaeSecao,
                        CPFResponsavel as cpfResponsavel, NomeResponsavel as nomeResponsavel, '${fonte_dado}' as fonte

                FROM ReceitaNovo.dbo.PessoaJuridica
                    LEFT OUTER JOIN Receita_Full.Receita.TipoCNAE ON (PessoaJuridica.CNAEFiscal = TipoCNAE.CodigoCNAE)
                WHERE CPFRESPONSAVEL=@cpf
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar a empresa ${cpf}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pela empresa ${cpf}.`, 'getEmpresaDetalhadoCPFResponsavel_ReceitaNovo_PessoaJuridica'); });
};

export let getEmpresaSimplificadoCNPJ_ReceitaNovo_PessoaJuridica = function (cnpj: string){

    return connection.getConn().request()
        .input('cnpj_parameter', mssql.Char(14), cnpj)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        CNPJ as cnpj, NomeFantasia as nomeFantasia, RazaoSocial as razaoSocial, Municipio as municipio, UF as uf,
                        cast(DataInicioAtividade as date) as dataInicioAtividade, '${fonte_dado}' as fonte

                FROM ReceitaNovo.dbo.PessoaJuridica
                WHERE CNPJ=@cnpj_parameter
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar a empresa ${cnpj}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pela empresa ${cnpj}.`, 'getEmpresaSimplificadoCNPJ_ReceitaNovo_PessoaJuridica'); });
};

export let getEmpresaSimplificadoLogradouro_ReceitaNovo_PessoaJuridica = function (logradouro: string){

    return connection.getConn().request()
        .input('LOGRADOURO', mssql.VarChar, logradouro)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        CNPJ as cnpj, NomeFantasia as nomeFantasia, RazaoSocial as razaoSocial, Municipio as municipio, UF as uf,
                        cast(DataInicioAtividade as date) as dataInicioAtividade, '${fonte_dado}' as fonte

                FROM ReceitaNovo.dbo.PessoaJuridica
                WHERE CONTAINS(Logradouro, @LOGRADOURO)
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar a empresa de endereco ${logradouro}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pela empresa de endereco ${logradouro}.`, 'getEmpresaSimplificadoLogradouro_ReceitaNovo_PessoaJuridica'); });
};

export let getEmpresaSimplificadoRazaoSocial_ReceitaNovo_PessoaJuridica = function (razaoSocial: string){

    return connection.getConn().request()
        .input('razao_social_parameter', mssql.VarChar, razaoSocial)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        CNPJ as cnpj, NomeFantasia as nomeFantasia, RazaoSocial as razaoSocial, Municipio as municipio, UF as uf,
                        cast(DataInicioAtividade as date) as dataInicioAtividade, '${fonte_dado}' as fonte

                FROM ReceitaNovo.dbo.PessoaJuridica
                WHERE CONTAINS(RazaoSocial, @razao_social_parameter)
                `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar a empresa ${razaoSocial}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pela empresa ${razaoSocial}.`, 'getEmpresaSimplificadoRazaoSocial_ReceitaNovo_PessoaJuridica'); });
};

export let getEmpresaSimplificadoNomeFantasia_ReceitaNovo_PessoaJuridica = function (nomeFantasia: string){

    return connection.getConn().request()
        .input('nome_fantasia_parameter', mssql.VarChar, nomeFantasia)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        CNPJ as cnpj, NomeFantasia as nomeFantasia, RazaoSocial as razaoSocial, Municipio as municipio, UF as uf,
                        cast(DataInicioAtividade as date) as dataInicioAtividade, '${fonte_dado}' as fonte

                FROM ReceitaNovo.dbo.PessoaJuridica
                WHERE CONTAINS(NomeFantasia, @nome_fantasia_parameter)
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar a empresa ${nomeFantasia}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pela empresa ${nomeFantasia}.`, 'getEmpresaSimplificadoNomeFantasia_ReceitaNovo_PessoaJuridica'); });
};

// Retorna todas as empresas que o CPF é sócio
export let getEmpresaSimplificadoSocioPFCPF_ReceitaNovo = function (cpf: string) {
    return connection.getConn().request()
        .input('cpf', mssql.Char(14), '000' + cpf)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        S.NomeSocio as socio_nome, RIGHT(S.CpfSocio, 11) as socio_cpf,
                        CAST(S.PercentualCapitalSocial AS float)/100 as socio_percCapital,

                        S.CNPJ as cnpj, PJ.RazaoSocial as razaoSocial, PJ.NomeFantasia as nomeFantasia,
                        PJ.CPFResponsavel as cpfResponsavel, PJ.NomeResponsavel as nomeResponsavel,
                        PJ.Municipio as municipio, PJ.UF as uf,
                        cast(PJ.DataInicioAtividade as date) as dataInicioAtividade, '${fonte_dado}' as fonte

                FROM ReceitaNovo.dbo.Socio AS S
                    INNER JOIN ReceitaNovo.dbo.PessoaJuridica AS PJ ON (S.CNPJ = PJ.CNPJ)
                WHERE S.CpfSocio=@cpf AND S.IdentificadorSocio=2
                `)
        .then(result => {
            if (result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar as empresas as quais ${cpf} é sócio.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelas empresas de ${cpf}.`, 'getEmpresaSimplificadoSocioPFCPF_ReceitaNovo'); });
};

export let getEmpresaSimplificadoSocioPFNome_ReceitaNovo = function (nome: string) {
    return connection.getConn().request()
        .input('nome', mssql.NVarChar, nome)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        S.NomeSocio as socio_nome, RIGHT(S.CpfSocio, 11) as socio_cpf,
                        CAST(S.PercentualCapitalSocial AS float)/100 as socio_percCapital,

                        S.CNPJ as cnpj, PJ.RazaoSocial as razaoSocial, PJ.NomeFantasia as nomeFantasia,
                        PJ.CPFResponsavel as cpfResponsavel, PJ.NomeResponsavel as nomeResponsavel,
                        PJ.Municipio as municipio, PJ.UF as uf,
                        cast(PJ.DataInicioAtividade as date) as dataInicioAtividade, '${fonte_dado}' as fonte

                FROM ReceitaNovo.dbo.Socio AS S
                    INNER JOIN ReceitaNovo.dbo.PessoaJuridica AS PJ ON (S.CNPJ = PJ.CNPJ)
                WHERE CONTAINS(S.NomeSocio, @nome) AND S.IdentificadorSocio=2
        `)
        .then(result => {
            if (result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar as empresas as quais ${nome} é sócio.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelas empresas de ${nome}.`, 'getEmpresaSimplificadoSocioPFNome_ReceitaNovo'); });
};

export let getEmpresaSimplificadoSocioPJCNPJ_ReceitaNovo = function (cnpj: string) {
    return connection.getConn().request()
        .input('cnpj', mssql.Char(14), cnpj)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        S.NomeSocio as socio_razaoSocial, S.CpfSocio as socio_cnpj,
                        CAST(S.PercentualCapitalSocial AS float)/100 as socio_percCapital,

                        S.CNPJ as cnpj, PJ.RazaoSocial as razaoSocial, PJ.NomeFantasia as nomeFantasia,
                        PJ.CPFResponsavel as cpfResponsavel, PJ.NomeResponsavel as nomeResponsavel,
                        PJ.Municipio as municipio, PJ.UF as uf,
                        cast(PJ.DataInicioAtividade as date) as dataInicioAtividade, '${fonte_dado}' as fonte

                FROM ReceitaNovo.dbo.Socio AS S
                    INNER JOIN ReceitaNovo.dbo.PessoaJuridica AS PJ ON (S.CNPJ = PJ.CNPJ)
                WHERE S.CpfSocio=@cnpj AND S.IdentificadorSocio=1
        `)
        .then(result => {
            if (result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar as empresas as quais ${cnpj} é sócio.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelas empresas de ${cnpj}.`, 'getEmpresaSimplificadoSocioPJCNPJ_ReceitaNovo'); });
};
