import * as mssql from 'mssql';

import { connection }  from '../../services/db.service';
import { Pessoa } from '../schemas';
import { trataResultado, resultNotFound, resultFound, logErroBuscaBD } from '../../utils';
import { MODEL_PRIORITY, API_CONFIG } from './../../config';
import { SIGLA_FONTE } from '../_fontes';

const fonte_dado = SIGLA_FONTE['receitanovo'];

const fontePessoas = MODEL_PRIORITY['sispesquisa.emails.pessoa'].fonte;
const rankPessoas  = MODEL_PRIORITY['sispesquisa.emails.pessoa'].rank;
const grupoPessoas = MODEL_PRIORITY['sispesquisa.emails.pessoa'].grupo;

export let getEmpresaEmail_Sispesquisa_Emails = function (email: string){

    return connection.getConn().request()
        .input('EMAIL', mssql.VarChar, email)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                    CNPJ as cnpj, NomeFantasia as nomeFantasia, RazaoSocial as razaoSocial, Municipio as municipio, UF as uf,
                    cast(DataInicioAtividade as date) as dataInicioAtividade, '${fonte_dado}' as fonte

            FROM SISPESQUISA.sispesquisa.emails E
                INNER JOIN ReceitaNovo.dbo.PessoaJuridica PJ ON (E.cpf_cnpj = PJ.CNPJ)
            WHERE EMAIL = @EMAIL
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar a pessoa com email ${email}.`, fontePessoas);
            else
                return resultFound(result, fontePessoas, rankPessoas, grupoPessoas);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pela pessoa ${email}.`, 'getEmpresaEmail_Sispesquisa_Emails'); });
};