import * as mssql from 'mssql';

import { connection }  from '../../services/db.service';
import { trataResultado, resultNotFound, resultFound, logErroBuscaBD } from '../../utils';
import { MODEL_PRIORITY, API_CONFIG } from './../../config';

const fonte = MODEL_PRIORITY['sispesquisa.telefones.empresa'].fonte;
const rank  = MODEL_PRIORITY['sispesquisa.telefones.empresa'].rank;
const grupo = MODEL_PRIORITY['sispesquisa.telefones.empresa'].grupo;

export let getEmpresaTelefone_Sispesquisa_Telefones = function (telefone: string){

    return connection.getConn().request()
        .input('TELEFONE', mssql.NVarChar, telefone)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                    PJ.CNPJ as cnpj,
                    PJ.NomeFantasia as nomeFantasia,
                    PJ.RazaoSocial as razaoSocial,
                    PJ.Municipio as municipio,
                    PJ.UF as uf,
                    cast(DataInicioAtividade as date) as dataInicioAtividade,
                    fonte

                FROM SISPESQUISA.sispesquisa.telefones T
                    INNER JOIN ReceitaNovo.dbo.PessoaJuridica PJ ON (T.cpf_cnpj = PJ.CNPJ)
                WHERE telefone=@TELEFONE
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar a empresa com telefone ${telefone}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pela empresa ${telefone}.`, 'getPessoaTelefone_Sispesquisa_Telefones'); });
};