import * as mssql from 'mssql';

import { connection }  from '../../services/db.service';
import { MODEL_PRIORITY, API_CONFIG } from '../../config';

import {
    trataResultado,
    resultNotFound,
    resultFound,
    logErroBuscaBD
} from '../../utils';

import { SIGLA_FONTE } from './../_fontes';

const fonte_dado = SIGLA_FONTE['receita_full'];

const fonte  = MODEL_PRIORITY['receita_full.pj.empresa_responsavel'].fonte;
const rank   = MODEL_PRIORITY['receita_full.pj.empresa_responsavel'].rank;
const grupo  = MODEL_PRIORITY['receita_full.pj.empresa_responsavel'].grupo;

export let getEmpresaSimplificadoResponsavelCPF_ReceitaFull = function (cpf: string){
    return connection.getConn().request()
        .input('cpf_parameter', mssql.VarChar, cpf)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        CPFResponsavel as cpfResponsavel, NomeResponsavel as nomeResponsavel,
                        CNPJ as cnpj, RazaoSocial as razaoSocial, NomeFantasia as nomeFantasia,
                        cast(DataInicioAtividade as date) as dataInicioAtividade, '${fonte_dado}' as fonte

                FROM Receita_Full.Receita.PJ
                WHERE PJ.CPFResponsavel=@cpf_parameter
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar as empresas as quais ${cpf} é responsavel.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pelas empresas de ${cpf}.`, 'getEmpresaSimplificadoResponsavelCPF_ReceitaFull'); });
};
