import * as soap from 'soap';

import { Pessoa } from '../schemas';

import {
    API_CONFIG,
    MODEL_PRIORITY,
    API_CODES,
    API_MSGS
} from './../../config';

import {
    resultIPCFound,
    resultNotFound,
    resultFoundRaw,
    logErroBuscaBD,
    trataErro,
    removerAcentos
} from '../../utils';

import { SIGLA_FONTE } from './../_fontes';

const fonte_dado = SIGLA_FONTE['ipc'];

const fonte = MODEL_PRIORITY['ipc.endereco'].fonte;
const rank  = MODEL_PRIORITY['ipc.endereco'].rank;
const grupo = MODEL_PRIORITY['ipc.endereco'].grupo;

export let getEnderecoCPF_IPC = function (cpf: string){
    const cpfFormatado = cpf.slice(0,3)+"."+cpf.slice(3,6)+"."+cpf.slice(6,9)+"-"+cpf.slice(9);

    return Promise_getEndereco_IPC('CPF', cpfFormatado)
        .then((result:any) => {
            if (result.length !== 0)
                return resultFoundRaw(result, fonte, rank, grupo);
            else
                return resultNotFound(`Nao foi possivel encontrar o endereco de ${cpf}.`, fonte);
        })
        .catch(error => { return logErroBuscaBD(error, `Falha na busca pelo endereco de ${cpf}.`, 'getEnderecoCPF_IPC'); });
}

let Promise_createClientIPC = function () {
    return new Promise((resolve, reject) => {
        soap.createClient(API_CONFIG.IPC_WSDL_URL, (error, client: any) => {
            if (error) {
                reject(error);
                return;
            } else {
                resolve(client);
            }
        });

        setTimeout(() => {
            reject(trataErro(API_CODES.CODE_RECURSO_NAO_ENCONTRADO, API_MSGS.MSG_IPC_TIMEOUT));
        }, API_CONFIG.IPC_CREATE_CLIENT_TIMEOUT);
    });
}

let Promise_clientPesquisarSimplificado = function (client, tipoPesquisa: string, valorPesquisa: string){
    return new Promise((resolve, reject) => {
        const params = {
            chaveAcesso: API_CONFIG.SIAP_IPC_CHAVE_ACESSO,
            tipoPesquisa: tipoPesquisa,
            valorPesquisa: valorPesquisa
        }

        client.Pesquisar(params, (err, result, raw, soapHeader) => {
            if (err) {
                reject(err);
                return
            } else {
                let dados;
                if (result && result.PesquisarResult) {
                    dados = result.PesquisarResult.Pessoa.map(registro => {
                        let { Logradouro, Numero, Bairro, Cidade, Estado, CEP, CPF } = registro;
                        return {
                            cpf:        (CPF) ? CPF.replace(/[^0-9]/g, ''):      null,
                            logradouro: (Logradouro) ? Logradouro.toUpperCase(): null,
                            numero:     (Numero) ? Numero:                       null,
                            bairro:     (Bairro) ? Bairro.toUpperCase():         null,
                            municipio:  (Cidade) ? Cidade.toUpperCase():         null,
                            uf:         (Estado) ? Estado.toUpperCase():         null,
                            cep:        (CEP) ? CEP:                             null,
                            fonte:      fonte_dado
                        };
                    });
                } else {
                    dados = [];
                }
                resolve(dados);
            }
        }, { timeout: API_CONFIG.IPC_REQUEST_TIMEOUT })
    });
}

let Promise_getEndereco_IPC = function (tipoPesquisa: string, valorPesquisa: string) {

    return Promise_createClientIPC()
            .then((client: any) => Promise_clientPesquisarSimplificado(client, tipoPesquisa, valorPesquisa))
            .catch(error => { throw error; })
}