import * as mssql from 'mssql';

import { connection }  from '../../services/db.service';
import { MODEL_PRIORITY, API_CONFIG } from '../../config';
import {
    trataResultado,
    resultNotFound,
    resultFound,
    logErroBuscaBD
} from '../../utils';

import { SIGLA_FONTE } from './../_fontes';

const fonte_dado = SIGLA_FONTE['receita'];

const fonte_pf = MODEL_PRIORITY['receita.pf.endereco'].fonte;
const rank_pf  = MODEL_PRIORITY['receita.pf.endereco'].rank;
const grupo_pf = MODEL_PRIORITY['receita.pf.endereco'].grupo;

export let getEnderecoCPF_Receita_PF = function (cpf: string){

    return connection.getConn().request()
        .input('cpf_parameter', mssql.Char(11), cpf)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        CPF as cpf, TipoLogradouro as tipoLogradouro, Logradouro as logradouro, NumeroLogradouro as numero,
                        Complemento as complemento, Bairro as bairro, CEP as cep, UF as uf, Municipio as municipio, '${fonte_dado}' as fonte

                FROM Receita.dbo.PF
                WHERE CPF=@cpf_parameter
        `)
        .then((result: any) => {
            result.recordset = result.recordset.filter(registro => ((registro.bairro.trim().length !== 0) && (registro.cep.trim().length !== 0) && (registro.logradouro.trim().length !== 0) && (registro.municipio.trim().length !== 0) && (registro.uf.trim().length !== 0)));

            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o endereço de ${cpf}.`, fonte_pf);
            else
                return resultFound(result, fonte_pf, rank_pf, grupo_pf);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pelo endereço de ${cpf}.`, 'getEnderecoCPF_Receita_PF')});
};

const fonte_pj = MODEL_PRIORITY['receita.pj.endereco'].fonte;
const rank_pj  = MODEL_PRIORITY['receita.pj.endereco'].rank;
const grupo_pj = MODEL_PRIORITY['receita.pj.endereco'].grupo;

export let getEnderecoCNPJ_Receita_PJ = function (cnpj: string){

    return connection.getConn().request()
        .input('cnpj_parameter', mssql.NVarChar, cnpj)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        CNPJ as cnpj, TIPO_LOGRADOURO as tipoLogradouro, LOGRADOURO as logradouro, NUMERO as numero,
                        COMPLEMENTO as complemento, BAIRRO as bairro, CEP as cep, UF as uf, Municipio as municipio, '${fonte_dado}' as fonte

                FROM Receita.dbo.PJ
                WHERE CNPJ=@cnpj_parameter`)
        .then((result: any) => {
            result.recordset = result.recordset.filter(registro => ((registro.bairro.trim().length !== 0) && (registro.cep.trim().length !== 0) && (registro.logradouro.trim().length !== 0) && (registro.municipio.trim().length !== 0) && (registro.uf.trim().length !== 0)));

            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o endereço de ${cnpj}.`, fonte_pj);
            else
                return resultFound(result, fonte_pj, rank_pj, grupo_pj);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pelo endereço de ${cnpj}.`, 'getEnderecoCNPJ_Receita_PJ')});
};