import * as mssql from 'mssql';

import { connection }  from '../../services/db.service';
import { MODEL_PRIORITY, API_CONFIG } from '../../config';
import {
    trataResultado,
    resultNotFound,
    resultFound,
    logErroBuscaBD
} from '../../utils';

import { SIGLA_FONTE } from './../_fontes';

const fonte_dado = SIGLA_FONTE['receitanovo'];

const fonte_pf = MODEL_PRIORITY['receitanovo.pessoafisica.endereco'].fonte;
const rank_pf  = MODEL_PRIORITY['receitanovo.pessoafisica.endereco'].rank;
const grupo_pf = MODEL_PRIORITY['receitanovo.pessoafisica.endereco'].grupo;

export let getEnderecoCPF_ReceitaNovo_PessoaFisica = function (cpf: string){

    return connection.getConn().request()
        .input('cpf_parameter', mssql.Char(11), cpf)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        CPF as cpf, TipoLogradouro as tipoLogradouro, Logradouro as logradouro, NumeroLogradouro as numero,
                        Complemento as complemento, Bairro as bairro, CEP as cep, UF as uf, Municipio as municipio, '${fonte_dado}' as fonte

                FROM ReceitaNovo.dbo.PessoaFisica
                WHERE CPF=@cpf_parameter
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o endereço de ${cpf}.`, fonte_pf);
            else
                return resultFound(result, fonte_pf, rank_pf, grupo_pf);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pelo endereço de ${cpf}.`, 'getEnderecoCPF_ReceitaNovo_PessoaFisica')});
};

const fonte_pj = MODEL_PRIORITY['receitanovo.pessoajuridica.endereco'].fonte;
const rank_pj  = MODEL_PRIORITY['receitanovo.pessoajuridica.endereco'].rank;
const grupo_pj = MODEL_PRIORITY['receitanovo.pessoajuridica.endereco'].grupo;

export let getEnderecoCNPJ_ReceitaNovo_PessoaJuridica = function (cnpj: string){

    return connection.getConn().request()
        .input('cnpj_parameter', mssql.Char(14), cnpj)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        CNPJ as cnpj, DescricaoTipoLogradouro as tipoLogradouro, Logradouro as logradouro, Numero as numero,
                        Complemento as complemento, Bairro as bairro, CEP as cep, UF as uf, Municipio as municipio, '${fonte_dado}' as fonte

                FROM ReceitaNovo.dbo.PessoaJuridica
                WHERE CNPJ=@cnpj_parameter
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o endereço de ${cnpj}.`, fonte_pj);
            else
                return resultFound(result, fonte_pj, rank_pj, grupo_pj);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pelo endereço de ${cnpj}.`, 'getEnderecoCNPJ_ReceitaNovo_PessoaJuridica')});
};