import * as mssql from 'mssql';

import { connection }  from '../../services/db.service';
import { MODEL_PRIORITY, API_CONFIG } from '../../config';
import {
    trataResultado,
    resultNotFound,
    resultFound,
    logErroBuscaBD
} from '../../utils';

import { SIGLA_FONTE } from './../_fontes';

const fonte_dado = SIGLA_FONTE['renach_2014'];

const fonte = MODEL_PRIORITY['renach.endereco'].fonte;
const rank  = MODEL_PRIORITY['renach.endereco'].rank;
const grupo = MODEL_PRIORITY['renach.endereco'].grupo;

export let getEnderecoCPF_Renach_2016_08 = function (cpf: string){

    return connection.getConn().request()
        .input('cpf_parameter', mssql.Char(11), cpf)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                    nr_cpf as cpf, ds_endereco as logradouro
                    ,nr_endereco as numero, ds_municipio_endereco as municipio
                    ,sg_uf_endereco as uf, nr_cep_endereco as cep
                    ,ds_complemento_endereco as complemento, '${fonte_dado}' as fonte

                FROM Renach_2014.dbo.renach_2016_08
                WHERE nr_cpf=@cpf_parameter
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o endereço de ${cpf}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelo endereço de ${cpf}.`, 'getEnderecoCPF_Renach_2016_08')});
};