import * as mssql from 'mssql';

import { connection }  from '../../services/db.service';
import { MODEL_PRIORITY, API_CODES, API_CONFIG } from '../../config';
import {
    trataResultado,
    resultNotFound,
    resultFound,
    logErroBuscaBD
} from '../../utils';

const fonte = MODEL_PRIORITY['sispesquisa.enderecos'].fonte;
const rank  = MODEL_PRIORITY['sispesquisa.enderecos'].rank;
const grupo = MODEL_PRIORITY['sispesquisa.enderecos'].grupo;

export let getEnderecoCPF_Sispesquisa_Enderecos = function (cpf: string){

    return connection.getConn().request()
        .input('cpf_parameter', mssql.Char(11), cpf)
        // .query(`SELECT  cpf, tipoLogradouro, UPPER(logradouro) as logradouro,
        //                 numeroLogradouro as numero, UPPER(complemento) as complemento,
        //                 UPPER(bairro) as bairro, cep, UPPER(municipio) as municipio, uf, fonte

        //         FROM SISPESQUISA.sispesquisa.enderecos
        //         WHERE cpf=@cpf_parameter
        // `)
        .query(`
            ;WITH ENDERECOS(CPF, LOGRADOURO, NUMERO, COMPLEMENTO, BAIRRO, CEP, MUNICIPIO, UF, FONTE, DATAREGISTRO)
            AS
            (
                SELECT cpf_cnpj
                    ,UPPER(logradouro)
                    ,CASE
                        WHEN TRY_CAST(numeroLogradouro as int) IS NULL THEN 0
                        ELSE CAST(numeroLogradouro as int)
                    END
                    ,UPPER(complemento)
                    ,UPPER(bairro)
                    ,cep
                    ,UPPER(municipio)
                    ,UPPER(uf)
                    ,fonte
                    ,DataRegistro
                FROM SISPESQUISA.sispesquisa.enderecos
                WHERE cpf_cnpj=@cpf_parameter
            )
            ,PESOS(CPF, NUMERO, CEP, PESO)
            AS
            (
                SELECT CPF, NUMERO, CEP, COUNT(*)
                FROM ENDERECOS
                GROUP BY cpf, NUMERO, cep
            )
            ,ENDERECOS_COM_PESO(CPF, LOGRADOURO, NUMERO, COMPLEMENTO, BAIRRO, CEP, MUNICIPIO, UF, FONTE, DATAREGISTRO, PESO)
            AS
            (
                SELECT E.CPF, E.LOGRADOURO, E.NUMERO, MAX(E.COMPLEMENTO), E.BAIRRO, E.CEP, E.MUNICIPIO, E.UF, MAX(E.FONTE), MAX(E.DATAREGISTRO), P.PESO * COUNT(*)
                FROM ENDERECOS E
                    INNER JOIN PESOS P ON (P.NUMERO = E.NUMERO AND P.CEP = E.CEP)
                GROUP BY E.CPF, LOGRADOURO, E.NUMERO, BAIRRO, E.CEP, MUNICIPIO, UF, P.PESO
            )

            SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                CPF as cpf, LOGRADOURO as logradouro, NUMERO as numero, COMPLEMENTO as complemento, BAIRRO as bairro, CEP as cep, MUNICIPIO as municipio, UF as uf, FONTE as fonte, PESO as peso
            FROM ENDERECOS_COM_PESO
            ORDER BY PESO DESC, FONTE DESC
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o endereço de ${cpf}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pelo endereço de ${cpf}.`, 'getEnderecoCPF_Sispesquisa_Enderecos')});
};

export let getEnderecoCNPJ_Sispesquisa_Enderecos = function (cnpj: string){

    return connection.getConn().request()
        .input('CNPJ', mssql.Char(14), cnpj)
        .query(`
            ;WITH ENDERECOS(CNPJ, LOGRADOURO, NUMERO, COMPLEMENTO, BAIRRO, CEP, MUNICIPIO, UF, FONTE, DATAREGISTRO)
            AS
            (
                SELECT cpf_cnpj
                    ,UPPER(logradouro)
                    ,CASE
                        WHEN TRY_CAST(numeroLogradouro as int) IS NULL THEN 0
                        ELSE CAST(numeroLogradouro as int)
                    END
                    ,UPPER(complemento)
                    ,UPPER(bairro)
                    ,cep
                    ,UPPER(municipio)
                    ,UPPER(uf)
                    ,fonte
                    ,DataRegistro
                FROM SISPESQUISA.sispesquisa.enderecos
                WHERE cpf_cnpj=@CNPJ
            )
            ,PESOS(CNPJ, NUMERO, CEP, PESO)
            AS
            (
                SELECT CNPJ, NUMERO, CEP, COUNT(*)
                FROM ENDERECOS
                GROUP BY CNPJ, NUMERO, cep
            )
            ,ENDERECOS_COM_PESO(CNPJ, LOGRADOURO, NUMERO, COMPLEMENTO, BAIRRO, CEP, MUNICIPIO, UF, FONTE, DATAREGISTRO, PESO)
            AS
            (
                SELECT E.CNPJ, E.LOGRADOURO, E.NUMERO, MAX(E.COMPLEMENTO), E.BAIRRO, E.CEP, E.MUNICIPIO, E.UF, MAX(E.FONTE), MAX(E.DATAREGISTRO), P.PESO * COUNT(*)
                FROM ENDERECOS E
                    INNER JOIN PESOS P ON (P.NUMERO = E.NUMERO AND P.CEP = E.CEP)
                GROUP BY E.CNPJ, LOGRADOURO, E.NUMERO, BAIRRO, E.CEP, MUNICIPIO, UF, P.PESO
            )

            SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                CNPJ as cnpj, LOGRADOURO as logradouro, NUMERO as numero, COMPLEMENTO as complemento, BAIRRO as bairro, CEP as cep, MUNICIPIO as municipio, UF as uf, FONTE as fonte, PESO as peso
            FROM ENDERECOS_COM_PESO
            ORDER BY PESO DESC, FONTE DESC
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o endereço de ${cnpj}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelo endereço de ${cnpj}.`, 'getEnderecoCNPJ_Sispesquisa_Enderecos')});
};

export let insertEnderecoCPF_Sispesquisa_Enderecos = function (form){

    return connection.getConn().request()
        .input('cpf_parameter', mssql.VarChar, form.cpf)
        .input('tipoLogradouro_parameter', mssql.VarChar, form.tipoLogradouro)
        .input('logradouro_parameter', mssql.VarChar, form.logradouro)
        .input('numeroLogradouro_parameter', mssql.VarChar, form.numeroLogradouro)
        .input('complemento_parameter', mssql.VarChar, form.complemento)
        .input('bairro_parameter', mssql.VarChar, form.bairro)
        .input('cep_parameter', mssql.VarChar, form.cep)
        .input('municipio_parameter', mssql.VarChar, form.municipio)
        .input('uf_parameter', mssql.VarChar, form.uf)
        .input('fonte_parameter', mssql.VarChar, form.fonte)
        .query(`INSERT INTO SISPESQUISA.sispesquisa.enderecos
                (cpf_cnpj, tipoLogradouro, logradouro, numeroLogradouro,
                complemento, bairro, cep, municipio, uf, fonte)
                VALUES
                (@cpf_parameter, @tipoLogradouro_parameter, @logradouro_parameter,
                @numeroLogradouro_parameter, @complemento_parameter, @bairro_parameter,
                @cep_parameter, @municipio_parameter, @uf_parameter, @fonte_parameter)
        `)
        .then(result => {
            return {
                status: API_CODES.CODE_SUCESSO,
                msg: 'Endereço cadastrado com sucesso.'
            }
        }).catch( error => {logErroBuscaBD(error, `Falha no cadastro do endereço.`, 'insertEnderecoCPF_Sispesquisa_Enderecos')});
};