import * as mssql from 'mssql';

import { connection }  from '../../services/db.service';
import { MODEL_PRIORITY, API_CONFIG } from '../../config';
import {
    trataResultado,
    resultNotFound,
    resultFound,
    logErroBuscaBD
} from '../../utils';

import { SIGLA_FONTE } from './../_fontes';

const fonte_dado = SIGLA_FONTE['sispesquisa.veiculos_19_07_2017'];

const fonte = MODEL_PRIORITY['sispesquisa.veiculos_novo.endereco'].fonte;
const rank  = MODEL_PRIORITY['sispesquisa.veiculos_novo.endereco'].rank;
const grupo = MODEL_PRIORITY['sispesquisa.veiculos_novo.endereco'].grupo;

export let getEnderecoCPF_Sispesquisa_VeiculosNovo = function (cpf: string){

    return connection.getConn().request()
        .input('cpf_parameter', mssql.VarChar, cpf)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        CPF_CNPJ_Proprietario as cpf, Endereco as logradouro, Bairro as bairro,
                        CEP as cep, Municipio_Residencia as municipio, '${fonte_dado}' as fonte

                FROM SISPESQUISA.sispesquisa.veiculos_19_07_2017
                WHERE CPF_CNPJ_Proprietario=@cpf_parameter
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o endereço de ${cpf}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pelo endereço de ${cpf}.`, 'getEnderecoCPF_Sispesquisa_VeiculosNovo')});
};

export let getEnderecoCNPJ_Sispesquisa_VeiculosNovo = function (cnpj: string){

    return connection.getConn().request()
        .input('cnpj_parameter', mssql.VarChar, cnpj)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        CPF_CNPJ_Proprietario as cnpj, Endereco as logradouro, Bairro as bairro,
                        CEP as cep, Municipio_Residencia as municipio, '${fonte_dado}' as fonte

                FROM SISPESQUISA.sispesquisa.veiculos_19_07_2017
                WHERE CPF_CNPJ_Proprietario=@cnpj_parameter
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o endereço de ${cnpj}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pelo endereço de ${cnpj}.`, 'getEnderecoCNPJ_Sispesquisa_VeiculosNovo')});
};