import * as mssql from 'mssql';

import { connection }  from '../../services/db.service';
import { MODEL_PRIORITY, API_CONFIG } from '../../config';
import {
    trataResultado,
    resultNotFound,
    resultFound,
    logErroBuscaBD
} from '../../utils';


import { SIGLA_FONTE } from './../_fontes';

const fonte_dado = SIGLA_FONTE['vep'];

const fonte_pf = MODEL_PRIORITY['vep.endereco'].fonte;
const rank_pf  = MODEL_PRIORITY['vep.endereco'].rank;
const grupo_pf = MODEL_PRIORITY['vep.endereco'].grupo;

export let getEnderecoCPF_VEP = function (cpf: string){

    return connection.getConn().request()
        .input('cpf_parameter', mssql.Char(11), cpf)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                    PF.CPF as cpf
                    ,UPPER(L.DS_LOCALIDADE) AS logradouro
                    ,E.NUMERO AS numero
                    ,UPPER(E.DS_COMPLEMENTO) AS complemento
                    ,UPPER(B.DS_BAIRRO) AS bairro
                    ,L.CEPLOCALIDADE AS cep
                    ,UF.DS_SIGLA AS uf
                    ,UPPER(C2.DS_CIDADE) AS municipio
                    ,'${fonte_dado}' as fonte

                FROM VEP.dbo.VEP_CNJ_TB_PESSOAFISICA PF
                    LEFT OUTER JOIN VEP.dbo.VEP_CNJ_TB_PESSOA P ON (P.ID_PESSOA = PF.FK_PESSOA)

                    LEFT OUTER JOIN VEP.dbo.VEP_CNJ_TB_ENDERECOPESSOA EP ON (EP.FK_PESSOA = P.ID_PESSOA)
                    LEFT OUTER JOIN VEP.dbo.VEP_CNJ_TB_ENDERECO E ON (E.ID_ENDERECO = EP.FK_ENDERECO)
                    LEFT OUTER JOIN VEP.dbo.VEP_CNJ_TB_LOCALIDADE L ON (L.ID_LOCALIDADE = E.FK_LOCALIDADE)
                    LEFT OUTER JOIN VEP.dbo.VEP_CNJ_TB_BAIRRO B ON (B.ID_BAIRRO = L.FK_BAIRRO)
                    LEFT OUTER JOIN VEP.dbo.VEP_CNJ_TB_CIDADE C2 ON (C2.ID_CIDADE = B.FK_CIDADE)
                    LEFT OUTER JOIN VEP.dbo.VEP_CNJ_TB_UF UF ON (UF.ID_UF = C2.FK_UF)

                WHERE PF.CPF=@cpf_parameter
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o endereço de ${cpf}.`, fonte_pf);
            else
                return resultFound(result, fonte_pf, rank_pf, grupo_pf);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelo endereço de ${cpf}.`, 'getEnderecoCPF_VEP')});
};