import * as mssql from 'mssql';

import { connection }  from '../services/db.service';
import { MODEL_PRIORITY, API_CONFIG } from '../config';
import {
    resultNotFound,
    resultFound,
    logErroBuscaBD
} from '../utils';

const fonte = MODEL_PRIORITY['filiacao'].fonte;
const rank  = MODEL_PRIORITY['filiacao'].rank;
const grupo = MODEL_PRIORITY['filiacao'].grupo;

export let getFiliacaoPartidariaDetalhadoCPF_Filiados = function (cpf: string){

    return connection.getConn().request()
        .input('cpf_parameter', mssql.Char(11), cpf)
        .query(`
            SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                dataExtracao, cpf, siglaPartido, nomePartido
                ,uf, municipio, zonaEleitoral, secaoEleitoral
                ,dataFiliacao, situacaoRegistro, tipoRegistro
                ,dataProcessamento, dataDesfiliacao, dataCancelamento
                ,dataRegularizacao, UPPER(motivoCancelamento) AS motivoCancelamento

            FROM SISPESQUISA.dbo.filiados_pb
            WHERE cpf=@cpf_parameter
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar a pessoa ${cpf}.`, fonte);
            else {
                return resultFound(result, fonte, rank, grupo);
            }
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pela pessoa ${cpf}.`, 'getFiliacaoPartidariaDetalhadoCPF_Filiados'); });
};