import * as mssql from 'mssql';

import { connection } from '../services/db.service';
import { MODEL_PRIORITY, API_CONFIG } from '../config';
import { BenchmarkService } from '../services/benchmark.service';
import {
    resultNotFound,
    resultFound,
    logErroBuscaBD,
    printTempoExecucao
} from '../utils';

const fonte = MODEL_PRIORITY['sagres.folha'].fonte;
const rank  = MODEL_PRIORITY['sagres.folha'].rank;
const grupo = MODEL_PRIORITY['sagres.folha'].grupo;

export let getFolhaMunicipalMesAnoCdugestora = function (cdugestora: string, mes: string, ano: string) {

    const tempoInicial = new BenchmarkService();

    console.log(cdugestora, ano, mes);

    return connection.getConn().request()
        .input('MESANO', mssql.Char(6), ano + mes)
        .input('CDUGESTORA', mssql.Int, cdugestora)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                    Nome as nome, CPF as cpf, Servidor as matricula, CARGO as cargo,
                    UPPER(Natureza) as natureza, Remuneracao_Bruta as vlBruto, Descontos as descontos,
                    Remuneracao_Liquida as vlLiquido, CAST(Data_Admissao as date) as dataAdmissao
                FROM SAGRES_MUNICIPAL.dbo.[FN_FOLHA_PAGAMENTO_por_Mes](@MESANO, null, @CDUGESTORA)
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getFolhaMunicipalMesAnoCdugestora');

            if (result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar a folha de ${cdugestora} no periodo ${mes + ano}.`, fonte);
            else {
                return resultFound(result, fonte, rank, grupo);
            }
        }).catch(error => logErroBuscaBD(error, `Falha na busca pela folha de ${cdugestora} no periodo ${mes + ano}.`, 'getFolhaMunicipalMesAnoCdugestora'));
};

export let getFolhaEstadualMesAnoUgestora = function (ugestora: string, mes: string, ano: string) {

    const tempoInicial = new BenchmarkService();

    return connection.getConn().request()
        .input('MESANO', mssql.Char(6), mes + ano)
        .input('UGESTORA', mssql.VarChar, ugestora)
        .query(`
            SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                    S.CPF as cpf, PF.Nome as nome, SERVIDOR as matricula, CARGO as cargo,
                    NATUREZA as natureza, PROVENTO as vlBruto, DESCONTO as descontos, LIQUIDO as vlLiquido, NULL as dataAdmissao
            FROM SAGRES_ESTADUAL.dbo.[FN_FOLHA_PAGAMENTO_por_Mes](@MESANO, null, @UGESTORA) AS S
                INNER JOIN ReceitaNovo.dbo.PessoaFisica PF ON (S.CPF COLLATE Latin1_General_CI_AS = PF.CPF)
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getFolhaEstadualMesAnoUgestora');

            if (result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar a folha de ${ugestora} no periodo ${mes + ano}.`, fonte);
            else {
                return resultFound(result, fonte, rank, grupo);
            }
        }).catch(error => logErroBuscaBD(error, `Falha na busca pela folha de ${ugestora} no periodo ${mes + ano}.`, 'getFolhaEstadualMesAnoUgestora'));
};