import * as soap from 'soap';

import { Pessoa } from '../schemas';
import {
    API_CONFIG,
    MODEL_PRIORITY,
    API_CODES,
    API_MSGS
} from './../../config';

import {
    resultIPCFound,
    resultNotFound,
    resultFoundRaw,
    logErroBuscaBD,
    trataErro,
    removerAcentos
} from '../../utils';


import { SIGLA_FONTE } from './../_fontes';

const fonte_dado = SIGLA_FONTE['ipc'];

const fonte  = MODEL_PRIORITY['ipc.foto'].fonte;
const rank   = MODEL_PRIORITY['ipc.foto'].rank;
const grupo  = MODEL_PRIORITY['ipc.foto'].grupo;

export let getFotosCPF_IPC = function (cpf: string){
    const cpfFormatado = cpf.slice(0,3)+"."+cpf.slice(3,6)+"."+cpf.slice(6,9)+"-"+cpf.slice(9);

    return Promise_getFotos_IPC('CPF', cpfFormatado)
        .then((result:any) => {
            if (Object.keys(result).length !== 0)
                return resultFoundRaw(result, fonte, rank, grupo);
            else
                return resultNotFound(`Nao foi possivel encontrar a pessoa ${cpf}.`, fonte);
        })
        .catch(error => { return logErroBuscaBD(error, `Falha na busca pela pessoa ${cpf}.`, 'getFotosCPF_IPC'); });
}

let Promise_createClientIPC = function () {
    return new Promise((resolve, reject) => {
        soap.createClient(API_CONFIG.IPC_WSDL_URL, (error, client: any) => {
            if (error) {
                reject(error);
                return;
            } else {
                resolve(client);
            }
        });

        setTimeout(() => {
            reject(trataErro(API_CODES.CODE_RECURSO_NAO_ENCONTRADO, API_MSGS.MSG_IPC_TIMEOUT));
        }, API_CONFIG.IPC_CREATE_CLIENT_TIMEOUT);
    });
}

let Promise_getFotos_IPC = function (tipoPesquisa: string, valorPesquisa: string){
    return Promise_createClientIPC()
            .then(client => Promise_clientPesquisarFotos(client, tipoPesquisa, valorPesquisa))
            .catch(error => {throw error;})
}

let Promise_clientPesquisarFotos = function (client, tipoPesquisa, valorPesquisa) {
    return new Promise((resolve, reject) => {
        const params = {
            chaveAcesso: API_CONFIG.SIAP_IPC_CHAVE_ACESSO,
            tipoPesquisa: tipoPesquisa,
            valorPesquisa: valorPesquisa
        }

        client.Pesquisar(params, (err, result, raw, soapHeader) => {
            if (err) {
                reject(err);
                return;
            } else {
                if (result && result.PesquisarResult) {
                    const codigo = result.PesquisarResult.Pessoa[0].Codigo;
                    const promiseFotos = Promise.all([Promise_getFotoFace_IPC(codigo, valorPesquisa),
                                                      Promise_getFotoAssinatura_IPC(codigo, valorPesquisa)])
                                            .then(fotos => {
                                                let img: any = {};

                                                if (fotos[0]) { img.face = fotos[0]; };
                                                if (fotos[1]) { img.assinatura = fotos[1]; };
                                                if (fotos[0] || fotos[1]){ img.fonte = fonte_dado;}

                                                return img;
                                            });
                    resolve(promiseFotos);
                } else {
                    resolve([]);
                }
            }
        }, { timeout: API_CONFIG.IPC_REQUEST_TIMEOUT })
    });
}

let Promise_getFotoAssinatura_IPC = function (codigo: string, valorPesquisa: string) {
    return Promise_createClientIPC()
        .then(client => Promise_obterImagem(client, codigo, 'Assinatura'))
        .catch(error => { logErroBuscaBD(error, `Falha na busca pela foto de ${valorPesquisa}.`, 'Promise_getFotoAssinatura_IPC'); return null; })
}

let Promise_getFotoFace_IPC = function (codigo: string, valorPesquisa: string){
    return Promise_createClientIPC()
        .then(client => Promise_obterImagem(client, codigo, 'Face'))
        .catch(error => { logErroBuscaBD(error, `Falha na busca pela foto de ${valorPesquisa}.`, 'Promise_getFotoFace_IPC'); return null;})
}

let Promise_obterImagem = function (client, codigo, tipo) {
    return new Promise((resolve, reject) => {
        const params = {
            chaveAcesso: API_CONFIG.SIAP_IPC_CHAVE_ACESSO,
            codigo: codigo,
            tipo: tipo
        }

        client.ObterImagem(params, (err, result, raw, soapHeader) => {
            if (err) {
                reject(err);
                return;
            } else {
                resolve(result.ObterImagemResult);
            }
        }, { timeout: API_CONFIG.IPC_REQUEST_TIMEOUT })
    });
}