import * as mssql from 'mssql';

import { connection } from '../../services/db.service';

import { API_CONFIG, MODEL_PRIORITY } from './../../config';
import { Pessoa } from '../schemas';
import {
    resultNotFound,
    resultFound,
    resultFoundRaw,
    hashFotosSispesquisa,
    logErroBuscaBD,
    print
} from '../../utils';

import { SIGLA_FONTE } from './../_fontes';

const fonte_dado = SIGLA_FONTE['sisdepen'];

const fonte = MODEL_PRIORITY['sisdepen.foto'].fonte;
const rank  = MODEL_PRIORITY['sisdepen.foto'].rank;
const grupo = MODEL_PRIORITY['sisdepen.foto'].grupo;

export let getFotoCPF_SISDEPEN = function (cpf: string){
    return connection.getConn().request()
        .input('cpf', mssql.VarChar, cpf)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                    CPF as cpf, foto

                FROM SISDEPEN.dbo.VW_SISDEPEN
                WHERE cpf=@cpf
        `)
        .then(result => {
            if (result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar a foto de ${cpf}.`, fonte);
            else {
                const img = { face: result.recordset[0].foto, fonte: fonte_dado};
                return resultFoundRaw(img, fonte, rank, grupo);
            }
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pela foto da pessoa ${cpf}.`, 'getFotoCPF_SISDEPEN'); });
};