const promisify  = require('util').promisify;
const rf = promisify(require('fs').readFile);

import { connection }  from '../../services/db.service';

import { API_CONFIG, MODEL_PRIORITY } from './../../config';
import { Pessoa } from '../schemas';
import {
    resultNotFound,
    resultFound,
    resultFoundRaw,
    hashFotosSispesquisa,
    logErroBuscaBD,
    print
} from '../../utils';


import { SIGLA_FONTE } from './../_fontes';

const fonte_dado = SIGLA_FONTE['sispesquisa.foto'];

const fonte_renach = MODEL_PRIORITY['sispesquisa.foto.renach'].fonte;
const rank_renach  = MODEL_PRIORITY['sispesquisa.foto.renach'].rank;
const grupo_renach = MODEL_PRIORITY['sispesquisa.foto.renach'].grupo;

export let getFotoCPF_FileSystem_Renach = function (cpf: string){
    const pwd = '/home/labld/fotos/renach/';

    return getFotos(cpf, pwd)
            .then((result: any) => {
                if (Object.keys(result).length !== 0)
                    return resultFoundRaw(result, fonte_renach, rank_renach, grupo_renach);
                else
                    return resultNotFound(`Nao foi possivel encontrar fotos da pessoa ${cpf}.`, fonte_renach);
            }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelas fotos da pessoa ${cpf}.`, 'getFotoCPF_FileSystem_Renach'); });
};

const fonte = MODEL_PRIORITY['sispesquisa.foto'].fonte;
const rank  = MODEL_PRIORITY['sispesquisa.foto'].rank;
const grupo = MODEL_PRIORITY['sispesquisa.foto'].grupo;

export let getFotoCPF_FileSystem_Fotos = function (cpf: string){
    const pwd = '/home/labld/fotos/fotos/';

    return getFotos(cpf, pwd)
            .then((result: any) => {
                if (Object.keys(result).length !== 0)
                    return resultFoundRaw(result, fonte, rank, grupo);
                else
                    return resultNotFound(`Nao foi possivel encontrar fotos da pessoa ${cpf}.`, fonte);
            }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelas fotos da pessoa ${cpf}.`, 'getFotoCPF_FileSystem_Fotos'); });
};

let getFotos = function (cpf, pwd) {

    return Promise.all([getFotoFace(cpf, pwd),
                        getFotoAssinatura(cpf, pwd)])
            .then(fotos => {
                let img: any = {};

                if (fotos[0]) { img.face = fotos[0]; };
                if (fotos[1]) { img.assinatura = fotos[1]; };
                if (fotos[0] || fotos[1]){ img.fonte = fonte_dado; }

                return img;
            })
}

let getFotoFace = function (cpf, pwd) {

    const img = `${cpf}-F.jpg`;
    
    return rf(`${pwd}${img}`)
        .then(bitmap => new Buffer(bitmap).toString('base64'))
        .catch(error => {
            // Arquivo nao existe
            if (error.code === 'ENOENT') { return null; }
            else { logErroBuscaBD(error, `Falha na busca pela Foto de ${cpf}.`, 'getFotoFace'); return null; }
        });
}

let getFotoAssinatura = function (cpf, pwd) {

    const img = `${cpf}-A.jpg`;

    return rf(`${pwd}${img}`)
        .then(bitmap => new Buffer(bitmap).toString('base64'))
        .catch(error => {
            // Arquivo nao existe
            if (error.code === 'ENOENT') { return null; }
            else { logErroBuscaBD(error, `Falha na busca pela Foto de ${cpf}.`, 'getFotoAssinatura'); return null; }
        });
}