import * as mssql from 'mssql';

import { connection }  from '../../services/db.service';
import { MODEL_PRIORITY, API_CONFIG } from '../../config';
import {
    trataResultado,
    resultNotFound,
    resultFound,
    logErroBuscaBD
} from '../../utils';

const fonte = MODEL_PRIORITY['historico_quadro_societario.cne'].fonte;
const rank  = MODEL_PRIORITY['historico_quadro_societario.cne'].rank;
const grupo = MODEL_PRIORITY['historico_quadro_societario.cne'].grupo;

export let getHistoricoQuadroPFDetalhadoCPF_CNE = function (cpf: string){

    return connection.getConn().request()
        .input('cpf', mssql.VarChar, cpf)
        .execute(`CNE.dbo._SP_HISTORICO_PARTICIPACAO_PF_EM_SOCIEDADES`)
        .then((result : any) => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar historico de quadro societario de ${cpf}.`, fonte);
            else{
                result.recordset = result.recordset.map(r => { r.tipo = 'pf'; return r; })
                result.recordset = result.recordset.slice(0, API_CONFIG.SIAP_MAX_RESULTS);

                return resultFound(result, fonte, rank, grupo);
            }
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelo historico de quadro societario de ${cpf}.`, 'getHistoricoQuadroPFDetalhadoCPF_CNE'); });
};

export let getHistoricoQuadroPJDetalhadoCNPJ_CNE = function (cnpj: string){

    return connection.getConn().request()
        .input('cnpj', mssql.VarChar, cnpj)
        .execute(`CNE.dbo._SP_HISTORICO_PARTICIPACAO_PJ_EM_SOCIEDADES`)
        .then((result: any) => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar historico de quadro societario de ${cnpj}.`, fonte);
            else{
                result.recordset = result.recordset.map(r => { r.tipo = 'pj-pj'; return r; });
                result.recordset = result.recordset.slice(0, API_CONFIG.SIAP_MAX_RESULTS);

                return resultFound(result, fonte, rank, grupo);
            }
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelo historico de quadro societario de ${cnpj}.`, 'getHistoricoQuadroPJDetalhadoCNPJ_CNE'); });
};

export let getHistoricoQuadroPFDetalhadoCNPJ_CNE = function (cnpj: string){

    return connection.getConn().request()
        .input('cnpj', mssql.VarChar, cnpj)
        .execute(`CNE.dbo._SP_HISTORICO_SOCIOS_PF_E_PJ_EMPRESAS`)
        .then((result: any) => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar historico de quadro societario de ${cnpj}.`, fonte);
            else{
                result.recordset = result.recordset.map(r => { r.tipo = 'pj-pf'; return r; });
                result.recordset = result.recordset.slice(0, API_CONFIG.SIAP_MAX_RESULTS);

                return resultFound(result, fonte, rank, grupo);
            }
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelo historico de quadro societario de ${cnpj}.`, 'getHistoricoQuadroPFDetalhadoCNPJ_CNE'); });
};