import { Request } from 'express';
import * as mssql from 'mssql';

import { connection }  from '../services/db.service';
import {
    resultNotFound,
    resultFound,
    logErroBuscaBD
} from '../utils';
import {
    MODEL_PRIORITY,
    API_CODES,
    API_CONFIG,
    API_MSGS
} from '../config';

const fonte   = MODEL_PRIORITY['integra.requisicoes'].fonte;
const rank    = MODEL_PRIORITY['integra.requisicoes'].rank;
const grupo   = MODEL_PRIORITY['integra.requisicoes'].grupo;


export let insereRequisicao = function (requisicao){
    return connection.getConn().request()
        .input('nome', mssql.NVarChar, requisicao.nome)
        .input('email', mssql.NVarChar, requisicao.email)
        .input('tipoReferencia', mssql.NVarChar, requisicao.tipoReferencia)
        .input('doc', mssql.NVarChar, requisicao.doc)
        .input('tipoArea', mssql.NVarChar, requisicao.tipoArea)
        .input('tipoGrupo', mssql.NVarChar, requisicao.tipoGrupo)
        .input('detalhesResumoDosFatos', mssql.NVarChar, requisicao.detalhesResumoDosFatos)
        .input('detalhesFatoresAdversos', mssql.NVarChar, requisicao.detalhesFatoresAdversos)
        .input('detalhesFinalidade', mssql.NVarChar, requisicao.detalhesFinalidade)
        .input('tempoInformacao', mssql.Int, requisicao.tempoInformacao)
        .input('idPromotoria', mssql.Int, requisicao.idPromotoria)
        .query(`INSERT INTO Integra.dbo.REQUISICOES
                (NOME, EMAIL, TIPO_REFERENCIA, DOC, TIPO_AREA, TIPO_GRUPO, DETALHES_RESUMO_FATOS,
                DETALHES_FATORES_ADVERSOS, DETALHES_FINALIDADE, TEMPO_INFORMACAO, ID_PROMOTORIA )
                
                OUTPUT INSERTED.ID_REQUISICAO

                VALUES
                (@nome, @email, @tipoReferencia, @doc, @tipoArea, @tipoGrupo, @detalhesResumoDosFatos,
                @detalhesFatoresAdversos, @detalhesFinalidade, @tempoInformacao, @idPromotoria)
        `)
        .then(result => {
            return result.recordset[0].ID_REQUISICAO;
            // {
            //     status: API_CODES.CODE_SUCESSO,
            //     msg: 'Requisição cadastrada com sucesso.',
            //     dados: [{
            //         idRequisicao: result.recordset[0].ID_REQUISICAO
            //     }]
            // }
        }).catch( error => { return logErroBuscaBD(error, `Falha na insercao da requisicao.`, 'insereRequisicao'); });
};

export let insereImovel = function (requisicao, idRequisicao){
    return connection.getConn().request()
        .input('idRequisicao', mssql.Int, idRequisicao)
        .input('endereco', mssql.NVarChar, requisicao.endereco)
        .input('necessidade', mssql.NVarChar, requisicao.necessidade)
        .query(`INSERT INTO Integra.dbo.IMOVEIS
                (ID_REQUISICAO, ENDERECO, NECESSIDADE)
                
                VALUES
                (@idRequisicao, @endereco, @necessidade)
        `)
        .then(result => {
            return true;
        }).catch( error => { return logErroBuscaBD(error, `Falha na insercao do imovel.`, 'insereImovel'); });
};

export let insereVeiculo = function (requisicao, idRequisicao){
    return connection.getConn().request()
        .input('idRequisicao', mssql.Int, idRequisicao)
        .input('cpf', mssql.NVarChar, requisicao.cpf)
        .input('placa', mssql.NVarChar, requisicao.placa)
        .input('necessidade', mssql.NVarChar, requisicao.necessidade)
        .query(`INSERT INTO Integra.dbo.VEICULOS
                (ID_REQUISICAO, CPF, PLACA, NECESSIDADE)
                VALUES
                (@idRequisicao, @cpf, @placa, @necessidade)
        `)
        .then(result => {
            return true;
        }).catch( error => { return logErroBuscaBD(error, `Falha na insercao do veiculo.`, 'insereVeiculo'); });
};

export let inserePF = function (requisicao, idRequisicao){
    return connection.getConn().request()
        .input('idRequisicao', mssql.Int, idRequisicao)
        .input('cpf', mssql.NVarChar, requisicao.cpf)
        .input('identidade', mssql.NVarChar, requisicao.identidade)
        .input('orgaoExpeditor', mssql.NVarChar, requisicao.orgaoExpeditor)
        .input('ufOrgaoExpeditor', mssql.NVarChar, requisicao.ufOrgaoExpeditor)
        .input('necessidade', mssql.NVarChar, requisicao.necessidade)
        .input('nome', mssql.NVarChar, requisicao.nome)
        .input('nomeMae', mssql.NVarChar, requisicao.nomeMae)
        .input('nomePai', mssql.NVarChar, requisicao.nomePai)
        .query(`INSERT INTO Integra.dbo.PESSOAS_FISICAS
                (ID_REQUISICAO, CPF, IDENTIDADE, ORGAO_EXPEDITOR, UF_ORGAO_EXPEDITOR, 
                NECESSIDADE, NOME, NOMEMAE, NOMEPAI )
                VALUES
                (@idRequisicao, @cpf, @identidade, @orgaoExpeditor, @ufOrgaoExpeditor,
                @necessidade, @nome, @nomeMae, @nomePai)
        `)
        .then(result => {
            return true;
        }).catch( error => { return logErroBuscaBD(error, `Falha na insercao da pessoa fisica.`, 'inserePF'); });
};

export let inserePJ = function (requisicao, idRequisicao){
    return connection.getConn().request()
        .input('idRequisicao', mssql.Int, idRequisicao)
        .input('cnpj', mssql.NVarChar, requisicao.cnpj)
        .input('nomeFantasia', mssql.NVarChar, requisicao.nomeFantasia)
        .input('razaoSocial', mssql.NVarChar, requisicao.razaoSocial)
        .input('necessidade', mssql.NVarChar, requisicao.necessidade)
        .query(`INSERT INTO Integra.dbo.PESSOAS_JURIDICAS
                (ID_REQUISICAO, CNPJ, NOMEFANTASIA, RAZAOSOCIAL, NECESSIDADE)
                VALUES
                (@idRequisicao, @cnpj, @nomeFantasia, @razaoSocial, @necessidade)
        `)
        .then(result => {
            return true;
        }).catch( error => { return logErroBuscaBD(error, `Falha na insercao da pessoa juridica.`, 'inserePJ'); });
};

export let insereAnexo = function (requisicao, idRequisicao){
    return connection.getConn().request()
        .input('idRequisicao', mssql.Int, idRequisicao)
        .input('nomeArquivo', mssql.NVarChar, requisicao.originalname)
        .input('arquivo', mssql.VarBinary, requisicao.buffer)
        .input('mime', mssql.NVarChar, requisicao.mimetype)
        .query(`INSERT INTO Integra.dbo.ANEXOS
                (ID_REQUISICAO, NOME_ARQUIVO, ARQUIVO, MIME)
                VALUES
                (@idRequisicao, @nomeArquivo, @arquivo, @mime)
        `)
        .then(result => {
            return true;
        }).catch( error => { return logErroBuscaBD(error, `Falha na insercao do anexo ${requisicao.nomeArquivo}.`, 'insereAnexo'); });
};

export let getRequisicoes = function (){
    let resultadoRequisicoes;

    return connection.getConn().request()
        .query(`SELECT  ID_REQUISICAO as idRequisicao, NOME as nome, EMAIL as email,
                        TIPO_REFERENCIA as tipoReferencia, DOC as doc, TIPO_AREA as tipoArea,
                        TIPO_GRUPO as tipoGrupo, DETALHES_RESUMO_FATOS as detalhesResumoDosFatos,
                        DETALHES_FATORES_ADVERSOS as detalhesFatoresAdversos,
                        DETALHES_FINALIDADE as detalhesFinalidade, TEMPO_INFORMACAO as tempoInformacao,
                        FINALIZADO as finalizado, PROMOTORIAS.DESCRICAO as promotoria,
                        DT_REQUISICAO as dataRequisicao


                FROM Integra.dbo.REQUISICOES
                INNER JOIN Integra.dbo.PROMOTORIAS
                    ON (REQUISICOES.ID_PROMOTORIA = PROMOTORIAS.ID_PROMOTORIA)
        `)
        .then(result => resultFound(result, fonte, rank, grupo))
        .then(result => {
            resultadoRequisicoes = result;
            return Promise.all(result.resultado.dados.map(requisicao => getExtras(requisicao)));
        })
        .then(r => {
            resultadoRequisicoes.resultado.dados = r;
            return resultadoRequisicoes;
        })
        .catch( error => { return logErroBuscaBD(error, `Falha na busca das requisicoes do sistema.`, 'getRequisicoes'); });
};

let getExtras = function(requisicao) {
    return Promise.all([getAnexosRequisicao(requisicao.idRequisicao),
                        getImoveisRequisicao(requisicao.idRequisicao),
                        getPFsRequisicao(requisicao.idRequisicao),
                        getPJsRequisicao(requisicao.idRequisicao),
                        getVeiculosRequisicao(requisicao.idRequisicao),
                    ])
                .then(extras => {
                    extras.forEach(extra => {
                        let nomeAtributo = Object.keys(extra)[0];
                        let atributo = extra[nomeAtributo];
                        requisicao[nomeAtributo] = atributo;
                    })
                    return requisicao;
                });
}

export let getAnexo = function (idAnexo){
    return connection.getConn().request()
        .input('idAnexo', mssql.Int, idAnexo)
        .query(`SELECT  ARQUIVO as arquivo, NOME_ARQUIVO as nomeArquivo,
                        DATALENGTH(ARQUIVO) AS tamanho, MIME as mime

                FROM Integra.dbo.ANEXOS
                WHERE ID_ANEXO=@idAnexo
        `)
        .then(result => result.recordset[0])
        .catch( error => { return logErroBuscaBD(error, `Falha na busca do anexo ${idAnexo}.`, 'getAnexo'); });
};

let getAnexosRequisicao = function (idRequisicao){
    return connection.getConn().request()
        .input('idRequisicao', mssql.Int, idRequisicao)
        .query(`SELECT  ID_ANEXO as idAnexo, NOME_ARQUIVO as nomeArquivo

                FROM Integra.dbo.ANEXOS
                WHERE ID_REQUISICAO=@idRequisicao
        `)
        .then(result => {
            return {
                anexos: result.recordset
            };
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca dos anexos de ${idRequisicao}.`, 'getAnexosRequisicao'); });
};

let getImoveisRequisicao = function (idRequisicao){
    return connection.getConn().request()
        .input('idRequisicao', mssql.Int, idRequisicao)
        .query(`SELECT  ENDERECO as endereco, NECESSIDADE as necessidade

                FROM Integra.dbo.IMOVEIS
                WHERE ID_REQUISICAO=@idRequisicao
        `)
        .then(result => {
            return {
                imoveis: result.recordset
            };
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca dos imoveis de ${idRequisicao}.`, 'getImoveisRequisicao'); });
};

let getPFsRequisicao = function (idRequisicao){
    return connection.getConn().request()
        .input('idRequisicao', mssql.Int, idRequisicao)
        .query(`SELECT  CPF as cpf, IDENTIDADE as identidade,
                        ORGAO_EXPEDITOR as orgaoExpeditor, UF_ORGAO_EXPEDITOR as ufOrgaoExpeditor,
                        NECESSIDADE as necessidade, NOME as nome, NOMEMAE as nomeMae,
                        NOMEPAI as nomePai

                FROM Integra.dbo.PESSOAS_FISICAS
                WHERE ID_REQUISICAO=@idRequisicao
        `)
        .then(result => {
            return {
                pessoasFisicas: result.recordset
            };
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca das pessoas fisicas de ${idRequisicao}.`, 'getPFsRequisicao'); });
};

let getPJsRequisicao = function (idRequisicao){
    return connection.getConn().request()
        .input('idRequisicao', mssql.Int, idRequisicao)
        .query(`SELECT  CNPJ as cnpj, NOMEFANTASIA as nomeFantasia,
                        RAZAOSOCIAL as razaoSocial, NECESSIDADE as necessidade

                FROM Integra.dbo.PESSOAS_JURIDICAS
                WHERE ID_REQUISICAO=@idRequisicao
        `)
        .then(result => {
            return {
                pessoasJuridicas: result.recordset
            };
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca das pessoas juridicas de ${idRequisicao}.`, 'getPJsRequisicao'); });
};

let getVeiculosRequisicao = function (idRequisicao){
    return connection.getConn().request()
        .input('idRequisicao', mssql.Int, idRequisicao)
        .query(`SELECT  CPF as cpf, PLACA as placa, NECESSIDADE as necessidade

                FROM Integra.dbo.VEICULOS
                WHERE ID_REQUISICAO=@idRequisicao
        `)
        .then(result => {
            return {
                veiculos: result.recordset
            };
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca dos veiculos de ${idRequisicao}.`, 'getVeiculosRequisicao'); });
};

export let finalizaRequisicao = function (id: string){
    return connection.getConn().request()
        .input('idRequisicao', mssql.Int, id)
        .query(`UPDATE  Integra.dbo.REQUISICOES
        
                SET FINALIZADO=1
                WHERE ID_REQUISICAO=@idRequisicao
        `)
        .then(result => {
            return {
                status: API_CODES.CODE_SUCESSO,
                msg: 'Requisição finalizada com sucesso.'
            }
        }).catch( error => { return logErroBuscaBD(error, `Falha na finalização da requisição ${id}.`, 'finalizaRequisicao'); });
};

//https://www.robtex.com/api/
//https://freeapi.robtex.com/ipquery/186.215.248.218
//http://ipinfo.io/186.215.248.218

//https://github.com/IonicaBizau/node-ipinfo
//https://github.com/bluesmoon/node-geoip