import * as mssql from 'mssql';

import { connection }  from '../services/db.service';
import { MODEL_PRIORITY, API_CONFIG } from '../config';
import {
    resultNotFound,
    resultFound,
    logErroBuscaBD
} from '../utils';

const fonte = MODEL_PRIORITY['sagres_municipal.licitacao'].fonte;
const rank  = MODEL_PRIORITY['sagres_municipal.licitacao'].rank;
const grupo = MODEL_PRIORITY['sagres_municipal.licitacao'].grupo;

export let getDadosLicitacoesMunicipalEmpresaCNPJ = function (cnpj: string) {

    return connection.getConn().request()
        .input('CNPJ', mssql.Char(14), cnpj)
        .query(`
            EXEC SAGRES_MUNICIPAL.DBO.USP_DADOS_LICITACOES @CNPJ
        `)
        .then((result: any) => {
            result.recordset = result.recordset.map(d => {
                return {
                    ugestora: d.de_Ugestora.toUpperCase(),
                    dtAno: d.dt_Ano,
                    // mesAno: d.MesAno,
                    tipoLicitacao: d.de_TipoLicitacao.toUpperCase(),
                    numeroLicitacao: d.nu_Licitacao,
                    dataHomologacao: d.dt_Homologacao,
                    numeroPropostas: d.nu_Propostas,
                    vlLicitacao: d.vl_Licitacao,
                    tipoObjeto: d.de_TipoObjeto,
                    cpfCnpj: d.nu_CPFCNPJ,
                    razaoSocial: d.RazaoSocial,
                    qtdOfertada: d.qt_Ofertada,
                    uniMedida: d.de_UnidadeMedida,
                    vlOfertado: d.vl_Ofertado,
                    situacaoProposta: d.de_SituacaoProposta,
                    temTipologia: d.Tem_Tipologia,
                    totalPesoTipologia: d.TotalPesoTipologia,
                }
            }).slice(0, API_CONFIG.SIAP_MAX_RESULTS);

            if (result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar dados de licitacoes de ${cnpj}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => logErroBuscaBD(error, `Falha na busca pelos dados de licitacoes de ${cnpj}.`, 'getDadosLicitacoesMunicipalEmpresaCNPJ'));
};
