import * as mssql from 'mssql';

import { connection }  from '../services/db.service';
import { MODEL_PRIORITY, API_CONFIG } from '../config';
import { BenchmarkService } from '../services/benchmark.service';
import {
    resultNotFound,
    resultFound,
    logErroBuscaBD,
    printTempoExecucao
} from '../utils';

const fonte  =  'sagres'; //MODEL_PRIORITY['sisobi.obito'].fonte;
const rank   =  0; //MODEL_PRIORITY['sisobi.obito'].rank;
const grupo  =  'nepotismo'; //MODEL_PRIORITY['sisobi.obito'].grupo;

export let getNepotismoUgestoraEsferaAno = function (lotacao: string, esfera: string, ano: string){

    const tempoInicial = new BenchmarkService();

    return connection.getConn().request()
        .input('LOTACAO', mssql.Char(50), lotacao)
        .input('ESFERA_ANALISADA', mssql.Char(1), esfera)
        .input('ANO', mssql.Int, ano)
        .query(`
            EXEC DBO.USP_NEPOTISMO @ANO, @ESFERA_ANALISADA, @LOTACAO
        `)
        .then((result: any) => {
            printTempoExecucao(tempoInicial, 'getNepotismoUgestoraEsferaAno');

            result.recordset = result.recordset.slice(0, API_CONFIG.SIAP_MAX_RESULTS);

            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar nepotismo em ${lotacao}.`, fonte);
            else {
                return resultFound(result, fonte, rank, grupo);
            }
        }).catch(error => logErroBuscaBD(error, `Falha na busca pelo nepotismo em ${lotacao}.`, 'getNepotismoUgestoraEsferaAno'));
};

export let getNepotismoCPFAno = function (cpf: string, ano: string){

    const tempoInicial = new BenchmarkService();

    return connection.getConn().request()
        .input('CPF', mssql.Char(11), cpf)
        .input('ANO', mssql.Int, ano)
        .query(`
            EXEC DBO.USP_NEPOTISMO_CPF @ANO, @CPF
        `)
        .then((result: any) => {
            printTempoExecucao(tempoInicial, 'getNepotismoCPFAno');

            result.recordset = result.recordset.slice(0, API_CONFIG.SIAP_MAX_RESULTS);

            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar nepotismo de ${cpf}.`, fonte);
            else {
                return resultFound(result, fonte, rank, grupo);
            }
        }).catch(error => logErroBuscaBD(error, `Falha na busca pelo nepotismo de ${cpf}.`, 'getNepotismoCPFAno'));
};