import * as mssql from 'mssql';

import { connection }  from '../services/db.service';
import { MODEL_PRIORITY, API_CONFIG } from '../config';
import {
    resultNotFound,
    resultFound,
    fixObitoCPF,
    logErroBuscaBD
} from '../utils';

const fonte  = MODEL_PRIORITY['sisobi.obito'].fonte;
const rank   = MODEL_PRIORITY['sisobi.obito'].rank;
const grupo  = MODEL_PRIORITY['sisobi.obito'].grupo;

export let getObitoDetalhadoCPF_Sisobi_Obito = function (cpf: string){

    return connection.getConn().request()
        .input('cpf_parameter', mssql.Char(11), cpf)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        Cartorio.CartorioNome as cartorio_nome, Cartorio.Endereco as cartorio_endereco,
                        Cartorio.Bairro as cartorio_bairro, Cartorio.CEP as cartorio_cep,
                        Cartorio.Municipio as cartorio_municipio, Cartorio.UF as cartorio_uf,
                        Cartorio.DDD as cartorio_ddd, Cartorio.Telefone1 as cartorio_telefone,
                        Cartorio.Telefone2 as cartorio_telefone2, Cartorio.Titular as cartorio_titular,
                        Cartorio.Substituto as cartorio_substituto,

                        Obito.Livro as obito_livro, Obito.Folha as obito_folha,
                        Obito.Termo as obito_termo, Obito.Lavratura as obito_lavratura,
                        Obito.Obito as obito_dataObito,

                        Obito.Falecido as obito_nome,
                        Obito.Mae as obito_nomeMae, Obito.NascimentoData as obito_dataNascimento,
                        Obito.CPF_RF as obito_cpf, Obito.Identidade as obito_rg,
                        Obito.OrgaoEmissor as obito_rgOrgEmissor, Obito.UFEmissao as obito_rgUFEmissao,

                        Obito.CITipo as obito_ciTipo, Obito.CILivro as obito_ciLivro,
                        Obito.CIFolha as obito_ciFolha, Obito.CITermo as obito_ciTermo,

                        Obito.CTPSNumero as obito_ctpsNumero, Obito.CTPSSerie as obito_ctpsSerie,
                        Obito.CTPSUF as obito_ctpsUF, Obito.Beneficio1 as obito_beneficio1,
                        Obito.Beneficio2 as obito_beneficio2, Obito.Beneficio3 as obito_beneficio3

                FROM Sisobi.dbo.Obito
                    LEFT OUTER JOIN Sisobi.dbo.Cartorio ON (Obito.CartorioID = Cartorio.CartorioID)
                WHERE CPF_RF=@cpf_parameter
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o obito de ${cpf}.`, fonte);
            else {
                return resultFound(result, fonte, rank, grupo);
            }
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelo obito de ${cpf}.`, 'getObitoDetalhadoCPF_Sisobi_Obito'); });
};

export let getObitoSimplificadoCPF_Sisobi_Obito = function (cpf: string){

    return connection.getConn().request()
        .input('cpf_parameter', mssql.Char(11), cpf)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        Obito.CPF_RF as obito_cpf, Obito.Falecido as obito_nome, Obito.Obito as obito_dataObito,
                        Obito.Mae as obito_nomeMae, Obito.NascimentoData as obito_dataNascimento,
                        Cartorio.Municipio as cartorio_municipio, Cartorio.UF as cartorio_uf

                FROM Sisobi.dbo.Obito
                    LEFT OUTER JOIN Sisobi.dbo.Cartorio ON (Obito.CartorioID = Cartorio.CartorioID)
                WHERE CPF_RF=@cpf_parameter
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o obito de ${cpf}.`, fonte);
            else{
                return resultFound(result, fonte, rank, grupo);
            }
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelo obito de ${cpf}.`, 'getObitoSimplificadoCPF_Sisobi_Obito'); });
};

export let getObitoSimplificadoNome_Sisobi_Obito = function (nome: string){

    return connection.getConn().request()
        .input('nome_parameter', mssql.VarChar, nome)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        Obito.CPF_RF as obito_cpf, Obito.Falecido as obito_nome, Obito.Obito as obito_dataObito,
                        Obito.Mae as obito_nomeMae, Obito.NascimentoData as obito_dataNascimento,
                        Cartorio.Municipio as cartorio_municipio, Cartorio.UF as cartorio_uf

                FROM Sisobi.dbo.Obito
                    LEFT OUTER JOIN Sisobi.dbo.Cartorio ON (Obito.CartorioID = Cartorio.CartorioID)
                WHERE CONTAINS(Obito.Falecido, @nome_parameter)
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o obito de ${nome}.`, fonte);
            else {
                return resultFound(result, fonte, rank, grupo);
            }
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelo obito de ${nome}.`, 'getObitoSimplificadoNome_Sisobi_Obito'); });
};