import * as mssql from 'mssql';

import { connection }  from '../services/db.service';

import {
    resultNotFound,
    resultFound,
    logErroBuscaBD
} from '../utils';

import {
    MODEL_PRIORITY,
    API_CODES,
    API_CONFIG,
    API_MSGS
} from '../config';

const fonte   = MODEL_PRIORITY['operacoes'].fonte;
const rank    = MODEL_PRIORITY['operacoes'].rank;
const grupo   = MODEL_PRIORITY['operacoes'].grupo;

export let getOperacoesCPF_Sispesquisa_Operacoes = function (cpf: string){
    return connection.getConn().request()
        .input('cpf', mssql.NVarChar, cpf)
        .query(`SELECT DISTINCT OPERACOES.OPERACAO AS nomeOperacao, OPERACOES.DATA AS dataOperacao
        
                FROM SISPESQUISA.operacoes.ALVOS_PF AS PF
                INNER JOIN SISPESQUISA.operacoes.OPERACOES
                    ON (PF.ID_OPERACAO = OPERACOES.ID)
                    WHERE PF.DOCUMENTO=@cpf
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar operacoes para a pessoa ${cpf}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        })
        .catch( error => { return logErroBuscaBD(error, `Falha na busca das operacoes para a pessoa ${cpf}.`, 'getOperacoesCPF_Sispesquisa_Operacoes'); });
};

export let getOperacoesCNPJ_Sispesquisa_Operacoes = function (cnpj: string){
    return connection.getConn().request()
        .input('cnpj', mssql.NVarChar, cnpj)
        .query(`SELECT DISTINCT OPERACOES.OPERACAO AS nomeOperacao, OPERACOES.DATA AS dataOperacao
        
                FROM SISPESQUISA.operacoes.ALVOS_PJ AS PJ
                INNER JOIN SISPESQUISA.operacoes.OPERACOES
                    ON (PJ.ID_OPERACAO = OPERACOES.ID)
                    WHERE PJ.DOCUMENTO=@cnpj
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar operacoes para a pessoa ${cnpj}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        })
        .catch( error => { return logErroBuscaBD(error, `Falha na busca das operacoes para a pessoa ${cnpj}.`, 'getOperacoesNPJ_Sispesquisa_Operacoes'); });
};