import * as soap from 'soap';

import { Pessoa } from '../schemas';

import {
    API_CONFIG,
    MODEL_PRIORITY,
    API_CODES,
    API_MSGS
} from './../../config';

import {
    resultIPCFound,
    resultNotFound,
    resultFoundRaw,
    logErroBuscaBD,
    trataErro,
    removerAcentos
} from '../../utils';

import { SIGLA_FONTE } from './../_fontes';

const fonte_dado = SIGLA_FONTE['ipc'];

const fonte  = MODEL_PRIORITY['ipc'].fonte;
const rank   = MODEL_PRIORITY['ipc'].rank;
const grupo  = MODEL_PRIORITY['ipc'].grupo;

export let getPessoaSimplificadoNome_IPC = function (nome: string){
    return Promise_getPessoaSimplificado_IPC('Nome' , nome)
        .then((result:any) => {
            if (result.length !== 0)
                return resultFoundRaw(result, fonte, rank, grupo);
            else
                return resultNotFound(`Nao foi possivel encontrar a pessoa ${nome}.`, fonte);
        })
        .catch(error => { return logErroBuscaBD(error, `Falha na busca pela pessoa ${nome}.`, 'getPessoaSimplificadoNome_IPC'); });
}

export let getPessoaSimplificadoRG_IPC = function (rg: string){
    return Promise_getPessoaSimplificado_IPC('RG', rg)
        .then((result:any) => {
            if (result.length !== 0)
                return resultFoundRaw(result, fonte, rank, grupo);
            else
                return resultNotFound(`Nao foi possivel encontrar a pessoa ${rg}.`, fonte);
        })
        .catch(error => { return logErroBuscaBD(error, `Falha na busca pela pessoa ${rg}.`, 'getPessoaSimplificadoRG_IPC'); });
}

export let getPessoaSimplificadoNomePai_IPC = function (nomePai: string){
    return Promise_getPessoaSimplificado_IPC('NomePai', nomePai)
        .then((result:any) => {
            if (result.length !== 0)
                return resultFoundRaw(result, fonte, rank, grupo);
            else
                return resultNotFound(`Nao foi possivel encontrar a pessoa ${nomePai}.`, fonte);
        })
        .catch(error => { return logErroBuscaBD(error, `Falha na busca pela pessoa ${nomePai}.`, 'getPessoaSimplificadoNomePai_IPC'); });
}

export let getPessoaSimplificadoNomeMae_IPC = function (nomeMae: string){
    return Promise_getPessoaSimplificado_IPC('NomeMae', nomeMae)
        .then((result:any) => {
            if (result.length !== 0)
                return resultFoundRaw(result, fonte, rank, grupo);
            else
                return resultNotFound(`Nao foi possivel encontrar a pessoa ${nomeMae}.`, fonte);
        })
        .catch(error => { return logErroBuscaBD(error, `Falha na busca pela pessoa ${nomeMae}.`, 'getPessoaSimplificadoNomeMae_IPC'); });
}

export let getPessoaSimplificadoCPF_IPC = function (cpf: string){
    const cpfFormatado = cpf.slice(0,3)+"."+cpf.slice(3,6)+"."+cpf.slice(6,9)+"-"+cpf.slice(9);

    return Promise_getPessoaSimplificado_IPC('CPF', cpfFormatado)
        .then((result:any) => {
            if (result.length !== 0)
                return resultFoundRaw(result, fonte, rank, grupo);
            else
                return resultNotFound(`Nao foi possivel encontrar a pessoa ${cpf}.`, fonte);
        })
        .catch(error => { return logErroBuscaBD(error, `Falha na busca pela pessoa ${cpf}.`, 'getPessoaSimplificadoCPF_IPC'); });
}

export let getPessoaDetalhadoCPF_IPC = function (cpf: string){
    const cpfFormatado = cpf.slice(0,3)+"."+cpf.slice(3,6)+"."+cpf.slice(6,9)+"-"+cpf.slice(9);

    return Promise_getPessoaDetalhado_IPC('CPF', cpfFormatado)
        .then((result:any) => {
            if (result.length !== 0)
                return resultFoundRaw(result, fonte, rank, grupo);
            else
                return resultNotFound(`Nao foi possivel encontrar a pessoa ${cpf}.`, fonte);
        })
        .catch(error => { return logErroBuscaBD(error, `Falha na busca pela pessoa ${cpf}.`, 'getPessoaDetalhadoCPF_IPC'); });
}

let Promise_createClientIPC = function () {
    return new Promise((resolve, reject) => {
        soap.createClient(API_CONFIG.IPC_WSDL_URL, (error, client: any) => {
            if (error) {
                reject(error);
                return;
            } else {
                resolve(client);
            }
        });

        setTimeout(() => {
            reject(trataErro(API_CODES.CODE_RECURSO_NAO_ENCONTRADO, API_MSGS.MSG_IPC_TIMEOUT));
        }, API_CONFIG.IPC_CREATE_CLIENT_TIMEOUT);
    });
}

let Promise_clientPesquisarSimplificado = function (client, tipoPesquisa: string, valorPesquisa: string){
    return new Promise((resolve, reject) => {
        const params = {
            chaveAcesso: API_CONFIG.SIAP_IPC_CHAVE_ACESSO,
            tipoPesquisa: tipoPesquisa,
            valorPesquisa: valorPesquisa
        }

        client.Pesquisar(params, (err, result, raw, soapHeader) => {
            if (err) {
                reject(err);
                return
            } else {
                let dados;
                if (result && result.PesquisarResult) {
                    dados = result.PesquisarResult.Pessoa.map(registro => {
                        let { Nome, RG, DataExpedicao, NomeMae, NomePai, Naturalidade, UFNaturalidade, DataNascimento, CPF, Cidade, Estado } = registro;
                        return {
                            nome:               (Nome) ? Nome : null,
                            nomeMae:            (NomeMae) ? NomeMae : null,
                            municipio:          (Cidade) ? Cidade : null,
                            uf:                 (Estado) ? Estado : null,
                            cpf:                (CPF) ? CPF.replace(/[^0-9]/g, '') : null,
                            dataNascimento:     (DataNascimento) ? DataNascimento : null,
                            fonte:              fonte_dado
                        };
                    });
                } else {
                    dados = [];
                }
                resolve(dados);
            }
        }, { timeout: API_CONFIG.IPC_REQUEST_TIMEOUT })
    });
}

let Promise_clientPesquisarDetalhado = function (client, tipoPesquisa: string, valorPesquisa: string){
    return new Promise((resolve, reject) => {
        const params = {
            chaveAcesso: API_CONFIG.SIAP_IPC_CHAVE_ACESSO,
            tipoPesquisa: tipoPesquisa,
            valorPesquisa: valorPesquisa
        }

        client.Pesquisar(params, (err, result, raw, soapHeader) => {
            if (err) {
                reject(err);
                return
            } else {
                let dados;
                if (result && result.PesquisarResult) {
                    dados = result.PesquisarResult.Pessoa.map(registro => {
                        let { Nome, RG, DataExpedicao, NomeMae, NomePai, Naturalidade, UFNaturalidade, DataNascimento, CPF, Cidade, Estado } = registro;
                        return {
                            nome:           (Nome)      ? Nome : null,
                            rg:             (RG)        ? RG : null,
                            nomeMae:        (NomeMae)   ? NomeMae : null,
                            nomePai:        (NomePai    ? NomePai : null),
                            cpf:            (CPF)       ? CPF.replace(/[^0-9]/g, '') : null,
                            municipio:      (Cidade)    ? Cidade : null,
                            uf:             (Estado)    ? Estado : null,
                            dataExpedicao:  (DataExpedicao)     ? DataExpedicao : null,
                            naturalidade:   (Naturalidade)      ? Naturalidade : null,
                            ufNaturalidade: (UFNaturalidade)    ? UFNaturalidade : null,
                            dataNascimento: (DataNascimento)    ? DataNascimento : null,
                            fonte:          fonte_dado
                        };
                    });
                } else {
                    dados = [];
                }
                resolve(dados);
            }
        }, { timeout: API_CONFIG.IPC_REQUEST_TIMEOUT })
    });
}

let Promise_getPessoaSimplificado_IPC = function (tipoPesquisa: string, valorPesquisa: string) {

    return Promise_createClientIPC()
            .then((client: any) => Promise_clientPesquisarSimplificado(client, tipoPesquisa, valorPesquisa))
            .catch(error => { throw error; })
}

let Promise_getPessoaDetalhado_IPC = function (tipoPesquisa: string, valorPesquisa: string){
    return Promise_createClientIPC()
        .then((client: any) => Promise_clientPesquisarDetalhado(client, tipoPesquisa, valorPesquisa))
        .catch(error => { throw error; })
}