import * as mssql from 'mssql';

import { connection }  from '../../services/db.service';
import {
    resultNotFound,
    resultFound,
    modelAddWhereTipoComparacao,
    modelAddWhereDataNascimento,
    modelAddWherePrimeiroNome,
    removerAcentos,
    logErroBuscaBD
} from '../../utils';
import { MODEL_PRIORITY, API_CONFIG } from './../../config';

import { SIGLA_FONTE } from './../_fontes';

const fonte_dado = SIGLA_FONTE['receita'];

const fonte  = MODEL_PRIORITY['receita.pf'].fonte;
const rank   = MODEL_PRIORITY['receita.pf'].rank;
const grupo  = MODEL_PRIORITY['receita.pf'].grupo;

export let getPessoaSimplificadoNome_Receita_PF = function (nome: string,
                                                            tipoComparacao='fulltext',
                                                            dataNascimentoFilho=undefined,
                                                            travaPrimeiroNome=false){

    let condicao = modelAddWhereTipoComparacao('Nome', '@nome_parameter', tipoComparacao);
    let c1 = modelAddWhereDataNascimento('DataNascimento', '@dt_nasc_filho_parameter', dataNascimentoFilho);
    let c2 = modelAddWherePrimeiroNome('Nome', '@primeiro_nome_parameter', travaPrimeiroNome);
    condicao += (c1) ? c1 : '';
    condicao += (c2) ? c2 : '';

    let primeiroNome = removerAcentos(nome.split(" ")[0]);
    return connection.getConn().request()
        .input('nome_parameter', mssql.VarChar, nome)
        .input('primeiro_nome_parameter', mssql.VarChar, primeiroNome + ' %')
        .input('dt_nasc_filho_parameter', mssql.Date, dataNascimentoFilho)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        CPF as cpf,
                        UPPER(Nome) as nome,
                        UPPER(NomeMae) as nomeMae,
                        UPPER(Municipio) as municipio,
                        UPPER(UF) as uf,
                        '${fonte_dado}' as fonte,
                        CASE
                            WHEN isdate(DataNascimento) = 1 THEN cast(DataNascimento as date)
                            ELSE NULL
                        END AS dataNascimento

                FROM Receita.dbo.PF
                ${condicao}
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar a pessoa ${nome}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pela pessoa ${nome}.`, 'getPessoaSimplificadoNome_Receita_PF'); });
};

export let getPessoaNomeMae_Receita_PF = function (nomeMae: string, tipoComparacao='fulltext'){

    let condicao = modelAddWhereTipoComparacao('NomeMae', '@nome_mae_parameter', tipoComparacao);

    return connection.getConn().request()
        .input('nome_mae_parameter', mssql.VarChar, nomeMae)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        CPF as cpf,
                        UPPER(Nome) as nome,
                        UPPER(NomeMae) as nomeMae,
                        UPPER(Municipio) as municipio,
                        UPPER(UF) as uf,
                        '${fonte_dado}' as fonte,
                        CASE
                            WHEN isdate(DataNascimento) = 1 THEN cast(DataNascimento as date)
                            ELSE NULL
                        END AS dataNascimento

                FROM Receita.dbo.PF
                ${condicao}
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar a pessoa ${nomeMae}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pela pessoa ${nomeMae}.`, 'getPessoaNomeMae_Receita_PF'); });
}

export let getPessoaDetalhadoCPF_Receita_PF = function (cpf: String){

    return connection.getConn().request()
        .input('cpf_parameter', mssql.Char(11), cpf)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        CPF as cpf,
                        UPPER(Nome) as nome,
                        UPPER(NomeMae) as nomeMae,
                        '${fonte_dado}' as fonte,
                        CASE
                            WHEN isdate(DataNascimento) = 1 THEN cast(DataNascimento as date)
                            ELSE NULL
                        END AS dataNascimento,
                        UPPER(TipoSexo.Descricao) as sexo,
                        UPPER(ExercicioOcupacao) as anoExercicioOcupacao,
                        UPPER(TipoNaturezaOcupacao.Descricao) as naturezaOcupacao,
                        UPPER(TipoOcupacaoPrincipal.Tipo) as tipoOcupacaoPrincipal,
                        UPPER(TipoOcupacaoPrincipal.Descricao) as ocupacaoPrincipal

                FROM Receita.dbo.PF
                    INNER JOIN Receita_Full.Receita.TipoSexo ON (PF.Sexo = TipoSexo.Sexo)
                    LEFT OUTER JOIN Receita_Full.Receita.TipoNaturezaOcupacao ON (PF.NaturezaOcupacao = TipoNaturezaOcupacao.NaturezaOcupacao)
                    LEFT OUTER JOIN Receita_Full.Receita.TipoOcupacaoPrincipal ON (PF.OcupacaoPrincipal = TipoOcupacaoPrincipal.OcupacaoPrincipal)
                WHERE cpf=@cpf_parameter
       `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar a pessoa ${cpf}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pela pessoa ${cpf}.`, 'getPessoaDetalhadoCPF_Receita_PF'); });
};

export let getPessoaSimplificadoCPF_Receita_PF = function (cpf: String){

    return connection.getConn().request()
        .input('cpf_parameter', mssql.Char(11), cpf)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        CPF as cpf,
                        UPPER(Nome) as nome,
                        UPPER(NomeMae) as nomeMae,
                        UPPER(Municipio) as municipio,
                        UPPER(UF) as uf,
                        '${fonte_dado}' as fonte,
                        CASE
                            WHEN isdate(DataNascimento) = 1 THEN cast(DataNascimento as date)
                            ELSE NULL
                        END AS dataNascimento

                FROM Receita.dbo.PF
                WHERE cpf=@cpf_parameter
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar a pessoa ${cpf}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pela pessoa ${cpf}.`, 'getPessoaSimplificadoCPF_Receita_PF'); });
};