import * as mssql from 'mssql';

import { connection }  from '../../services/db.service';
import { Pessoa } from '../schemas'
;
import {
    resultNotFound,
    modelAddWhereTipoComparacao,
    modelAddWhereDataNascimento,
    modelAddWherePrimeiroNome,
    removerAcentos,
    resultFound,
    logErroBuscaBD
} from '../../utils';

import { MODEL_PRIORITY, API_CONFIG } from './../../config';

import { SIGLA_FONTE } from './../_fontes';

const fonte_dado = SIGLA_FONTE['receitanovo'];

const fonte  = MODEL_PRIORITY['receitanovo.pessoafisica'].fonte;
const rank   = MODEL_PRIORITY['receitanovo.pessoafisica'].rank;
const grupo  = MODEL_PRIORITY['receitanovo.pessoafisica'].grupo;

export let getPessoaSimplificadoNome_ReceitaNovo_PessoaFisica = function (nome: string,
                                                                        tipoComparacao='fulltext',
                                                                        dataNascimentoFilho=undefined,
                                                                        dataComparacao = '<',
                                                                        travaPrimeiroNome=false){

    let condicao = modelAddWhereTipoComparacao('Nome', '@nome_parameter', tipoComparacao);
    let c1 = modelAddWhereDataNascimento('DataNascimento', '@dt_nasc_filho_parameter', dataNascimentoFilho, dataComparacao);
    let c2 = modelAddWherePrimeiroNome('Nome', '@primeiro_nome_parameter', travaPrimeiroNome);
    condicao += (c1) ? c1 : '';
    condicao += (c2) ? c2 : '';

    let primeiroNome = removerAcentos(nome.split(" ")[0]);

    return connection.getConn().request()
        .input('nome_parameter', mssql.VarChar, nome)
        .input('primeiro_nome_parameter', mssql.VarChar, primeiroNome + ' %')
        .input('dt_nasc_filho_parameter', mssql.Date, dataNascimentoFilho)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        CPF as cpf,
                        UPPER(Nome) as nome,
                        UPPER(NomeMae) as nomeMae,
                        UPPER(Municipio) as municipio,
                        UPPER(UF) as uf,
                        UPPER(Sexo.Descricao) as sexo,
                        DataNascimento AS dataNascimento,
                        '${fonte_dado}' as fonte

                FROM ReceitaNovo.dbo.PessoaFisica
                    INNER JOIN ReceitaNovo.dbo.Sexo ON (PessoaFisica.Sexo = Sexo.SexoID)
                ${condicao}
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar a pessoa ${nome}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pela pessoa ${nome}.`, 'getPessoaSimplificadoNome_ReceitaNovo_PessoaFisica') });

};

export let getPessoaNomeMae_ReceitaNovo_PessoaFisica = function (nomeMae: string,
                                                                tipoComparacao = 'fulltext',
                                                                dataNascimento = undefined,
                                                                dataComparacao = '<'){

    let condicao = modelAddWhereTipoComparacao('NomeMae', '@nomeMae', tipoComparacao);
    let c1 = modelAddWhereDataNascimento('DataNascimento', '@dataNascimento', dataNascimento, dataComparacao);
    condicao += (c1) ? c1 : '';

    return connection.getConn().request()
        .input('nomeMae', mssql.VarChar, nomeMae)
        .input('dataNascimento', mssql.Date, dataNascimento)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        CPF as cpf,
                        UPPER(Nome) as nome,
                        UPPER(NomeMae) as nomeMae,
                        UPPER(Municipio) as municipio,
                        UPPER(UF) as uf,
                        UPPER(Sexo.Descricao) as sexo,
                        DataNascimento AS dataNascimento,
                        '${fonte_dado}' as fonte

                FROM ReceitaNovo.dbo.PessoaFisica
                    INNER JOIN ReceitaNovo.dbo.Sexo ON (PessoaFisica.Sexo = Sexo.SexoID)
                ${condicao}
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar a pessoa ${nomeMae}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pela pessoa ${nomeMae}.`, 'getPessoaNomeMae_ReceitaNovo_PessoaFisica'); });
}

export let getPessoaDetalhadoCPF_ReceitaNovo_PessoaFisica = function (cpf: string){

    return connection.getConn().request()
        .input('cpf_parameter', mssql.Char(11), cpf)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        CPF as cpf,
                        UPPER(Nome) as nome,
                        UPPER(NomeMae) as nomeMae,
                        UPPER(Municipio) as municipio,
                        UPPER(UF) as uf,
                        UPPER(Sexo.Descricao) as sexo,
                        UPPER(NaturezaOcupacao.Descricao) as naturezaOcupacao,
                        AnoObito as anoObito,
                        DataNascimento AS dataNascimento,
                        CASE
                            WHEN ExercicioOcupacao = '0000' THEN NULL
                            ELSE ExercicioOcupacao
                        END as anoExercicioOcupacao,
                        UPPER(OcupacaoPrincipal.Descricao) as ocupacaoPrincipal,
                        '${fonte_dado}' as fonte

                FROM ReceitaNovo.dbo.PessoaFisica
                    INNER JOIN ReceitaNovo.dbo.Sexo ON (PessoaFisica.Sexo = Sexo.SexoID)
                    LEFT OUTER JOIN ReceitaNovo.dbo.NaturezaOcupacao ON (PessoaFisica.NaturezaOcupacao = NaturezaOcupacao.NaturezaOcupacaoID)
                    LEFT OUTER JOIN ReceitaNovo.dbo.OcupacaoPrincipal ON (PessoaFisica.OcupacaoPrincipal = OcupacaoPrincipal.OcupacaoPrincipalID)
                WHERE CPF=@cpf_parameter
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar a pessoa ${cpf}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pela pessoa ${cpf}.`, 'getPessoaDetalhadoCPF_ReceitaNovo_PessoaFisica'); });
};

export let getPessoaSimplificadoCPF_ReceitaNovo_PessoaFisica = function (cpf: string){

    return connection.getConn().request()
        .input('cpf_parameter', mssql.Char(11), cpf)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        CPF as cpf,
                        UPPER(Nome) as nome,
                        UPPER(NomeMae) as nomeMae,
                        UPPER(Municipio) as municipio,
                        UPPER(UF) as uf,
                        UPPER(Sexo.Descricao) as sexo,
                        DataNascimento AS dataNascimento,
                        '${fonte_dado}' as fonte

                FROM ReceitaNovo.dbo.PessoaFisica
                    INNER JOIN ReceitaNovo.dbo.Sexo ON (PessoaFisica.Sexo = Sexo.SexoID)
                WHERE CPF=@cpf_parameter
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar a pessoa ${cpf}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pela pessoa ${cpf}.`, 'getPessoaSimplificadoCPF_ReceitaNovo_PessoaFisica'); });
};

export let getPessoaSimplificadoLogradouro_ReceitaNovo_PessoaFisica = function (logradouro: string){

    return connection.getConn().request()
        .input('LOGRADOURO', mssql.VarChar, logradouro)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        CPF as cpf,
                        UPPER(Nome) as nome,
                        UPPER(NomeMae) as nomeMae,
                        UPPER(Municipio) as municipio,
                        UPPER(UF) as uf,
                        UPPER(Sexo.Descricao) as sexo,
                        DataNascimento AS dataNascimento,
                        '${fonte_dado}' as fonte

                FROM ReceitaNovo.dbo.PessoaFisica
                    INNER JOIN ReceitaNovo.dbo.Sexo ON (PessoaFisica.Sexo = Sexo.SexoID)
                WHERE CONTAINS(Logradouro, @LOGRADOURO)
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar a pessoa de endereco ${logradouro}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pela pessoa de endereco ${logradouro}.`, 'getPessoaSimplificadoLogradouro_ReceitaNovo_PessoaFisica'); });
};

// export let getPessoaTelefone_ReceitaFull_PF = function (telefone: string){
//     return connection.getConn().request()
//         .input('telefone_parameter', mssql.NVarChar, telefone)
//         .query(`SELECT  CPF as cpf, CONCAT(DDD,Telefone) as telefone, '${fonte_dado}' as fonte

//                 FROM Receita_Full.Receita.PF
//                 WHERE Telefone=@telefone_parameter`)
//         .then(result => {
//             if(result.recordset.length === 0)
//                 return resultNotFound(`Nao foi possivel encontrar a pessoa com telefone ${telefone}.`, fonte);
//             else
//                 return resultFound(result, fonte, rank, grupo);
//         }).catch( error => { return logErroBuscaBD(error, `Falha na busca pela pessoa com telefone ${telefone}.`, 'getPessoaTelefone_ReceitaFull_PF'); });
// }