import * as mssql from 'mssql';

import { connection }  from '../../services/db.service';
import {
    resultNotFound,
    resultFound,
    logErroBuscaBD
} from '../../utils';
import { MODEL_PRIORITY, API_CONFIG } from './../../config';

import { SIGLA_FONTE } from './../_fontes';

const fonte_dado = SIGLA_FONTE['renach_2014'];

const fonte  = MODEL_PRIORITY['renach.pessoa'].fonte;
const rank   = MODEL_PRIORITY['renach.pessoa'].rank;
const grupo  = MODEL_PRIORITY['renach.pessoa'].grupo;

export let getPessoaDetalhadoCPF_Renach_2016_08 = function (cpf: string){

    return connection.getConn().request()
        .input('cpf_parameter', mssql.Char(11), cpf)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                    nr_cpf as cpf
                    ,UPPER(nm_condutor) as nome
                    ,CASE
                        WHEN isdate(dt_nascimento) = 1 THEN cast(dt_nascimento as date)
                        ELSE NULL
                    END as dataNascimento
                    ,UPPER(ds_local_nascimento) as naturalidade
                    ,CASE
                        WHEN cd_sexo = 1 THEN 'MASCULINO'
                        WHEN cd_sexo = 2 THEN 'FEMININO'
                        ELSE 'OUTROS'
                    END as sexo
                    ,CASE
                        WHEN isdate(dt_cadastramento) = 1 THEN cast(dt_cadastramento as date)
                        ELSE NULL
                    END as dataCadastro
                    ,UPPER(nm_mae) as nomeMae
                    ,UPPER(nm_pai) as nomePai
                    ,nr_registro as cnh
                    ,nr_documento as rg
                    ,ds_orgao_emissor as orgEmissorRg
                    ,sg_uf_documento as ufOrgEmissorRG
                    ,'${fonte_dado}' as fonte

                FROM Renach_2014.dbo.renach_2016_08
                WHERE nr_cpf=@cpf_parameter
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar a pessoa ${cpf}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pela pessoa ${cpf}.`, 'getPessoaDetalhadoCPF_Renach_2016_08'); });
};