import * as mssql from 'mssql';

import { connection }  from '../../services/db.service';
import {
    resultNotFound,
    resultFound,
    modelAddWhereTipoComparacao,
    modelAddWhereDataNascimento,
    logErroBuscaBD
} from '../../utils';
import { MODEL_PRIORITY, API_CONFIG } from './../../config';


import { SIGLA_FONTE } from './../_fontes';

const fonte_dado = SIGLA_FONTE['sispesquisa.cnh'];

const fonteCNH  = MODEL_PRIORITY['sispesquisa.cnh'].fonte;
const rankCNH   = MODEL_PRIORITY['sispesquisa.cnh'].rank;
const grupoCNH  = MODEL_PRIORITY['sispesquisa.cnh'].grupo;

export let getPessoaSimplificadoCNH_Sispesquisa_CNH = function (cnh : string){
    return connection.getConn().request()
        .input('cnh_parameter', mssql.Char(11), cnh)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                    CNH.CPF as cpf,
                    UPPER(CNH.Nome) as nome,
                    UPPER(Nome_da_Mae) as nomeMae,
                    UPPER(Nome_do_Pai) as nomePai,
                    Identidade as rg,
                    Orgao_Emissor_Identidade as orgEmissor,
                    UF_Emissao_Identidade as ufOrgEmissor,
                    Renach as renach,
                    CNH as cnh,
                    UPPER(PF.Municipio) as municipio,
                    UPPER(PF.UF) as uf,
                    PF.DataNascimento as dataNascimento,
                    '${fonte_dado}' as fonte

                FROM SISPESQUISA.dbo.CNH
                    LEFT OUTER JOIN ReceitaNovo.dbo.PessoaFisica AS PF ON (PF.CPF = CNH.CPF )
                WHERE cnh=@cnh_parameter
                ORDER BY Data_Exame_Medico DESC
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar a pessoa ${cnh}.`, fonteCNH);
            else
                return resultFound(result, fonteCNH, rankCNH, grupoCNH);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pela pessoa ${cnh}.`, 'getPessoaSimplificadoCNH_Sispesquisa_CNH'); });
}

export let getPessoaDetalhadoCPF_Sispesquisa_CNH = function (cpf: string){
    return connection.getConn().request()
        .input('cpf_parameter', mssql.Char(11), cpf)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        CPF as cpf, Nome as nome, Nome_do_Pai as nomePai, Nome_da_Mae as nomeMae, '${fonte_dado}' as fonte,
                        Identidade as rg, Orgao_Emissor_Identidade as orgEmissorRg, UF_Emissao_Identidade as ufOrgEmissorRG,
                        Renach as renach, CNH as cnh, Categoria_Atual as catAtual, Resultado_Exame_Psicotecnico as resPsicotecnico,
                        Data_Exame_Psicotecnico as dtPsicotecnico, Resultado_Exame_Medico as resMedico, Data_Exame_Medico as dtMedico

                FROM SISPESQUISA.dbo.CNH
                WHERE CPF=@cpf_parameter
                ORDER BY Data_Exame_Medico DESC
                `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar a pessoa ${cpf}.`, fonteCNH);
            else
                return resultFound(result, fonteCNH, rankCNH, grupoCNH);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pela pessoa ${cpf}.`, 'getPessoaDetalhadoCPF_Sispesquisa_CNH'); });
}

export let getPessoaSimplificadoCPF_Sispesquisa_CNH = function (cpf : string){
    return connection.getConn().request()
        .input('cpf_parameter', mssql.Char(11), cpf)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        CNH.CPF as cpf, CNH.Nome as nome, Nome_da_Mae as nomeMae, Nome_do_Pai as nomePai, Identidade as rg,  '${fonte_dado}' as fonte,
                        Orgao_Emissor_Identidade as orgEmissor, UF_Emissao_Identidade as ufOrgEmissor, Renach as renach, CNH as cnh,
                        PF.Municipio as municipio, PF.UF as uf,
                        PF.DataNascimento as dataNascimento

                FROM SISPESQUISA.dbo.CNH
                    LEFT OUTER JOIN ReceitaNovo.dbo.PessoaFisica AS PF ON (PF.CPF = CNH.CPF )
                WHERE CNH.CPF=@cpf_parameter
                ORDER BY Data_Exame_Medico DESC
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar a pessoa ${cpf}.`, fonteCNH);
            else
                return resultFound(result, fonteCNH, rankCNH, grupoCNH);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pela pessoa ${cpf}.`, 'getPessoaSimplificadoCPF_Sispesquisa_CNH'); });
}

export let getPessoaSimplificadoRG_Sispesquisa_CNH = function (rg: string){
    return connection.getConn().request()
        .input('rg_parameter', mssql.VarChar, rg)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        CNH.CPF as cpf, CNH.Nome as nome, Nome_da_Mae as nomeMae, Nome_do_Pai as nomePai, Identidade as rg, '${fonte_dado}' as fonte,
                        Orgao_Emissor_Identidade as orgEmissor, UF_Emissao_Identidade as ufOrgEmissor, Renach as renach, CNH as cnh,
                        PF.Municipio as municipio, PF.UF as uf,
                        PF.DataNascimento as dataNascimento

                FROM SISPESQUISA.dbo.CNH
                    LEFT OUTER JOIN ReceitaNovo.dbo.PessoaFisica AS PF ON (PF.CPF = CNH.CPF )
                WHERE Identidade=@rg_parameter
                ORDER BY Data_Exame_Medico DESC
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar a pessoa ${rg}.`, fonteCNH);
            else
                return resultFound(result, fonteCNH, rankCNH, grupoCNH);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pela pessoa ${rg}.`, 'getPessoaSimplificadoRG_Sispesquisa_CNH'); });
}

export let getPessoaSimplificadoNome_Sispesquisa_CNH = function (nome: String){
    return connection.getConn().request()
        .input('nome_parameter', mssql.VarChar, nome)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        CNH.CPF as cpf, CNH.Nome as nome, Nome_da_Mae as nomeMae, Nome_do_Pai as nomePai, Identidade as rg, '${fonte_dado}' as fonte,
                        PF.Municipio as municipio, PF.UF as uf,
                        PF.DataNascimento as dataNascimento

                FROM SISPESQUISA.dbo.CNH
                    LEFT OUTER JOIN ReceitaNovo.dbo.PessoaFisica AS PF ON (PF.CPF = CNH.CPF )
                WHERE CNH.Nome=@nome_parameter
                ORDER BY Data_Exame_Medico DESC
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar a pessoa ${nome}.`, fonteCNH);
            else
                return resultFound(result, fonteCNH, rankCNH, grupoCNH);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pela pessoa ${nome}.`, 'getPessoaSimplificadoNome_Sispesquisa_CNH'); });
};

export let getPessoaNomePai_Sispesquisa_CNH = function (nomePai: string,
                                                        tipoComparacao='fulltext',
                                                        dataNascimento=undefined,
                                                        dataComparacao = '<'){

    let condicao = modelAddWhereTipoComparacao('Nome_do_Pai', '@nomePai', tipoComparacao);
    let c1 = modelAddWhereDataNascimento('PF.DataNascimento', '@dataNascimento', dataNascimento, dataComparacao);
    condicao += (c1) ? c1 : '';

    return connection.getConn().request()
            .input('nomePai', mssql.VarChar, nomePai)
            .input('dataNascimento', mssql.Date, dataNascimento)
            .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                            CNH.CPF as cpf, CNH.Nome as nome, Nome_da_Mae as nomeMae, '${fonte_dado}' as fonte,
                            Nome_do_Pai as nomePai,
                            PF.Municipio as municipio, PF.UF as uf,
                            PF.DataNascimento as dataNascimento

                    FROM SISPESQUISA.dbo.CNH
                        LEFT OUTER JOIN ReceitaNovo.dbo.PessoaFisica AS PF ON (PF.CPF = CNH.CPF )
                    ${condicao}
                    ORDER BY Data_Exame_Medico DESC
            `)
            .then(result => {
                if(result.recordset.length === 0)
                    return resultNotFound(`Nao foi possivel encontrar a pessoa ${nomePai}.`, fonteCNH);
                else
                    return resultFound(result, fonteCNH, rankCNH, grupoCNH);
            }).catch( error => { return logErroBuscaBD(error, `Falha na busca pela pessoa ${nomePai}.`, 'getPessoaNomePai_Sispesquisa_CNH'); });
}

export let getPessoaNomeMae_Sispesquisa_CNH = function (nomeMae: string, tipoComparacao='fulltext'){
    let condicao = modelAddWhereTipoComparacao('Nome_da_Mae', '@nome_mae_parameter', tipoComparacao);

    return connection.getConn().request()
        .input('nome_mae_parameter', mssql.VarChar, nomeMae)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        CNH.CPF as cpf, CNH.Nome as nome, Nome_da_Mae as nomeMae, '${fonte_dado}' as fonte,
                        Nome_do_Pai as nomePai,
                        PF.Municipio as municipio, PF.UF as uf,
                        PF.DataNascimento as dataNascimento

                FROM SISPESQUISA.dbo.CNH
                    LEFT OUTER JOIN ReceitaNovo.dbo.PessoaFisica AS PF ON (PF.CPF = CNH.CPF )
                ${condicao}
                ORDER BY Data_Exame_Medico DESC
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar a pessoa ${nomeMae}.`, fonteCNH);
            else
                return resultFound(result, fonteCNH, rankCNH, grupoCNH);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pela pessoa ${nomeMae}.`, 'getPessoaNomeMae_Sispesquisa_CNH'); });
}
