import * as mssql from 'mssql';

import { connection }  from '../../services/db.service';
import { Pessoa } from '../schemas';
import { trataResultado, resultNotFound, resultFound, logErroBuscaBD } from '../../utils';
import { MODEL_PRIORITY, API_CONFIG } from './../../config';
import { SIGLA_FONTE } from '../_fontes';

const fonte_dado = SIGLA_FONTE['receitanovo'];

const fontePessoas = MODEL_PRIORITY['sispesquisa.emails.pessoa'].fonte;
const rankPessoas  = MODEL_PRIORITY['sispesquisa.emails.pessoa'].rank;
const grupoPessoas = MODEL_PRIORITY['sispesquisa.emails.pessoa'].grupo;

export let getPessoaEmail_Sispesquisa_Emails = function (email: string){

    return connection.getConn().request()
        .input('EMAIL', mssql.VarChar, email)
        .query(`
            SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                CPF as cpf,
                UPPER(Nome) as nome,
                UPPER(NomeMae) as nomeMae,
                UPPER(Municipio) as municipio,
                UPPER(UF) as uf,
                UPPER(Sexo.Descricao) as sexo,
                DataNascimento AS dataNascimento,
                '${fonte_dado}' as fonte

            FROM SISPESQUISA.sispesquisa.emails E
                INNER JOIN ReceitaNovo.dbo.PessoaFisica PF ON (E.cpf_cnpj = PF.CPF)
                INNER JOIN ReceitaNovo.dbo.Sexo ON (PF.Sexo = Sexo.SexoID)
            WHERE EMAIL=@EMAIL
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar a pessoa com email ${email}.`, fontePessoas);
            else
                return resultFound(result, fontePessoas, rankPessoas, grupoPessoas);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pela pessoa ${email}.`, 'getPessoaEmail_Sispesquisa_Emails'); });
};