import * as mssql from 'mssql';

import { connection } from '../../services/db.service';
import { Pessoa } from '../schemas';
import { MODEL_PRIORITY, API_CONFIG } from './../../config';

import {
    trataResultado,
    resultNotFound,
    resultFound,
    modelAddWhereTipoComparacao,
    modelAddWherePrimeiroNome,
    logErroBuscaBD
} from '../../utils';

import { SIGLA_FONTE } from './../_fontes';

const fonte_dado = SIGLA_FONTE['sispesquisa.prisional'];

const fontePrisional   = MODEL_PRIORITY['sispesquisa.prisional.pessoa'].fonte;
const rankPrisional    = MODEL_PRIORITY['sispesquisa.prisional.pessoa'].rank;
const grupoPrisional   = MODEL_PRIORITY['sispesquisa.prisional.pessoa'].grupo;

export let getPessoaDetalhadoCPF_Sispesquisa_Prisional = function (cpf: string){

    return connection.getConn().request()
        .input('cpf_parameter', mssql.NChar(11), cpf)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        PF.cpf, nome_prisional as nome, pai as nomePai, mae as nomeMae, '${fonte_dado}' as fonte,
                        PF.Municipio as municipio, PF.UF as uf, PF.DataNascimento AS dataNascimento

                FROM SISPESQUISA.sispesquisa.prisional
                    LEFT OUTER JOIN ReceitaNovo.dbo.PessoaFisica AS PF ON (PF.CPF = prisional.CPF)
                WHERE prisional.cpf=@cpf_parameter
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar a pessoa ${cpf}.`, fontePrisional);
            else
                return resultFound(result, fontePrisional, rankPrisional, grupoPrisional);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pela pessoa ${cpf}.`, 'getPessoaDetalhadoCPF_Sispesquisa_Prisional'); });
};

export let getPessoaSimplificadoCPF_Sispesquisa_Prisional = function (cpf: string){

    return connection.getConn().request()
        .input('cpf_parameter', mssql.NChar(11), cpf)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        PF.cpf, nome_prisional as nome, pai as nomePai, mae as nomeMae, '${fonte_dado}' as fonte,
                        PF.Municipio as municipio, PF.UF as uf, PF.DataNascimento AS dataNascimento

                FROM SISPESQUISA.sispesquisa.prisional
                    LEFT OUTER JOIN ReceitaNovo.dbo.PessoaFisica AS PF ON (PF.CPF = prisional.CPF)
                WHERE prisional.cpf=@cpf_parameter
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar a pessoa ${cpf}.`, fontePrisional);
            else
                return resultFound(result, fontePrisional, rankPrisional, grupoPrisional);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pela pessoa ${cpf}.`, 'getPessoaSimplificadoCPF_Sispesquisa_Prisional'); });
};

export let getPessoaSimplificadoNome_Sispesquisa_Prisional = function (nome: string){

    return connection.getConn().request()
        .input('nome_parameter', mssql.NVarChar, nome)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        PF.cpf, nome_prisional as nome, pai as nomePai, mae as nomeMae, '${fonte_dado}' as fonte,
                        PF.Municipio as municipio, PF.UF as uf, PF.DataNascimento AS dataNascimento

                FROM SISPESQUISA.sispesquisa.prisional
                    LEFT OUTER JOIN ReceitaNovo.dbo.PessoaFisica AS PF ON (PF.CPF = prisional.CPF)
                WHERE CONTAINS(nome_prisional, @nome_parameter)
            `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar a pessoa ${nome}.`, fontePrisional);
            else
                return resultFound(result, fontePrisional);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pela pessoa ${nome}.`, 'getPessoaSimplificadoNome_Sispesquisa_Prisional'); });
};

export let getPessoaNomeMae_Sispesquisa_Prisional = function (nomeMae: string, tipoComparacao='fulltext'){
    let condicao = modelAddWhereTipoComparacao('mae', '@nome_mae_parameter', tipoComparacao);

    return connection.getConn().request()
        .input('nome_mae_parameter', mssql.NVarChar, nomeMae)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        PF.cpf, nome_prisional as nome, pai as nomePai, mae as nomeMae, '${fonte_dado}' as fonte,
                        PF.Municipio as municipio, PF.UF as uf, PF.DataNascimento AS dataNascimento

                FROM SISPESQUISA.sispesquisa.prisional
                    LEFT OUTER JOIN ReceitaNovo.dbo.PessoaFisica AS PF ON (PF.CPF = prisional.CPF)
                ${condicao}
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar a pessoa ${nomeMae}.`, fontePrisional);
            else
                return resultFound(result, fontePrisional, rankPrisional, grupoPrisional);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pela pessoa ${nomeMae}.`, 'getPessoaNomeMae_Sispesquisa_Prisional'); });
}

export let getPessoaNomePai_Sispesquisa_Prisional = function (nomePai: string, tipoComparacao='fulltext'){
    let condicao = modelAddWhereTipoComparacao('pai', '@nome_pai_parameter', tipoComparacao);

    return connection.getConn().request()
        .input('nome_pai_parameter', mssql.NVarChar, nomePai)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        PF.cpf, nome_prisional as nome, pai as nomePai, mae as nomeMae, '${fonte_dado}' as fonte,
                        PF.Municipio as municipio, PF.UF as uf, PF.DataNascimento AS dataNascimento

                FROM SISPESQUISA.sispesquisa.prisional
                    LEFT OUTER JOIN ReceitaNovo.dbo.PessoaFisica AS PF ON (PF.CPF = prisional.CPF)
                ${condicao}
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar a pessoa ${nomePai}.`, fontePrisional);
            else
                return resultFound(result, fontePrisional, rankPrisional, grupoPrisional);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pela pessoa ${nomePai}.`, 'getPessoaNomePai_Sispesquisa_Prisional'); });
}