import * as mssql from 'mssql';

import { connection }  from '../../services/db.service';
import { Pessoa } from '../schemas';
import { trataResultado, resultNotFound, resultFound, logErroBuscaBD } from '../../utils';
import { MODEL_PRIORITY, API_CONFIG } from './../../config';

const fontePessoas   = MODEL_PRIORITY['sispesquisa.telefones.pessoa'].fonte;
const rankPessoas    = MODEL_PRIORITY['sispesquisa.telefones.pessoa'].rank;
const grupoPessoas   = MODEL_PRIORITY['sispesquisa.telefones.pessoa'].grupo;

export let getPessoaTelefone_Sispesquisa_Telefones = function (telefone: string){

    return connection.getConn().request()
        .input('telefone_parameter', mssql.NVarChar, telefone)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                    cpf_cnpj as cpf, telefone, fonte

                FROM SISPESQUISA.sispesquisa.telefones
                WHERE telefone=@telefone_parameter
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar a pessoa com telefone ${telefone}.`, fontePessoas);
            else
                return resultFound(result, fontePessoas, rankPessoas, grupoPessoas);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pela pessoa ${telefone}.`, 'getPessoaTelefone_Sispesquisa_Telefones'); });
};