import * as mssql from 'mssql';

import { connection }  from '../../services/db.service';
import { Pessoa } from '../schemas';
import {
    resultNotFound,
    modelAddWhereTipoComparacao,
    modelAddWhereDataNascimento,
    modelAddWherePrimeiroNome,
    removerAcentos,
    resultFound,
    logErroBuscaBD
} from '../../utils';

import { MODEL_PRIORITY, API_CONFIG } from './../../config';

import { SIGLA_FONTE } from './../_fontes';

const fonte_dado = SIGLA_FONTE['vep'];

const fonte  = MODEL_PRIORITY['vep.pessoa'].fonte;
const rank   = MODEL_PRIORITY['vep.pessoa'].rank;
const grupo  = MODEL_PRIORITY['vep.pessoa'].grupo;

export let getPessoaDetalhadoCPF_VEP = function (cpf: string){

    return connection.getConn().request()
        .input('cpf_parameter', mssql.VarChar(11), cpf)
        .query(`
            SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                UPPER(P.DS_PESSOA) as nome
                ,PF.DT_NASCIMENTO as dataNascimento
                ,UPPER(PF.PAI) as nomePai
                ,UPPER(PF.MAE) as nomeMae
                --,CASE
                --    WHEN PF.FK_SEXO = 21 THEN 'MASCULINO'
                --    WHEN PF.FK_SEXO = 42 THEN 'FEMININO'
                --    ELSE 'OUTRO'
                --END AS sexo
                ,UPPER(PF.CONJUGE) as conjuge
                ,PF.IDENTIDADE_NUM as rg
                ,UPPER(PF.IDENTIDADE_ORGAO) as orgEmissorRg
                ,UPPER(UF_IDENT.DS_SIGLA) AS ufOrgEmissorRG
                ,PF.CPF as cpf
                ,PF.RESERVISTA_NUM as reservista
                ,PF.RESERVISTA_CIRC as reservistaCirc
                ,PF.RESERVISTA_SERIE as reservistaSerie
                ,PF.TITULO_NUM as tituloEleitor
                ,PF.TITULO_SECAO as tituloSecao
                ,PF.TITULO_ZONA as tituloZona
                ,PF.CART_PROF_NUM as carteiraProfNum
                ,PF.CART_PROF_SERIE as carteiraProfSerie
                ,UF_CART_TRAB.DS_UF as carteiraProfUf
                ,UPPER(C.DS_CIDADE) AS naturalidade
                ,UPPER(ALC.DS_ALCUNHA) as vulgo
                ,'${fonte_dado}' as fonte

            FROM VEP.dbo.VEP_CNJ_TB_PESSOAFISICA PF
                LEFT OUTER JOIN VEP.dbo.VEP_CNJ_TB_PESSOA P ON (P.ID_PESSOA = PF.FK_PESSOA)
                LEFT OUTER JOIN VEP.dbo.VEP_CNJ_TB_UF UF_IDENT ON (UF_IDENT.ID_UF = PF.FK_IDENTIDADE_UF)
                LEFT OUTER JOIN VEP.dbo.VEP_CNJ_TB_CIDADE C ON (C.ID_CIDADE = PF.FK_NATURALIDADE)
                LEFT OUTER JOIN VEP.dbo.VEP_CNJ_TB_UF UF_CART_TRAB ON (UF_CART_TRAB.ID_UF = PF.FK_CART_PROF_UF)
                OUTER APPLY(
                    SELECT STUFF((
                        SELECT ', ' + ALC2.DS_ALCUNHA
                        FROM VEP.dbo.VEP_CNJ_TB_ALCUNHA ALC2
                            INNER JOIN VEP.dbo.VEP_CNJ_TB_PARTE PT2 ON (PT2.ID_PARTE = ALC2.FK_PARTE)
                        WHERE PT2.FK_PESSOAFISICA = PF.ID_PESSOAFISICA
                        FOR XML PATH('')
                    ), 1, 2, '') AS DS_ALCUNHA
                ) ALC
             WHERE CPF = @cpf_parameter
                AND  NOT EXISTS(
                SELECT 1
                FROM VEP.dbo.VEP_CNJ_TB_PARTEPROCESSOADV PPA
                INNER JOIN VEP.dbo.VEP_CNJ_TB_PARTEPROCESSO PRO ON (PPA.FK_PARTEPROCESSO = PRO.ID_PARTEPROCESSO)
                INNER JOIN VEP.dbo.VEP_CNJ_TB_PARTE P ON (P.ID_PARTE = PRO.FK_PARTE)
                WHERE (PF.ID_PESSOAFISICA = P.FK_PESSOAFISICA)
            )
        `)
        .then(result => {
            if (result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar a pessoa ${cpf}.`, fonte);
            else {
                return resultFound(result, fonte, rank, grupo);
            }
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pela pessoa ${cpf}.`, 'getPessoaDetalhadoCPF_VEP'); });
};