import * as mssql from 'mssql';

import { connection }  from './../../services/db.service';
import { MODEL_PRIORITY, API_CONFIG } from './../../config';
import {
    trataResultado,
    resultNotFound,
    resultFound,
    logErroBuscaBD
} from './../../utils';


import { SIGLA_FONTE } from './../_fontes';

const fonte_dado = SIGLA_FONTE['sds'];

const fonte = MODEL_PRIORITY['sispesquisa.dbo.tbl_prontuarios'].fonte;
const rank  = MODEL_PRIORITY['sispesquisa.dbo.tbl_prontuarios'].rank;
const grupo = MODEL_PRIORITY['sispesquisa.dbo.tbl_prontuarios'].grupo;

export let getPresoSimplificadoNomeMae_SDS_Prisional = function (nomeMae: string){

    return connection.getConn().request()
        .input('nomeMae', mssql.VarChar, nomeMae)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        cpf, UPPER(nome) as nome, UPPER(mae) as nomeMae, UPPER(vulgo) as vulgo, rg,
                        UPPER(principal_atividade_criminosa) as principalAtividade, '${fonte_dado}' as fonte

                FROM SISPESQUISA.dbo.tbl_prontuarios
                WHERE CONTAINS(mae, @nomeMae)
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o preso com mãe ${nomeMae}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pela pessoa ${nomeMae}.`, 'getPresoSimplificadoNomeMae_SDS_Prisional'); });
}

export let getPresoSimplificadoNome_SDS_Prisional = function (nome: string){

    return connection.getConn().request()
        .input('nome', mssql.VarChar, nome)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        cpf, UPPER(nome) as nome, UPPER(mae) as nomeMae, UPPER(vulgo) as vulgo, rg,
                        UPPER(principal_atividade_criminosa) as principalAtividade, '${fonte_dado}' as fonte

                FROM SISPESQUISA.dbo.tbl_prontuarios
                WHERE CONTAINS(nome, @nome)
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o preso com nome ${nome}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pela pessoa ${nome}.`, 'getPresoSimplificadoNome_SDS_Prisional'); });
}

export let getPresoSimplificadoVulgo_SDS_Prisional = function (vulgo: string){

    return connection.getConn().request()
        .input('vulgo', mssql.VarChar, '%' + vulgo + '%')
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        cpf, UPPER(nome) as nome, UPPER(mae) as nomeMae, UPPER(vulgo) as vulgo, rg,
                        UPPER(principal_atividade_criminosa) as principalAtividade, '${fonte_dado}' as fonte

                FROM SISPESQUISA.dbo.tbl_prontuarios
                WHERE vulgo LIKE @vulgo
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o preso com vulgo ${vulgo}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pela pessoa ${vulgo}.`, 'getPresoSimplificadoVulgo_SDS_Prisional'); });
}

export let getPresoSimplificadoCPF_SDS_Prisional = function (cpf: string) {

    return connection.getConn().request()
        .input('cpf', mssql.VarChar, cpf)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        cpf, UPPER(nome) as nome, UPPER(mae) as nomeMae, UPPER(vulgo) as vulgo, rg,
                        UPPER(principal_atividade_criminosa) as principalAtividade, '${fonte_dado}' as fonte

                FROM SISPESQUISA.dbo.tbl_prontuarios
                WHERE cpf=@cpf
        `)
        .then(result => {
            if (result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o preso com cpf ${cpf}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pela pessoa ${cpf}.`, 'getPresoSimplificadoCPF_SDS_Prisional'); });
}

export let getPresoDetalhadoCPF_SDS_Prisional = function (cpf: string) {

    return connection.getConn().request()
        .input('cpf', mssql.VarChar, cpf)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        cpf, UPPER(nome) as nome, UPPER(mae) as nomeMae, rg, UPPER(orgao_exp) as orgEmissor, UPPER(pai) as nomePai,
                        cabelo, olhos, cutis, UPPER(vulgo) as vulgo, UPPER(comparsas) as comparsas, faccao, barba, cicatriz,
                        tatuagem, UPPER(principal_atividade_criminosa) as principalAtividade,
                        informacoes_adicionais_varchar1 as infoAdicional,
                        informacoes_adicionais_varchar2 as infoAdicional2,
                        '${fonte_dado}' as fonte

                FROM SISPESQUISA.dbo.tbl_prontuarios
                WHERE cpf=@cpf
        `)
        .then(result => {
            if (result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o preso com cpf ${cpf}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pela pessoa ${cpf}.`, 'getPresoDetalhadoCPF_SDS_Prisional'); });
}