import * as mssql from 'mssql';

import { connection }  from '../../services/db.service';
import { MODEL_PRIORITY, API_CONFIG } from '../../config';
import {
    trataResultado,
    resultNotFound,
    resultFound,
    logErroBuscaBD
} from '../../utils';


import { SIGLA_FONTE } from '../_fontes';

const fonte_dado = SIGLA_FONTE['sisdepen'];

const fonte = MODEL_PRIORITY['sisdepen'].fonte;
const rank  = MODEL_PRIORITY['sisdepen'].rank;
const grupo = MODEL_PRIORITY['sisdepen'].grupo;

export let getPresoSimplificadoNome_SISDEPEN = function (nome: string){

    return connection.getConn().request()
        .input('nome', mssql.VarChar, nome)
        .query(`
                SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        cpf, UPPER(nome) as nome, UPPER(alcunha) as vulgo, rg, cnc,
                        cadeia, situacao, '${fonte_dado}' as fonte

                FROM SISDEPEN.dbo.VW_SISDEPEN
                WHERE CONTAINS(nome, @nome)
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o preso com nome ${nome}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pela pessoa ${nome}.`, 'getPresoSimplificadoNome_SISDEPEN'); });
}

export let getPresoSimplificadoVulgo_SISDEPEN = function (vulgo: string){

    return connection.getConn().request()
        .input('vulgo', mssql.VarChar, '%' + vulgo + '%')
        .query(`
                SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        cpf, UPPER(nome) as nome, UPPER(alcunha) as vulgo, rg, cnc,
                        cadeia, situacao, '${fonte_dado}' as fonte

                FROM SISDEPEN.dbo.VW_SISDEPEN
                WHERE alcunha LIKE @vulgo
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o preso com vulgo ${vulgo}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pela pessoa ${vulgo}.`, 'getPresoSimplificadoVulgo_SISDEPEN'); });
}

export let getPresoSimplificadoCPF_SISDEPEN = function (cpf: string) {

    return connection.getConn().request()
        .input('cpf', mssql.VarChar, cpf)
        .query(`
                SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        cpf, UPPER(nome) as nome, UPPER(alcunha) as vulgo, rg, cnc,
                        cadeia, situacao, '${fonte_dado}' as fonte

                FROM SISDEPEN.dbo.VW_SISDEPEN
                WHERE cpf=@cpf
        `)
        .then(result => {
            if (result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o preso com cpf ${cpf}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pela pessoa ${cpf}.`, 'getPresoSimplificadoCPF_SISDEPEN'); });
}

export let getPresoSimplificadoCNC_SISDEPEN = function (cnc: string) {

    return connection.getConn().request()
        .input('CNC', mssql.VarChar, cnc)
        .query(`
                SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        cpf, UPPER(nome) as nome, UPPER(alcunha) as vulgo, rg, cnc,
                        cadeia, situacao, '${fonte_dado}' as fonte

                FROM SISDEPEN.dbo.VW_SISDEPEN
                WHERE cnc=@CNC
        `)
        .then(result => {
            if (result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o preso com cnc ${cnc}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pela pessoa ${cnc}.`, 'getPresoSimplificadoCNC_SISDEPEN'); });
}

export let getPresoDetalhadoCNC_SISDEPEN = function (cnc: string) {

    return connection.getConn().request()
        .input('CNC', mssql.VarChar, cnc)
        .query(`
                SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                    cpf,
                    UPPER(nome) as nome,
                    UPPER(nomeApresentacao) as nomeApresentacao,
                    UPPER(nomeSocial) as nomeSocial,
                    UPPER(alcunha) as vulgo,
                    rg,
                    rgOrgaoEmissor,
                    rgUf,
                    rgDataExpedicao,
                    nacionalidade,
                    naturalidade,
                    naturalidadeUF,
                    cnc,
                    cadeia,
                    cadeiaUF,
                    cadeiaAmbito,
                    tipificacao,
                    situacao,
                    tipoRecolhimento,
                    regimePrisional,
                    foto,
                    CAST(dataInformacao as date) as dataInformacao,
                    '${fonte_dado}' as fonte

                FROM SISDEPEN.dbo.VW_SISDEPEN
                WHERE cnc=@CNC
        `)
        .then(result => {
            if (result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o preso com cnc ${cnc}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pela pessoa ${cnc}.`, 'getPresoDetalhadoCNC_SISDEPEN'); });
}

export let getPresoDetalhadoCPF_SISDEPEN = function (cpf: string) {

    return connection.getConn().request()
        .input('cpf', mssql.VarChar, cpf)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        cpf, cnc, cadeia, cadeiaUF, cadeiaAmbito, situacao, tipoRecolhimento,
                        regimePrisional, nome, nomeApresentacao, nomeSocial, sexo, nacionalidade,
                        naturalidadeUF, naturalidade, escolaridade, rg, rgOrgaoEmissor, rgUf, rgDataExpedicao,
                        alcunha as vulgo, dataInformacao, '${fonte_dado}' as fonte

                FROM SISDEPEN.dbo.VW_SISDEPEN
                WHERE cpf=@cpf
        `)
        .then(result => {
            if (result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o preso com cpf ${cpf}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pela pessoa ${cpf}.`, 'getPresoDetalhadoCPF_SISDEPEN'); });
}