import * as mssql from 'mssql';

import { connection } from './../../services/db.service';
import { MODEL_PRIORITY, API_CONFIG } from './../../config';
import {
    trataResultado,
    resultNotFound,
    resultFound,
    logErroBuscaBD
} from './../../utils';


import { SIGLA_FONTE } from './../_fontes';

const fonte_dado = SIGLA_FONTE['sispesquisa.prisional'];

const fonte   = MODEL_PRIORITY['sispesquisa.prisional.preso'].fonte;
const rank    = MODEL_PRIORITY['sispesquisa.prisional.preso'].rank;
const grupo   = MODEL_PRIORITY['sispesquisa.prisional.preso'].grupo;

export let getPrisionalDetalhado = function (paramBusca: string, valor: string){

    return connection.getConn().request()
        .input('search_parameter', mssql.NVarChar, valor)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        cpf, nome_prisional as nome, pai as nomePai, mae as nomeMae,
                        dtEntrada as dataEntrada, prontuario, status, apelido_vulgo as vulgo,
                        '${fonte_dado}' as fonte, FACCAO as faccao, id_unidade_prisional as unidade_prisional

                FROM SISPESQUISA.sispesquisa.prisional
                WHERE ${paramBusca}=@search_parameter
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o preso ${valor}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pelo preso ${valor}.`, 'getPrisionalDetalhado'); });
};

let getPrisionalSimplificado = function (paramBusca: string, valor: string){

    return connection.getConn().request()
        .input('search_parameter', mssql.NVarChar, valor)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        cpf, nome_prisional as nome, dtEntrada as dataEntrada,
                        apelido_vulgo as vulgo, status, '${fonte_dado}' as fonte

                FROM SISPESQUISA.sispesquisa.prisional
                WHERE ${paramBusca}=@search_parameter
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o preso ${valor}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pelo preso ${valor}.`, 'getPrisionalSimplificado'); });
};

let getPrisionalSimplificadoLike = function (paramBusca: string, valor: string){

    return connection.getConn().request()
        .input('search_parameter', mssql.NVarChar, valor)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        cpf, nome_prisional as nome, dtEntrada as dataEntrada,
                        apelido_vulgo as vulgo, status, '${fonte_dado}' as fonte

                FROM SISPESQUISA.sispesquisa.prisional
                WHERE ${paramBusca} LIKE @search_parameter
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o preso ${valor}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);

        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pelo preso ${valor}.`, 'getPrisionalSimplificadoLike'); });
};

let getPrisionalSimplificadoFullText = function (paramBusca: string, valor: string){

    return connection.getConn().request()
        .input('search_parameter', mssql.NVarChar, valor)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        cpf, nome_prisional as nome, dtEntrada as dataEntrada,
                        apelido_vulgo as vulgo, status, '${fonte_dado}' as fonte

                FROM SISPESQUISA.sispesquisa.prisional
                WHERE CONTAINS(${paramBusca}, @search_parameter)
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o preso ${valor}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);

        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelo preso ${valor}.`, 'getPrisionalSimplificadoFullText'); });
};

export let getPresoDetalhadoCPF_Sispesquisa_Prisional = function (cpf: string){
    return getPrisionalDetalhado("cpf", cpf);
}

export let getPresoSimplificadoCPF_Sispesquisa_Prisional = function (cpf: string){
    return getPrisionalSimplificado("cpf", cpf);
};

export let getPresoSimplificadoVulgo_Sispesquisa_Prisional = function(vulgo: string){
    return getPrisionalSimplificadoLike("apelido_vulgo", '%' + vulgo + '%');
}

export let getPresoSimplificadoNome_Sispesquisa_Prisional = function (nome: string){
    return getPrisionalSimplificadoFullText("nome_prisional", nome);
};

export let getPresoSimplificadoNomeMae_Sispesquisa_Prisional = function (nomeMae: string){

    return connection.getConn().request()
        .input('nome_mae_parameter', mssql.NVarChar, nomeMae)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        cpf, nome_prisional as nome, dtEntrada as dataEntrada,
                        apelido_vulgo as vulgo, status, '${fonte_dado}' as fonte

                FROM SISPESQUISA.sispesquisa.prisional
                WHERE CONTAINS(mae, @nome_mae_parameter)
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o preso com mãe ${nomeMae}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pela pessoa ${nomeMae}.`, 'getPresoSimplificadoNomeMae_Sispesquisa_Prisional'); });
}
