import { Request } from 'express';
import * as mssql from 'mssql';

import { connection }  from '../services/db.service';
import {
    trataResultado,
    resultNotFound,
    resultFound,
    logErroBuscaBD
} from '../utils';
import {
    MODEL_PRIORITY,
    API_CODES
} from '../config';

const fonte  = MODEL_PRIORITY['sispesquisa.cadastro_usuario.preusuario'].fonte;
const rank   = MODEL_PRIORITY['sispesquisa.cadastro_usuario.preusuario'].rank;
const grupo  = MODEL_PRIORITY['sispesquisa.cadastro_usuario.preusuario'].grupo;

export let getPreUsuariosInativos_Sispesquisa_CadastroUsuario = function (){

    return connection.getConn().request()
        .query(`SELECT  id, nome, nacionalidade, cpf, identidade, org_emissor as orgEmissor, uf_org_emissor as ufOrgEmissor,
                        matricula, titularidade, promotoria, email, telefone, dt_cadastro as dataCadastro, ativado
                FROM SISPESQUISA.sispesquisa.cadastro_usuario
                WHERE ativado='N'
                ORDER BY dataCadastro`)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar pre usuarios.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch( error => {logErroBuscaBD(error, `Falha na busca pela lista de pre usuarios.`, 'getPreUsuariosInativos_Sispesquisa_CadastroUsuario')});
};

export let insertPreUsuarios_Sispesquisa_CadastroUsuario = function (dados, ativado='N'){

    return connection.getConn().request()
        .input('nome', mssql.NVarChar, dados.nome)
        .input('nacionalidade', mssql.NVarChar, dados.nacionalidade)
        .input('cpf', mssql.NVarChar, dados.cpf)
        .input('identidade', mssql.NVarChar, dados.identidade)
        .input('orgEmisssor', mssql.NVarChar, dados.orgEmissor)
        .input('ufOrgEmissor', mssql.NVarChar, dados.ufOrgEmissor)
        .input('matricula', mssql.NVarChar, dados.matricula)
        .input('titularidade', mssql.NVarChar, dados.titularidade)
        .input('promotoria', mssql.NVarChar, dados.promotoria)
        .input('email', mssql.NVarChar, dados.email)
        .input('telefone', mssql.NVarChar, dados.telefone)
        .input('dataCadastro', mssql.NVarChar, dados.dataCadastro)
        .input('ativado', mssql.NVarChar, ativado)
        .query(`INSERT INTO SISPESQUISA.sispesquisa.cadastro_usuario
                VALUES
                    (@nome, @nacionalidade, @cpf, @identidade, @orgEmisssor,
                    @ufOrgEmissor, @matricula, @titularidade, @promotoria, @email,
                    @telefone, @dataCadastro, @ativado)
        `)
        .then(result => {
            return {
                status: API_CODES.CODE_SUCESSO,
                msg: 'Usuário cadastrado com sucesso.'
            }
        }).catch( error => {logErroBuscaBD(error, `Falha no cadastro do usuario.`, 'insertPreUsuarios_Sispesquisa_CadastroUsuario')});

}
export let checkCPFPreUsuarios_Sispesquisa_CadastroUsuario = function (dados){

    return connection.getConn().request()
        .input('cpf', mssql.NVarChar, dados.cpf)
        .query(`SELECT id
                FROM SISPESQUISA.sispesquisa.cadastro_usuario
                WHERE CPF=@cpf
        `)
        .then(result => {
            if(result.recordset.length !== 0)
                return true;
            else
                return false;
        }).catch( error => {logErroBuscaBD(error, `Falha no busca por usuario.`, 'checkCPFPreUsuarios_Sispesquisa_CadastroUsuario')});

}

export let deletePreUsuario_Sispesquisa_CadastroUsuario = function (id: string){

    return connection.getConn().request()
        .input('id_parameter', mssql.NVarChar, id)
        .query(`DELETE FROM SISPESQUISA.sispesquisa.cadastro_usuario
                WHERE id=@id_parameter`)
    .then(result => {
        return {
            status: API_CODES.CODE_SUCESSO,
            msg: 'Usuário deletado com sucesso.'
        }
    }).catch( error => {logErroBuscaBD(error, `Falha na exclusão do usuário ${id}.`, 'deletePreUsuario_Sispesquisa_CadastroUsuario')});

}

export let ativarPreUsuario_Sispesquisa_CadastroUsuario = function(id: string){

    return connection.getConn().request()
        .input('id_parameter', mssql.NVarChar, id)
        .query(`UPDATE SISPESQUISA.sispesquisa.cadastro_usuario

                SET ativado='S'
                WHERE id=@id_parameter
        `)
        .then(result => {
            return {
                status: API_CODES.CODE_SUCESSO,
                msg: 'Usuário ativado com sucesso.'
            }
        }).catch( error => {
            logErroBuscaBD(error, `Falha na ativação do usuario.`, 'ativarPreUsuario_Sispesquisa_CadastroUsuario');
            throw error;
        });
}

export let desativarPreUsuario_Sispesquisa_CadastroUsuario = function(id: string){

    return connection.getConn().request()
        .input('id_parameter', mssql.NVarChar, id)
        .query(`UPDATE SISPESQUISA.sispesquisa.cadastro_usuario

                SET ativado='N'
                WHERE id=@id_parameter
        `)
        .then(result => {
            return {
                status: API_CODES.CODE_SUCESSO,
                msg: 'Usuário desativado com sucesso.'
            }
        }).catch( error => {
            logErroBuscaBD(error, `Falha na desativação do usuario.`, 'desativarPreUsuario_Sispesquisa_CadastroUsuario');
            throw error;
        });
}