import * as mssql from 'mssql';

import * as pessoa from './pessoa.model';

import { connection }  from '../services/db.service';
import { MODEL_PRIORITY, API_CONFIG } from '../config';
import {
    resultNotFound,
    resultFound,
    logErroBuscaBD,
    logErroBuscaWS,
    resultFoundRaw
} from '../utils';
import { getPessoaSimplificadoRG_Sispesquisa_CNH } from './pessoa/pessoa.model.sispesquisa.cnh';
import bodyParser = require('body-parser');

const fonte = MODEL_PRIORITY['vep.processo'].fonte;
const rank  = MODEL_PRIORITY['vep.processo'].rank;
const grupo = MODEL_PRIORITY['vep.processo'].grupo;

const request = require('request-promise');

/*
export let getPessoaPorNome = async function (cpf: string){
    return pessoa.getPessoaDetalhadoCPF_ReceitaNovo_PessoaFisica(cpf).then(res => {
        if(res.resultado.dados.length === 1) {
            return res.resultado.dados[0].nome;
        }
    });
}
*/

const validate = (cb) => {
    try {
      return cb();
    } catch (e) {
      return false;
    }
  }

export let getProcessoCPF_TJPB = async function (cpf: string){

    //var timeout = 12000; //600000 = 10 min.
    var timeout = 12000;
    var sistemas = [1, 2, 3, 4, 5, 6, 7, 8, 9];
    var processos = [];

    //identificando o nome vinculado ao cpf
    var nome = await pessoa.getPessoaDetalhadoCPF_ReceitaNovo_PessoaFisica(cpf).then(res => {
        if(res.resultado.dados.length === 1) {
            return res.resultado.dados[0].nome;
        }
    });

    //console.log('******************** Iniciando execução do WS!');
    //o objetivo desse try/catch é interceptar qualquer erro na execucao dos webservices
    //try {

        //para cada sistema, execute os webservices:
        for(let sistema of sistemas) {

            //identificando os codigos de pessoas no banco do TJPB via cpf:
            let codigosPessoaCPF = await request({
                timeout: timeout,
                url: `http://esb.tjpb.jus.br/rpjud-core/v1.1/sistemas/${sistema}/pessoas?nome=&nomeSanitizado=NAO&nome=&nomeSanitizado=SIM&tipoPessoa=F&numeroPessoaRF=${cpf}&codigosClasses=[]&codigosSituacoesProcessos=[]&codigosUnidadesJudiciarias=[]&codigosPolos=[]&codigosSituacoesParte=[]`
            }).then(res => {
                if(res.length === 0) {
                    return [];
                }else {
                    let codigos = [];
                    res = JSON.parse(res);
                    for(let cod of res) {
                        if(!cod.nenhumProcesso) {
                            codigos.push(cod.codigoPessoa);
                        }
                    }
                    return codigos;
                }
            //}).catch(error => { throw new Error(`Erro ao executar WS do TJPB (codigosPessoaCPF: ${cpf}, sistema: ${sistema})`); });
            }).catch(error => { return [] } );

            //identificando os codigos de pessoas no banco do TJPB via nome:
            let codigosPessoaNome = await request({
                timeout: timeout,
                url: `http://esb.tjpb.jus.br/rpjud-core/v1.1/sistemas/${sistema}/pessoas?nome=${nome}&nomeSanitizado=SIM&tipoPessoa=F&numeroPessoaRF=&codigosClasses=[]&codigosSituacoesProcessos=[]&codigosUnidadesJudiciarias=[]&codigosPolos=[]&codigosSituacoesParte=[]`
            }).then(res => {
                if(res.length === 0) {
                    return [];
                }else {
                    let codigos = [];
                    res = JSON.parse(res);
                    for(let cod of res) {
                        if(!cod.nenhumProcesso) {
                            codigos.push(cod.codigoPessoa);
                        }
                    }
                    return codigos;
                }
            //}).catch(error => { throw new Error(`Erro ao executar WS do TJPB (codigosPessoaNome: ${nome}, sistema: ${sistema})`); });
            }).catch(error => { return [] } );

            //unifica os resultados de codigosPessoaCPF e codigosPessoaNome:
            var codigos = await codigosPessoaCPF.concat(codigosPessoaNome.filter(function (item) {
                return codigosPessoaCPF.indexOf(item) < 0;
            }));

            //para cada codigo de pessoa encontrada, identifique os processos
            for (let c of codigos) {
                //console.log('******************** codigo da pessoa: ' + c);

                let processosPorCodigo = await request({
                    timeout: timeout,
                    url: `http://esb.tjpb.jus.br/rpjud-core/v1.1/sistemas/${sistema}/processos?codigoPessoa=${c}&tipoPessoa=F&codigosClasses=[]&codigosSituacoesProcessos=[]&codigosUnidadesJudiciarias=[]&codigosPolos=[]&codigosSituacoesParte=[]`
                }).then(res => JSON.parse(res))
                //.catch(error => { throw new Error(`Erro ao executar WS do TJPB (processosPorCodigo: ${c}, sistema: ${sistema})`); });
                .catch(error => { return []; });

                //normalizando todos os processos encontrados de determinado codigo em um unico array
                for(let p of processosPorCodigo) {

                    let processoCompleto = await request({
                        timeout: timeout,
                        url: `http://esb.tjpb.jus.br/rpjud-core/v1.1/sistemas/${sistema}/processos/${p.numero}`
                    }).then(res => JSON.parse(res))
                    //.catch(error => { throw new Error(`Erro ao executar WS do TJPB (processoCompleto: ${p.numero}, sistema: ${sistema})`); });
                    .catch(error => { return [] });

                    if(processoCompleto !== null) {
                        console.log('*********** processo numero: ' + p.numero + ' sistema: ' + sistema + ' codigo: ' + c);
                        let processo = {
                            numero: p.numero,
                            numeroAntigo: processoCompleto.numeroAntigo,
                            classe: (validate(() => p.classe.descricao)) ? p.classe.descricao : '',
                            assunto: (validate(() => p.assuntoPrincipal.descricao)) ? p.assuntoPrincipal.descricao : '',
                            gratuito: processoCompleto.gratuito,
                            unidadeJudiciaria: (validate(() => p.unidadeJudiciaria.descricao)) ? p.unidadeJudiciaria.descricao : '',
                            dataDistribuicao: processoCompleto.dataDistribuicaoInstant,
                            sigilo: processoCompleto.sigilo,
                            status: (validate(() => p.status.descricao)) ? p.status.descricao : '',
                            transitouJulgado: processoCompleto.transitouEmJulgado,
                            valorAcao: processoCompleto.valorAcao,
                            sistema: (validate(() => processoCompleto.sistema.sigla)) ? processoCompleto.sistema.sigla : '',
                            tipo: 'processo_tjpb'
                        };
                        processos.push(processo);
                    }else {
                        console.log('(*********** [erro] processo numero: ' + p.numero + ' sistema: ' + sistema + ' codigo: ' + c);
                    }
                }
            }
        }
    //} catch (err) {
    //    return logErroBuscaWS(err, '', '');
    //}
    //console.log('******************** Execução do WS concluida!');

    //return numeros
    if(processos.length === 0)
        return resultNotFound(`Nao foi possivel encontrar processos TJPB para ${cpf}.`, 'TJPB');
    else {
        return resultFoundRaw(processos, 'TJPB', rank, grupo);
    }
}; 

export let getProcessoCPF_VEP = function (cpf: string){

    return connection.getConn().request()
        .input('cpf_parameter', mssql.Char(11), cpf)
        .query(`
            SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                PF.CPF AS cpf,
                PRO.NUMEROPROCESSO AS numeroProcesso,
                PRO.DT_RECEBIMENTO AS dataRecebimento,
                PRO.DT_ARQUIVAMENTO as dataArquivamento,
                PRO.STATUS as status,
                PRO.PRIORIDADE as prioridade,
                PRO.SEGREDOJUSTICA as segredoJustica,
                PRO.NUMEROPROCESSOUNIFICADO as numeroProcessoUnificado,
                PRO.NUMEROPROCESSODEPENDENTE as numeroProcessoDependente,
                'processo' as tipo

            FROM VEP.dbo.VEP_CNJ_TB_PESSOAFISICA PF
                INNER JOIN VEP.dbo.VEP_CNJ_TB_PARTE PT ON (PT.FK_PESSOAFISICA = PF.ID_PESSOAFISICA)
                INNER JOIN VEP.dbo.VEP_CNJ_TB_PARTEPROCESSO PP ON (PP.FK_PARTE = PT.ID_PARTE)
                INNER JOIN VEP.dbo.VEP_CNJ_TB_PROCESSO PRO ON (PRO.NUMEROPROCESSO = PP.FK_PROCESSO)

            WHERE PF.CPF=@cpf_parameter
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar processos VEP para ${cpf}.`, fonte);
            else {
                return resultFound(result, fonte, rank, grupo);
            }
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelos processos VEP de ${cpf}.`, 'getProcessoCPF_VEP'); });
};

export let getPenaProcessoCPF_VEP = function (cpf: string){

    return connection.getConn().request()
        .input('cpf_parameter', mssql.Char(11), cpf)
        .query(`
            SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                PF.CPF as cpf
                ,PRO.NUMEROPROCESSO as numeroProcesso
                ,PENAACUMPRIR as penaACumprir
                ,TOTALPENA as totalPena
                ,HSP.STATUS as status
                ,SP.DS_SITUACAOPENAL as situacaoPenal
                ,DT_SITUACAO as dataSituacao
                ,DT_FIMPENA as dataFimPena
                ,DT_INICIOPENA as dataInicioPena
                ,'progressaopena' as tipo

            FROM VEP.dbo.VEP_CNJ_TB_HIST_SOMAPENA HSP
                INNER JOIN VEP.dbo.VEP_CNJ_TB_SITUACAOPENAL SP ON (SP.ID_SITUACAOPENAL = HSP.FK_SITUACAOPENAL)
                INNER JOIN VEP.dbo.VEP_CNJ_TB_PARTEPROCESSO PP ON (PP.ID_PARTEPROCESSO = HSP.FK_PARTEPROCESSO)
                INNER JOIN VEP.dbo.VEP_CNJ_TB_PROCESSO PRO ON (PRO.NUMEROPROCESSO = PP.FK_PROCESSO)
                INNER JOIN VEP.dbo.VEP_CNJ_TB_PARTE P ON (P.ID_PARTE = PP.FK_PARTE)
                INNER JOIN VEP.dbo.VEP_CNJ_TB_PESSOAFISICA PF ON (PF.ID_PESSOAFISICA = P.FK_PESSOAFISICA)
            WHERE CPF=@cpf_parameter
            ORDER BY DT_SITUACAO DESC
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar progressao de pena dos processos VEP para ${cpf}.`, fonte);
            else {
                return resultFound(result, fonte, rank, grupo);
            }
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pela progressao de pena dos processos VEP de ${cpf}.`, 'getPenaProcessoCPF_VEP'); });
};