import * as pessoa from './../pessoa.model';

import { MODEL_PRIORITY, API_CONFIG } from '../../config';
import {
    resultNotFound,
    resultFound,
    logErroBuscaBD,
    resultFoundRaw
} from '../../utils';
import bodyParser = require('body-parser');

const fonte = MODEL_PRIORITY['tjpb.processo'].fonte;
const rank  = MODEL_PRIORITY['tjpb.processo'].rank;
const grupo = MODEL_PRIORITY['tjpb.processo'].grupo;

const request = require('request-promise');

export let getProcessoCPF_TJPB = async function (cpf: string){

    var timeout = 12000; //600000 = 10 min.
    var sistema = 5;

    //identificando os codigos de pessoas no banco do TJPB:
    let codigosPessoaCPF = await request({
        timeout: timeout,
        url: `http://esb.tjpb.jus.br/rpjud-core/v1.1/sistemas/${sistema}/pessoas?nome=&nomeSanitizado=NAO&nome=&nomeSanitizado=SIM&tipoPessoa=F&numeroPessoaRF=${cpf}&codigosClasses=[]&codigosSituacoesProcessos=[]&codigosUnidadesJudiciarias=[]&codigosPolos=[]&codigosSituacoesParte=[]`
    }).then(res => {
        if(res.length === 0) {
            return [];
        }else {
            let codigos = [];
            res = JSON.parse(res);
            for(let cod of res) {
                //if(cod.nenhumProcesso == false) {
                    codigos.push(cod.codigoPessoa);
                //}
            }
            return codigos;
        }
    }).catch(error => { return resultNotFound(`Erro ao executar WS do TJPB (codigosPessoaCPF - ${cpf})`, 'TJPB'); });

    //identificando o nome vinculado ao cpf
    let nome = pessoa.getPessoaDetalhadoCPF_ReceitaNovo_PessoaFisica(cpf).then(res => {
        if(res.resultado.dados.length === 1) {
            return res.resultado.dados[0].nome;
        }
    });

    //identificando os codigos de pessoas no banco do TJPB:
    let codigosPessoaNome = await request({
        timeout: timeout,
        url: `http://esb.tjpb.jus.br/rpjud-core/v1.1/sistemas/${sistema}/pessoas?nome=${nome}&nomeSanitizado=SIM&tipoPessoa=F&numeroPessoaRF=&codigosClasses=[]&codigosSituacoesProcessos=[]&codigosUnidadesJudiciarias=[]&codigosPolos=[]&codigosSituacoesParte=[]`
    }).then(res => {
        if(res.length === 0) {
            return [];
        }else {
            let codigos = [];
            res = JSON.parse(res);
            for(let cod of res) {
                //if(cod.nenhumProcesso == false) {
                    codigos.push(cod.codigoPessoa);
                //}            
            }
            return codigos;
        }
    }).catch(error => { return resultNotFound(`Erro ao executar WS do TJPB (codigosPessoaNome - ${cpf})`, 'TJPB'); });

    //var codigos = codigosPessoaNome;    
    //unifica os resultados de codigosPessoaCPF e codigosPessoaNome:
    var codigos = codigosPessoaCPF.concat(codigosPessoaNome.filter(function (item) {
        return codigosPessoaCPF.indexOf(item) < 0;
    }));

    //para cada codigo de pessoa encontrada, identifique os processos
    let processos = [];
    for (let c of codigos) {
        console.log('******************** codigo da pessoa: ' + c);

        let processosPorCodigo = await request({
            timeout: timeout,
            url: `http://esb.tjpb.jus.br/rpjud-core/v1.1/sistemas/${sistema}/processos?codigoPessoa=${c}&tipoPessoa=F&codigosClasses=[]&codigosSituacoesProcessos=[]&codigosUnidadesJudiciarias=[]&codigosPolos=[]&codigosSituacoesParte=[]`
        }).then(res => JSON.parse(res))
        .catch(error => { return resultNotFound(`Erro ao executar WS do TJPB (processosPorCodigo - ${c})`, 'TJPB'); });

        //normalizando todos os processos encontrados de determinado codigo em um unico array
        for(let p of processosPorCodigo) {
            console.log('******************** processo numero: ' + p.numero);

            let processoCompleto = await request({
                timeout: timeout,
                url: `http://esb.tjpb.jus.br/rpjud-core/v1.1/sistemas/${sistema}/processos/${p.numero}`
            }).then(res => JSON.parse(res))
            .catch(error => { return resultNotFound(`Erro ao executar WS do TJPB (processoCompleto - ${p.numero})`, 'TJPB'); });

            let processo = {
                numero: p.numero,
                numeroAntigo: processoCompleto.numeroAntigo,
                classe: p.classe.descricao,
                assunto: (p.hasOwnProperty('assuntoPrincipal.descricao')) ? p.assuntoPrincipal.descricao : '',
                gratuito: processoCompleto.gratuito,
                unidadeJudiciaria: p.unidadeJudiciaria.descricao,
                dataDistribuicao: processoCompleto.dataDistribuicaoInstant,
                sigilo: processoCompleto.sigilo,
                status: p.status.descricao,
                transitouJulgado: processoCompleto.transitouEmJulgado,
                valorAcao: processoCompleto.valorAcao,
                sistema: processoCompleto.sistema.sigla,
                tipo: 'processo_tjpb'
            };
            processos.push(processo);
        }
    }

    //return numeros
    if(processos.length === 0)
        return resultNotFound(`Nao foi possivel encontrar processos TJPB para ${cpf}.`, 'TJPB');
    else {
        return resultFoundRaw(processos, 'TJPB', rank, grupo);
    }
}; 