import * as mssql from 'mssql';

import * as pessoa from './../pessoa.model';

import { connection }  from '../../services/db.service';
import { MODEL_PRIORITY, API_CONFIG } from '../../config';
import {
    resultNotFound,
    resultFound,
    logErroBuscaBD,
    resultFoundRaw
} from '../../utils';
import { getPessoaSimplificadoRG_Sispesquisa_CNH } from './../pessoa/pessoa.model.sispesquisa.cnh';
import bodyParser = require('body-parser');

const fonte = MODEL_PRIORITY['vep.processo'].fonte;
const rank  = MODEL_PRIORITY['vep.processo'].rank;
const grupo = MODEL_PRIORITY['vep.processo'].grupo;

export let getProcessoCPF_VEP = function (cpf: string){

    return connection.getConn().request()
        .input('cpf_parameter', mssql.Char(11), cpf)
        .query(`
            SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                PF.CPF AS cpf,
                PRO.NUMEROPROCESSO AS numeroProcesso,
                PRO.DT_RECEBIMENTO AS dataRecebimento,
                PRO.DT_ARQUIVAMENTO as dataArquivamento,
                PRO.STATUS as status,
                PRO.PRIORIDADE as prioridade,
                PRO.SEGREDOJUSTICA as segredoJustica,
                PRO.NUMEROPROCESSOUNIFICADO as numeroProcessoUnificado,
                PRO.NUMEROPROCESSODEPENDENTE as numeroProcessoDependente,
                'processo' as tipo

            FROM VEP.dbo.VEP_CNJ_TB_PESSOAFISICA PF
                INNER JOIN VEP.dbo.VEP_CNJ_TB_PARTE PT ON (PT.FK_PESSOAFISICA = PF.ID_PESSOAFISICA)
                INNER JOIN VEP.dbo.VEP_CNJ_TB_PARTEPROCESSO PP ON (PP.FK_PARTE = PT.ID_PARTE)
                INNER JOIN VEP.dbo.VEP_CNJ_TB_PROCESSO PRO ON (PRO.NUMEROPROCESSO = PP.FK_PROCESSO)

            WHERE PF.CPF=@cpf_parameter
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar processos VEP para ${cpf}.`, fonte);
            else {
                return resultFound(result, fonte, rank, grupo);
            }
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelos processos VEP de ${cpf}.`, 'getProcessoCPF_VEP'); });
};

export let getPenaProcessoCPF_VEP = function (cpf: string){

    return connection.getConn().request()
        .input('cpf_parameter', mssql.Char(11), cpf)
        .query(`
            SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                PF.CPF as cpf
                ,PRO.NUMEROPROCESSO as numeroProcesso
                ,PENAACUMPRIR as penaACumprir
                ,TOTALPENA as totalPena
                ,HSP.STATUS as status
                ,SP.DS_SITUACAOPENAL as situacaoPenal
                ,DT_SITUACAO as dataSituacao
                ,DT_FIMPENA as dataFimPena
                ,DT_INICIOPENA as dataInicioPena
                ,'progressaopena' as tipo

            FROM VEP.dbo.VEP_CNJ_TB_HIST_SOMAPENA HSP
                INNER JOIN VEP.dbo.VEP_CNJ_TB_SITUACAOPENAL SP ON (SP.ID_SITUACAOPENAL = HSP.FK_SITUACAOPENAL)
                INNER JOIN VEP.dbo.VEP_CNJ_TB_PARTEPROCESSO PP ON (PP.ID_PARTEPROCESSO = HSP.FK_PARTEPROCESSO)
                INNER JOIN VEP.dbo.VEP_CNJ_TB_PROCESSO PRO ON (PRO.NUMEROPROCESSO = PP.FK_PROCESSO)
                INNER JOIN VEP.dbo.VEP_CNJ_TB_PARTE P ON (P.ID_PARTE = PP.FK_PARTE)
                INNER JOIN VEP.dbo.VEP_CNJ_TB_PESSOAFISICA PF ON (PF.ID_PESSOAFISICA = P.FK_PESSOAFISICA)
            WHERE CPF=@cpf_parameter
            ORDER BY DT_SITUACAO DESC
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar progressao de pena dos processos VEP para ${cpf}.`, fonte);
            else {
                return resultFound(result, fonte, rank, grupo);
            }
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pela progressao de pena dos processos VEP de ${cpf}.`, 'getPenaProcessoCPF_VEP'); });
};