import * as mssql from 'mssql';

import { BenchmarkService } from './../../services/benchmark.service';
import { connection } from '../../services/db.service';
import { MODEL_PRIORITY } from './../../config';

import {
    trataResultado,
    resultNotFound,
    resultFound,
    logErroBuscaBD,
    printTempoExecucao
} from '../../utils';

export let getNodesParentescosPessoaCPF = function (listaCpf: Array<string>, tipobusca: string = 'completa', tipoparentesco: string = 'restrito'){

    if (tipobusca !== 'parentesco' && tipobusca !== 'completa') { return []; }
    if (listaCpf.length === 0) { return []; }

    const lista = "'" + listaCpf.join("','") + "'";
    const restricao = (tipoparentesco === 'restrito') ? `WHERE relacao IN ('MÃE', 'PAI', 'IRMÃO', 'IRMÃ', 'CÔNJUGE/COMPANHEIRO', 'CÔNJUGE/COMPANHEIRA', 'PAI/PADRASTO', 'MÃE/MADRASTA', 'COMPANHEIRO', 'COMPANHEIRA', 'CÔNJUGE')` : '';
    const tempoInicial = new BenchmarkService();

    return connection.getConn().request()
        .query(`
            ;WITH ALVO_PF_INICIAL(CPF, NOME, NomeMae, DataNascimento, Sexo, Municipio, UF)
            AS
            (
                SELECT CPF, NOME, NomeMae, DataNascimento, UPPER(Sexo.Descricao), Municipio, UF
                FROM ReceitaNovo.dbo.PessoaFisica PF
                    INNER JOIN ReceitaNovo.dbo.Sexo ON (PF.Sexo = Sexo.SexoID)
                WHERE cpf IN (${lista})
            )
            ,VINCULOS_DIRETOS_A(CPF, RELACAO)
            AS
            (
                SELECT CPF2, RELACAO
                FROM ALVO_PF_INICIAL A
                    INNER JOIN BD_RELACAO.dbo.RELACAO_PF_PF R ON (A.CPF = R.CPF1)
                ${restricao}
            )
            ,VINCULOS_DIRETOS_B(CPF, RELACAO)
            AS
            (
                SELECT CPF1, RELACAO
                FROM ALVO_PF_INICIAL A
                    INNER JOIN BD_RELACAO.dbo.RELACAO_PF_PF R ON (A.CPF = R.CPF2)
                ${restricao}
            )
            ,VINCULOS_DIRETOS(CPF, RELACAO)
            AS
            (
                SELECT CPF, RELACAO	FROM VINCULOS_DIRETOS_A
                UNION
                SELECT CPF, RELACAO	FROM VINCULOS_DIRETOS_B
            )
            ,ALVO_PF_COM_VINCULOS(CPF, NOME, NomeMae, DataNascimento, Sexo, Municipio, UF)
            AS
            (
                SELECT PF.CPF, NOME, NomeMae, DataNascimento, UPPER(Sexo.Descricao), Municipio, UF
                FROM VINCULOS_DIRETOS V
                    INNER JOIN ReceitaNovo.dbo.PessoaFisica PF ON (V.CPF = PF.CPF)
                    INNER JOIN ReceitaNovo.dbo.Sexo ON (PF.Sexo = Sexo.SexoID)

                UNION

                SELECT *
                FROM ALVO_PF_INICIAL
            )
            ,ALVO_PF(CPF, NOME, NomeMae, NomePai, DataNascimento, Sexo, Municipio, UF)
            AS
            (
                SELECT A.CPF, A.NOME, A.NomeMae, E.NOM_PAI, A.DataNascimento, A.Sexo, A.Municipio, A.UF
                FROM ALVO_PF_COM_VINCULOS A
                    LEFT OUTER JOIN TSE.dbo.ELEITOR E ON (A.CPF = E.CPF_RF)
            )

            SELECT CPF as id, 'pessoa' as entidade, CPF as cpf, NOME as nome, NomeMae as nomeMae, NomePai as nomePai, DataNascimento as dataNascimento, Sexo as sexo, Municipio as municipio, UF as uf
            FROM ALVO_PF
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getNodesParentescosPessoaCPF');

            return result.recordset.map(resultado => trataResultado(resultado));
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelos nodes de ${listaCpf}.`, 'getNodesParentescosPessoaCPF'); });
};

export let getEdgesParentescosPessoaCPF = function (listaCpf: Array<string>, tipobusca: string = 'completa', tipoparentesco: string = 'restrito') {

    if (tipobusca !== 'parentesco' && tipobusca !== 'completa') { return []; }
    if (listaCpf.length === 0) { return []; }

    const lista = "'" + listaCpf.join("','") + "'";
    const restricao = (tipoparentesco === 'restrito') ? `WHERE relacao IN ('MÃE', 'PAI', 'IRMÃO', 'IRMÃ', 'CÔNJUGE/COMPANHEIRO', 'CÔNJUGE/COMPANHEIRA', 'PAI/PADRASTO', 'MÃE/MADRASTA', 'COMPANHEIRO', 'COMPANHEIRA', 'CÔNJUGE')` : '';
    const tempoInicial = new BenchmarkService();

    return connection.getConn().request()
        .query(`
            ;WITH ALVOS_PF_INICIAL(CPF, NOME, NomeMae, DataNascimento, Sexo, Municipio, UF)
            AS
            (
                SELECT CPF, NOME, NomeMae, DataNascimento, Sexo, Municipio, UF
                FROM ReceitaNovo.dbo.PessoaFisica PF
                WHERE cpf IN (${lista})
            )
            ,VINCULOS_PARENTES_A(CPF_ORIGEM, RELACAO, CPF_DESTINO)
            AS
            (
                SELECT CPF1, RELACAO, CPF2
                FROM ALVOS_PF_INICIAL A
                    INNER JOIN BD_RELACAO.dbo.RELACAO_PF_PF R ON (A.CPF = R.CPF1)
                ${restricao}
            )
            ,VINCULOS_PARENTES_B(CPF_ORIGEM, RELACAO, CPF_DESTINO)
            AS
            (
                SELECT CPF2, RELACAO, CPF1
                FROM ALVOS_PF_INICIAL A
                    INNER JOIN BD_RELACAO.dbo.RELACAO_PF_PF R ON (A.CPF = R.CPF2)
                ${restricao}
            )
            ,VINCULOS_DIRETOS(ORIGEM, RELACAO, DESTINO)
            AS
            (
                SELECT * FROM VINCULOS_PARENTES_A
                UNION
                SELECT * FROM VINCULOS_PARENTES_B
            )

            SELECT ORIGEM as origem, RELACAO as relacao, DESTINO as destino
            FROM VINCULOS_DIRETOS
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getEdgesParentescosPessoaCPF');

            return result.recordset.map(resultado => trataResultado(resultado));
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelos nodes de ${listaCpf}.`, 'getEdgesParentescosPessoaCPF'); });
};

export let getNodesEmpresasResponsavelPessoaCPF = function (listaCpf: Array<string>, tipobusca: string = 'completa') {

    if (tipobusca !== 'empresasresponsavel' && tipobusca !== 'completa') { return []; }
    if (listaCpf.length === 0) { return []; }

    const lista = "'" + listaCpf.join("','") + "'";
    const tempoInicial = new BenchmarkService();

    return connection.getConn().request()
        .query(`
            ;WITH ALVO_PF_INICIAL(CPF, NOME, NomeMae, DataNascimento, Sexo, Municipio, UF)
            AS
            (
                SELECT CPF, NOME, NomeMae, DataNascimento, Sexo, Municipio, UF
                FROM ReceitaNovo.dbo.PessoaFisica PF
                WHERE cpf IN (${lista})
            )
            ,RESPONSAVEL_EMPRESA(CNPJ, RAZAOSOCIAL, NOMEFANTASIA, DTINICIOATIVIDADE, DTSITUACAOCADASTRAL, MUNICIPIO, UF)
            AS
            (
                SELECT PJ.CNPJ, PJ.RazaoSocial, PJ.NomeFantasia, PJ.DataInicioAtividade, PJ.DataSituacaoCadastral, PJ.Municipio, PJ.UF
                FROM ALVO_PF_INICIAL A
                    INNER JOIN ReceitaNovo.dbo.PessoaJuridica PJ ON (A.CPF = PJ.CpfResponsavel)
            )
            ,EMPRESAS(CNPJ, RAZAOSOCIAL, NOMEFANTASIA, DTINICIOATIVIDADE, DTSITUACAOCADASTRAL, MUNICIPIO, UF)
            AS
            (
                SELECT * FROM RESPONSAVEL_EMPRESA
            )

            SELECT CNPJ as id, 'empresa' as entidade, CNPJ as cnpj, RAZAOSOCIAL as razaoSocial, NOMEFANTASIA as nomeFantasia, cast(DTINICIOATIVIDADE as date) as dataInicioAtividade, MUNICIPIO as municipio, UF as uf
            FROM EMPRESAS
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getNodesEmpresasResponsavelPessoaCPF');

            return result.recordset.map(resultado => trataResultado(resultado));
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelos nodes de ${listaCpf}.`, 'getNodesEmpresasResponsavelPessoaCPF'); });
};

export let getEdgesEmpresasResponsavelPessoaCPF = function (listaCpf: Array<string>, tipobusca: string = 'completa') {

    if (tipobusca !== 'empresasresponsavel' && tipobusca !== 'completa') { return []; }
    if (listaCpf.length === 0) { return []; }

    const lista = "'" + listaCpf.join("','") + "'";
    const tempoInicial = new BenchmarkService();

    return connection.getConn().request()
        .query(`
            ;WITH ALVOS_PF_INICIAL(CPF, NOME, NomeMae, DataNascimento, Sexo, Municipio, UF)
            AS
            (
                SELECT CPF, NOME, NomeMae, DataNascimento, Sexo, Municipio, UF
                FROM ReceitaNovo.dbo.PessoaFisica PF
                WHERE cpf IN (${lista})
            )
            ,RESPONSAVEL_EMPRESA(CPF, RELACAO, CNPJ)
            AS
            (
                SELECT A.CPF, 'RESPONSAVEL', PJ.CNPJ
                FROM ALVOS_PF_INICIAL A
                    INNER JOIN ReceitaNovo.dbo.PessoaJuridica PJ ON (A.CPF = PJ.CpfResponsavel)
            )
            ,VINCULOS_DIRETOS(ORIGEM, RELACAO, DESTINO)
            AS
            (
                SELECT * FROM RESPONSAVEL_EMPRESA
            )

            SELECT ORIGEM as origem, RELACAO as relacao, DESTINO as destino
            FROM VINCULOS_DIRETOS
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getEdgesEmpresasResponsavelPessoaCPF');

            return result.recordset.map(resultado => trataResultado(resultado));
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelos nodes de ${listaCpf}.`, 'getEdgesEmpresasResponsavelPessoaCPF'); });
};

export let getNodesEmpresasSocioPessoaCPF = function (listaCpf: Array<string>, tipobusca: string = 'completa') {

    if (tipobusca !== 'empresassocio' && tipobusca !== 'completa') { return []; }
    if (listaCpf.length === 0) { return []; }

    const lista = "'" + listaCpf.join("','") + "'";
    const tempoInicial = new BenchmarkService();

    return connection.getConn().request()
        .query(`
            ;WITH ALVO_PF_INICIAL(CPF, NOME, NomeMae, DataNascimento, Sexo, Municipio, UF)
            AS
            (
                SELECT CPF, NOME, NomeMae, DataNascimento, Sexo, Municipio, UF
                FROM ReceitaNovo.dbo.PessoaFisica PF
                WHERE cpf IN (${lista})
            )
            ,SOCIOS_EMPRESA(CNPJ, RAZAOSOCIAL, NOMEFANTASIA, DTINICIOATIVIDADE, DTSITUACAOCADASTRAL, MUNICIPIO, UF)
            AS
            (
                SELECT PJ.CNPJ, PJ.RazaoSocial, PJ.NomeFantasia, PJ.DataInicioAtividade, PJ.DataSituacaoCadastral, PJ.Municipio, PJ.UF
                FROM ALVO_PF_INICIAL A
                    INNER JOIN ReceitaNovo.dbo.Socio S ON ('000' + A.CPF = S.CpfSocio)
                    INNER JOIN ReceitaNovo.dbo.PessoaJuridica PJ ON (S.Cnpj = PJ.CNPJ)
            )
            ,EMPRESAS(CNPJ, RAZAOSOCIAL, NOMEFANTASIA, DTINICIOATIVIDADE, DTSITUACAOCADASTRAL, MUNICIPIO, UF)
            AS
            (
                SELECT * FROM SOCIOS_EMPRESA
            )

            SELECT CNPJ as id, 'empresa' as entidade, CNPJ as cnpj, RAZAOSOCIAL as razaoSocial, NOMEFANTASIA as nomeFantasia, cast(DTINICIOATIVIDADE as date) as dataInicioAtividade, MUNICIPIO as municipio, UF as uf
            FROM EMPRESAS
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getNodesEmpresasSocioPessoaCPF');

            return result.recordset.map(resultado => trataResultado(resultado));
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelos nodes de ${listaCpf}.`, 'getNodesEmpresasSocioPessoaCPF'); });
};

export let getEdgesEmpresasSocioPessoaCPF = function (listaCpf: Array<string>, tipobusca: string = 'completa') {

    if (tipobusca !== 'empresassocio' && tipobusca !== 'completa') { return []; }
    if (listaCpf.length === 0) { return []; }

    const lista = "'" + listaCpf.join("','") + "'";
    const tempoInicial = new BenchmarkService();

    return connection.getConn().request()
        .query(`
            ;WITH ALVOS_PF_INICIAL(CPF, NOME, NomeMae, DataNascimento, Sexo, Municipio, UF)
            AS
            (
                SELECT CPF, NOME, NomeMae, DataNascimento, Sexo, Municipio, UF
                FROM ReceitaNovo.dbo.PessoaFisica PF
                WHERE cpf IN (${lista})
            )
            ,SOCIOS_EMPRESA(CPF, RELACAO, CNPJ)
            AS
            (
                SELECT A.CPF, 'SOCIO', PJ.CNPJ
                FROM ALVOS_PF_INICIAL A
                    INNER JOIN ReceitaNovo.dbo.Socio S ON ('000' + A.CPF = S.CpfSocio)
                    INNER JOIN ReceitaNovo.dbo.PessoaJuridica PJ ON (S.Cnpj = PJ.CNPJ)
            )
            ,VINCULOS_DIRETOS(ORIGEM, RELACAO, DESTINO)
            AS
            (
                SELECT * FROM SOCIOS_EMPRESA
            )

            SELECT ORIGEM as origem, RELACAO as relacao, DESTINO as destino
            FROM VINCULOS_DIRETOS
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getEdgesEmpresasSocioPessoaCPF');

            return result.recordset.map(resultado => trataResultado(resultado));
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelos nodes de ${listaCpf}.`, 'getEdgesEmpresasSocioPessoaCPF'); });
};

export let getNodesTelefonesPessoaCPF = function (listaCpf: Array<string>, tipobusca: string = 'completa') {

    if (tipobusca !== 'telefones' && tipobusca !== 'completa') { return []; }
    if (listaCpf.length === 0) { return []; }

    const lista = "'" + listaCpf.join("','") + "'";
    const tempoInicial = new BenchmarkService();

    return connection.getConn().request()
        .query(`
            ;WITH TELEFONES(TELEFONE)
            AS
            (
                SELECT DISTINCT telefone
                FROM SISPESQUISA.sispesquisa.telefones
                WHERE cpf_cnpj IN (${lista})
            )

            SELECT CAST(telefone as varchar) as id, 'telefone' as entidade, CAST(telefone as varchar) as telefone
            FROM TELEFONES
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getNodesTelefonesPessoaCPF');

            return result.recordset.map(resultado => trataResultado(resultado));
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelos nodes de ${listaCpf}.`, 'getNodesTelefonesPessoaCPF'); });
};

export let getEdgesTelefonesPessoaCPF = function (listaCpf: Array<string>, tipobusca: string = 'completa') {

    if (tipobusca !== 'telefones' && tipobusca !== 'completa') { return []; }
    if (listaCpf.length === 0) { return []; }

    const lista = "'" + listaCpf.join("','") + "'";
    const tempoInicial = new BenchmarkService();

    return connection.getConn().request()
        .query(`
            ;WITH TELEFONES(TELEFONE, cpf)
            AS
            (
                SELECT DISTINCT telefone, cpf_cnpj
                FROM SISPESQUISA.sispesquisa.telefones
                WHERE cpf_cnpj IN (${lista})
            )

            SELECT cpf as origem, 'proprietario' as relacao, telefone as destino
            FROM TELEFONES
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getEdgesTelefonesPessoaCPF');

            return result.recordset.map(resultado => trataResultado(resultado));
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelos nodes de ${listaCpf}.`, 'getEdgesTelefonesPessoaCPF'); });
};

export let getNodesEnderecosPessoaCPF = function (listaCpf: Array<string>, tipobusca: string = 'completa') {

    if (tipobusca !== 'enderecos' && tipobusca !== 'completa') { return []; }
    if (listaCpf.length === 0) { return []; }

    const lista = "'" + listaCpf.join("','") + "'";
    const tempoInicial = new BenchmarkService();

    return connection.getConn().request()
        .query(`
            ;WITH ENDERECOS(TIPOLOGRADOURO, LOGRADOURO, NUMERO, COMPLEMENTO, BAIRRO, CEP, MUNICIPIO, UF)
            AS
            (
                SELECT TRIM(TipoLogradouro), TRIM(Logradouro), TRIM(NumeroLogradouro), TRIM(Complemento),
                    TRIM(Bairro), TRIM(Cep), TRIM(Municipio), TRIM(UF)
                FROM ReceitaNovo.dbo.PessoaFisica
                WHERE cpf IN (${lista})
            )
            SELECT CONCAT(LOGRADOURO,'-', NUMERO) as id, 'endereco' as entidade, TIPOLOGRADOURO as tipoLogradouro,
                LOGRADOURO as logradouro, NUMERO as numero, COMPLEMENTO as complemento, BAIRRO as bairro,
                CEP as cep, MUNICIPIO as municipio, UF as uf
            FROM ENDERECOS
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getNodesEnderecosPessoaCPF');

            return result.recordset.map(resultado => trataResultado(resultado));
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelos nodes de ${listaCpf}.`, 'getNodesEnderecosPessoaCPF'); });
};

export let getEdgesEnderecosPessoaCPF = function (listaCpf: Array<string>, tipobusca: string = 'completa') {

    if (tipobusca !== 'enderecos' && tipobusca !== 'completa') { return []; }
    if (listaCpf.length === 0) { return []; }

    const lista = "'" + listaCpf.join("','") + "'";
    const tempoInicial = new BenchmarkService();

    return connection.getConn().request()
        .query(`
            ;WITH ENDERECOS(CPF, LOGRADOURO, NUMERO)
            AS
            (
                SELECT TRIM(CPF), TRIM(Logradouro), TRIM(NumeroLogradouro)
                FROM ReceitaNovo.dbo.PessoaFisica
                WHERE cpf IN (${lista})
            )
            SELECT cpf as origem, 'reside' as relacao, CONCAT(LOGRADOURO, '-', NUMERO) as destino
            FROM ENDERECOS
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getEdgesEnderecosPessoaCPF');

            return result.recordset.map(resultado => trataResultado(resultado));
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelos nodes de ${listaCpf}.`, 'getEdgesEnderecosPessoaCPF'); });
};

export let getNodesOrgaosPublicosMunicipaisPessoaCPF = function (listaCpf: Array<string>, tipobusca: string = 'completa') {

    if (tipobusca !== 'orgaospublicos' && tipobusca !== 'completa') { return []; }
    if (listaCpf.length === 0) { return []; }

    const lista = "'" + listaCpf.join("','") + "'";
    const tempoInicial = new BenchmarkService();

    return connection.getConn().request()
        .query(`
            ;WITH VINCULOS_MUNICIPAIS(ID, UGESTORA)
            AS
            (
                SELECT DISTINCT fp.cd_ugestora, UPPER(UG.de_Ugestora) AS de_Ugestora
                FROM SAGRES_MUNICIPAL.dbo.FolhaPagamento FP
                OUTER APPLY (
                    SELECT TOP 1 de_Ugestora
                    FROM SAGRES_CONTROLE.dbo.Codigo_Ugestora UG
                    WHERE UG.cd_Ugestora=FP.cd_UGestora
                    ORDER BY UG.dt_Ano DESC
                ) UG
                WHERE cd_CPF IN (${lista})

                UNION

                SELECT DISTINCT fp.cd_ugestora, UPPER(UG.de_Ugestora) AS de_Ugestora
                FROM SAGRES_MUNICIPAL.dbo.FolhaPagamento_N FP
                OUTER APPLY (
                    SELECT TOP 1 de_Ugestora
                    FROM SAGRES_CONTROLE.dbo.Codigo_Ugestora UG
                    WHERE UG.cd_Ugestora=FP.cd_UGestora
                    ORDER BY UG.dt_Ano DESC
                ) UG
                WHERE cd_CPF IN (${lista})

                UNION

                SELECT DISTINCT fp.cd_ugestora, UPPER(UG.de_Ugestora) AS de_Ugestora
                FROM SAGRES_MUNICIPAL.dbo.FolhaPagamento_N_2017 FP
                OUTER APPLY (
                    SELECT TOP 1 de_Ugestora
                    FROM SAGRES_CONTROLE.dbo.Codigo_Ugestora UG
                    WHERE UG.cd_Ugestora=FP.cd_UGestora
                    ORDER BY UG.dt_Ano DESC
                ) UG
                WHERE cd_CPF IN (${lista})

            )

            SELECT ID as id, 'orgaopublico' as entidade, UGESTORA as uGestora, 'M' as esfera
            FROM VINCULOS_MUNICIPAIS
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getNodesOrgaosPublicosMunicipaisPessoaCPF');

            return result.recordset.map(resultado => trataResultado(resultado));
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelos nodes de ${listaCpf}.`, 'getNodesOrgaosPublicosMunicipaisPessoaCPF'); });
};

export let getEdgesOrgaosPublicosMunicipaisPessoaCPF = function (listaCpf: Array<string>, tipobusca: string = 'completa') {

    if (tipobusca !== 'orgaospublicos' && tipobusca !== 'completa') { return []; }
    if (listaCpf.length === 0) { return []; }

    const lista = "'" + listaCpf.join("','") + "'";
    const tempoInicial = new BenchmarkService();

    return connection.getConn().request()
        .query(`
            ;WITH VINCULOS_MUNICIPAIS(ID, UGESTORA, CPF, CARGO, PRIMEIRA, ULTIMA)
            AS
            (
                SELECT DISTINCT fp.cd_UGestora, UPPER(UG.de_Ugestora) AS de_Ugestora, cd_CPF, C.de_Cargo, min(dt_ano) as primeira_ocorrencia, max(dt_ano) as ultima_ocorrencia
                FROM SAGRES_MUNICIPAL.dbo.FolhaPagamento FP
                OUTER APPLY (
                    SELECT TOP 1 de_Cargo
                    FROM SAGRES_MUNICIPAL.dbo.Cargos C
                    WHERE C.cd_Cargo=FP.cd_Cargo AND C.cd_UGestora=FP.cd_UGestora
                    ORDER BY C.dt_Ano DESC
                ) C
                OUTER APPLY (
                    SELECT TOP 1 de_Ugestora
                    FROM SAGRES_CONTROLE.dbo.Codigo_Ugestora UG
                    WHERE UG.cd_Ugestora=FP.cd_UGestora
                    ORDER BY UG.dt_Ano DESC
                ) UG
                WHERE cd_CPF IN (${lista})
                GROUP BY fp.cd_UGestora, de_Ugestora, cd_CPF, de_Cargo

                UNION

                SELECT DISTINCT fp.cd_UGestora, UPPER(UG.de_Ugestora) AS de_Ugestora, cd_CPF, C.de_Cargo, min(dt_ano) as primeira_ocorrencia, max(dt_ano) as ultima_ocorrencia
                FROM SAGRES_MUNICIPAL.dbo.FolhaPagamento_N FP
                OUTER APPLY (
                    SELECT TOP 1 de_Cargo
                    FROM SAGRES_MUNICIPAL.dbo.Cargos C
                    WHERE C.cd_Cargo=FP.cd_Cargo AND C.cd_UGestora=FP.cd_UGestora
                    ORDER BY C.dt_Ano DESC
                ) C
                OUTER APPLY (
                    SELECT TOP 1 de_Ugestora
                    FROM SAGRES_CONTROLE.dbo.Codigo_Ugestora UG
                    WHERE UG.cd_Ugestora=FP.cd_UGestora
                    ORDER BY UG.dt_Ano DESC
                ) UG
                WHERE cd_CPF IN (${lista})
                GROUP BY fp.cd_UGestora, de_Ugestora, cd_CPF, de_Cargo

                UNION

                SELECT DISTINCT fp.cd_UGestora, UPPER(UG.de_Ugestora) AS de_Ugestora, cd_CPF, C.de_Cargo, min(dt_ano) as primeira_ocorrencia, max(dt_ano) as ultima_ocorrencia
                FROM SAGRES_MUNICIPAL.dbo.FolhaPagamento_N_2017 FP
                OUTER APPLY (
                    SELECT TOP 1 de_Cargo
                    FROM SAGRES_MUNICIPAL.dbo.Cargos C
                    WHERE C.cd_Cargo=FP.cd_Cargo AND C.cd_UGestora=FP.cd_UGestora
                    ORDER BY C.dt_Ano DESC
                ) C
                OUTER APPLY (
                    SELECT TOP 1 de_Ugestora
                    FROM SAGRES_CONTROLE.dbo.Codigo_Ugestora UG
                    WHERE UG.cd_Ugestora=FP.cd_UGestora
                    ORDER BY UG.dt_Ano DESC
                ) UG
                WHERE cd_CPF IN (${lista})
                GROUP BY fp.cd_UGestora, de_Ugestora, cd_CPF, de_Cargo
            )

            SELECT CONCAT(CPF, '-', CARGO, '-', ID) as id, CPF as origem, CARGO as relacao, ID as destino, MIN(PRIMEIRA) as pOcorrencia, MAX(ULTIMA) as uOcorrencia
            FROM VINCULOS_MUNICIPAIS
            GROUP BY ID, UGESTORA, CPF, CARGO
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getEdgesOrgaosPublicosMunicipaisPessoaCPF');

            return result.recordset.map(resultado => trataResultado(resultado));
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelos nodes de ${listaCpf}.`, 'getEdgesOrgaosPublicosMunicipaisPessoaCPF'); });
};

export let getNodesOrgaosPublicosEstaduaisPessoaCPF = function (listaCpf: Array<string>, tipobusca: string = 'completa') {

    if (tipobusca !== 'orgaospublicos' && tipobusca !== 'completa') { return []; }
    if (listaCpf.length === 0) { return []; }

    const lista = "'" + listaCpf.join("','") + "'";
    const tempoInicial = new BenchmarkService();

    return connection.getConn().request()
        .query(`
            ;WITH VINCULOS_ESTADUAIS(LOTACAO)
            AS
            (
                SELECT de_lotacao
                FROM SAGRES_ESTADUAL.dbo.t_FL_Servidor f
                    LEFT OUTER JOIN SAGRES_ESTADUAL.dbo.Ambito A on (f.cd_Ambito = a.cd_ambito)
                    LEFT OUTER JOIN SAGRES_ESTADUAL.dbo.t_FL_Cargo_02 c on (f.cd_Cargo = c.cd_CargoNovo and f.tp_Cargo = c.tp_Cargo and f.dt_Ano = c.dt_Ano and f.cd_Ambito = c.cd_Ambito)
                    LEFT OUTER JOIN SAGRES_ESTADUAL.dbo.t_FL_Lotacao l on (f.cd_Lotacao = l.cd_Lotacao)
                WHERE nu_CPF IN (${lista})

                UNION

                SELECT de_lotacao
                FROM SAGRES_ESTADUAL.dbo.FL_Servidor f
                    LEFT OUTER JOIN SAGRES_ESTADUAL.dbo.Ambito A on (f.cd_Ambito = a.cd_ambito)
                    LEFT OUTER JOIN SAGRES_ESTADUAL.dbo.FL_Cargo c on (f.cd_Cargo = c.cd_Cargo and f.tp_Cargo = c.tp_Cargo and f.dt_Ano = c.dt_Ano and f.cd_Ambito = c.cd_Ambito)
                    LEFT OUTER JOIN SAGRES_ESTADUAL.dbo.FL_Lotacao l on (f.cd_Lotacao = l.cd_Lotacao)
                WHERE nu_CPF IN (${lista})
            )

            SELECT LOTACAO as id, 'orgaopublico' as entidade, LOTACAO as uGestora, 'E' as esfera
            FROM VINCULOS_ESTADUAIS

        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getNodesOrgaosPublicosEstaduaisPessoaCPF');

            return result.recordset.map(resultado => trataResultado(resultado));
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelos nodes de ${listaCpf}.`, 'getNodesOrgaosPublicosEstaduaisPessoaCPF'); });
};

export let getEdgesOrgaosPublicosEstaduaisPessoaCPF = function (listaCpf: Array<string>, tipobusca: string = 'completa') {

    if (tipobusca !== 'orgaospublicos' && tipobusca !== 'completa') { return []; }
    if (listaCpf.length === 0) { return []; }

    const lista = "'" + listaCpf.join("','") + "'";
    const tempoInicial = new BenchmarkService();

    return connection.getConn().request()
        .query(`
            ;WITH VINCULOS_ESTADUAIS(AMBITO, LOTACAO, CPF, CARGO, PRIMEIRA, ULTIMA)
            AS
            (
                SELECT UPPER(A.de_ambito) as ambito , TRIM(l.de_Lotacao), f.nu_CPF, TRIM(c.de_Cargo), min(f.dt_Ano) as minimo , max(f.dt_ano) as maximo
                FROM SAGRES_ESTADUAL.dbo.t_FL_Servidor f
                    LEFT OUTER JOIN SAGRES_ESTADUAL.dbo.Ambito A on (f.cd_Ambito = a.cd_ambito)
                    LEFT OUTER JOIN SAGRES_ESTADUAL.dbo.t_FL_Cargo_02 c on (f.cd_Cargo = c.cd_CargoNovo and f.tp_Cargo = c.tp_Cargo and f.dt_Ano = c.dt_Ano and f.cd_Ambito = c.cd_Ambito)
                    LEFT OUTER JOIN SAGRES_ESTADUAL.dbo.t_FL_Lotacao l on (f.cd_Lotacao = l.cd_Lotacao)
                WHERE nu_CPF IN (${lista})
                GROUP BY A.de_ambito, l.de_Lotacao, f.nu_CPF, c.de_Cargo

                UNION

                SELECT UPPER(A.de_ambito) as ambito , TRIM(l.de_Lotacao), f.nu_CPF, TRIM(c.de_Cargo), min(f.dt_Ano) as minimo , max(f.dt_ano) as maximo
                FROM SAGRES_ESTADUAL.dbo.FL_Servidor f
                    LEFT OUTER JOIN SAGRES_ESTADUAL.dbo.Ambito A on (f.cd_Ambito = a.cd_ambito)
                    LEFT OUTER JOIN SAGRES_ESTADUAL.dbo.FL_Cargo c on (f.cd_Cargo = c.cd_Cargo and f.tp_Cargo = c.tp_Cargo and f.dt_Ano = c.dt_Ano and f.cd_Ambito = c.cd_Ambito)
                    LEFT OUTER JOIN SAGRES_ESTADUAL.dbo.FL_Lotacao l on (f.cd_Lotacao = l.cd_Lotacao)
                WHERE nu_CPF IN (${lista})
                GROUP BY A.de_ambito, l.de_Lotacao, f.nu_CPF, c.de_Cargo
            )

            SELECT CONCAT(CPF, '-', CARGO, '-', LOTACAO) as id, CPF as origem, CARGO as relacao, LOTACAO as destino, MIN(PRIMEIRA) as pOcorrencia, MAX(ULTIMA) as uOcorrencia, AMBITO as ambito
            FROM VINCULOS_ESTADUAIS
            GROUP BY AMBITO, LOTACAO, CPF, CARGO
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getEdgesOrgaosPublicosEstaduaisPessoaCPF');

            return result.recordset.map(resultado => trataResultado(resultado));
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelos nodes de ${listaCpf}.`, 'getEdgesOrgaosPublicosEstaduaisPessoaCPF'); });
};



export let getParentescosCPF = function (cpf: string) {

    const tempoInicial = new BenchmarkService();
    return connection.getConn().request()
        .input('cpf', mssql.Char(11), cpf)
        .query(`
            ;WITH VINCULOS(NIVEL, CATEGORIA, CPF, NOME, SEXO)
            AS
            (
                SELECT NIVEL, CATEGORIA, CPF, NOME, SEXO
                FROM BD_RELACAO.dbo.FN_RELACOES_ATE_TERCEIRO_GRAU_MAIS_PRIMOS_COM_NOMES_E_SEXO_RESUMIDO (
                    @cpf,
                    NULL,
                    NULL
                )
            )
            ,PESSOAS(NIVEL, CATEGORIA, CPF, NOME, SEXO, IDADE, DATANASCIMENTO, MUNICIPIO, UF)
            AS
            (
                SELECT NIVEL, CATEGORIA, PF.CPF, PF.NOME, UPPER(Sexo.Descricao), DATEDIFF(YEAR, PF.DataNascimento, GETDATE()), PF.DataNascimento, PF.Municipio, PF.uf
                FROM VINCULOS V
                    INNER JOIN ReceitaNovo.dbo.PessoaFisica PF ON (V.CPF = PF.CPF)
                    INNER JOIN ReceitaNovo.dbo.Sexo ON (PF.Sexo = Sexo.SexoID)
            )

            SELECT NIVEL as nivel, CATEGORIA as categoria, CPF as cpf, NOME as nome, SEXO as sexo, IDADE as idade, DATANASCIMENTO as dataNascimento, MUNICIPIO as municipio, UF as uf
            FROM PESSOAS
            ORDER BY NIVEL, CATEGORIA, NOME
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getParentescosCPF');

            if (result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar a pessoa ${cpf}.`, 'BD_RELACAO');
            else
                return resultFound(result, 'BD_RELACAO', 0, 'parentesco');
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pela pessoa ${cpf}.`, 'getParentescosCPF'); });
};