import * as mssql from 'mssql';

import { BenchmarkService } from './../../services/benchmark.service';
import { connection } from '../../services/db.service';
import { MODEL_PRIORITY } from './../../config';

import {
    trataResultado,
    resultNotFound,
    resultFound,
    logErroBuscaBD,
    printTempoExecucao
} from '../../utils';

/**
 * Retorna as entidades 'Pessoa' que possuem esse telefone.
 * @param cpf 
 * @param tipobusca 
 */
export let getNodesPessoasTelefone = function (listaTelefone: Array<string>, tipobusca: string = 'completa'){

    if (tipobusca !== 'parentesco' && tipobusca !== 'completa') { return []; }
    if (listaTelefone.length === 0) { return []; }

    const lista = "'" + listaTelefone.join("','") + "'";
    const tempoInicial = new BenchmarkService();

    return connection.getConn().request()
        .query(`
            ;WITH PESSOAS_COMUNS(cpf)
            AS
            (
                SELECT DISTINCT cpf_cnpj
                FROM SISPESQUISA.sispesquisa.telefones
                WHERE telefone IN (${lista})
            )
            ,PESSOAS(CPF, NOME, NomeMae, DataNascimento, Sexo, Municipio, UF)
            AS
            (
                SELECT PF.CPF, NOME, NomeMae, DataNascimento, Sexo, Municipio, UF
                FROM ReceitaNovo.dbo.PessoaFisica PF
                    INNER JOIN PESSOAS_COMUNS P ON (PF.CPF = P.CPF)
            )
            ,DADOS_COM_PAI(CPF, NOME, NomeMae, NomePai, DataNascimento, Sexo, Municipio, UF)
            AS
            (
                SELECT P.CPF, P.NOME, P.NomeMae, E.NOM_PAI, P.DataNascimento, P.Sexo, P.Municipio, P.UF
                FROM PESSOAS P
                    LEFT OUTER JOIN TSE.dbo.ELEITOR E ON (P.CPF = E.CPF_RF)
            )

            SELECT CPF as id, 'pessoa' as entidade, CPF as cpf, NOME as nome, NomeMae as nomeMae, NomePai as nomePai, DataNascimento as dataNascimento, Sexo as sexo, Municipio as municipio, UF as uf
            FROM DADOS_COM_PAI
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getNodesPessoasTelefone');

            return result.recordset.map(resultado => trataResultado(resultado));
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelos nodes de ${listaTelefone}.`, 'getNodesPessoasTelefone'); });
};

export let getEdgesPessoasTelefone = function (listaTelefone: Array<string>, tipobusca: string = 'completa') {

    if (tipobusca !== 'parentesco' && tipobusca !== 'completa') { return []; }
    if (listaTelefone.length === 0) { return []; }

    const lista = "'" + listaTelefone.join("','") + "'";
    const tempoInicial = new BenchmarkService();

    return connection.getConn().request()
        .query(`
            ;WITH PESSOAS(cpf, TELEFONE)
            AS
            (
                SELECT DISTINCT cpf_cnpj, telefone
                FROM SISPESQUISA.sispesquisa.telefones
                WHERE telefone IN (${lista})
            )

            SELECT telefone as origem, 'propriedade' as relacao, cpf as destino
            FROM PESSOAS
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getEdgesPessoasTelefone');

            return result.recordset.map(resultado => trataResultado(resultado));
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelos edges de ${listaTelefone}.`, 'getEdgesPessoasTelefone'); });
};
