import * as mssql from 'mssql';

import { BenchmarkService } from './../../services/benchmark.service';
import { connection } from '../../services/db.service';
import { MODEL_PRIORITY } from './../../config';

import {
    trataResultado,
    resultNotFound,
    resultFound,
    logErroBuscaBD,
    printTempoExecucao
} from '../../utils';

const fonte   = MODEL_PRIORITY['sagres_estadual.empenhos'].fonte;
const rank    = MODEL_PRIORITY['sagres_estadual.empenhos'].rank;
const grupo   = MODEL_PRIORITY['sagres_estadual.empenhos'].grupo;

export let getNodesPFCPF = function (listaCpf){

    if (listaCpf.length === 0) { return []; }

    const lista = "'" + listaCpf.join("','") + "'";
    const tempoInicial = new BenchmarkService();

    return connection.getConn().request()
        .query(`
            ;WITH ALVO_PF_INICIAL(CPF, NOME, NomeMae, DataNascimento, Sexo, Municipio, UF)
            AS
            (   
                SELECT CPF, NOME, NomeMae, DataNascimento, UPPER(Sexo.Descricao), Municipio, UF
                FROM ReceitaNovo.dbo.PessoaFisica PF
                    INNER JOIN ReceitaNovo.dbo.Sexo ON (PF.Sexo = Sexo.SexoID)
                WHERE PF.CPF IN (${lista})
            )
            ,VINCULOS_DIRETOS_A(CPF, RELACAO)
            AS
            (
                SELECT CPF2, RELACAO
                FROM ALVO_PF_INICIAL A 
                    INNER JOIN BD_RELACAO.dbo.RELACAO_PF_PF R ON (A.CPF = R.CPF1)
                WHERE relacao IN ('MÃE', 'PAI', 'IRMÃO', 'IRMÃ', 'CÔNJUGE/COMPANHEIRO', 'CÔNJUGE/COMPANHEIRA', 'PAI/PADRASTO', 'MÃE/MADRASTA', 'COMPANHEIRO', 'COMPANHEIRA', 'CÔNJUGE')	
            )
            ,VINCULOS_DIRETOS_B(CPF, RELACAO)
            AS
            (
                SELECT CPF1, RELACAO
                FROM ALVO_PF_INICIAL A 
                    INNER JOIN BD_RELACAO.dbo.RELACAO_PF_PF R ON (A.CPF = R.CPF2)
                WHERE relacao IN ('MÃE', 'PAI', 'IRMÃO', 'IRMÃ', 'CÔNJUGE/COMPANHEIRO', 'CÔNJUGE/COMPANHEIRA', 'PAI/PADRASTO', 'MÃE/MADRASTA', 'COMPANHEIRO', 'COMPANHEIRA', 'CÔNJUGE')	
            )
            ,VINCULOS_DIRETOS(CPF, RELACAO)
            AS
            (
                SELECT CPF, RELACAO	FROM VINCULOS_DIRETOS_A
                UNION
                SELECT CPF, RELACAO	FROM VINCULOS_DIRETOS_B
            )
            ,ALVO_PF_COM_VINCULOS(CPF, NOME, NomeMae, DataNascimento, Sexo, Municipio, UF)
            AS
            (
                SELECT PF.CPF, NOME, NomeMae, DataNascimento, UPPER(Sexo.Descricao), Municipio, UF
                FROM VINCULOS_DIRETOS V 
                    INNER JOIN ReceitaNovo.dbo.PessoaFisica PF ON (V.CPF = PF.CPF)
                    INNER JOIN ReceitaNovo.dbo.Sexo ON (PF.Sexo = Sexo.SexoID)

                UNION

                SELECT *
                FROM ALVO_PF_INICIAL
            )
            ,ALVO_PF(CPF, NOME, NomeMae, NomePai, DataNascimento, Sexo, Municipio, UF)
            AS
            (
                SELECT A.CPF, A.NOME, A.NomeMae, E.NOM_PAI, A.DataNascimento, A.Sexo, A.Municipio, A.UF
                FROM ALVO_PF_COM_VINCULOS A
                    LEFT OUTER JOIN TSE.dbo.ELEITOR E ON (A.CPF = E.CPF_RF)
            )

            SELECT CPF as id, 'pessoa' as entidade, CPF as cpf, NOME as nome, NomeMae as nomeMae, NomePai as nomePai, DataNascimento as dataNascimento, Sexo as sexo, Municipio as municipio, UF as uf
            FROM ALVO_PF
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getNodesPFCPF');

            return result.recordset.map(resultado => trataResultado(resultado));
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelos nodes de ${lista}.`, 'getNodesPFCPF'); });
};


export let getVinculosPFCPF = function (listaCpf) {

    if (listaCpf.length === 0) { return []; }

    const lista = "'" + listaCpf.join("','") + "'";
    const tempoInicial = new BenchmarkService();

    return connection.getConn().request()
        .query(`
            ;WITH ALVOS_PF_INICIAL(CPF, NOME, NomeMae, DataNascimento, Sexo, Municipio, UF)
            AS
            (   
                SELECT CPF, NOME, NomeMae, DataNascimento, Sexo, Municipio, UF
                FROM ReceitaNovo.dbo.PessoaFisica PF
                WHERE PF.CPF IN (${lista})
            )
            ,VINCULOS_PARENTES_A(CPF_ORIGEM, RELACAO, CPF_DESTINO)
            AS
            (
                SELECT CPF1, RELACAO, CPF2
                FROM ALVOS_PF_INICIAL A 
                    INNER JOIN BD_RELACAO.dbo.RELACAO_PF_PF R ON (A.CPF = R.CPF1)		
                WHERE relacao IN ('MÃE', 'PAI', 'IRMÃO', 'IRMÃ', 'CÔNJUGE/COMPANHEIRO', 'CÔNJUGE/COMPANHEIRA', 'PAI/PADRASTO', 'MÃE/MADRASTA', 'COMPANHEIRO', 'COMPANHEIRA', 'CÔNJUGE')	
            )
            ,VINCULOS_PARENTES_B(CPF_ORIGEM, RELACAO, CPF_DESTINO)
            AS
            (
                SELECT CPF2, RELACAO, CPF1
                FROM ALVOS_PF_INICIAL A 
                    INNER JOIN BD_RELACAO.dbo.RELACAO_PF_PF R ON (A.CPF = R.CPF2)		
                WHERE relacao IN ('MÃE', 'PAI', 'IRMÃO', 'IRMÃ', 'CÔNJUGE/COMPANHEIRO', 'CÔNJUGE/COMPANHEIRA', 'PAI/PADRASTO', 'MÃE/MADRASTA', 'COMPANHEIRO', 'COMPANHEIRA', 'CÔNJUGE')	
            )
            ,SOCIOS_EMPRESA(CPF, RELACAO, CNPJ)
            AS
            (
                SELECT A.CPF, 'SOCIO', PJ.CNPJ
                FROM ALVOS_PF_INICIAL A 
                    INNER JOIN ReceitaNovo.dbo.Socio S ON ('000' + A.CPF = S.CpfSocio)	
                    INNER JOIN ReceitaNovo.dbo.PessoaJuridica PJ ON (S.Cnpj = PJ.CNPJ)  
            )
            ,RESPONSAVEL_EMPRESA(CPF, RELACAO, CNPJ)
            AS
            (
                SELECT A.CPF, 'RESPONSAVEL', PJ.CNPJ
                FROM ALVOS_PF_INICIAL A 
                    INNER JOIN ReceitaNovo.dbo.PessoaJuridica PJ ON (A.CPF = PJ.CpfResponsavel)  
            )
            ,VINCULOS_DIRETOS(ORIGEM, RELACAO, DESTINO)
            AS
            (
                SELECT * FROM VINCULOS_PARENTES_A
                UNION
                SELECT * FROM VINCULOS_PARENTES_B
                UNION
                SELECT * FROM SOCIOS_EMPRESA
                UNION
                SELECT * FROM RESPONSAVEL_EMPRESA
            )

            SELECT ORIGEM as origem, RELACAO as relacao, DESTINO as destino
            FROM VINCULOS_DIRETOS 
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getVinculosPFCPF');

            return result.recordset.map(resultado => trataResultado(resultado));
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelos nodes de ${lista}.`, 'getVinculosPFCPF'); });
};


export let getNodesPFCNPJ = function (listaCnpj) {

    if (listaCnpj.length === 0) { return []; }

    const lista = "'" + listaCnpj.join("','") + "'";
    const tempoInicial = new BenchmarkService();

    return connection.getConn().request()
        .query(`
        ;WITH ALVO_PJ_INICIAL(CNPJ, RAZAOSOCIAL, NOMEFANTASIA, DTINICIOATIVIDADE, Municipio, UF)
        AS
        (   
            SELECT CNPJ, RazaoSocial, NomeFantasia, cast(DataInicioAtividade as date), Municipio, uf
            FROM ReceitaNovo.dbo.PessoaJuridica PJ
            WHERE PJ.CNPJ IN (${lista})
        )
        ,SOCIOS_PF_EMPRESA(CPF, NOME, NomeMae, DataNascimento, Sexo, Municipio, UF)
        AS
        (
            SELECT CPF, NOME, NomeMae, DataNascimento, UPPER(Sexo.Descricao), PF.Municipio, PF.UF
            FROM ALVO_PJ_INICIAL A 
                INNER JOIN ReceitaNovo.dbo.Socio S ON (A.CNPJ = S.Cnpj)	
                INNER JOIN ReceitaNovo.dbo.PessoaFisica PF ON (RIGHT(S.CpfSocio,11) = PF.CPF)
                INNER JOIN ReceitaNovo.dbo.Sexo ON (PF.Sexo = Sexo.SexoID)
            WHERE S.IdentificadorSocio = 2
        )
        ,RESPONSAVEL_EMPRESA(CPF, NOME, NomeMae, DataNascimento, Sexo, Municipio, UF)
        AS
        (
            SELECT CPF, NOME, NomeMae, DataNascimento, UPPER(Sexo.Descricao), PF.Municipio, PF.UF
            FROM ALVO_PJ_INICIAL A 
                INNER JOIN ReceitaNovo.dbo.PessoaJuridica PJ ON (A.CNPJ = PJ.CNPJ)
                INNER JOIN ReceitaNovo.dbo.PessoaFisica PF ON (PJ.CpfResponsavel = PF.CPF)
                INNER JOIN ReceitaNovo.dbo.Sexo ON (PF.Sexo = Sexo.SexoID)
        )
        ,PESSOAS(CPF, NOME, NomeMae, DataNascimento, Sexo, Municipio, UF)
        AS
        (
            SELECT * FROM SOCIOS_PF_EMPRESA
            UNION
            SELECT * FROM RESPONSAVEL_EMPRESA	
        )

        SELECT CPF as id, 'pessoa' as entidade, CPF as cpf, NOME as nome, NomeMae as nomeMae, DataNascimento as dataNascimento, Sexo as sexo, Municipio as municipio, UF as uf
        FROM PESSOAS
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getNodesPFCNPJ');

            return result.recordset.map(resultado => trataResultado(resultado));
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelos nodes de ${lista}.`, 'getNodesPFCNPJ'); });
};