import * as mssql from 'mssql';

import { BenchmarkService } from './../../services/benchmark.service';
import { connection } from '../../services/db.service';
import { MODEL_PRIORITY } from './../../config';

import {
    trataResultado,
    resultNotFound,
    resultFound,
    logErroBuscaBD,
    printTempoExecucao
} from '../../utils';

const fonte = MODEL_PRIORITY['sagres_estadual.empenhos'].fonte;
const rank  = MODEL_PRIORITY['sagres_estadual.empenhos'].rank;
const grupo = MODEL_PRIORITY['sagres_estadual.empenhos'].grupo;

export let getNodesPJSocioCPF = function (listaCpf){

    if (listaCpf.length === 0) { return []; }

    const lista = "'" + listaCpf.join("','") + "'";
    const tempoInicial = new BenchmarkService();

    return connection.getConn().request()
        .query(`
            ;WITH ALVO_PF_INICIAL(CPF, NOME, NomeMae, DataNascimento, Sexo, Municipio, UF)
            AS
            (
                SELECT CPF, NOME, NomeMae, DataNascimento, Sexo, Municipio, UF
                FROM ReceitaNovo.dbo.PessoaFisica PF
                WHERE PF.CPF IN (${lista})
            )
            ,SOCIOS_EMPRESA(CNPJ, RAZAOSOCIAL, NOMEFANTASIA, DTINICIOATIVIDADE, DTSITUACAOCADASTRAL, MUNICIPIO, UF)
            AS
            (
                SELECT PJ.CNPJ, PJ.RazaoSocial, PJ.NomeFantasia, PJ.DataInicioAtividade, PJ.DataSituacaoCadastral, PJ.Municipio, PJ.UF
                FROM ALVO_PF_INICIAL A 
                    INNER JOIN ReceitaNovo.dbo.Socio S ON ('000' + A.CPF = S.CpfSocio)
                    INNER JOIN ReceitaNovo.dbo.PessoaJuridica PJ ON (S.Cnpj = PJ.CNPJ)
            )
            ,RESPONSAVEL_EMPRESA(CNPJ, RAZAOSOCIAL, NOMEFANTASIA, DTINICIOATIVIDADE, DTSITUACAOCADASTRAL, MUNICIPIO, UF)
            AS
            (
                SELECT PJ.CNPJ, PJ.RazaoSocial, PJ.NomeFantasia, PJ.DataInicioAtividade, PJ.DataSituacaoCadastral, PJ.Municipio, PJ.UF
                FROM ALVO_PF_INICIAL A 
                    INNER JOIN ReceitaNovo.dbo.PessoaJuridica PJ ON (A.CPF = PJ.CpfResponsavel)
            )
            ,EMPRESAS(CNPJ, RAZAOSOCIAL, NOMEFANTASIA, DTINICIOATIVIDADE, DTSITUACAOCADASTRAL, MUNICIPIO, UF)
            AS
            (
                SELECT * FROM SOCIOS_EMPRESA
                UNION ALL
                SELECT * FROM RESPONSAVEL_EMPRESA
            )

            SELECT CNPJ as id, 'empresa' as entidade, CNPJ as cnpj, RAZAOSOCIAL as razaoSocial, NOMEFANTASIA as nomeFantasia, cast(DTINICIOATIVIDADE as date) as dataInicioAtividade, MUNICIPIO as municipio, UF as uf
            FROM EMPRESAS
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getNodesPJSocioCPF');
            
            return result.recordset.map(resultado => trataResultado(resultado));
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelos nodes de ${lista}.`, 'getNodesPJSocioCPF'); });
};


export let getNodesPJCNPJ = function (listaCnpj){

    if (listaCnpj.length === 0) { return []; }
    const lista = "'" + listaCnpj.join("','") + "'";
    const tempoInicial = new BenchmarkService();

    return connection.getConn().request()
        .query(`
            ;WITH ALVO_PJ_INICIAL(ID, CNPJ, RAZAOSOCIAL, NOMEFANTASIA, DTINICIOATIVIDADE, Municipio, UF)
            AS
            (   
                SELECT CNPJ, CNPJ, RazaoSocial, NomeFantasia, cast(DataInicioAtividade as date), Municipio, uf
                FROM ReceitaNovo.dbo.PessoaJuridica PJ
                WHERE PJ.CNPJ IN (${lista})
            )
            ,SOCIOS_EMPRESA_A(ID, CNPJ, RAZAOSOCIAL, NOMEFANTASIA, DTINICIOATIVIDADE, MUNICIPIO, UF)
            AS
            (
                SELECT PJ.CNPJ, PJ.CNPJ, PJ.RazaoSocial, PJ.NomeFantasia, PJ.DataInicioAtividade, PJ.Municipio, PJ.UF
                FROM ALVO_PJ_INICIAL A 
                    INNER JOIN ReceitaNovo.dbo.Socio S ON (A.CNPJ = S.CpfSocio)	
                    INNER JOIN ReceitaNovo.dbo.PessoaJuridica PJ ON (S.Cnpj = PJ.CNPJ)  
            )
            ,SOCIOS_EMPRESA_B(ID, CNPJ, RAZAOSOCIAL, NOMEFANTASIA, DTINICIOATIVIDADE, MUNICIPIO, UF)
            AS
            (
                SELECT PJ.CNPJ, PJ.CNPJ, PJ.RazaoSocial, PJ.NomeFantasia, PJ.DataInicioAtividade, PJ.Municipio, PJ.UF
                FROM ALVO_PJ_INICIAL A 
                    INNER JOIN ReceitaNovo.dbo.Socio S ON (A.CNPJ = S.Cnpj)	
                    INNER JOIN ReceitaNovo.dbo.PessoaJuridica PJ ON (S.CpfSocio = PJ.CNPJ) 
                WHERE S.IdentificadorSocio = 1 
            )
            ,SOCIOS_ESTRANGEIRO_EMPRESA(ID, CNPJ, RAZAOSOCIAL, NOMEFANTASIA, DTINICIOATIVIDADE, MUNICIPIO, UF)
            AS
            (
                SELECT S.NomeSocio, NULL, S.NomeSocio COLLATE Latin1_General_CI_AI, NULL, NULL, S.NomePaisSocio COLLATE Latin1_General_CI_AI, NULL
                FROM ALVO_PJ_INICIAL A 
                    INNER JOIN ReceitaNovo.dbo.Socio S ON (A.CNPJ = S.Cnpj)	
                WHERE S.IdentificadorSocio = 3
            )
            ,EMPRESAS(ID, CNPJ, RAZAOSOCIAL, NOMEFANTASIA, DTINICIOATIVIDADE, MUNICIPIO, UF)
            AS
            (
                SELECT * FROM ALVO_PJ_INICIAL
                UNION
                SELECT * FROM SOCIOS_EMPRESA_A
                UNION 
                SELECT * FROM SOCIOS_EMPRESA_B
                UNION
                SELECT * FROM SOCIOS_ESTRANGEIRO_EMPRESA
            )

            SELECT ID as id, 'empresa' as entidade, CNPJ as cnpj, RAZAOSOCIAL as razaoSocial, NOMEFANTASIA as nomeFantasia, cast(DTINICIOATIVIDADE as date) as dataInicioAtividade, MUNICIPIO as municipio, UF as uf
            FROM EMPRESAS
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getNodesPJCNPJ');

            return result.recordset.map(resultado => trataResultado(resultado));
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelos nodes de ${lista}.`, 'getNodesPJCNPJ'); });
};

export let getVinculosPJCNPJ = function (listaCnpj){

    if (listaCnpj.length === 0) { return []; }
    const lista = "'" + listaCnpj.join("','") + "'";
    const tempoInicial = new BenchmarkService();

    return connection.getConn().request()
        .query(`
            ;WITH ALVO_PJ_INICIAL(CNPJ, RAZAOSOCIAL, NOMEFANTASIA, DTINICIOATIVIDADE, Municipio, UF)
            AS
            (   
                SELECT CNPJ, RazaoSocial, NomeFantasia, cast(DataInicioAtividade as date), Municipio, uf
                FROM ReceitaNovo.dbo.PessoaJuridica PJ
                WHERE PJ.CNPJ IN (${lista})
            )
            ,SOCIOS_PJ_EMPRESA_A(ORIGEM, RELACAO, DESTINO)
            AS
            (
            SELECT S.CpfSocio, 'SOCIOPJA', S.Cnpj
            FROM ALVO_PJ_INICIAL A 
                INNER JOIN ReceitaNovo.dbo.Socio S ON (A.CNPJ = S.CpfSocio)	
            )
            ,SOCIOS_PJ_EMPRESA_B(ORIGEM, RELACAO, DESTINO)
            AS
            (
            SELECT S.Cnpj, 'SOCIOPJB', S.CpfSocio
            FROM ALVO_PJ_INICIAL A 
                INNER JOIN ReceitaNovo.dbo.Socio S ON (A.CNPJ = S.Cnpj)	
            WHERE S.IdentificadorSocio = 1
            )
            ,SOCIOS_PF_EMPRESA(ORIGEM, RELACAO, DESTINO)
            AS
            (
            SELECT S.Cnpj, 'SOCIOPF', RIGHT(S.CpfSocio,11)
            FROM ALVO_PJ_INICIAL A 
                INNER JOIN ReceitaNovo.dbo.Socio S ON (A.CNPJ = S.Cnpj)	
            WHERE S.IdentificadorSocio = 2
            )
            ,SOCIOS_ESTRANGEIROS_EMPRESA(ORIGEM, RELACAO, DESTINO)
            AS
            (
            SELECT S.Cnpj, 'SOCIOESTRANGEIRO', S.NomeSocio
            FROM ALVO_PJ_INICIAL A 
                INNER JOIN ReceitaNovo.dbo.Socio S ON (A.CNPJ = S.Cnpj)	
            WHERE S.IdentificadorSocio = 3
            )
            ,VINCULOS(ORIGEM, RELACAO, DESTINO)
            AS
            (
                SELECT * FROM SOCIOS_PF_EMPRESA
                UNION ALL
                SELECT * FROM SOCIOS_PJ_EMPRESA_A
                UNION ALL
                SELECT * FROM SOCIOS_PJ_EMPRESA_B
                UNION ALL
                SELECT * FROM SOCIOS_ESTRANGEIROS_EMPRESA
            )

            SELECT ORIGEM as origem, RELACAO as relacao, DESTINO as destino
            FROM VINCULOS

        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getVinculosPJCNPJ');

            return result.recordset.map(resultado => trataResultado(resultado));
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelos nodes de ${lista}.`, 'getVinculosPJCNPJ'); });
};
