import * as mssql from 'mssql';

import { connection }  from '../services/db.service';
import { Pessoa } from './schemas';
import { MODEL_PRIORITY } from './../config';
import {
    trataResultado,
    resultNotFound,
    resultFound,
    logErroBuscaBD
} from './../utils';

const fonte   = MODEL_PRIORITY['rif'].fonte;
const rank    = MODEL_PRIORITY['rif'].rank;
const grupo   = MODEL_PRIORITY['rif'].grupo;

export let existeRIFCPF_Sispesquisa_RIF = function (cpf: string){

    return connection.getConn().request()
        .input('cpf', mssql.NVarChar, cpf)
        .query(`
            SELECT qtd
            FROM (
                SELECT COUNT(*) as qtd
                FROM SISPESQUISA.dbo.RIF_PF
                WHERE Documento=@cpf
            ) AS RIF
            WHERE RIF.qtd > 0
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar RIF de ${cpf}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pelo RIF de ${cpf}.`, 'existeRIFCPF_Sispesquisa_RIF'); });
};

export let existeRIFCNPJ_Sispesquisa_RIF = function (cnpj: string){

    return connection.getConn().request()
        .input('cnpj', mssql.NVarChar, cnpj)
        .query(`
            SELECT qtd
            FROM (
                SELECT COUNT(*) as qtd
                FROM SISPESQUISA.dbo.RIF_PJ
                WHERE Documento=@cnpj
            ) AS RIF
            WHERE RIF.qtd > 0
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar RIF de ${cnpj}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pelo RIF de ${cnpj}.`, 'existeRIFCNPJ_Sispesquisa_RIF'); });
};
