import * as mssql from 'mssql';

import { connection }  from '../../services/db.service';
import { resultNotFound, resultFound, logErroBuscaBD } from '../../utils';
import { MODEL_PRIORITY, API_CONFIG } from '../../config';

const fonte  = MODEL_PRIORITY['sagres_estadual.vw_folha'].fonte;
const rank   = MODEL_PRIORITY['sagres_estadual.vw_folha'].rank;
const grupo  = MODEL_PRIORITY['sagres_estadual.vw_folha'].grupo;

export let getServidorEstadualDetalhadoCPF_SAGRES_ESTADUAL_VW_FOLHA = function (cpf: string){

    return connection.getConn().request()
        .input('cpf_parameter', mssql.Char(11), cpf)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                    dt_Ano as dtAno, vl_Liquido as vLiquido, UPPER(de_Cargo) as cargo,
                    UPPER(de_Tipo) as tipoCargo, UPPER(de_Lotacao) as lotacao

                FROM SAGRES_ESTADUAL.dbo.VW_FOLHA
                WHERE nu_CPF=@cpf_parameter
                ORDER BY dt_ano desc
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o servidor estadual ${cpf}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelo servidor estadual ${cpf}.`, 'getServidorEstadualDetalhadoCPF_SAGRES_ESTADUAL_VW_FOLHA')});
};

export let getServidorEstadualSimplificadoCPF_SAGRES_ESTADUAL_VW_FOLHA = function(cpf: string){

    return connection.getConn().request()
        .input('cpf_parameter', mssql.Char(11), cpf)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                    dt_Ano as dtAno, vl_Liquido as vLiquido, UPPER(de_Cargo) as cargo,
                    UPPER(de_Tipo) as tipoCargo, UPPER(de_Lotacao) as lotacao

                FROM SAGRES_ESTADUAL.dbo.VW_FOLHA
                WHERE nu_CPF=@cpf_parameter
                ORDER BY dt_ano desc
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o servidor estadual ${cpf}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelo servidor estadual ${cpf}.`, 'getServidorEstadualSimplificadoCPF_SAGRES_ESTADUAL_VW_FOLHA') })
}