import * as mssql from 'mssql';

import { connection }  from '../../services/db.service';
import { MODEL_PRIORITY, API_CONFIG } from '../../config';
import {
    resultNotFound,
    resultFound,
    logErroBuscaBD
} from '../../utils';

const fonte  = MODEL_PRIORITY['sispesquisa.servidores_estaduais'].fonte;
const rank   = MODEL_PRIORITY['sispesquisa.servidores_estaduais'].rank;
const grupo  = MODEL_PRIORITY['sispesquisa.servidores_estaduais'].grupo;

export let getServidorEstadualDetalhadoCPF_Sispesquisa_Servidores_Estaduais = function (cpf: string){

    return connection.getConn().request()
        .input('cpf_parameter', mssql.NVarChar, cpf)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        dt_Ano as dtAno, nu_CPF as cpf,  nu_Matricula as matricula, dt_Nascimento as dtNascimento,
                        dt_Admissao as dtAdmissao, dt_Aposentadoria as dtAposentadoria, de_Cargo as cargo, de_Lotacao as lotacao,
                        vl_Provento as provento, vl_Desconto as desconto, fonte

                FROM SISPESQUISA.sispesquisa.servidores_estaduais
                WHERE nu_CPF=@cpf_parameter
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o servidor estadual ${cpf}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pelo servidor estadual ${cpf}.`, 'getServidorEstadualDetalhadoCPF_Sispesquisa_Servidores_Estaduais')});
};

export let getServidorEstadualSimplificadoCPF_Sispesquisa_Servidores_Estaduais = function(cpf: string){

    return connection.getConn().request()
        .input('cpf_parameter', mssql.NVarChar, cpf)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                    de_Cargo as cargo, de_Lotacao as lotacao, dt_Ano as dtAno, SUM(vl_Provento-vl_Desconto) as valorLiquido

                FROM SISPESQUISA.sispesquisa.servidores_estaduais
                WHERE nu_CPF=@cpf_parameter
                GROUP BY dt_Ano, de_Cargo, de_Lotacao
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o servidor estadual ${cpf}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pelo servidor estadual ${cpf}.`, 'getServidorEstadualSimplificadoCPF_Sispesquisa_Servidores_Estaduais')});
}