import * as mssql from 'mssql';

import { connection }  from '../../services/db.service';
import { MODEL_PRIORITY, API_CONFIG } from '../../config';
import {
    resultNotFound,
    resultFound,
    logErroBuscaBD
} from '../../utils';


import { SIGLA_FONTE } from './../_fontes';

const tipo = SIGLA_FONTE['sispesquisa.servidores_federais_nordeste'];

const fonte = MODEL_PRIORITY['sispesquisa.servidores_federais_nordeste'].fonte;
const rank  = MODEL_PRIORITY['sispesquisa.servidores_federais_nordeste'].rank;
const grupo = MODEL_PRIORITY['sispesquisa.servidores_federais_nordeste'].grupo;

export let getServidorFederalCPF_Sispesquisa_Servidores_Federais_Nordeste = function (cpf: string){

    return connection.getConn().request()
        .input('cpf_parameter', mssql.NVarChar, cpf)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                    CPF_SERVIDOR as cpf, CARGO as cargo, ORGAO as orgLotacao, UORG as unidOrgLotacao,
                    UF_ORGANIZACAO as ufOrg, UPAG as uPag, JORNADA_TRABALHO as jornada,
                    SITUACAO_VINCULO as situacao, REMUN as remuneracao, '${tipo}' as tipo

                FROM SISPESQUISA.dbo.SERVIDORES_FEDERAIS_NORDESTE
                WHERE CPF_SERVIDOR=@cpf_parameter
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o servidor federal ${cpf}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelo servidor federal ${cpf}.`, 'getServidorFederalCPF_Sispesquisa_Servidores_Federais_Nordeste')});
};