import * as mssql from 'mssql';

import { connection }  from '../../services/db.service';
import { MODEL_PRIORITY, API_CONFIG } from '../../config';
import {
    resultNotFound,
    resultFound,
    logErroBuscaBD
} from '../../utils';


import { SIGLA_FONTE } from './../_fontes';

const tipo = SIGLA_FONTE['sispesquisa.servidores_federais'];

const fonte  = MODEL_PRIORITY['sispesquisa.servidores_federais'].fonte;
const rank   = MODEL_PRIORITY['sispesquisa.servidores_federais'].rank;
const grupo  = MODEL_PRIORITY['sispesquisa.servidores_federais'].grupo;

export let getServidorFederalCPF_Sispesquisa_Servidores_Federais = function (cpf: String){

    return connection.getConn().request()
        .input('cpf_parameter', mssql.NVarChar, cpf)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        CPF as cpf,  DESCRICAO_CARGO as cargo, ATIVIDADE as atividade,
                        UORG_LOTACAO as unidOrgLotacao, ORG_LOTACAO as orgLotacao, DATA_INGRESSO_CARGOFUNCAO as dataIngressoCargo,
                        DATA_INGRESSO_ORGAO as dataIngressoOrgao, UF_EXERCICIO as ufExercicio, dt_cadastro as dataCadastro, fonte,
                        '${tipo}' as tipo

                FROM SISPESQUISA.sispesquisa.servidores_federais
                WHERE CPF=@cpf_parameter
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o servidor federal ${cpf}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelo servidor federal ${cpf}.`, 'getServidorFederalCPF_Sispesquisa_Servidores_Federais')});
};