import * as mssql from 'mssql';

import { connection }  from '../../services/db.service';
import { MODEL_PRIORITY, API_CONFIG } from '../../config';
import {
    resultNotFound,
    resultFound,
    logErroBuscaBD
} from '../../utils';

const fonte  = MODEL_PRIORITY['sagres_municipal.v_folhapagamento'].fonte;
const rank   = MODEL_PRIORITY['sagres_municipal.v_folhapagamento'].rank;
const grupo  = MODEL_PRIORITY['sagres_municipal.v_folhapagamento'].grupo;

export let getServidorMunicipalDetalhadoCPF_SAGRES_MUNICIPAL_FolhaPagamento = function (cpf: String){

    return connection.getConn().request()
        .input('cpf_parameter', mssql.Char(11), cpf)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        V_FolhaPagamento.dt_Ano as dtAno, cd_CPF as cpf,
                        UPPER(Cargos.de_Cargo) as cargo, UPPER(Codigo_Ugestora.de_Ugestora) as unidadeGestora,
                        UPPER(Tipo_Cargo.de_TipoCargo) as tipoCargo,
                        SUM(CASE
                                WHEN tp_lancamento = 0 THEN vl_lancamento
                                ELSE vl_lancamento*-1
                            END
                        ) as vLiquido

                FROM SAGRES_MUNICIPAL.dbo.V_FolhaPagamento
                    OUTER APPLY (
                        SELECT TOP 1 de_Ugestora
                        FROM SAGRES_CONTROLE.dbo.Codigo_Ugestora
                        WHERE (V_FolhaPagamento.cd_UGestora = Codigo_Ugestora.cd_Ugestora)
                        ) AS Codigo_Ugestora
                    OUTER APPLY (
                        SELECT TOP 1 de_Cargo, tp_Cargo
                        FROM SAGRES_MUNICIPAL.dbo.VW_CARGOS
                        WHERE (V_FolhaPagamento.cd_UGestora = VW_CARGOS.cd_UGestora AND V_FolhaPagamento.cd_Cargo = VW_CARGOS.cd_Cargo)
                        ) AS Cargos
                    LEFT OUTER JOIN SAGRES_MUNICIPAL.dbo.Tipo_Cargo
                        ON (Cargos.tp_Cargo = Tipo_Cargo.tp_Cargo)
                WHERE cd_CPF=@cpf_parameter
                GROUP BY V_FolhaPagamento.dt_Ano, cd_CPF, Cargos.de_Cargo, Codigo_Ugestora.de_Ugestora, Tipo_Cargo.de_TipoCargo
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o servidor municipal ${cpf}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pelo servidor municipal ${cpf}.`, 'getServidorMunicipalDetalhadoCPF_SAGRES_MUNICIPAL_FolhaPagamento')});
};

export let getServidorMunicipalSimplificadoCPF_SAGRES_MUNICIPAL_FolhaPagamento = function (cpf: String){

    return connection.getConn().request()
        .input('cpf_parameter', mssql.Char(11), cpf)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        V_FolhaPagamento.dt_Ano as dtAno, cd_CPF as cpf,
                        UPPER(Cargos.de_Cargo) as cargo, UPPER(Codigo_Ugestora.de_Ugestora) as unidadeGestora,
                        UPPER(Tipo_Cargo.de_TipoCargo) as tipoCargo,
                        SUM(CASE
                                WHEN tp_lancamento = 0 THEN vl_lancamento
                                ELSE vl_lancamento*-1
                            END
                        ) as vLiquido

                FROM SAGRES_MUNICIPAL.dbo.V_FolhaPagamento
                    OUTER APPLY (
                        SELECT TOP 1 de_Ugestora
                        FROM SAGRES_CONTROLE.dbo.Codigo_Ugestora
                        WHERE (V_FolhaPagamento.cd_UGestora = Codigo_Ugestora.cd_Ugestora)
                        ) AS Codigo_Ugestora
                    OUTER APPLY (
                        SELECT TOP 1 de_Cargo, tp_Cargo
                        FROM SAGRES_MUNICIPAL.dbo.VW_CARGOS
                        WHERE (V_FolhaPagamento.cd_UGestora = VW_CARGOS.cd_UGestora AND V_FolhaPagamento.cd_Cargo = VW_CARGOS.cd_Cargo)
                        ) AS Cargos
                    LEFT OUTER JOIN SAGRES_MUNICIPAL.dbo.Tipo_Cargo
                        ON (Cargos.tp_Cargo = Tipo_Cargo.tp_Cargo)
                    WHERE cd_CPF=@cpf_parameter
                GROUP BY V_FolhaPagamento.dt_Ano, cd_CPF, Cargos.de_Cargo, Codigo_Ugestora.de_Ugestora, Tipo_Cargo.de_TipoCargo
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o servidor municipal ${cpf}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pelo servidor municipal ${cpf}.`, 'getServidorMunicipalSimplificadoCPF_SAGRES_MUNICIPAL_FolhaPagamento')});
};