import * as mssql from 'mssql';

import { connection }  from '../../services/db.service';
import { MODEL_PRIORITY, API_CONFIG } from '../../config';
import {
    resultNotFound,
    resultFound,
    logErroBuscaBD
} from '../../utils';

const fonte  = MODEL_PRIORITY['sispesquisa.servidores_municipais'].fonte;
const rank   = MODEL_PRIORITY['sispesquisa.servidores_municipais'].rank;
const grupo  = MODEL_PRIORITY['sispesquisa.servidores_municipais'].grupo;

export let getServidorMunicipalDetalhadoCPF_Sispesquisa_Servidores_Municipais = function (cpf: string){

    return connection.getConn().request()
        .input('cpf_parameter', mssql.NVarChar, cpf)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        de_ugestora as unidadeGestora, de_cargo as cargo, de_tipocargo as tipoCargo,
                        cd_CPF as cpf, de_UOrcamentaria as unidadeOrcamentaria, ano as dtAno, fonte

                FROM SISPESQUISA.sispesquisa.servidores_municipais
                WHERE cd_CPF=@cpf_parameter
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o servidor municipal ${cpf}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pelo servidor municipal ${cpf}.`, 'getServidorMunicipalDetalhadoCPF_Sispesquisa_Servidores_Municipais')});
};

export let getServidorMunicipalSimplificadoCPF_Sispesquisa_Servidores_Municipais = function (cpf: string){

    return connection.getConn().request()
        .input('cpf_parameter', mssql.NVarChar, cpf)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        ano as dtAno, de_cargo as cargo, de_ugestora as unidadeGestora, de_UOrcamentaria as unidadeOrcamentaria,
                        de_tipocargo as tipoCargo

                FROM SISPESQUISA.sispesquisa.servidores_municipais
                WHERE cd_CPF=@cpf_parameter
                GROUP BY ano, de_cargo, de_tipocargo, de_ugestora, de_UOrcamentaria
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o servidor municipal ${cpf}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pelo servidor municipal ${cpf}.`, 'getServidorMunicipalSimplificadoCPF_Sispesquisa_Servidores_Municipais')});
};