import * as mssql from 'mssql';

import { connection } from './../../services/db.service';
import { MODEL_PRIORITY, API_CONFIG } from './../../config';
import {
    resultNotFound,
    resultFound,
    logErroBuscaBD
} from './../../utils';

const fonte_pf = MODEL_PRIORITY['receita_full.sociopf'].fonte;
const rank_pf  = MODEL_PRIORITY['receita_full.sociopf'].rank;
const grupo_pf = MODEL_PRIORITY['receita_full.sociopf'].grupo;

export let getSocioPFEmpresaSimplificadoCNPJ_ReceitaFull = function (cnpj: string) {
    return connection.getConn().request()
        .input('cnpj_parameter', mssql.NVarChar, cnpj)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        S.Nome as nome, S.CPF as cpf, S.PercentualCapitalSocial as percCapital,
                        S.CNPJ as empresa_cnpj, PJ.RazaoSocial as empresa_razaoSocial, PJ.NomeFantasia as empresa_nomeFantasia

                FROM Receita_Full.Receita.SocioPF AS S
                    INNER JOIN Receita_Full.Receita.PJ ON (S.CNPJ = PJ.CNPJ)
                WHERE S.CNPJ=@cnpj_parameter
                `)
        .then(result => {
            if (result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar os sócios da empresa ${cnpj}.`, fonte_pf);
            else
                return resultFound(result, fonte_pf, rank_pf, grupo_pf);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelos sócios de ${cnpj}.`, 'getSocioPFEmpresaSimplificadoCNPJ_ReceitaFull'); });
};

const fonte_pj = MODEL_PRIORITY['receita_full.sociopj'].fonte;
const rank_pj  = MODEL_PRIORITY['receita_full.sociopj'].rank;
const grupo_pj = MODEL_PRIORITY['receita_full.sociopj'].grupo;

export let getSocioPJEmpresaSimplificadoCNPJ_ReceitaFull = function (cnpj: string) {

    return connection.getConn().request()
        .input('cnpj_parameter', mssql.NVarChar, cnpj)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        S.RazaoSocial as razaoSocial, S.CNPJSocio as cnpj, S.PercentualCapitalSocial as percCapital,
                        S.CNPJ as empresa_cnpj, PJ.RazaoSocial as empresa_razaoSocial, PJ.NomeFantasia as empresa_nomeFantasia

                FROM Receita_Full.Receita.SocioPJ AS S
                    INNER JOIN Receita_Full.Receita.PJ ON (S.CNPJ = PJ.CNPJ)
                WHERE S.CNPJ=@cnpj_parameter
                `)
        .then(result => {
            if (result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar os sócios da empresa ${cnpj}.`, fonte_pj);
            else
                return resultFound(result, fonte_pj, rank_pj, grupo_pj);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelos sócios de ${cnpj}.`, 'getSocioPJEmpresaSimplificadoCNPJ_ReceitaFull'); });
};

const fonte_es = MODEL_PRIORITY['receita_full.socioestrangeiro'].fonte;
const rank_es  = MODEL_PRIORITY['receita_full.socioestrangeiro'].rank;
const grupo_es = MODEL_PRIORITY['receita_full.socioestrangeiro'].grupo;

export let getSocioEstrangeiroEmpresaSimplificadoCNPJ_ReceitaFull = function (cnpj: string) {

    return connection.getConn().request()
        .input('cnpj_parameter', mssql.NVarChar, cnpj)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        S.Nome as nome, S.NomePais as nomePais, S.PercentualCapitalSocial as percCapital,
                        S.CNPJ as empresa_cnpj, PJ.RazaoSocial as empresa_razaoSocial, PJ.NomeFantasia as empresa_nomeFantasia

                FROM Receita_Full.Receita.SocioEstrangeiro AS S
                    INNER JOIN Receita_Full.Receita.PJ ON (S.CNPJ = PJ.CNPJ)
                WHERE S.CNPJ=@cnpj_parameter
                `)
        .then(result => {
            if (result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar os sócios da empresa ${cnpj}.`, fonte_es);
            else
                return resultFound(result, fonte_es, rank_es, grupo_es);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelos sócios de ${cnpj}.`, 'getSocioEstrangeiroEmpresaSimplificadoCNPJ_ReceitaFull'); });
};
