import * as mssql from 'mssql';

import { connection } from './../../services/db.service';
import { MODEL_PRIORITY, API_CONFIG } from './../../config';
import {
    resultNotFound,
    resultFound,
    logErroBuscaBD
} from './../../utils';

const fonte_pf = MODEL_PRIORITY['receitanovo.sociopf'].fonte;
const rank_pf  = MODEL_PRIORITY['receitanovo.sociopf'].rank;
const grupo_pf = MODEL_PRIORITY['receitanovo.sociopf'].grupo;


export let getSocioPFEmpresaSimplificadoCNPJ_ReceitaNovo = function (cnpj: string) {
    return connection.getConn().request()
        .input('cnpj_parameter', mssql.Char(14), cnpj)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        S.NomeSocio as nome, RIGHT(S.CpfSocio, 11) as cpf, CAST(S.PercentualCapitalSocial AS float)/100 as percCapital,
                        S.CNPJ as empresa_cnpj, PJ.RazaoSocial as empresa_razaoSocial, PJ.NomeFantasia as empresa_nomeFantasia

                FROM ReceitaNovo.dbo.Socio AS S
                    INNER JOIN ReceitaNovo.dbo.PessoaJuridica AS PJ ON (S.CNPJ = PJ.CNPJ)
                WHERE S.CNPJ=@cnpj_parameter AND S.IdentificadorSocio=2
                `)
        .then(result => {
            if (result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar os sócios da empresa ${cnpj}.`, fonte_pf);
            else
                return resultFound(result, fonte_pf, rank_pf, grupo_pf);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelos sócios de ${cnpj}.`, 'getSocioPFEmpresaSimplificadoCNPJ_ReceitaNovo'); });
};

const fonte_pj = MODEL_PRIORITY['receitanovo.sociopj'].fonte;
const rank_pj  = MODEL_PRIORITY['receitanovo.sociopj'].rank;
const grupo_pj = MODEL_PRIORITY['receitanovo.sociopj'].grupo;

export let getSocioPJEmpresaSimplificadoCNPJ_ReceitaNovo = function (cnpj: string) {

    return connection.getConn().request()
        .input('cnpj_parameter', mssql.Char(14), cnpj)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        S.NomeSocio as razaoSocial, S.CpfSocio as cnpj, CAST(S.PercentualCapitalSocial AS float)/100 as percCapital,
                        S.CNPJ as empresa_cnpj, PJ.RazaoSocial as empresa_razaoSocial, PJ.NomeFantasia as empresa_nomeFantasia

                FROM ReceitaNovo.dbo.Socio AS S
                    INNER JOIN ReceitaNovo.dbo.PessoaJuridica AS PJ ON (S.CNPJ = PJ.CNPJ)
                WHERE S.CNPJ=@cnpj_parameter AND S.IdentificadorSocio=1
                `)
        .then(result => {
            if (result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar os sócios da empresa ${cnpj}.`, fonte_pj);
            else
                return resultFound(result, fonte_pj, rank_pj, grupo_pj);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelos sócios de ${cnpj}.`, 'getSocioPJEmpresaSimplificadoCNPJ_ReceitaNovo'); });
};

const fonte_es = MODEL_PRIORITY['receitanovo.socioestrangeiro'].fonte;
const rank_es  = MODEL_PRIORITY['receitanovo.socioestrangeiro'].rank;
const grupo_es = MODEL_PRIORITY['receitanovo.socioestrangeiro'].grupo;

export let getSocioEstrangeiroEmpresaSimplificadoCNPJ_ReceitaNovo = function (cnpj: string) {

    return connection.getConn().request()
        .input('cnpj_parameter', mssql.Char(14), cnpj)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        S.NomeSocio as razaoSocial, S.NomePaisSocio as nomePais, CAST(S.PercentualCapitalSocial AS float)/100 as percCapital,
                        S.CNPJ as empresa_cnpj, PJ.RazaoSocial as empresa_razaoSocial, PJ.NomeFantasia as empresa_nomeFantasia

                FROM ReceitaNovo.dbo.Socio AS S
                    INNER JOIN ReceitaNovo.dbo.PessoaJuridica AS PJ ON (S.CNPJ = PJ.CNPJ)
                WHERE S.CNPJ=@cnpj_parameter AND S.IdentificadorSocio=3
                `)
        .then(result => {
            if (result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar os sócios da empresa ${cnpj}.`, fonte_es);
            else
                return resultFound(result, fonte_es, rank_es, grupo_es);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelos sócios de ${cnpj}.`, 'getSocioEstrangeiroEmpresaSimplificadoCNPJ_ReceitaFull'); });
};
