import * as mssql from 'mssql';

import { connection }  from '../../services/db.service';
import { MODEL_PRIORITY, API_CONFIG } from './../../config';
import {
    resultNotFound,
    resultFound,
    logErroBuscaBD
} from '../../utils';

import { SIGLA_FONTE } from './../_fontes';

const fonte_dado = SIGLA_FONTE['receita_full'];

const fonte_pf   = MODEL_PRIORITY['receita_full.pf.telefone'].fonte;
const rank_pf    = MODEL_PRIORITY['receita_full.pf.telefone'].rank;
const grupo_pf   = MODEL_PRIORITY['receita_full.pf.telefone'].grupo;

export let getTelefoneCPF_ReceitaFull_PF = function (cpf: String){

    return connection.getConn().request()
        .input('cpf_parameter', mssql.VarChar, cpf)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                    CPF as cpf, DDD as ddd, Telefone as telefone,
                    Nome as nome, '${fonte_dado}' as fonte

                FROM Receita_Full.Receita.PF
                WHERE CPF=@cpf_parameter
        `)
        .then((result: any) => {
            result.recordset = result.recordset.filter(registro => (((registro.ddd.trim().length !== 0) && (registro.telefone.trim().length !== 0)))
                                                                && (registro.telefone !== '00000000'));

            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o telefone da pessoa ${cpf}.`, fonte_pf);
            else{
                return resultFound(result, fonte_pf, rank_pf, grupo_pf);
            }
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pelo telefone da pessoa ${cpf}.`, 'getTelefoneCPF_ReceitaFull_PF'); });
};

export let getTelefoneNome_ReceitaFull_PF = function (nome: string){

    return connection.getConn().request()
        .input('nome_parameter', mssql.VarChar, nome)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                    CPF as cpf, DDD as ddd, Telefone as telefone,
                    Nome as nome, '${fonte_dado}' as fonte

                FROM Receita_Full.Receita.PF
                WHERE CONTAINS(Nome, @nome_parameter)
        `)
        .then((result: any) => {
            result.recordset = result.recordset.filter(registro => (((registro.ddd.trim().length !== 0) && (registro.telefone.trim().length !== 0)))
                                                                && (registro.telefone !== '00000000'));

            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o telefone da pessoa ${nome}.`, fonte_pf);
            else {
                return resultFound(result, fonte_pf, rank_pf, grupo_pf);
            }
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pelo telefone da pessoa ${nome}.`, 'getTelefoneNome_ReceitaFull_PF'); });
};

export let getTelefoneTelefone_ReceitaFull_PF = function (telefone: string){

    return connection.getConn().request()
        .input('telefone_parameter', mssql.VarChar, telefone )
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                    CPF as cpf, DDD as ddd, Telefone as telefone,
                    Nome as nome, '${fonte_dado}' as fonte

                FROM Receita_Full.Receita.PF
                WHERE telefone = @telefone_parameter
        `)
        .then((result: any) => {
            result.recordset = result.recordset.filter(registro => (((registro.ddd.trim().length !== 0) && (registro.telefone.trim().length !== 0)))
                                                                && (registro.telefone !== '00000000'));

            if (result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o telefone da pessoa ${telefone}.`, fonte_pf);
            else {
                return resultFound(result, fonte_pf, rank_pf, grupo_pf);
            }
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pelo telefone da pessoa ${telefone}.`, 'getTelefoneTelefone_ReceitaFull_PF'); });
};

const fonte_pj   = MODEL_PRIORITY['receita_full.pj.telefone'].fonte;
const rank_pj    = MODEL_PRIORITY['receita_full.pj.telefone'].rank;
const grupo_pj   = MODEL_PRIORITY['receita_full.pj.telefone'].grupo;

export let getTelefoneCNPJ_ReceitaFull_PJ = function (cnpj: String){

    return connection.getConn().request()
        .input('cnpj_parameter', mssql.VarChar, cnpj)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        CNPJ as cnpj, DDD as ddd, Telefone as telefone,
                        DDD2 as ddd2, Telefone2 as telefone2,
                        DDDFax as dddfax, TelefoneFax as telefoneFax,
                        RazaoSocial as razaoSocial, '${fonte_dado}' as fonte

                FROM Receita_Full.Receita.PJ
                WHERE CNPJ=@cnpj_parameter
        `)
        .then((result: any) => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o telefone da empresa ${cnpj}.`, fonte_pj);
            else{
                result.recordset = result.recordset.reduce((acc, registro) => {
                    acc.push({ ddd: registro.ddd, telefone: registro.telefone, cnpj: registro.cnpj, razaoSocial: registro.razaoSocial, fonte: registro.fonte},
                        { ddd: registro.ddd2, telefone: registro.telefone2, cnpj: registro.cnpj, razaoSocial: registro.razaoSocial, fonte: registro.fonte},
                        { ddd: registro.dddfax, telefone: registro.telefoneFax, cnpj: registro.cnpj, razaoSocial: registro.razaoSocial, fonte: registro.fonte});
                    return acc;
                }, [] );
                result.recordset = result.recordset.filter(registro => (((registro.ddd.trim().length !== 0) && (registro.telefone.trim().length !== 0)))
                                                                    && (registro.telefone !== '00000000'));

                if(result.recordset.length === 0) {
                    return resultNotFound(`Nao foi possivel encontrar o telefone da empresa ${cnpj}.`, fonte_pj);
                } else {
                    return resultFound(result, fonte_pj, rank_pj, grupo_pj);
                }
            }
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pelo telefone da empresa ${cnpj}.`, 'getTelefoneCNPJ_ReceitaFull_PJ'); });
};

export let getTelefoneRazaoSocial_ReceitaFull_PJ = function (razaoSocial: string){

    return connection.getConn().request()
        .input('razao_social_parameter', mssql.VarChar, razaoSocial)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        CNPJ as cnpj, DDD as ddd, Telefone as telefone,
                        DDD2 as ddd2, Telefone2 as telefone2,
                        DDDFax as dddfax, TelefoneFax as telefoneFax,
                        RazaoSocial as razaoSocial, '${fonte_dado}' as fonte

                FROM Receita_Full.Receita.PJ
                WHERE CONTAINS(RazaoSocial, @razao_social_parameter)
        `)
        .then((result: any) => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o telefone da empresa ${razaoSocial}.`, fonte_pj);
            else{
                result.recordset = result.recordset.reduce((acc, registro) => {
                    acc.push({ ddd: registro.ddd, telefone: registro.telefone, cnpj: registro.cnpj, razaoSocial: registro.razaoSocial, fonte: registro.fonte},
                        { ddd: registro.ddd2, telefone: registro.telefone2, cnpj: registro.cnpj, razaoSocial: registro.razaoSocial, fonte: registro.fonte},
                        { ddd: registro.dddfax, telefone: registro.telefoneFax, cnpj: registro.cnpj, razaoSocial: registro.razaoSocial, fonte: registro.fonte});
                    return acc;
                }, [] );
                result.recordset = result.recordset.filter(registro => (((registro.ddd.trim().length !== 0) && (registro.telefone.trim().length !== 0)))
                                                                    && (registro.telefone !== '00000000'));

                if(result.recordset.length === 0) {
                    return resultNotFound(`Nao foi possivel encontrar o telefone da empresa ${razaoSocial}.`, fonte_pj);
                } else {
                    return resultFound(result, fonte_pj, rank_pj, grupo_pj);
                }
            }
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pelo telefone da empresa ${razaoSocial}.`, 'getTelefoneRazaoSocial_ReceitaFull_PJ'); });
};


export let getTelefoneTelefone_ReceitaFull_PJ = function (telefone: string) {

    return connection.getConn().request()
        .input('telefone_parameter', mssql.VarChar, telefone)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        CNPJ as cnpj, DDD as ddd, Telefone as telefone,
                        RazaoSocial as razaoSocial, '${fonte_dado}' as fonte

                FROM Receita_Full.Receita.PJ
                WHERE Telefone = @telefone_parameter OR Telefone2 = @telefone_parameter
        `)
        .then((result: any) => {
            if (result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o telefone da empresa ${telefone}.`, fonte_pj);
            else {
                result.recordset = result.recordset.reduce((acc, registro) => {
                    acc.push({ ddd: registro.ddd, telefone: registro.telefone, cnpj: registro.cnpj, razaoSocial: registro.razaoSocial, fonte: registro.fonte },
                        { ddd: registro.ddd2, telefone: registro.telefone2, cnpj: registro.cnpj, razaoSocial: registro.razaoSocial, fonte: registro.fonte },
                        { ddd: registro.dddfax, telefone: registro.telefoneFax, cnpj: registro.cnpj, razaoSocial: registro.razaoSocial, fonte: registro.fonte });
                    return acc;
                }, []);
                result.recordset = result.recordset.filter(registro => (((registro.ddd.trim().length !== 0) && (registro.telefone.trim().length !== 0)))
                                                                    && (registro.telefone !== '00000000'));

                if (result.recordset.length === 0) {
                    return resultNotFound(`Nao foi possivel encontrar o telefone da pessoa ${telefone}.`, fonte_pj);
                } else {
                    return resultFound(result, fonte_pj, rank_pj, grupo_pj);
                }
            }
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelo telefone da empresa ${telefone}.`, 'getTelefoneTelefone_ReceitaFull_PJ'); });
};
