import * as mssql from 'mssql';

import { connection }  from '../../services/db.service';
import { MODEL_PRIORITY, API_CODES, API_CONFIG } from './../../config';
import {
    trataResultado,
    resultNotFound,
    resultFound,
    logErroBuscaBD
} from '../../utils';

const fonte   = MODEL_PRIORITY['sispesquisa.telefones'].fonte;
const rank    = MODEL_PRIORITY['sispesquisa.telefones'].rank;
const grupo   = MODEL_PRIORITY['sispesquisa.telefones'].grupo;

export let getTelefoneCPF_Sispesquisa_Telefones = function (cpf: string){

    return connection.getConn().request()
        .input('CPF', mssql.NVarChar, cpf)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                    T.cpf_cnpj as cpf, T.telefone, T.fonte, PF.nome as nome

                FROM SISPESQUISA.sispesquisa.telefones T
                    LEFT OUTER JOIN ReceitaNovo.dbo.PessoaFisica PF ON (PF.CPF = T.cpf_cnpj)
                WHERE T.cpf_cnpj=@CPF
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o telefone da pessoa ${cpf}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pelo telefone da pessoa ${cpf}.`, 'getTelefoneCPF_Sispesquisa_Telefones'); });
};

export let getTelefoneCNPJ_Sispesquisa_Telefones = function (cnpj: string){

    return connection.getConn().request()
        .input('CNPJ', mssql.NVarChar, cnpj)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                    T.cpf_cnpj as cnpj, T.telefone, T.fonte, PJ.RazaoSocial as razaoSocial

                FROM SISPESQUISA.sispesquisa.telefones T
                    LEFT OUTER JOIN ReceitaNovo.dbo.PessoaJuridica PJ ON (PJ.CNPJ = T.cpf_cnpj)
                WHERE T.cpf_cnpj=@CNPJ
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o telefone da empresa ${cnpj}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelo telefone da empresa ${cnpj}.`, 'getTelefoneCNPJ_Sispesquisa_Telefones'); });
};

export let getTelefoneNome_Sispesquisa_Telefones = function (nome: string){

    return connection.getConn().request()
        .input('nome_parameter', mssql.NVarChar, nome)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                    T.cpf_cnpj as cpf, T.telefone, T.fonte, PF.nome as nome

                FROM SISPESQUISA.sispesquisa.telefones T
                    INNER JOIN ReceitaNovo.dbo.PessoaFisica PF ON (PF.CPF = T.cpf_cnpj)
                WHERE CONTAINS(PF.nome, @nome_parameter)
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o telefone da pessoa ${nome}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pelo telefone da pessoa ${nome}.`, 'getTelefoneNome_Sispesquisa_Telefones'); });
};

export let getTelefonePFTelefone_Sispesquisa_Telefones = function (telefone: string){

    return connection.getConn().request()
        .input('telefone_parameter', mssql.NVarChar, telefone)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                    T.cpf_cnpj as cpf, T.telefone, T.fonte, PF.nome as nome

                FROM SISPESQUISA.sispesquisa.telefones T
                    LEFT OUTER JOIN ReceitaNovo.dbo.PessoaFisica PF ON (PF.CPF = T.cpf_cnpj)
                WHERE T.telefone=@telefone_parameter AND LEN(cpf_cnpj) = 11
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o telefone da pessoa ${telefone}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelo telefone da pessoa ${telefone}.`, 'getTelefonePFTelefone_Sispesquisa_Telefones'); });
};

export let getTelefonePJTelefone_Sispesquisa_Telefones = function (telefone: string){

    return connection.getConn().request()
        .input('telefone_parameter', mssql.NVarChar, telefone)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                    T.cpf_cnpj as cnpj, T.telefone, T.fonte, PJ.RazaoSocial as razaoSocial

                FROM SISPESQUISA.sispesquisa.telefones T
                    LEFT OUTER JOIN ReceitaNovo.dbo.PessoaJuridica PJ ON (PJ.CNPJ = T.cpf_cnpj)
                WHERE T.telefone=@telefone_parameter AND LEN(cpf_cnpj) = 14
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o telefone da pessoa ${telefone}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelo telefone da pessoa ${telefone}.`, 'getTelefonePJTelefone_Sispesquisa_Telefones'); });
};

export let getTelefoneBuscaProfundaTelefone_Sispesquisa_Telefones = function (telefone: string){

    return connection.getConn().request()
        .input('TELEFONE', mssql.NVarChar, '%' + telefone + '%')
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                    T.cpf_cnpj as cpf, T.telefone, T.fonte, PF.nome as nome

                FROM SISPESQUISA.sispesquisa.telefones T
                    LEFT OUTER JOIN ReceitaNovo.dbo.PessoaFisica PF ON (PF.CPF = T.cpf_cnpj)
                WHERE T.telefone LIKE @TELEFONE
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o telefone da pessoa ${telefone}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelo telefone da pessoa ${telefone}.`, 'getTelefoneBuscaProfundaTelefone_Sispesquisa_Telefones'); });
};

export let insertTelefone_Sispesquisa_Telefones = function (form){

    return connection.getConn().request()
        .input('cpf_parameter', form.cpf)
        .input('telefone_parameter', form.telefone)
        .input('fonte_parameter', form.fonte)
        .query(`INSERT INTO SISPESQUISA.sispesquisa.telefones
                    (cpf_cnpj, telefone, fonte)
                VALUES
                    (@cpf_parameter, @telefone_parameter, @fonte_parameter)
        `)
        .then(result => {
            return {
                status: API_CODES.CODE_SUCESSO,
                msg: 'Telefone cadastrado com sucesso.'
            }
        }).catch( error => {logErroBuscaBD(error, `Falha no cadastro do telefone.`, 'insertTelefone_Sispesquisa_Telefones')});
};