import * as mssql from 'mssql';

import { connection }  from '../../services/db.service';
import { MODEL_PRIORITY, API_CODES, API_CONFIG } from './../../config';
import {
    trataResultado,
    resultNotFound,
    resultFound,
    logErroBuscaBD
} from '../../utils';

import { SIGLA_FONTE } from './../_fontes';

const fonte_dado = SIGLA_FONTE['vep'];

const fonte   = MODEL_PRIORITY['vep.telefone'].fonte;
const rank    = MODEL_PRIORITY['vep.telefone'].rank;
const grupo   = MODEL_PRIORITY['vep.telefone'].grupo;

export let getTelefoneCPF_VEP = function (cpf: string){

    return connection.getConn().request()
        .input('cpf_parameter', mssql.NVarChar, cpf)
        .query(` SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                    PF.CPF as cpf
                    ,TEL.TELEFONE as telefone
                    ,'${fonte_dado}' as fonte

                FROM VEP.dbo.VEP_CNJ_TB_PESSOAFISICA PF
                    LEFT OUTER JOIN VEP.dbo.VEP_CNJ_TB_PESSOA P ON (P.ID_PESSOA = PF.FK_PESSOA)
                    OUTER APPLY(
                        SELECT STUFF((
                            SELECT ', ' + CAST(PTEL3.DDDTELEFONE AS VARCHAR) + CAST(PTEL3.NUMEROTELEFONE AS VARCHAR)
                            FROM VEP.dbo.VEP_CNJ_TB_PESSOATELEFONE PTEL3
                            WHERE (P.ID_PESSOA = PTEL3.FK_PESSOA)
                            FOR XML PATH('')
                        ), 1, 2, '') AS TELEFONE
                    ) AS TEL
                WHERE NOT EXISTS(
                    SELECT 1
                    FROM VEP.dbo.VEP_CNJ_TB_PARTEPROCESSOADV PPA
                    INNER JOIN VEP.dbo.VEP_CNJ_TB_PARTEPROCESSO PRO ON (PPA.FK_PARTEPROCESSO = PRO.ID_PARTEPROCESSO)
                    INNER JOIN VEP.dbo.VEP_CNJ_TB_PARTE P ON (P.ID_PARTE = PRO.FK_PARTE)
                    WHERE (PF.ID_PESSOAFISICA = P.FK_PESSOAFISICA)
                ) AND PF.CPF=@cpf_parameter AND TEL.TELEFONE IS NOT NULL
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o telefone da pessoa ${cpf}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelo telefone da pessoa ${cpf}.`, 'getTelefoneCPF_VEP'); });
};