import * as mssql from 'mssql';

import { connection } from './../../services/db.service';
import { BenchmarkService } from './../../services/benchmark.service';
import { MODEL_PRIORITY } from './../../config';

import {
    resultNotFound,
    resultFound,
    logErroBuscaBD,
    trataResultadosCruzamentoSisobi,
    insereAtributoTipologia,
    printTempoExecucao
} from './../../utils';

const fonte = MODEL_PRIORITY['tipologias.cacafantasmas'].fonte;
const rank  = MODEL_PRIORITY['tipologias.cacafantasmas'].rank;
const grupo = MODEL_PRIORITY['tipologias.cacafantasmas'].grupo;

const tipologias = {
    'fp_sisobi':            { codigo: 't1', peso: 5 },
    'empenho_sisobi':       { codigo: 't2', peso: 5 },

    'fp_enderecosrf':       { codigo: 't3', peso: 3 },
    'fp_enderecosre':       { codigo: 't4', peso: 1 },

    'fp_sociopj':           { codigo: 't5', peso: 2 },
    'fp_responsavelpj':     { codigo: 't6', peso: 2 },
    'fp_sociopj_empenho':   { codigo: 't7', peso: 10 },

    'fp_extravinculo':      { codigo: 't8', peso: 8 },
    'fp_sociopj_caged':     { codigo: 't9', peso: 50 },
}


export let getTipologiaDetalhadaFP_SISOBI = function (ugestora: string, dtInicio: string, dtFim: string) {

    const tempoInicial = new BenchmarkService();
    return connection.getConn().request()
        .input('DT_INICIAL', mssql.NVarChar, dtInicio)
        .input('DT_FINAL', mssql.NVarChar, dtFim)
        .input('UGESTORA', mssql.NVarChar, ugestora)
        .query(`
            SELECT *
            FROM (
                SELECT
                    PF.Nome as nome
                    ,O.Falecido as nomeFalecido
                    ,FP.cd_CPF AS cpf
                    ,FP.nu_Matricula AS matricula
                    ,CAR.de_Cargo AS cargo
                    ,FP.MESANO_ORDENADO AS mesAno
                    ,O.Obito as dataObito
                    ,DATEDIFF(M,O.OBITO,FP.MESANO_ORDENADO + '01') AS mesesFalecido
                    ,CAR.de_TipoCargo AS natureza
                    ,SUM(
                        CASE
                            WHEN FP.tp_Lancamento = 0 THEN FP.vl_Lancamento
                            ELSE 0
                        END
                    ) AS remuneracaoBruta
                    ,SUM(
                        CASE
                            WHEN FP.tp_Lancamento = 1 THEN FP.vl_Lancamento
                            ELSE 0
                        END
                    ) AS descontos
                    ,SUM(
                        CASE
                            WHEN FP.tp_Lancamento = 0 THEN FP.vl_Lancamento
                            ELSE 0
                        END
                    )
                    -
                    SUM(
                        CASE
                            WHEN FP.tp_Lancamento = 1 THEN FP.vl_Lancamento
                            ELSE 0
                        END
                    )
                    AS remuneracaoLiquida

                FROM SAGRES_MUNICIPAL.dbo.FolhaPagamento_N FP
                    LEFT OUTER JOIN ReceitaNovo.dbo.PessoaFisica PF ON (PF.CPF = FP.cd_CPF)
                    LEFT OUTER JOIN Sisobi.dbo.Obito O ON (FP.cd_CPF = O.CPF_RF)

                    CROSS APPLY(
                        SELECT TOP 1 C.DE_CARGO, TC.DE_TIPOCARGO
                        FROM SAGRES_MUNICIPAL.dbo.Cargos C
                        INNER JOIN SAGRES_MUNICIPAL.dbo.Tipo_Cargo TC ON (TC.TP_CARGO = C.TP_CARGO)
                        WHERE (C.CD_UGESTORA = FP.CD_UGESTORA AND C.CD_CARGO = FP.CD_CARGO)-- AND C.DT_MESANO = FP.DT_MESANO)
                    ) AS CAR

                WHERE FP.MESANO_ORDENADO BETWEEN @DT_INICIAL AND @DT_FINAL
                    AND FP.cd_UGestora = @UGESTORA
                    AND FP.MESANO_ORDENADO + '01' > O.Obito

                GROUP BY
                    PF.Nome
                    ,O.Falecido
                    ,O.Falecido
                    ,FP.cd_CPF
                    ,FP.nu_Matricula
                    ,CAR.de_Cargo
                    ,FP.dt_MesAno
                    ,FP.MESANO_ORDENADO
                    ,CAR.de_TipoCargo
                    ,O.Obito

                UNION ALL

                SELECT
                    PF.Nome as nome
                    ,O.Falecido as nomeFalecido
                    ,FP.cd_CPF AS cpf
                    ,FP.nu_Matricula AS servidor
                    ,CAR.de_Cargo AS cargo
                    ,FP.MESANO_ORDENADO AS mesAno
                    ,O.Obito AS dataObito
                    ,DATEDIFF(M,O.OBITO,FP.MESANO_ORDENADO + '01') AS mesesFalecido
                    ,CAR.de_TipoCargo AS Natureza
                    ,SUM(
                        CASE
                            WHEN FP.tp_Lancamento = 0 THEN FP.vl_Lancamento
                            ELSE 0
                        END
                    ) AS remuneracaoBruta
                    ,SUM(
                        CASE
                            WHEN FP.tp_Lancamento = 1 THEN FP.vl_Lancamento
                            ELSE 0
                        END
                    ) AS descontos
                    ,SUM(
                        CASE
                            WHEN FP.tp_Lancamento = 0 THEN FP.vl_Lancamento
                            ELSE 0
                        END
                    )
                    -
                    SUM(
                        CASE
                            WHEN FP.tp_Lancamento = 1 THEN FP.vl_Lancamento
                            ELSE 0
                        END
                    )
                    AS remuneracaoLiquida

                FROM SAGRES_MUNICIPAL.dbo.FolhaPagamento_N_2017 FP
                    LEFT OUTER JOIN ReceitaNovo.dbo.PessoaFisica PF ON (PF.CPF = FP.cd_CPF)
                    LEFT OUTER JOIN Sisobi.dbo.Obito O ON (FP.cd_CPF = O.CPF_RF)

                    CROSS APPLY(
                        SELECT TOP 1 C.DE_CARGO, TC.DE_TIPOCARGO
                        FROM SAGRES_MUNICIPAL.dbo.Cargos_2017 C
                        INNER JOIN SAGRES_MUNICIPAL.dbo.Tipo_Cargo TC ON (TC.TP_CARGO = C.TP_CARGO)
                        WHERE (C.CD_UGESTORA = FP.CD_UGESTORA AND C.CD_CARGO = FP.CD_CARGO)-- AND C.DT_MESANO = FP.DT_MESANO)
                    ) AS CAR

                WHERE FP.MESANO_ORDENADO BETWEEN @DT_INICIAL AND @DT_FINAL
                    AND FP.cd_UGestora = @UGESTORA
                    AND FP.MESANO_ORDENADO + '01' > O.Obito

                GROUP BY
                    PF.Nome
                    ,O.Falecido
                    ,FP.cd_CPF
                    ,FP.nu_Matricula
                    ,CAR.de_Cargo
                    ,FP.dt_MesAno
                    ,FP.MESANO_ORDENADO
                    ,CAR.de_TipoCargo
                    ,O.Obito

                ) AS RESULTADO
            ORDER BY nome, mesAno DESC
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getTipologiaDetalhadaFP_SISOBI');
            result.recordset = trataResultadosCruzamentoSisobi(result.recordset);
            result.recordset = insereAtributoTipologia(result.recordset, tipologias['fp_sisobi'].codigo);

            if (result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar tipologias FP-SISOBI para ${ugestora}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pela tipologias de ${ugestora}.`, 'getTipologiaDetalhadaFP_SISOBI'); });
}

export let getTipologiaDetalhadaEmpenhos_SISOBI = function (ugestora: string, dtInicio: string, dtFim: string) {

    const tempoInicial = new BenchmarkService();
    return connection.getConn().request()
        .input('DT_INICIAL', mssql.NVarChar, dtInicio)
        .input('DT_FINAL', mssql.NVarChar, dtFim)
        .input('UGESTORA', mssql.NVarChar, ugestora)
        .query(`
            SELECT
                RIGHT(E.cd_Credor,11) as cpf,
                E.no_Credor as nome,
                O.Falecido as nomeFalecido,
                E.de_Historico as descricaoEmpenho,
                L.de_TipoLicitacao as descLicitacao,
                E.nu_Empenho as numEmpenho,
                E.dt_Empenho as dataEmpenho,
                P.dt_Pagamento AS dataPagamento,
                E.vl_Empenho as vlEmpenho,
                P.vl_Pagamento as vlPagamento,
                O.Obito as dataObito,
                DATEDIFF(M, O.OBITO, E.dt_Empenho) AS mesesFalecido

            FROM SAGRES_MUNICIPAL.dbo.Empenhos E
                INNER JOIN SAGRES_MUNICIPAL.dbo.PAGAMENTOS P ON (E.cd_Ugestora = P.cd_Ugestora) AND (E.dt_Ano = P.dt_Ano) AND (E.cd_UnidOrcamentaria = P.cd_UnidOrcamentaria) AND (E.nu_Empenho = P.nu_Empenho)
                INNER JOIN Sisobi.dbo.Obito O ON  (RIGHT(E.cd_Credor, 11) = O.CPF_RF)
                LEFT OUTER JOIN SAGRES_MUNICIPAL.dbo.Tipo_Modalidade_Licitacao L ON (E.tp_Licitacao = L.tp_Licitacao)

            WHERE E.cd_UGestora = @UGESTORA
                AND E.MESANO_ORDENADO BETWEEN @DT_INICIAL AND @DT_FINAL
                AND E.tp_Credor = 1 -- CREDOR PESSOA FISICA
                AND E.dt_Empenho > O.Obito	-- EMPENHADO POS MORTE
                AND O.CPF_RF <> '00000000000' -- CPF INVALIDO
                AND E.cd_Credor <> '00000000000191' -- CPF INVALIDO

            GROUP BY
                E.cd_Credor,
                E.no_Credor,
                E.de_Historico,
                L.de_TipoLicitacao,
                E.nu_Empenho,
                E.dt_Empenho,
                E.vl_Empenho,
                P.vl_Pagamento,
                P.dt_MesAno,
                P.dt_Pagamento,
                O.CPF_RF,
                O.Falecido,
                O.Obito

            ORDER BY cpf, dataEmpenho DESC
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getTipologiaDetalhadaEmpenhos_SISOBI');
            result.recordset = trataResultadosCruzamentoSisobi(result.recordset);
            result.recordset = insereAtributoTipologia(result.recordset, tipologias['empenho_sisobi'].codigo);

            if (result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar tipologias Empenhos-SISOBI para ${ugestora}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { console.error(error); return logErroBuscaBD(error, `Falha na busca pela tipologias de ${ugestora}.`, 'getTipologiaDetalhadaEmpenhos_SISOBI'); });
}

export let getTipologiaDetalhadaFP_ENDERECOSRF = function (ugestora: string, dtInicio: string, dtFim: string) {

    const tempoInicial = new BenchmarkService();
    return connection.getConn().request()
        .input('DT_INICIAL', mssql.NVarChar, dtInicio)
        .input('DT_FINAL', mssql.NVarChar, dtFim)
        .input('UGESTORA', mssql.NVarChar, ugestora)
        .query(`
            ;WITH SERVIDORES(CPF)
            AS
            (
                SELECT DISTINCT FP.cd_CPF
                FROM MISC.dbo.FolhaPagamento_Servidores FP
                WHERE FP.cd_UGestora = @UGESTORA
                    AND FP.MESANO_ORDENADO BETWEEN @DT_INICIAL AND @DT_FINAL
            )

            ,UG(CD_UGESTORA, MUNICIPIO, MICROREGIAO)
            AS
            (
                SELECT DISTINCT G.cd_Ugestora, M.no_Municipio, MMM.MICROREGIAO
                FROM SAGRES_CONTROLE.dbo.Codigo_Ugestora G
                    INNER JOIN SAGRES_CONTROLE.dbo.Codigo_Municipios M ON (G.cd_Municipio = M.cd_Municipio)
                    LEFT OUTER JOIN SISPESQUISA.dbo.MUNICIPIO_MESOMICROREGIAO MMM ON (MMM.MUNICIPIO COLLATE Latin1_General_CI_AI = M.no_Municipio COLLATE Latin1_General_CI_AI)
            )

            ,ENDERECOS_SERVIDORES(CPF, NOME, S_MUNICIPIO, S_UF, S_MICRORREGIAO, UG_MICRORREGIAO, CUSTO)
            AS
            (
                SELECT  S.CPF, PF.Nome, PF.Municipio, PF.UF, MMM.MICROREGIAO, UG.MICROREGIAO,
                        CASE
                            WHEN ISNULL(CUSTO.custo, -1) = -1 THEN 20
                            ELSE Custo.custo + 1
                        END as custo

                FROM ReceitaNovo.dbo.PessoaFisica PF
                    INNER JOIN SERVIDORES S ON (PF.CPF = S.CPF)
                    LEFT OUTER JOIN SISPESQUISA.dbo.MUNICIPIO_MESOMICROREGIAO MMM ON (MMM.MUNICIPIO COLLATE Latin1_General_CI_AI = PF.MUNICIPIO AND PF.UF = 'PB')
                    LEFT OUTER JOIN UG ON (UG.CD_UGESTORA = @UGESTORA)
                    LEFT OUTER JOIN SISPESQUISA.dbo.tabela_custo_microregioes CUSTO ON (CUSTO.origem = MMM.MICROREGIAO COLLATE Latin1_General_CI_AI AND CUSTO.destino = UG.MICROREGIAO COLLATE Latin1_General_CI_AI)
                WHERE CUSTO.custo > 0 OR CUSTO.custo IS NULL
            )

            SELECT CPF as cpf, NOME as nome, S_MUNICIPIO as municipio, S_UF as uf, S_MICRORREGIAO as microrregiao, UG_MICRORREGIAO as microrregiaoUG, CUSTO as custo
            FROM ENDERECOS_SERVIDORES
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getTipologiaDetalhadaFP_ENDERECOSRF');
            result.recordset = insereAtributoTipologia(result.recordset, tipologias['fp_enderecosrf'].codigo);

            if (result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar tipologias FP-ENDERECOSRF para ${ugestora}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pela tipologias de ${ugestora}.`, 'getTipologiaDetalhadaFP_ENDERECOSRF'); });
}

export let getTipologiaDetalhadaFP_ENDERECOSRE = function (ugestora: string, dtInicio: string, dtFim: string) {

    const tempoInicial = new BenchmarkService();
    return connection.getConn().request()
        .input('DT_INICIAL', mssql.NVarChar, dtInicio)
        .input('DT_FINAL', mssql.NVarChar, dtFim)
        .input('UGESTORA', mssql.NVarChar, ugestora)
        .query(`
                ;WITH SERVIDORES(CPF)
                AS
                (
                    SELECT DISTINCT FP.cd_CPF
                    FROM MISC.dbo.FolhaPagamento_Servidores FP
                    WHERE FP.cd_UGestora = @UGESTORA
                        AND FP.MESANO_ORDENADO BETWEEN @DT_INICIAL AND @DT_FINAL
                )

                ,UG(CD_UGESTORA, MUNICIPIO, MICROREGIAO)
                AS
                (
                    SELECT DISTINCT G.cd_Ugestora, M.no_Municipio, MMM.MICROREGIAO
                    FROM SAGRES_CONTROLE.dbo.Codigo_Ugestora G
                        INNER JOIN SAGRES_CONTROLE.dbo.Codigo_Municipios M ON (G.cd_Municipio = M.cd_Municipio)
                        LEFT OUTER JOIN SISPESQUISA.dbo.MUNICIPIO_MESOMICROREGIAO MMM ON (MMM.MUNICIPIO COLLATE Latin1_General_CI_AI = M.no_Municipio COLLATE Latin1_General_CI_AI)
                )

                ,ENDERECOS_SERVIDORES(CPF, S_MUNICIPIO, S_UF, S_MICRORREGIAO, UG_MICRORREGIAO, CUSTO)
                AS
                (
                    SELECT  S.CPF, UPPER(E.Municipio), E.UF, MMM.MICROREGIAO, UG.MICROREGIAO,
                            CASE
                                WHEN ISNULL(CUSTO.custo, -1) = -1 THEN 20
                                ELSE Custo.custo + 1
                            END as custo

                    FROM SERVIDORES S
                        INNER JOIN SISPESQUISA.sispesquisa.enderecos E ON (S.CPF = E.cpf_cnpj)
                        LEFT OUTER JOIN SISPESQUISA.dbo.MUNICIPIO_MESOMICROREGIAO MMM ON (MMM.MUNICIPIO COLLATE Latin1_General_CI_AI = E.MUNICIPIO AND E.UF = 'PB')
                        LEFT OUTER JOIN UG ON (UG.CD_UGESTORA = @UGESTORA)
                        LEFT OUTER JOIN SISPESQUISA.dbo.tabela_custo_microregioes CUSTO ON (CUSTO.origem = MMM.MICROREGIAO COLLATE Latin1_General_CI_AI AND CUSTO.destino = UG.MICROREGIAO COLLATE Latin1_General_CI_AI)
                    WHERE (CUSTO.custo > 0 OR CUSTO.custo IS NULL)
                        AND NOT(E.UF = 'PB' AND MMM.MICROREGIAO IS NULL )
                )

                SELECT PF.CPF as cpf, PF.Nome as nome, S_MUNICIPIO as municipio, S_UF as uf, S_MICRORREGIAO as microrregiao, UG_MICRORREGIAO as microrregiaoUG, CUSTO as custo, COUNT(*) as qtd
                FROM ENDERECOS_SERVIDORES E
                    INNER JOIN ReceitaNovo.dbo.PessoaFisica PF ON (E.CPF = PF.CPF)
                GROUP BY PF.CPF, PF.Nome, S_MUNICIPIO, S_UF, S_MICRORREGIAO, UG_MICRORREGIAO, CUSTO
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getTipologiaDetalhadaFP_ENDERECOSRE');
            result.recordset = insereAtributoTipologia(result.recordset, tipologias['fp_enderecosre'].codigo);

            if (result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar tipologias FP-ENDERECOSRE para ${ugestora}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pela tipologias de ${ugestora}.`, 'getTipologiaDetalhadaFP_ENDERECOSRE'); });
}

export let getTipologiaDetalhadaFP_SocioPJ = function (ugestora: string, dtInicio: string, dtFim: string) {

    const tempoInicial = new BenchmarkService();
    return connection.getConn().request()
        .input('DT_INICIAL', mssql.NVarChar, dtInicio)
        .input('DT_FINAL', mssql.NVarChar, dtFim)
        .input('UGESTORA', mssql.NVarChar, ugestora)
        .query(`
            ;WITH SERVIDORES(CPF)
            AS
            (
                SELECT DISTINCT FP.cd_CPF
                FROM MISC.dbo.FolhaPagamento_Servidores FP
                WHERE FP.cd_UGestora = @UGESTORA
                        AND FP.MESANO_ORDENADO BETWEEN @DT_INICIAL AND @DT_FINAL
            )

            ,UG(CD_UGESTORA, MUNICIPIO, MICROREGIAO)
            AS
            (
                SELECT DISTINCT G.cd_Ugestora, M.no_Municipio, MMM.MICROREGIAO
                FROM SAGRES_CONTROLE.dbo.Codigo_Ugestora G
                    INNER JOIN SAGRES_CONTROLE.dbo.Codigo_Municipios M ON (G.cd_Municipio = M.cd_Municipio)
                    LEFT OUTER JOIN SISPESQUISA.dbo.MUNICIPIO_MESOMICROREGIAO MMM ON (MMM.MUNICIPIO COLLATE Latin1_General_CI_AI = M.no_Municipio COLLATE Latin1_General_CI_AI)
            )

            ,SERVIDORES_SOCIOS(CPF, CNPJ, RAZAOSOCIAL, MUNICIPIO, UF, MICRORREGIAO, CUSTO, MICRORREGIAO_UG)
            AS
            (
                SELECT DISTINCT
                    SERV.CPF, PJ.CNPJ, PJ.RazaoSocial, PJ.Municipio, PJ.UF, MMM.MICROREGIAO,
                    CASE
                        WHEN ISNULL(CUSTO.custo, -1) = -1 THEN 20
                        ELSE Custo.custo + 1
                    END as custo,
                    UG.MICROREGIAO

                FROM ReceitaNovo.dbo.Socio S

                    INNER JOIN SERVIDORES SERV ON (('000' + SERV.CPF) = S.CpfSocio AND S.IdentificadorSocio = 2)
                    INNER JOIN ReceitaNovo.dbo.PessoaJuridica PJ ON (PJ.CNPJ = S.Cnpj)
                    LEFT OUTER JOIN SISPESQUISA.dbo.MUNICIPIO_MESOMICROREGIAO MMM ON (MMM.MUNICIPIO COLLATE Latin1_General_CI_AI = PJ.MUNICIPIO AND PJ.UF = 'PB')
                    LEFT OUTER JOIN UG ON (UG.CD_UGESTORA = @UGESTORA)
                    LEFT OUTER JOIN SISPESQUISA.dbo.tabela_custo_microregioes CUSTO ON (CUSTO.origem = MMM.MICROREGIAO COLLATE Latin1_General_CI_AI AND CUSTO.destino = UG.MICROREGIAO COLLATE Latin1_General_CI_AI)
            )

            SELECT PF.CPF as cpf, PF.Nome as nome, S.CNPJ as cnpj, S.RAZAOSOCIAL as razaoSocial, S.MUNICIPIO as municipio, S.UF as uf, S.MICRORREGIAO as microrregiao, S.CUSTO as custo, S.MICRORREGIAO_UG as microrregiaoUG
            FROM SERVIDORES_SOCIOS S
                LEFT OUTER JOIN ReceitaNovo.dbo.PessoaFisica PF ON (S.CPF = PF.CPF)
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getTipologiaDetalhadaFP_SocioPJ');
            result.recordset = insereAtributoTipologia(result.recordset, tipologias['fp_sociopj'].codigo);

            if (result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar tipologias FP-SocioPJ para ${ugestora}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pela tipologias de ${ugestora}.`, 'getTipologiaDetalhadaFP_SocioPJ'); });
}

export let getTipologiaDetalhadaFP_ExtraVinculos = function (ugestora: string, dtInicio: string, dtFim: string) {

    const tempoInicial = new BenchmarkService();
    return connection.getConn().request()
        .input('DT_INICIAL', mssql.NVarChar, dtInicio)
        .input('DT_FINAL', mssql.NVarChar, dtFim)
        .input('UGESTORA', mssql.NVarChar, ugestora)
        .query(`
            ;WITH SERVIDORES_DO_ORGAO(CPF, ANO)
            AS
            (
                SELECT DISTINCT FP.cd_CPF, FP.dt_Ano
                FROM MISC.dbo.FolhaPagamento_Servidores FP
                WHERE FP.cd_UGestora = @UGESTORA
                        AND FP.MESANO_ORDENADO BETWEEN @DT_INICIAL AND @DT_FINAL
            )

            ,UG(CD_UGESTORA, UGESTORA, MUNICIPIO, MICROREGIAO)
            AS
            (
                SELECT DISTINCT G.cd_Ugestora, UPPER(G.de_Ugestora), UPPER(M.no_Municipio), MMM.MICROREGIAO
                FROM SAGRES_CONTROLE.dbo.Codigo_Ugestora G
                    INNER JOIN SAGRES_CONTROLE.dbo.Codigo_Municipios M ON (G.cd_Municipio = M.cd_Municipio)
                    LEFT OUTER JOIN SISPESQUISA.dbo.MUNICIPIO_MESOMICROREGIAO MMM ON (MMM.MUNICIPIO COLLATE Latin1_General_CI_AI = M.no_Municipio COLLATE Latin1_General_CI_AI)
            )

            ,EXTRA_VINCULOS(CPF, ANO, CD_UGESTORA)
            AS
            (
                SELECT DISTINCT FP.cd_CPF, FP.dt_Ano, FP.cd_UGestora
                FROM MISC.dbo.FolhaPagamento_Servidores FP
                    INNER JOIN SERVIDORES_DO_ORGAO S ON (FP.cd_CPF = S.CPF AND FP.dt_Ano = ANO)
                WHERE FP.cd_UGestora <> @UGESTORA
                    AND FP.MESANO_ORDENADO BETWEEN @DT_INICIAL AND @DT_FINAL
            )

            ,SERVIDORES_COM_EXTRAVINCULOS(CPF, ANO, CD_UGESTORA, UGESTORA, MUNICIPIO, MICRORREGIAO)
            AS
            (
                SELECT E.CPF, E.ANO, E.CD_UGESTORA, UG.UGESTORA, UG.MUNICIPIO, UG.MICROREGIAO
                FROM EXTRA_VINCULOS E
                    LEFT OUTER JOIN UG ON (E.CD_UGESTORA = UG.CD_UGESTORA)
            )

            SELECT	S.ANO as ano, S.CPF as cpf, PF.Nome as nome, S.UGESTORA as ugestora, S.MUNICIPIO as municipio, S.MICRORREGIAO as microrregiao, CUSTO.custo, UG.MICROREGIAO as microrregiaoUG
            FROM SERVIDORES_COM_EXTRAVINCULOS S
                INNER JOIN ReceitaNovo.dbo.PessoaFisica PF ON (S.CPF = PF.CPF)
                LEFT OUTER JOIN UG ON (UG.CD_UGESTORA = @UGESTORA)
                LEFT OUTER JOIN SISPESQUISA.dbo.tabela_custo_microregioes CUSTO ON (CUSTO.origem = S.MICRORREGIAO COLLATE Latin1_General_CI_AI AND CUSTO.destino = UG.MICROREGIAO COLLATE Latin1_General_CI_AI)
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getTipologiaDetalhadaFP_ExtraVinculos');
            result.recordset = insereAtributoTipologia(result.recordset, tipologias['fp_extravinculo'].codigo);

            if (result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar tipologias FP-ExtraVinculo para ${ugestora}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pela tipologias de ${ugestora}.`, 'getTipologiaDetalhadaFP_ExtraVinculos'); });
}
