import * as mssql from 'mssql';

import { connection }  from './../../services/db.service';
import { Pessoa } from './../schemas';
import { MODEL_PRIORITY } from './../../config';
import { 
    trataResultado,
    resultNotFound,
    resultFound,
    logErroBuscaBD
} from './../../utils';

const fonte_pf   = MODEL_PRIORITY['tipologias.pf'].fonte;
const rank_pf    = MODEL_PRIORITY['tipologias.pf'].rank;
const grupo_pf   = MODEL_PRIORITY['tipologias.pf'].grupo;

export let getTipologiaSimplificadoCPF_Tipologias = function (cpf: string){

    return connection.getConn().request()
        .input('cpf_parameter', mssql.Char(11), cpf)
        .query(`SELECT  CPF as cpf, Credor as nome,
                        T1 as t1,   T2 as t2,   T3 as t3,   T4 as t4,   T5 as t5,
                        T6 as t6,   T7 as t7,   T8 as t8,   T9 as t9,   T10 as t10,
                        T11 as t11, T12 as t12, T13 as t13, T14 as t14, T15 as t15,
                        T16 as t16, T17 as t17, T18 as t18, T19 as t19, T20 as t20,
                        T21 as t21, T22 as t22, T23 as t23, T24 as t24, T25 as t25,
                        T26 as t26, T27 as t27, T28 as t28, T29 as t29, T30 as t30,
                        T31 as t31,
                        TotalOcorrencias as totalOcorrencias, TotalPesoTipologia as totalPeso,
                        DoacaoEleitoral as doacaoEleitoral, TaxaDoacao as taxaDoacao, 
                        TotalTCS as totalTCS

                FROM Tipologias.dbo.MATRIZ_COMPLETA_PF
                WHERE CPF=@cpf_parameter AND TotalOcorrencias > 0
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar a tipologia de ${cpf}.`, fonte_pf);
            else
                return resultFound(result, fonte_pf, rank_pf, grupo_pf);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pela tipologia de ${cpf}.`, 'getTipologiaCPF_Tipologias'); });
};

export let getTipologiaPFSimplificadoUFMunicipio_Tipologias = function (dados){

    return connection.getConn().request()
        .input('uf', mssql.NVarChar, dados.uf)
        .input('municipio', mssql.VarChar, dados.municipio)
        .query(`SELECT  TOP 100
                        PF.CPF,
                        PF.Nome,
                        TIPOLOGIA.TotalPesoTipologia,
                        TIPOLOGIA.TotalOcorrencias,
                        TIPOLOGIA.DoacaoEleitoral,
                        TIPOLOGIA.TaxaDoacao,
                        TIPOLOGIA.TotalTCS

                FROM ReceitaNovo.dbo.PessoaFisica AS PF
                    LEFT OUTER JOIN Tipologias.dbo.MATRIZ_COMPLETA_PF AS TIPOLOGIA ON (PF.CPF = TIPOLOGIA.CPF)
                WHERE PF.UF=@uf AND PF.Municipio=@municipio
                ORDER BY TIPOLOGIA.TotalPesoTipologia DESC, TIPOLOGIA.TotalTCS DESC
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar tipologias de pessoas de ${dados.municipio}.`, fonte_pf);
            else
                return resultFound(result, fonte_pf, rank_pf, grupo_pf);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pelas tipologias de pessoas de ${dados.municipio}.`, 'getTipologiaPFSimplificadoUFMunicipio_Tipologias'); });
};

const fonte_pj   = MODEL_PRIORITY['tipologias.pj'].fonte;
const rank_pj    = MODEL_PRIORITY['tipologias.pj'].rank;
const grupo_pj   = MODEL_PRIORITY['tipologias.pj'].grupo;

export let getTipologiaSimplificadoCNPJ_Tipologias = function (cnpj: string){
    return connection.getConn().request()
        .input('cnpj_parameter', mssql.Char(14), cnpj)
        .query(`SELECT  CNPJ as cnpj, Credor as razaoSocial, NomeFantasia as nomeFantasia, 
                        T1 as t1,   T2 as t2,   T3 as t3,   T4 as t4,   T5 as t5,
                        T6 as t6,   T7 as t7,   T8 as t8,   T9 as t9,   T10 as t10,
                        T11 as t11, T13 as t13, T14 as t14,
                        T16 as t16, T17 as t17, T18 as t18, T19 as t19, T20 as t20,
                        T21 as t21, T22 as t22, T23 as t23, T24 as t24, T25 as t25,
                        T26 as t26, T27 as t27, T28 as t28, T29 as t29, T30 as t30,
                        T31 as t31, T32 as t32, T33 as t33, T34 as t34, T35 as t35,
                        T36 as t36, T37 as t37, T38 as t38, T39 as t39, T40 as t40,
                        T41 as t41, T42 as t42, T43 as t43, T44 as t44, T45 as t45,
                        T46 as t46, T47 as t47, T48 as t48, T49 as t49, T50 as t50,
                        T51 as t51, T52 as t52, T53 as t53, T54 as t54,
                        TotalOcorrencias as totalOcorrencias, TotalPesoTipologia as totalPeso,
                        DoacaoEleitoral as doacaoEleitoral, TaxaDoacao as taxaDoacao, 
                        TotalTCS as totalTCS

                FROM Tipologias.dbo.MATRIZ_COMPLETA_PJ
                WHERE CNPJ=@cnpj_parameter  AND TotalOcorrencias > 0
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar a tipologia de ${cnpj}.`, fonte_pj);
            else
                return resultFound(result, fonte_pj, rank_pj, grupo_pj);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pela tipologia de ${cnpj}.`, 'getTipologiaCNPJ_Tipologias');  });
};

export let getTipologiaPJSimplificadoUFMunicipio_Tipologias = function (dados){
    return connection.getConn().request()
        .input('uf', mssql.NVarChar, dados.uf)
        .input('municipio', mssql.VarChar, dados.municipio)
        .query(`SELECT TOP 100
                    PJ.CNPJ AS cnpj,
                    PJ.RazaoSocial AS razaoSocial,
                    PJ.NomeFantasia AS nomeFantasia,
                    TIPOLOGIA.TotalPesoTipologia AS totalPeso,
                    TIPOLOGIA.TotalOcorrencias AS totalOcorrencias,
                    TIPOLOGIA.DoacaoEleitoral AS doacaoEleitoral,
                    TIPOLOGIA.TaxaDoacao AS taxaDoacao,
                    TIPOLOGIA.TotalTCS AS totalTCS
                
                FROM ReceitaNovo.dbo.PessoaJuridica AS PJ
                    LEFT OUTER JOIN Tipologias.dbo.MATRIZ_COMPLETA_PJ AS TIPOLOGIA ON (PJ.CNPJ = TIPOLOGIA.CNPJ)
                WHERE PJ.UF=@uf AND PJ.Municipio=@municipio
                ORDER BY TIPOLOGIA.TotalPesoTipologia DESC, TIPOLOGIA.TotalTCS DESC
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar tipologias de pessoas de ${dados.municipio}.`, fonte_pj);
            else
                return resultFound(result, fonte_pj, rank_pj, grupo_pj);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pelas tipologias de pessoas de ${dados.municipio}.`, 'getTipologiaPJSimplificadoUFMunicipio_Tipologias');  });
};