-- TIPOLOGIA
--
-- Lista os servidores que foram empenhados pelo órgão após sua
-- data de óbito.
--
-- Gera um score baseado na quantidade de empenhos efetuados pós óbito e
-- no pior dos pagamentos (o com mais distância da data de óbito)


DECLARE @DT_INICIAL VARCHAR(6), @DT_FINAL VARCHAR(6), @UGESTORA INT, @PESO_TIPOLOGIA INT

SET @PESO_TIPOLOGIA = 2
SET @DT_INICIAL     = '200801'
SET @DT_FINAL       = '201703'
SET @UGESTORA       = 201095

;WITH EMPENHOS_COM_OBITO(CPF, NOME, NOMEFALECIDO, DATAOBITO, QTD, PIOREMPENHO)  --SELECIONA OS CPFS DOS FUNCIONÁRIOS DO ORGÃO ANALISADO
AS
(   
    SELECT RIGHT(E.cd_Credor,11), E.no_Credor, O.Falecido, O.Obito, COUNT(*) as QTD, MAX(DATEDIFF(M, O.Obito, E.dt_Empenho))
    FROM SAGRES_MUNICIPAL.dbo.Empenhos E
        INNER JOIN SAGRES_MUNICIPAL.dbo.PAGAMENTOS P ON (E.cd_Ugestora = P.cd_Ugestora) AND (E.dt_Ano = P.dt_Ano) AND (E.cd_UnidOrcamentaria = P.cd_UnidOrcamentaria) AND (E.nu_Empenho = P.nu_Empenho)
        INNER JOIN Sisobi.dbo.Obito O ON (RIGHT(E.cd_Credor, 11) = O.CPF_RF)

    WHERE E.cd_UGestora = @UGESTORA
        AND E.MESANO_ORDENADO BETWEEN @DT_INICIAL AND @DT_FINAL
        AND E.tp_Credor = 1 -- CREDOR PESSOA FISICA
        AND E.dt_Empenho > O.Obito	-- EMPENHADO POS MORTE
        AND O.CPF_RF <> '00000000000' -- CPF INVALIDO
        AND E.cd_Credor <> '00000000000191' -- CPF INVALIDO
    
    GROUP BY E.cd_Credor, E.no_Credor, O.Falecido, O.Obito
)

, RESULTADO(CPF, NOME, NOMEFALECIDO, DATAOBITO, SCORE)
AS
(
    SELECT CPF, NOME, NOMEFALECIDO, DATAOBITO, QTD * @PESO_TIPOLOGIA + PIOREMPENHO/12
    FROM EMPENHOS_COM_OBITO 
)

SELECT CPF as cpf, NOME as nome, NOMEFALECIDO as nomeFalecido, DATAOBITO as dataObito, SCORE as score
FROM RESULTADO
ORDER BY SCORE DESC