-- TIPOLOGIA
--
-- Lista os servidores do órgão que apresentem residência declarada na
-- RF com microrregião diferente da microrregião de onde se encontra o
-- órgão.
--
-- Gera um score baseado na distância entre o município do endereço 
-- declarado na RF e o da unidade gestora.

DECLARE @DT_INICIAL VARCHAR(6) = '201201', @DT_FINAL VARCHAR(6) = '201703', @UGESTORA INT = 101025, @PESO_TIPOLOGIA INT = 2

DECLARE @DT_INICIAL VARCHAR(6) = '201201', @DT_FINAL VARCHAR(6) = '201703', @UGESTORA INT = 101025, @PESO_TIPOLOGIA INT = 5

;WITH SERVIDORES(CPF)
AS
(   
    SELECT DISTINCT FP.cd_CPF
    FROM SAGRES_MUNICIPAL.dbo.FolhaPagamento_Servidores FP
    WHERE FP.cd_UGestora = @UGESTORA
        AND FP.MESANO_ORDENADO BETWEEN @DT_INICIAL AND @DT_FINAL
)
,UG(CD_UGESTORA, MUNICIPIO, MICROREGIAO)
AS
(
    SELECT DISTINCT G.cd_Ugestora, M.no_Municipio, MMM.MICROREGIAO
    FROM SAGRES_CONTROLE.dbo.Codigo_Ugestora G
        INNER JOIN SAGRES_CONTROLE.dbo.Codigo_Municipios M ON (G.cd_Municipio = M.cd_Municipio)
        LEFT OUTER JOIN SISPESQUISA.dbo.MUNICIPIO_MESOMICROREGIAO MMM ON (MMM.MUNICIPIO COLLATE Latin1_General_CI_AI = M.no_Municipio COLLATE Latin1_General_CI_AI)	
)

,SERVIDORES_RESPONSAVEIS_EMPRESA(CPF, CNPJ, RAZAOSOCIAL, PJ_MUNICIPIO, PJ_UF, UG_MUNICIPIO, FX_DISTANCIA, CUSTO)  
AS
(
    SELECT S.CPF, PJ.CNPJ, PJ.RazaoSocial, PJ.Municipio,  PJ.UF, UPPER(UG.Municipio), 
			CASE
				WHEN CUSTO.distancia < 30 THEN 0
				WHEN CUSTO.distancia < 50 THEN 1
				WHEN CUSTO.distancia < 100 THEN 2
				WHEN CUSTO.distancia < 200 THEN 3
				WHEN CUSTO.distancia < 300 THEN 4
				WHEN CUSTO.distancia < 500 THEN 5
				WHEN CUSTO.distancia < 1000 THEN 6
				WHEN CUSTO.distancia < 2000 THEN 7
				WHEN CUSTO.distancia < 3000 THEN 8
				ELSE 9
			END AS FX_DISTANCIA, CUSTO.distancia
    FROM ReceitaNovo.dbo.PessoaJuridica PJ
        INNER JOIN SERVIDORES S ON (S.CPF = PJ.CpfResponsavel)
        INNER JOIN UG ON (UG.CD_UGESTORA = @UGESTORA)
		LEFT OUTER JOIN SISPESQUISA.dbo.resultado CUSTO ON 
					(CUSTO.MUNICIPIO_ORIGEM COLLATE Latin1_General_CI_AI = UG.MUNICIPIO 
					AND CUSTO.uf_origem = 'PB'
					AND CUSTO.MUNICIPIO_DESTINO COLLATE Latin1_General_CI_AI = PJ.Municipio					
					AND CUSTO.uf_destino = PJ.UF)
	WHERE (PJ.Municipio <> UG.Municipio COLLATE Latin1_General_CI_AI)
) 
,RESULTADO(CPF, CUSTO, QTD)
AS 
(
    SELECT CPF, SUM(FX_DISTANCIA), COUNT(*) AS QTD
    FROM SERVIDORES_RESPONSAVEIS_EMPRESA S
	WHERE FX_DISTANCIA <> 0
    GROUP BY CPF
)

SELECT PF.CPF as cpf, PF.Nome as nome, CUSTO * QTD as score
FROM RESULTADO
    LEFT OUTER JOIN ReceitaNovo.dbo.PessoaFisica PF ON (RESULTADO.CPF = PF.CPF)
ORDER BY score DESC