-- TIPOLOGIA
--
-- Lista os servidores do órgão que tenham vínculos diversos ao órgão analisado.
--
-- Gera um score baseado na quantidade e distancia das unidades gestoras dos
-- extra vínculos para a unidade gestora analisada.

DECLARE @DT_INICIAL VARCHAR(6) = '201201', @DT_FINAL VARCHAR(6) = '201703', @UGESTORA INT = 101095, @PESO_TIPOLOGIA INT = 5

;WITH SERVIDORES_DO_ORGAO(CPF, ANO)
AS
(    
    SELECT DISTINCT FP.cd_CPF, FP.dt_Ano
    FROM SAGRES_MUNICIPAL.dbo.FolhaPagamento_Servidores FP
    WHERE FP.cd_UGestora = @UGESTORA
            AND FP.MESANO_ORDENADO BETWEEN @DT_INICIAL AND @DT_FINAL
)

,UG(CD_UGESTORA, UGESTORA, MUNICIPIO, MICROREGIAO)
AS
(
    SELECT DISTINCT G.cd_Ugestora, UPPER(G.de_Ugestora), UPPER(M.no_Municipio), MMM.MICROREGIAO
    FROM SAGRES_CONTROLE.dbo.Codigo_Ugestora G
        INNER JOIN SAGRES_CONTROLE.dbo.Codigo_Municipios M ON (G.cd_Municipio = M.cd_Municipio)
        LEFT OUTER JOIN SISPESQUISA.dbo.MUNICIPIO_MESOMICROREGIAO MMM ON (MMM.MUNICIPIO COLLATE Latin1_General_CI_AI = M.no_Municipio COLLATE Latin1_General_CI_AI)	
)

,EXTRA_VINCULOS(CPF, ANO, CD_UGESTORA)
AS
(
    SELECT DISTINCT FP.cd_CPF, FP.dt_Ano, FP.cd_UGestora
    FROM SAGRES_MUNICIPAL.dbo.FolhaPagamento_Servidores FP
        INNER JOIN SERVIDORES_DO_ORGAO S ON (FP.cd_CPF = S.CPF AND FP.dt_Ano = ANO)
    WHERE FP.cd_UGestora <> @UGESTORA
        AND FP.MESANO_ORDENADO BETWEEN @DT_INICIAL AND @DT_FINAL
)

,SERVIDORES_COM_EXTRAVINCULOS(CPF, ANO, CD_UGESTORA, UGESTORA, MUNICIPIO, MICRORREGIAO)
AS
(
    SELECT E.CPF, E.ANO, E.CD_UGESTORA, UG.UGESTORA, UG.MUNICIPIO, UG.MICROREGIAO
    FROM EXTRA_VINCULOS E
        LEFT OUTER JOIN UG ON (E.CD_UGESTORA = UG.CD_UGESTORA)
)

SELECT	S.CPF as cpf, PF.Nome as nome, SUM((CUSTO.custo+1) * @PESO_TIPOLOGIA) as score
FROM SERVIDORES_COM_EXTRAVINCULOS S
    INNER JOIN ReceitaNovo.dbo.PessoaFisica PF ON (S.CPF = PF.CPF)
    LEFT OUTER JOIN UG ON (UG.CD_UGESTORA = @UGESTORA)
    LEFT OUTER JOIN SISPESQUISA.dbo.tabela_custo_microregioes CUSTO ON (CUSTO.origem = S.MICRORREGIAO COLLATE Latin1_General_CI_AI AND CUSTO.destino = UG.MICROREGIAO COLLATE Latin1_General_CI_AI)
GROUP BY S.CPF, PF.Nome
ORDER BY score DESC