-- TIPOLOGIA
--
-- Lista os servidores que receberam pagamentos após sua
-- data de óbito.
--
-- Gera um score baseado na quantidade de pagamentos recebidos pós óbito e
-- no pior dos pagamentos (o com mais distância da data de óbito)

DECLARE @DT_INICIAL VARCHAR(6) = '201501', @DT_FINAL VARCHAR(6) = '201703', @UGESTORA INT = 101095, @PESO_TIPOLOGIA INT = 5

;WITH SERVIDORES(CPF, MESANO)
AS
(   
    SELECT DISTINCT FP.cd_CPF, FP.MESANO_ORDENADO
    FROM SAGRES_MUNICIPAL.dbo.FolhaPagamento_Servidores FP
    WHERE FP.cd_UGestora = @UGESTORA
        AND FP.MESANO_ORDENADO BETWEEN @DT_INICIAL AND @DT_FINAL
)

,SERVIDORES_COM_OBITO(CPF, NOMEFALECIDO, DATAOBITO, QTD, PIORPAGAMENTO)  
AS
(
    SELECT S.CPF, O.Falecido, O.Obito, COUNT(*) as QTD, MAX(DATEDIFF(M, O.Obito, S.MESANO + '01'))
    FROM Sisobi.dbo.Obito O 
        INNER JOIN SERVIDORES S ON (S.CPF = O.CPF_RF)

    WHERE S.MESANO+'01' > O.Obito
    GROUP BY S.CPF, O.Falecido, O.Obito
)
, RESULTADO(CPF, NOME, NOMEFALECIDO, DATAOBITO, SCORE)
AS
(
    SELECT S.CPF, NOME, NOMEFALECIDO, DATAOBITO, QTD * @PESO_TIPOLOGIA + PIORPAGAMENTO/12
    FROM SERVIDORES_COM_OBITO  S
        INNER JOIN ReceitaNovo.dbo.PessoaFisica PF ON (S.CPF = PF.CPF)
)

SELECT CPF as cpf, NOME as nome, NOMEFALECIDO as nomeFalecido, SCORE as score
FROM RESULTADO
ORDER BY SCORE DESC