-- TIPOLOGIA
--
-- Lista os servidores do órgão que sejam sócios de PJ e que essas
-- licitaram junto ao órgão.
--
-- Gera um score baseado na quantidade de pagamentos, valor dos pagamentos 
-- e distancia das empresas para a unidade gestora.

DECLARE @DT_INICIAL VARCHAR(6) = '201501', @DT_FINAL VARCHAR(6) = '201703', @UGESTORA INT = 101095, @PESO_TIPOLOGIA INT = 20

;WITH SERVIDORES(CPF, ANO)
AS
(   
    SELECT DISTINCT FP.cd_CPF, FP.dt_Ano
    FROM SAGRES_MUNICIPAL.dbo.FolhaPagamento_Servidores FP
    WHERE FP.cd_UGestora = @UGESTORA
            AND FP.MESANO_ORDENADO BETWEEN @DT_INICIAL AND @DT_FINAL
)

,UG(CD_UGESTORA, MUNICIPIO, MICROREGIAO)
AS
(
    SELECT DISTINCT G.cd_Ugestora, M.no_Municipio, MMM.MICROREGIAO
    FROM SAGRES_CONTROLE.dbo.Codigo_Ugestora G
        INNER JOIN SAGRES_CONTROLE.dbo.Codigo_Municipios M ON (G.cd_Municipio = M.cd_Municipio)
        LEFT OUTER JOIN SISPESQUISA.dbo.MUNICIPIO_MESOMICROREGIAO MMM ON (MMM.MUNICIPIO COLLATE Latin1_General_CI_AI = M.no_Municipio COLLATE Latin1_General_CI_AI)	
)

,SERVIDORES_SOCIOS(ANO, CPF, CNPJ, RAZAOSOCIAL, MUNICIPIO, UF, MICRORREGIAO, CUSTO)
AS
(
    SELECT 
        SERV.ANO, SERV.CPF, PJ.CNPJ, PJ.RazaoSocial, PJ.Municipio, PJ.UF, MMM.MICROREGIAO, 
        CASE
            WHEN ISNULL(CUSTO.custo, -1) = -1 THEN 20
            ELSE Custo.custo + 1
        END as custo

    FROM ReceitaNovo.dbo.Socio S
        INNER JOIN SERVIDORES SERV ON (('000' + SERV.CPF) = S.CpfSocio AND S.IdentificadorSocio = 2)
        INNER JOIN ReceitaNovo.dbo.PessoaJuridica PJ ON (PJ.CNPJ = S.Cnpj)
        LEFT OUTER JOIN SISPESQUISA.dbo.MUNICIPIO_MESOMICROREGIAO MMM ON (MMM.MUNICIPIO COLLATE Latin1_General_CI_AI = PJ.MUNICIPIO AND PJ.UF = 'PB')
        LEFT OUTER JOIN UG ON (UG.CD_UGESTORA = @UGESTORA)
        LEFT OUTER JOIN SISPESQUISA.dbo.tabela_custo_microregioes CUSTO ON (CUSTO.origem = MMM.MICROREGIAO COLLATE Latin1_General_CI_AI AND CUSTO.destino = UG.MICROREGIAO COLLATE Latin1_General_CI_AI)
)

,EMPRESAS_EMPENHO(CPF,  CNPJ, RAZAOSOCIAL, MUNICIPIO, UF, MICRORREGIAO, CUSTO, EMPENHADO, PAGAMENTO, QTD)
AS
(
    SELECT	S.CPF, S.CNPJ, S.RAZAOSOCIAL, S.MUNICIPIO, S.UF, S.MICRORREGIAO, S.CUSTO,
            SUM(E.vl_Empenho) AS EMPENHADO, SUM(P.vl_Pagamento) AS PAGAMENTO, COUNT(*) AS QTD
        
    FROM SERVIDORES_SOCIOS S	
        INNER JOIN SAGRES_MUNICIPAL.dbo.Empenhos E ON (S.CNPJ = E.cd_Credor AND S.ANO = E.dt_Ano)
        INNER JOIN SAGRES_MUNICIPAL.dbo.PAGAMENTOS P ON (E.cd_Ugestora = P.cd_Ugestora) AND (E.dt_Ano = P.dt_Ano) AND (E.cd_UnidOrcamentaria = P.cd_UnidOrcamentaria) AND (E.nu_Empenho = P.nu_Empenho)
        --INNER JOIN ReceitaNovo.dbo.PessoaFisica PF ON (S.CPF = PF.CPF)

    WHERE E.cd_UGestora = @UGESTORA
        AND E.tp_Credor = 2
        AND E.MESANO_ORDENADO BETWEEN @DT_INICIAL AND @DT_FINAL

    GROUP BY S.CPF, S.CNPJ, S.RAZAOSOCIAL, S.MUNICIPIO, S.UF, S.MICRORREGIAO, S.CUSTO
)


,RESULTADO(CPF, NOME, CNPJ, RAZAOSOCIAL, MUNICIPIO, UF, MICRORREGIAO, CUSTO, QTD, FX_PAGO)
AS
(
    SELECT E.CPF, PF.Nome, CNPJ, RAZAOSOCIAL, E.MUNICIPIO, E.UF, MICRORREGIAO, CUSTO, QTD,
            CASE
                WHEN PAGAMENTO > 0 AND PAGAMENTO <= 10000 THEN 1
                WHEN PAGAMENTO > 10000 AND PAGAMENTO <= 100000 THEN 2
                WHEN PAGAMENTO > 100000 AND PAGAMENTO <= 250000 THEN 3
                WHEN PAGAMENTO > 250000 AND PAGAMENTO <= 500000 THEN 4
                WHEN PAGAMENTO > 500000 AND PAGAMENTO <= 750000 THEN 5
                WHEN PAGAMENTO > 750000 AND PAGAMENTO <= 1000000 THEN 6
                WHEN PAGAMENTO > 1000000 THEN 7
                ELSE 0
            END AS FX_PAGO
    FROM EMPRESAS_EMPENHO E
    INNER JOIN ReceitaNovo.dbo.PessoaFisica PF ON (E.CPF = PF.CPF)
)

SELECT CPF as cpf, NOME as nome, @PESO_TIPOLOGIA * ((FX_PAGO+1) * (FX_PAGO+1) + (QTD/CUSTO)) as score
FROM RESULTADO
ORDER BY 2 DESC