-- TIPOLOGIA
--
-- Lista os servidores do órgão que tenham PJ.
--
-- Gera um score baseado na quantidade e distancia das empresas
-- para a unidade gestora.

DECLARE @DT_INICIAL VARCHAR(6) = '201501', @DT_FINAL VARCHAR(6) = '201703', @UGESTORA INT = 101095, @PESO_TIPOLOGIA INT = 5

;WITH SERVIDORES(CPF)
AS
(   
    SELECT DISTINCT FP.cd_CPF
    FROM SAGRES_MUNICIPAL.dbo.FolhaPagamento_Servidores FP
    WHERE FP.cd_UGestora = @UGESTORA
            AND FP.MESANO_ORDENADO BETWEEN @DT_INICIAL AND @DT_FINAL 
)

,UG(CD_UGESTORA, MUNICIPIO, MICROREGIAO)
AS
(
    SELECT DISTINCT G.cd_Ugestora, M.no_Municipio, MMM.MICROREGIAO
    FROM SAGRES_CONTROLE.dbo.Codigo_Ugestora G
        INNER JOIN SAGRES_CONTROLE.dbo.Codigo_Municipios M ON (G.cd_Municipio = M.cd_Municipio)
        LEFT OUTER JOIN SISPESQUISA.dbo.MUNICIPIO_MESOMICROREGIAO MMM ON (MMM.MUNICIPIO COLLATE Latin1_General_CI_AI = M.no_Municipio COLLATE Latin1_General_CI_AI)	
)

,SERVIDORES_SOCIOS(CPF, CNPJ, RAZAOSOCIAL, MUNICIPIO, UF, MICRORREGIAO, CUSTO)
AS
(
    SELECT DISTINCT 
        SERV.CPF, PJ.CNPJ, PJ.RazaoSocial, PJ.Municipio, PJ.UF, MMM.MICROREGIAO, 
        CASE
            WHEN ISNULL(CUSTO.custo, -1) = -1 THEN 20
            ELSE Custo.custo + 1
        END as custo
    FROM ReceitaNovo.dbo.Socio S
        INNER JOIN SERVIDORES SERV ON (('000' + SERV.CPF) = S.CpfSocio AND S.IdentificadorSocio = 2)
        INNER JOIN ReceitaNovo.dbo.PessoaJuridica PJ ON (PJ.CNPJ = S.Cnpj)
        LEFT OUTER JOIN SISPESQUISA.dbo.MUNICIPIO_MESOMICROREGIAO MMM ON (MMM.MUNICIPIO COLLATE Latin1_General_CI_AI = PJ.MUNICIPIO AND PJ.UF = 'PB')
        LEFT OUTER JOIN UG ON (UG.CD_UGESTORA = @UGESTORA)
        LEFT OUTER JOIN SISPESQUISA.dbo.tabela_custo_microregioes CUSTO ON (CUSTO.origem = MMM.MICROREGIAO COLLATE Latin1_General_CI_AI AND CUSTO.destino = UG.MICROREGIAO COLLATE Latin1_General_CI_AI)
)
,RESULTADO(CPF, MICRORREGIAO, UF, CUSTO, QTD)
AS 
(
    SELECT CPF, MICRORREGIAO, UF, CUSTO, COUNT(*) AS QTD
    FROM SERVIDORES_SOCIOS S
    GROUP BY CPF, MICRORREGIAO, UF, CUSTO
)

SELECT PF.CPF as cpf, PF.Nome as nome, SUM(CUSTO * QTD * @PESO_TIPOLOGIA) as score
FROM RESULTADO
    LEFT OUTER JOIN ReceitaNovo.dbo.PessoaFisica PF ON (RESULTADO.CPF = PF.CPF)
GROUP BY PF.CPF, PF.Nome
ORDER BY 3 DESC