import * as mssql from 'mssql';

import { connection }  from '../../services/db.service';
import { MODEL_PRIORITY, API_CONFIG } from '../../config';
import {
    trataResultado,
    resultNotFound,
    resultFound,
    fixVeiculoCPF_CNPJ,
    logErroBuscaBD
} from '../../utils';

import { SIGLA_FONTE } from './../_fontes';

const fonte_dado = SIGLA_FONTE['renavam_2016'];

const fonte = MODEL_PRIORITY['sispesquisa.renavam_2016'].fonte;
const rank  = MODEL_PRIORITY['sispesquisa.renavam_2016'].rank;
const grupo = MODEL_PRIORITY['sispesquisa.renavam_2016'].grupo;

export let getVeiculoDetalhadoCPF_Renavam_2016 = function (cpf: string){

    return connection.getConn().request()
        .input('cpf_parameter', mssql.VarChar, cpf)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        PLACA as placa, CHASSI as chassi, CODIGO_RENAVAM as renavam,
                        ANO_FABRICACAO as anoFab, ANO_MODELO as anoMod, ds_tipo_veiculo as tipo,
                        ds_cor as cor, ds_marca_modelo as marcaModelo, ds_especie as especie,
                        ds_combustivel as combustivel, NUMERO_DOCUMENTO_PROPRIETARIO as cpf,
                        ds_restricao1 as situacao, ds_restricao2 as observacao,
                        ds_restricao3 as restricao, ds_restricao4 as restricao_,
                        CAST(DATA_ULTIMA_ATUALIZACAO as date) as dataAtualizacao,
                        PF.Nome as nome,
                        VAL.valor as valor,
                        '${fonte_dado}' as fonte

                FROM Renavam_2014.dbo.renavam_2016_08 V
                    LEFT OUTER JOIN ReceitaNovo.dbo.PessoaFisica AS PF ON (V.NUMERO_DOCUMENTO_PROPRIETARIO = PF.CPF)
                    LEFT OUTER JOIN Renavam_2014.gdf_ipva.valores_veiculos_ipva_2016_df VAL ON (V.CODIGO_MARCA_MODELO = VAL.CODIGO_MARCA_MODELO AND V.ANO_FABRICACAO = VAL.ano)
                WHERE NUMERO_DOCUMENTO_PROPRIETARIO=@cpf_parameter
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o veiculo de ${cpf}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelo veiculo de ${cpf}.`, 'getVeiculoDetalhadoCPF_Renavam_2016'); });
};

export let getVeiculoDetalhadoCNPJ_Renavam_2016 = function (cnpj: string){

    return connection.getConn().request()
        .input('cnpj_parameter', mssql.VarChar, cnpj)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        PLACA as placa, CHASSI as chassi, CODIGO_RENAVAM as renavam,
                        ANO_FABRICACAO as anoFab, ANO_MODELO as anoMod, ds_tipo_veiculo as tipo, ds_cor as cor,
                        ds_marca_modelo as marcaModelo, ds_especie as especie,
                        ds_combustivel as combustivel, NUMERO_DOCUMENTO_PROPRIETARIO as cnpj,
                        ds_restricao1 as situacao, ds_restricao2 as observacao,
                        ds_restricao3 as restricao, ds_restricao4 as restricao_,
                        CAST(DATA_ULTIMA_ATUALIZACAO as date) as dataAtualizacao,
                        PJ.RazaoSocial as razaoSocial,
                        VAL.valor as valor

                FROM Renavam_2014.dbo.renavam_2016_08 V
                    LEFT OUTER JOIN ReceitaNovo.dbo.PessoaJuridica AS PJ ON (V.NUMERO_DOCUMENTO_PROPRIETARIO = PJ.CNPJ)
                    LEFT OUTER JOIN Renavam_2014.gdf_ipva.valores_veiculos_ipva_2016_df VAL ON (V.CODIGO_MARCA_MODELO = VAL.CODIGO_MARCA_MODELO AND V.ANO_FABRICACAO = VAL.ano)
                WHERE NUMERO_DOCUMENTO_PROPRIETARIO=@cnpj_parameter
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o veiculo de ${cnpj}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelo veiculo de ${cnpj}.`, 'getVeiculoDetalhadoCNPJ_Renavam_2016'); });
};

export let getVeiculoDetalhadoChassi_Renavam_2016 = function (chassi: string){

    return connection.getConn().request()
        .input('chassi_parameter', mssql.VarChar, chassi)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        PLACA as placa, CHASSI as chassi, CODIGO_RENAVAM as renavam,
                        ANO_FABRICACAO as anoFab, ANO_MODELO as anoMod, ds_tipo_veiculo as tipo, ds_cor as cor,
                        ds_marca_modelo as marcaModelo, ds_especie as especie,
                        ds_combustivel as combustivel, NUMERO_DOCUMENTO_PROPRIETARIO as cpf,
                        ds_restricao1 as situacao, ds_restricao2 as observacao,
                        ds_restricao3 as restricao, ds_restricao4 as restricao_,
                        CAST(DATA_ULTIMA_ATUALIZACAO as date) as dataAtualizacao

                FROM Renavam_2014.dbo.renavam_2016_08
                WHERE CHASSI=@chassi_parameter
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o veiculo de ${chassi}.`, fonte);
            else{
                result.recordset = fixVeiculoCPF_CNPJ(result);
                return resultFound(result, fonte, rank, grupo);
            }
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelo veiculo de ${chassi}.`, 'getVeiculoDetalhadoChassi_Renavam_2016'); });
};

export let getVeiculoDetalhadoRenavam_Renavam_2016 = function (renavam: string){

    return connection.getConn().request()
        .input('renavam_parameter', mssql.VarChar, renavam)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        PLACA as placa, CHASSI as chassi, CODIGO_RENAVAM as renavam,
                        ANO_FABRICACAO as anoFab, ANO_MODELO as anoMod, ds_tipo_veiculo as tipo, ds_cor as cor,
                        ds_marca_modelo as marcaModelo, ds_especie as especie,
                        ds_combustivel as combustivel, NUMERO_DOCUMENTO_PROPRIETARIO as cpf,
                        ds_restricao1 as situacao, ds_restricao2 as observacao,
                        ds_restricao3 as restricao, ds_restricao4 as restricao_,
                        CAST(DATA_ULTIMA_ATUALIZACAO as date) as dataAtualizacao

                FROM Renavam_2014.dbo.renavam_2016_08
                WHERE CODIGO_RENAVAM=@renavam_parameter
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o veiculo de ${renavam}.`, fonte);
            else{
                result.recordset = fixVeiculoCPF_CNPJ(result);
                return resultFound(result, fonte, rank, grupo);
            }
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelo veiculo de ${renavam}.`, 'getVeiculoDetalhadoRenavam_Renavam_2016'); });
};

export let getVeiculoDetalhadoPlaca_Renavam_2016 = function (placa: string){

    return connection.getConn().request()
        .input('placa_parameter', mssql.VarChar, placa)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        PLACA as placa, CHASSI as chassi, CODIGO_RENAVAM as renavam,
                        ANO_FABRICACAO as anoFab, ANO_MODELO as anoMod, ds_tipo_veiculo as tipo, ds_cor as cor,
                        ds_marca_modelo as marcaModelo, ds_especie as especie,
                        ds_combustivel as combustivel, NUMERO_DOCUMENTO_PROPRIETARIO as cpf,
                        ds_restricao1 as situacao, ds_restricao2 as observacao,
                        ds_restricao3 as restricao, ds_restricao4 as restricao_,
                        CAST(DATA_ULTIMA_ATUALIZACAO as date) as dataAtualizacao

                FROM Renavam_2014.dbo.renavam_2016_08
                WHERE PLACA=@placa_parameter
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o veiculo de ${placa}.`, fonte);
            else{
                result.recordset = fixVeiculoCPF_CNPJ(result);
                return resultFound(result, fonte, rank, grupo);
            }
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelo veiculo de ${placa}.`, 'getVeiculoDetalhadoPlaca_Renavam_2016'); });
};

export let getVeiculoDetalhadoNome_Renavam_2016 = function (nome: string){

    return connection.getConn().request()
        .input('nome_parameter', mssql.VarChar, nome)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        PLACA as placa, CHASSI as chassi, CODIGO_RENAVAM as renavam,
                        ANO_FABRICACAO as anoFab, ANO_MODELO as anoMod, ds_tipo_veiculo as tipo, ds_cor as cor,
                        ds_marca_modelo as marcaModelo, ds_especie as especie,
                        ds_combustivel as combustivel, NUMERO_DOCUMENTO_PROPRIETARIO as cpf,
                        ds_restricao1 as situacao, ds_restricao2 as observacao,
                        ds_restricao3 as restricao, ds_restricao4 as restricao_,
                        CAST(DATA_ULTIMA_ATUALIZACAO as date) as dataAtualizacao

                FROM Renavam_2014.dbo.renavam_2016_08 AS Veiculos
                    INNER JOIN ReceitaNovo.dbo.PessoaFisica AS PF ON (Veiculos.NUMERO_DOCUMENTO_PROPRIETARIO = PF.CPF)
                WHERE CONTAINS(Nome, @nome_parameter)
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o veiculo de ${nome}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelo veiculo de ${nome}.`, 'getVeiculoDetalhadoNome_Renavam_2016'); });
};

