import * as mssql from 'mssql';

import { connection }  from '../../services/db.service';
import { MODEL_PRIORITY, API_CONFIG } from '../../config';
import {
    trataResultado,
    resultNotFound,
    resultFound,
    fixVeiculoCPF_CNPJ,
    logErroBuscaBD
} from '../../utils';

import { SIGLA_FONTE } from './../_fontes';

const fonte_dado = SIGLA_FONTE['renavam_2016'];

const fonte  = MODEL_PRIORITY['sispesquisa.veiculos'].fonte;
const rank   = MODEL_PRIORITY['sispesquisa.veiculos'].rank;
const grupo  = MODEL_PRIORITY['sispesquisa.veiculos'].grupo;

export let getVeiculoDetalhadoCPF_Sispesquisa_Veiculos = function (cpf: string){

    return connection.getConn().request()
        .input('cpf_parameter', mssql.NVarChar, cpf)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        PLACA as placa, CHASSI as chassi, TIPO_CHASSI as tipoChassi, RENAVAM as renavam, ANO_FAB as anoFab,
                        ANO_MOD as anoMod, TIPO as tipo, MARCA_MODELO as marcaModelo, ESPECIE as especie, PROCEDENCIA as procedencia,
                        COMBUSTIVEL as combustivel, CPF_CNPJ as cpf, SITUACAO as situacao, OBSERVACAO as observacao,
                        RESTRICAO as restricao, RESTRICAO_ as restricao_, DT_CAD as dataCadastro,
                        CASE
                            WHEN TIPO_PESSOA = 1 THEN 'Física'
                            ELSE 'Jurídica'
                        END AS tipoPessoa,
                        PF.Nome as nome

                FROM SISPESQUISA.sispesquisa.veiculos
                    LEFT OUTER JOIN ReceitaNovo.dbo.PessoaFisica AS PF ON (veiculos.CPF_CNPJ = PF.CPF)
                WHERE CPF_CNPJ=@cpf_parameter
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o veiculo de ${cpf}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelo veiculo de ${cpf}.`, 'getVeiculoDetalhadoCPF_Sispesquisa_Veiculos'); });
};

export let getVeiculoDetalhadoCNPJ_Sispesquisa_Veiculos = function (cnpj: string){

    return connection.getConn().request()
        .input('cnpj_parameter', mssql.NVarChar, cnpj)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        PLACA as placa, CHASSI as chassi, TIPO_CHASSI as tipoChassi, RENAVAM as renavam, ANO_FAB as anoFab,
                        ANO_MOD as anoMod, TIPO as tipo, MARCA_MODELO as marcaModelo, ESPECIE as especie, PROCEDENCIA as procedencia,
                        COMBUSTIVEL as combustivel, CPF_CNPJ as cnpj, SITUACAO as situacao, OBSERVACAO as observacao,
                        RESTRICAO as restricao, RESTRICAO_ as restricao_, DT_CAD as dataCadastro,
                        CASE
                            WHEN TIPO_PESSOA = 1 THEN 'Física'
                            ELSE 'Jurídica'
                        END AS tipoPessoa

                FROM SISPESQUISA.sispesquisa.veiculos
                    LEFT OUTER JOIN ReceitaNovo.dbo.PessoaJuridica AS PJ ON (veiculos.CPF_CNPJ = PJ.CNPJ)
                WHERE CPF_CNPJ=@cnpj_parameter
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o veiculo de ${cnpj}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelo veiculo de ${cnpj}.`, 'getVeiculoDetalhadoCNPJ_Sispesquisa_Veiculos'); });
};

export let getVeiculoDetalhadoChassi_Sispesquisa_Veiculos = function (chassi: string){

    return connection.getConn().request()
        .input('chassi_parameter', mssql.NVarChar, chassi)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        PLACA as placa, CHASSI as chassi, TIPO_CHASSI as tipoChassi, RENAVAM as renavam, ANO_FAB as anoFab,
                        ANO_MOD as anoMod, TIPO as tipo, MARCA_MODELO as marcaModelo, ESPECIE as especie, PROCEDENCIA as procedencia,
                        COMBUSTIVEL as combustivel, CPF_CNPJ as cpf, SITUACAO as situacao, OBSERVACAO as observacao,
                        RESTRICAO as restricao, RESTRICAO_ as restricao_, DT_CAD as dataCadastro,
                        CASE
                            WHEN TIPO_PESSOA = 1 THEN 'Física'
                            ELSE 'Jurídica'
                        END AS tipoPessoa

                FROM SISPESQUISA.sispesquisa.veiculos
                WHERE CHASSI=@chassi_parameter
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o veiculo de ${chassi}.`, fonte);
            else{
                result.recordset = fixVeiculoCPF_CNPJ(result);
                return resultFound(result, fonte, rank, grupo);
            }
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelo veiculo de ${chassi}.`, 'getVeiculoDetalhadoChassi_Sispesquisa_Veiculos'); });
};

export let getVeiculoDetalhadoRenavam_Sispesquisa_Veiculos = function (renavam: string){

    return connection.getConn().request()
        .input('renavam_parameter', mssql.NVarChar, renavam)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        PLACA as placa, CHASSI as chassi, TIPO_CHASSI as tipoChassi, RENAVAM as renavam, ANO_FAB as anoFab,
                        ANO_MOD as anoMod, TIPO as tipo, MARCA_MODELO as marcaModelo, ESPECIE as especie, PROCEDENCIA as procedencia,
                        COMBUSTIVEL as combustivel, CPF_CNPJ as cpf, SITUACAO as situacao, OBSERVACAO as observacao,
                        RESTRICAO as restricao, RESTRICAO_ as restricao_, DT_CAD as dataCadastro,
                        CASE
                            WHEN TIPO_PESSOA = 1 THEN 'Física'
                            ELSE 'Jurídica'
                        END AS tipoPessoa

                FROM SISPESQUISA.sispesquisa.veiculos
                WHERE RENAVAM=@renavam_parameter
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o veiculo de ${renavam}.`, fonte);
            else{
                result.recordset = fixVeiculoCPF_CNPJ(result);
                return resultFound(result, fonte, rank, grupo);
            }
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelo veiculo de ${renavam}.`, 'getVeiculoDetalhadoRenavam_Sispesquisa_Veiculos'); });
};

export let getVeiculoDetalhadoPlaca_Sispesquisa_Veiculos = function (placa: string){

    return connection.getConn().request()
        .input('placa_parameter', mssql.NVarChar, placa)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        PLACA as placa, CHASSI as chassi, TIPO_CHASSI as tipoChassi, RENAVAM as renavam, ANO_FAB as anoFab,
                        ANO_MOD as anoMod, TIPO as tipo, MARCA_MODELO as marcaModelo, ESPECIE as especie, PROCEDENCIA as procedencia,
                        COMBUSTIVEL as combustivel, CPF_CNPJ as cpf, SITUACAO as situacao, OBSERVACAO as observacao,
                        RESTRICAO as restricao, RESTRICAO_ as restricao_, DT_CAD as dataCadastro,
                        CASE
                            WHEN TIPO_PESSOA = 1 THEN 'Física'
                            ELSE 'Jurídica'
                        END AS tipoPessoa

                FROM SISPESQUISA.sispesquisa.veiculos
                WHERE PLACA=@placa_parameter
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o veiculo de ${placa}.`, fonte);
            else{
                result.recordset = fixVeiculoCPF_CNPJ(result);
                return resultFound(result, fonte, rank, grupo);
            }
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelo veiculo de ${placa}.`, 'getVeiculoDetalhadoPlaca_Sispesquisa_Veiculos'); });
};

export let getVeiculoDetalhadoPlaca_LIKE_Sispesquisa_Veiculos = function (placa: string){

    return connection.getConn().request()
        .input('placa_parameter', mssql.NVarChar, '%'+placa+'%')
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        PLACA as placa, CHASSI as chassi, TIPO_CHASSI as tipoChassi, RENAVAM as renavam, ANO_FAB as anoFab,
                        ANO_MOD as anoMod, TIPO as tipo, MARCA_MODELO as marcaModelo, ESPECIE as especie, PROCEDENCIA as procedencia,
                        COMBUSTIVEL as combustivel, CPF_CNPJ as cpf, SITUACAO as situacao, OBSERVACAO as observacao,
                        RESTRICAO as restricao, RESTRICAO_ as restricao_, DT_CAD as dataCadastro,
                        CASE
                            WHEN TIPO_PESSOA = 1 THEN 'Física'
                            ELSE 'Jurídica'
                        END AS tipoPessoa

                FROM SISPESQUISA.sispesquisa.veiculos
                WHERE PLACA LIKE @placa_parameter
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o veiculo de ${placa}.`, fonte);
            else{
                result.recordset = fixVeiculoCPF_CNPJ(result);
                return resultFound(result, fonte, rank, grupo);
            }
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelo veiculo de ${placa}.`, 'getVeiculoDetalhadoPlaca_Sispesquisa_Veiculos'); });
};

export let getVeiculoDetalhadoNome_Sispesquisa_Veiculos = function (nome: string){

    return connection.getConn().request()
        .input('nome_parameter', mssql.NVarChar, nome)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        PLACA as placa, CHASSI as chassi, TIPO_CHASSI as tipoChassi, RENAVAM as renavam, ANO_FAB as anoFab,
                        ANO_MOD as anoMod, TIPO as tipo, MARCA_MODELO as marcaModelo, ESPECIE as especie, PROCEDENCIA as procedencia,
                        COMBUSTIVEL as combustivel, CPF_CNPJ as cpf, SITUACAO as situacao, OBSERVACAO as observacao,
                        RESTRICAO as restricao, RESTRICAO_ as restricao_, DT_CAD as dataCadastro, PF.Nome as nomeProprietario,
                        CASE
                            WHEN TIPO_PESSOA = 1 THEN 'Física'
                            ELSE 'Jurídica'
                        END AS tipoPessoa

                FROM SISPESQUISA.sispesquisa.veiculos
                    INNER JOIN ReceitaNovo.dbo.PessoaFisica AS PF ON (Veiculos.CPF_CNPJ = PF.CPF)
                WHERE CONTAINS(Nome, @nome_parameter)
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o veiculo de ${nome}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelo veiculo de ${nome}.`, 'getVeiculoCPF_CNPJ_Sispesquisa_Veiculos'); });
};

