import * as mssql from 'mssql';

import { connection }  from '../../services/db.service';
import { MODEL_PRIORITY, API_CONFIG } from '../../config';
import {
    trataResultado,
    resultNotFound,
    resultFound,
    fixVeiculoCPF_CNPJ,
    logErroBuscaBD
} from '../../utils';

import { SIGLA_FONTE } from './../_fontes';

const fonte_dado = SIGLA_FONTE['sispesquisa.veiculos_19_07_2017'];

const fonte  = MODEL_PRIORITY['sispesquisa.veiculos_novo'].fonte;
const rank   = MODEL_PRIORITY['sispesquisa.veiculos_novo'].rank;
const grupo  = MODEL_PRIORITY['sispesquisa.veiculos_novo'].grupo;

export let getVeiculoDetalhadoCPF_Sispesquisa_VeiculosNovo = function (cpf: string){

    return connection.getConn().request()
        .input('cpf_parameter', mssql.VarChar, cpf)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        PLACA as placa, CHASSI as chassi, TIPO_CHASSI as tipoChassi, RENAVAM as renavam, Ano_Fabricacao as anoFab,
                        ANO_MOD as anoMod, Descricao_Tipo as tipo, Descricao_Marca_Modelo as marcaModelo, Categoria as especie, Especie as procedencia,
                        COMBUSTIVEL as combustivel, CPF_CNPJ_Proprietario as cpf, Ultimo_Ano_Licenciamento as ultLicenciamento, Restricao_Gravame as situacao,
                        Restricao_Tributaria as observacao, Restricao_Judicial as restricao, Restricao_Adm as restricao_,
                        CASE
                            WHEN isdate(Dt_Ultima_Atualizacao) = 1 THEN cast(Dt_Ultima_Atualizacao as date)
                            ELSE NULL
                        END AS dataAtualizacao,
                        PF.Nome as nome,
                        '${fonte_dado}' as fonte

                FROM SISPESQUISA.sispesquisa.veiculos_19_07_2017
                    LEFT OUTER JOIN ReceitaNovo.dbo.PessoaFisica AS PF ON (veiculos_19_07_2017.CPF_CNPJ_Proprietario = PF.CPF)
                WHERE CPF_CNPJ_Proprietario=@cpf_parameter
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o veiculo de ${cpf}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelo veiculo de ${cpf}.`, 'getVeiculoDetalhadoCPF_Sispesquisa_VeiculosNovo'); });
};

export let getVeiculoDetalhadoCNPJ_Sispesquisa_VeiculosNovo = function (cnpj: string){

    return connection.getConn().request()
        .input('cnpj_parameter', mssql.VarChar, cnpj)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        PLACA as placa, CHASSI as chassi, TIPO_CHASSI as tipoChassi, RENAVAM as renavam, Ano_Fabricacao as anoFab,
                        ANO_MOD as anoMod, Descricao_Tipo as tipo, Descricao_Marca_Modelo as marcaModelo, Categoria as especie, Especie as procedencia,
                        COMBUSTIVEL as combustivel, CPF_CNPJ_Proprietario as cnpj, Ultimo_Ano_Licenciamento as ultLicenciamento, Restricao_Gravame as situacao,
                        Restricao_Tributaria as observacao, Restricao_Judicial as restricao, Restricao_Adm as restricao_,
                        CASE
                            WHEN isdate(Dt_Ultima_Atualizacao) = 1 THEN cast(Dt_Ultima_Atualizacao as date)
                            ELSE NULL
                        END AS dataAtualizacao,
                        '${fonte_dado}' as fonte

                FROM SISPESQUISA.sispesquisa.veiculos_19_07_2017
                    LEFT OUTER JOIN ReceitaNovo.dbo.PessoaJuridica AS PJ ON (veiculos_19_07_2017.CPF_CNPJ_Proprietario = PJ.CNPJ)
                WHERE CPF_CNPJ_Proprietario=@cnpj_parameter
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o veiculo de ${cnpj}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelo veiculo de ${cnpj}.`, 'getVeiculoDetalhadoCNPJ_Sispesquisa_VeiculosNovo'); });
};

export let getVeiculoDetalhadoChassi_Sispesquisa_VeiculosNovo = function (chassi: string){

    return connection.getConn().request()
        .input('chassi_parameter', mssql.VarChar, chassi)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        PLACA as placa, CHASSI as chassi, TIPO_CHASSI as tipoChassi, RENAVAM as renavam, Ano_Fabricacao as anoFab,
                        ANO_MOD as anoMod, Descricao_Tipo as tipo, Descricao_Marca_Modelo as marcaModelo, Categoria as especie, Especie as procedencia,
                        COMBUSTIVEL as combustivel, CPF_CNPJ_Proprietario as cpf, Ultimo_Ano_Licenciamento as ultLicenciamento, Restricao_Gravame as situacao,
                        Restricao_Tributaria as observacao, Restricao_Judicial as restricao, Restricao_Adm as restricao_,
                        CASE
                            WHEN isdate(Dt_Ultima_Atualizacao) = 1 THEN cast(Dt_Ultima_Atualizacao as date)
                            ELSE NULL
                        END AS dataAtualizacao,
                        '${fonte_dado}' as fonte

                FROM SISPESQUISA.sispesquisa.veiculos_19_07_2017
                WHERE CHASSI=@chassi_parameter
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o veiculo de ${chassi}.`, fonte);
            else{
                result.recordset = fixVeiculoCPF_CNPJ(result);
                return resultFound(result, fonte, rank, grupo);
            }
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelo veiculo de ${chassi}.`, 'getVeiculoDetalhadoChassi_Sispesquisa_VeiculosNovo'); });
};

export let getVeiculoDetalhadoRenavam_Sispesquisa_VeiculosNovo = function (renavam: string){

    return connection.getConn().request()
        .input('renavam_parameter', mssql.VarChar, renavam)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        PLACA as placa, CHASSI as chassi, TIPO_CHASSI as tipoChassi, RENAVAM as renavam, Ano_Fabricacao as anoFab,
                        ANO_MOD as anoMod, Descricao_Tipo as tipo, Descricao_Marca_Modelo as marcaModelo, Categoria as especie, Especie as procedencia,
                        COMBUSTIVEL as combustivel, CPF_CNPJ_Proprietario as cpf, Ultimo_Ano_Licenciamento as ultLicenciamento, Restricao_Gravame as situacao,
                        Restricao_Tributaria as observacao, Restricao_Judicial as restricao, Restricao_Adm as restricao_,
                        CASE
                            WHEN isdate(Dt_Ultima_Atualizacao) = 1 THEN cast(Dt_Ultima_Atualizacao as date)
                            ELSE NULL
                        END AS dataAtualizacao,
                        '${fonte_dado}' as fonte

                FROM SISPESQUISA.sispesquisa.veiculos_19_07_2017
                WHERE RENAVAM=@renavam_parameter
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o veiculo de ${renavam}.`, fonte);
            else{
                result.recordset = fixVeiculoCPF_CNPJ(result);
                return resultFound(result, fonte, rank, grupo);
            }
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelo veiculo de ${renavam}.`, 'getVeiculoDetalhadoRenavam_Sispesquisa_VeiculosNovo'); });
};

export let getVeiculoDetalhadoPlaca_Sispesquisa_VeiculosNovo = function (placa: string){

    return connection.getConn().request()
        .input('placa_parameter', mssql.VarChar, placa)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        PLACA as placa, CHASSI as chassi, TIPO_CHASSI as tipoChassi, RENAVAM as renavam, Ano_Fabricacao as anoFab,
                        ANO_MOD as anoMod, Descricao_Tipo as tipo, Descricao_Marca_Modelo as marcaModelo, Categoria as especie, Especie as procedencia,
                        COMBUSTIVEL as combustivel, CPF_CNPJ_Proprietario as cpf, Ultimo_Ano_Licenciamento as ultLicenciamento, Restricao_Gravame as situacao,
                        Restricao_Tributaria as observacao, Restricao_Judicial as restricao, Restricao_Adm as restricao_,
                        CASE
                            WHEN isdate(Dt_Ultima_Atualizacao) = 1 THEN cast(Dt_Ultima_Atualizacao as date)
                            ELSE NULL
                        END AS dataAtualizacao,
                        '${fonte_dado}' as fonte

                FROM SISPESQUISA.sispesquisa.veiculos_19_07_2017
                WHERE PLACA=@placa_parameter
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o veiculo de ${placa}.`, fonte);
            else{
                result.recordset = fixVeiculoCPF_CNPJ(result);
                return resultFound(result, fonte, rank, grupo);
            }
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelo veiculo de ${placa}.`, 'getVeiculoDetalhadoPlaca_Sispesquisa_VeiculosNovo'); });
};

export let getVeiculoDetalhadoPlaca_LIKE_Sispesquisa_VeiculosNovo = function (placa: string){

    return connection.getConn().request()
        .input('placa_parameter', mssql.VarChar, '%'+placa+'%')
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        PLACA as placa, CHASSI as chassi, TIPO_CHASSI as tipoChassi, RENAVAM as renavam, Ano_Fabricacao as anoFab,
                        ANO_MOD as anoMod, Descricao_Tipo as tipo, Descricao_Marca_Modelo as marcaModelo, Categoria as especie, Especie as procedencia,
                        COMBUSTIVEL as combustivel, CPF_CNPJ_Proprietario as cpf, Ultimo_Ano_Licenciamento as ultLicenciamento, Restricao_Gravame as situacao,
                        Restricao_Tributaria as observacao, Restricao_Judicial as restricao, Restricao_Adm as restricao_,
                        CASE
                            WHEN isdate(Dt_Ultima_Atualizacao) = 1 THEN cast(Dt_Ultima_Atualizacao as date)
                            ELSE NULL
                        END AS dataAtualizacao,
                        '${fonte_dado}' as fonte

                FROM SISPESQUISA.sispesquisa.veiculos_19_07_2017
                WHERE PLACA LIKE @placa_parameter
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o veiculo de ${placa}.`, fonte);
            else{
                result.recordset = fixVeiculoCPF_CNPJ(result);
                return resultFound(result, fonte, rank, grupo);
            }
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelo veiculo de ${placa}.`, 'getVeiculoDetalhadoPlaca_Sispesquisa_VeiculosNovo'); });
};

export let getVeiculoDetalhadoNome_Sispesquisa_VeiculosNovo = function (nome: string){

    return connection.getConn().request()
        .input('nome_parameter', mssql.VarChar, nome)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        PLACA as placa, CHASSI as chassi, TIPO_CHASSI as tipoChassi, RENAVAM as renavam, Ano_Fabricacao as anoFab,
                        ANO_MOD as anoMod, Descricao_Tipo as tipo, Descricao_Marca_Modelo as marcaModelo, Categoria as especie, Especie as procedencia,
                        COMBUSTIVEL as combustivel, CPF_CNPJ_Proprietario as cpf, Ultimo_Ano_Licenciamento as ultLicenciamento, Restricao_Gravame as situacao,
                        Restricao_Tributaria as observacao, Restricao_Judicial as restricao, Restricao_Adm as restricao_,
                        CASE
                            WHEN isdate(Dt_Ultima_Atualizacao) = 1 THEN cast(Dt_Ultima_Atualizacao as date)
                            ELSE NULL
                        END AS dataAtualizacao,
                        PF.Nome as nome,
                        '${fonte_dado}' as fonte

                FROM SISPESQUISA.sispesquisa.veiculos_19_07_2017 AS Veiculos
                    INNER JOIN ReceitaNovo.dbo.PessoaFisica AS PF ON (Veiculos.CPF_CNPJ_Proprietario = PF.CPF)
                WHERE CONTAINS(Nome, @nome_parameter)
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o veiculo de ${nome}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelo veiculo de ${nome}.`, 'getVeiculoCPF_CNPJ_Sispesquisa_VeiculosNovo'); });
};

