import * as mssql from 'mssql';

import { connection }  from '../services/db.service';
import { MODEL_PRIORITY, API_CONFIG } from '../config';
import {
    trataResultado,
    resultNotFound,
    resultFound,
    fixObitoCPF,
    logErroBuscaBD
} from '../utils';

const fonte  = MODEL_PRIORITY['vep'].fonte;
const rank   = MODEL_PRIORITY['vep'].rank;
const grupo  = MODEL_PRIORITY['vep'].grupo;

export let getObitoDetalhadoCPF_Sisobi_Obito = function (cpf: string){

    return connection.getConn().request()
        .input('cpf_parameter', mssql.VarChar(11), cpf)
        .query(`
            SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                PRO.NUMEROPROCESSO as processos
                ,UPPER(P.DS_PESSOA) as nome
                ,PF.DT_NASCIMENTO
                ,UPPER(PF.PAI) as nomePai
                ,UPPER(PF.MAE) as nomeMae
                ,CASE
                    WHEN PF.FK_SEXO = 21 THEN 'MASCULINO'
                    WHEN PF.FK_SEXO = 42 THEN 'FEMININO'
                    ELSE 'OUTRO'
                END AS sexo
                ,UPPER(PF.CONJUGE) as conjuge
                ,PF.IDENTIDADE_NUM as rg
                ,UPPER(PF.IDENTIDADE_ORGAO) as orgEmissor
                ,UPPER(UF_IDENT.DS_SIGLA) AS ufOrgEmissor
                ,PF.CPF as cpf
                ,PF.RESERVISTA_NUM as reservista
                ,PF.RESERVISTA_CIRC as reservistaCirc
                ,PF.RESERVISTA_SERIE as reservistaSerie
                ,PF.TITULO_NUM as tituloEleitor
                ,PF.TITULO_SECAO as tituloSecao
                ,PF.TITULO_ZONA as tituloZona
                ,PF.CART_PROF_NUM as carteiraProfNum
                ,PF.CART_PROF_SERIE as carteiraProfSerie
                ,UF_CART_TRAB.DS_UF as carteiraProfUf
                ,UPPER(C.DS_CIDADE) AS naturalidade
                ,PE.DS_PESSOAEMAIL AS email
                ,UPPER(ALC.DS_ALCUNHA) as vulgo
                ,FP.IMAGEM as foto
                ,FP.MIMETIPE as mime
                ,FP.DT_DOCUMENTO as dataFoto
                ,TEL.TELEFONE as telefone

            FROM VEP.dbo.VEP_CNJ_TB_PESSOAFISICA PF
                LEFT OUTER JOIN VEP.dbo.VEP_CNJ_TB_PESSOA P ON (P.ID_PESSOA = PF.FK_PESSOA)
                LEFT OUTER JOIN VEP.dbo.VEP_CNJ_TB_PESSOAEMAIL PE ON (PE.FK_PESSOA = P.ID_PESSOA)
                LEFT OUTER JOIN VEP.dbo.VEP_CNJ_TB_UF UF_IDENT ON (UF_IDENT.ID_UF = PF.FK_IDENTIDADE_UF)
                LEFT OUTER JOIN VEP.dbo.VEP_CNJ_TB_CIDADE C ON (C.ID_CIDADE = PF.FK_NATURALIDADE)
                LEFT OUTER JOIN VEP.dbo.VEP_CNJ_TB_UF UF_CART_TRAB ON (UF_CART_TRAB.ID_UF = PF.FK_CART_PROF_UF)
                LEFT OUTER JOIN VEP.dbo.VEP_CNJ_TB_FOTOPESSOA FP ON (P.ID_PESSOA = FP.FK_PESSOA)
                OUTER APPLY(
                    SELECT STUFF((
                        SELECT ', ' + PRO1.NUMEROPROCESSO
                        FROM VEP.dbo.VEP_CNJ_TB_PARTEPROCESSO PP1
                            INNER JOIN VEP.dbo.VEP_CNJ_TB_PROCESSO PRO1 ON (PRO1.NUMEROPROCESSO = PP1.FK_PROCESSO)
                            INNER JOIN VEP.dbo.VEP_CNJ_TB_PARTE PT1 ON (PP1.FK_PARTE = PT1.ID_PARTE)
                            INNER JOIN VEP.dbo.VEP_CNJ_TB_PESSOAFISICA PF1 ON (PF1.ID_PESSOAFISICA = PT1.FK_PESSOAFISICA)
                        WHERE (PF1.ID_PESSOAFISICA = PF.ID_PESSOAFISICA)
                        FOR XML PATH('')
                        ), 1, 2, '') AS NUMEROPROCESSO
                ) PRO
                OUTER APPLY(
                    SELECT STUFF((
                        SELECT ', ' + ALC2.DS_ALCUNHA
                        FROM VEP.dbo.VEP_CNJ_TB_ALCUNHA ALC2
                            INNER JOIN VEP.dbo.VEP_CNJ_TB_PARTE PT2 ON (PT2.ID_PARTE = ALC2.FK_PARTE)
                        WHERE PT2.FK_PESSOAFISICA = PF.ID_PESSOAFISICA
                        FOR XML PATH('')
                    ), 1, 2, '') AS DS_ALCUNHA
                ) ALC
                OUTER APPLY(
                    SELECT STUFF((
                        SELECT ', ' + CAST(PTEL3.DDDTELEFONE AS VARCHAR) + CAST(PTEL3.NUMEROTELEFONE AS VARCHAR)
                        FROM VEP.dbo.VEP_CNJ_TB_PESSOATELEFONE PTEL3
                        WHERE (P.ID_PESSOA = PTEL3.FK_PESSOA)
                        FOR XML PATH('')
                    ), 1, 2, '') AS TELEFONE
                ) AS TEL
            WHERE CPF = @cpf_parameter
            AND NOT EXISTS(  --RETIRAR OS PROCESSOS NO QUAL ELE É ADVOGADO
                SELECT 1
                FROM VEP.dbo.VEP_CNJ_TB_PARTEPROCESSOADV PPA
                INNER JOIN VEP.dbo.VEP_CNJ_TB_PARTEPROCESSO PRO ON (PPA.FK_PARTEPROCESSO = PRO.ID_PARTEPROCESSO)
                INNER JOIN VEP.dbo.VEP_CNJ_TB_PARTE P ON (P.ID_PARTE = PRO.FK_PARTE)
                WHERE (PF.ID_PESSOAFISICA = P.FK_PESSOAFISICA)
            )
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar a pessoa ${cpf}.`, fonte);
            else {
                result.recordset = fixObitoCPF(result);
                return resultFound(result, fonte, rank, grupo);
            }
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pela pessoa ${cpf}.`, 'getObitoDetalhadoCPF_Sisobi_Obito'); });
};

export let getObitoSimplificadoCPF_Sisobi_Obito = function (cpf: string){

    return connection.getConn().request()
        .input('cpf_parameter', mssql.Char(11), cpf)
        .query(`
        =@cpf_parameter
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o obito de ${cpf}.`, fonte);
            else{
                result.recordset = fixObitoCPF(result);
                return resultFound(result, fonte, rank, grupo);
            }
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelo obito de ${cpf}.`, 'getObitoSimplificadoCPF_Sisobi_Obito'); });
};

export let getObitoSimplificadoNome_Sisobi_Obito = function (nome: string){

    return connection.getConn().request()
        .input('nome_parameter', mssql.VarChar, nome)
        .query(`
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o obito de ${nome}.`, fonte);
            else {
                result.recordset = fixObitoCPF(result);
                return resultFound(result, fonte, rank, grupo);
            }
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelo obito de ${nome}.`, 'getObitoSimplificadoNome_Sisobi_Obito'); });
};