import * as mssql from 'mssql';

import { connection }  from '../../services/db.service';
import { MODEL_PRIORITY, API_CONFIG } from '../../config';
import {
    resultNotFound,
    resultFound,
    logErroBuscaBD
} from '../../utils';

import { SIGLA_FONTE } from './../_fontes';

const fonte_dado = SIGLA_FONTE['vep'];

const fonte = MODEL_PRIORITY['vep.email'].fonte;
const rank  = MODEL_PRIORITY['vep.email'].rank;
const grupo = MODEL_PRIORITY['vep.email'].grupo;

export let getEmailCPF_VEP = function (cpf: string){

    return connection.getConn().request()
        .input('cpf_parameter', mssql.VarChar, cpf)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                    PF.CPF as cpf
                    ,LOWER(PE.DS_PESSOAEMAIL) AS email
                    ,'email' as tipo
                    ,'${fonte_dado}' as fonte

                FROM VEP.dbo.VEP_CNJ_TB_PESSOAFISICA PF
                    LEFT OUTER JOIN VEP.dbo.VEP_CNJ_TB_PESSOA P ON (P.ID_PESSOA = PF.FK_PESSOA)
                    LEFT OUTER JOIN VEP.dbo.VEP_CNJ_TB_PESSOAEMAIL PE ON (PE.FK_PESSOA = P.ID_PESSOA)

                WHERE PE.DS_PESSOAEMAIL IS NOT NULL AND PF.CPF=@cpf_parameter
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o email de ${cpf}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelo email de ${cpf}.`, 'getEmailCPF_VEP'); });
};