import * as mssql from 'mssql';

import { connection }  from '../services/db.service';
import { getEnderecoCPF_ReceitaNovo_PessoaFisica } from './endereco.model';
import { BenchmarkService } from '../services/benchmark.service';

import { MODEL_PRIORITY, API_CONFIG } from '../config';
import {
    trataResultado,
    resultNotFound,
    resultFound,
    fixObitoCPF,
    logErroBuscaBD,
    filtraLogradouro,
    toTextSearch,
    printTempoExecucao
} from '../utils';

const fonte = MODEL_PRIORITY['receitanovo.vizinho'].fonte;
const rank  = MODEL_PRIORITY['receitanovo.vizinho'].rank;
const grupo = MODEL_PRIORITY['receitanovo.vizinho'].grupo;

export let getVizinhosReceitaNovoCPF = function (cpf: string){

    return getEnderecoCPF_ReceitaNovo_PessoaFisica(cpf)
        .then(res => {
            const endereco = res.resultado.dados[0];
            const listaLogradouro = toTextSearch(filtraLogradouro(endereco.logradouro));

            return getVizinhoPFReceitaNovoLogradouro(cpf, listaLogradouro, endereco.numero, endereco.municipio)
        })
};

let getVizinhoPFReceitaNovoLogradouro = function (cpf: string, logradouroFullText: string, numero: string, municipio: string) {

    const _numero = parseInt(numero, 10);
    const _offset: number = 10;

    const condicao_numero = (isNaN(_numero)) ?
        `numero = 0` :
        `numero BETWEEN ${_numero - _offset} AND ${_numero + _offset}`;

    const tempoInicial = new BenchmarkService();

    return connection.getConn().request()
        .input('CPF', mssql.Char(11), cpf)
        .input('LOGRADOURO', mssql.VarChar, logradouroFullText)
        .input('NUMERO_MIN', mssql.Int, _numero - _offset)
        .input('NUMERO_MAX', mssql.Int, _numero + _offset)
        .input('MUNICIPIO', mssql.VarChar, municipio)
        .query(`
            ;WITH PESSOAS_MESMA_RUA(cpf, nome, dataNascimento, logradouro, numero, tipoLogradouro, complemento)
            AS
            (
                SELECT
                    CPF as cpf,
                    NOME as nome,
                    DataNascimento as dataNascimento,
                    Logradouro as logradouro,
                    CASE
                        WHEN TRY_CAST(numeroLogradouro as int) IS NULL THEN 0
                        ELSE CAST(numeroLogradouro as int)
                    END,
                    TipoLogradouro as tipoLogradouro,
                    Complemento as complemento

                FROM ReceitaNovo.dbo.PessoaFisica
                WHERE CONTAINS(Logradouro, @LOGRADOURO)
                    AND Municipio = @MUNICIPIO
                    AND CPF <> @CPF
            )

            SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS} *
            FROM PESSOAS_MESMA_RUA
            WHERE ${condicao_numero}
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getVizinhoPFReceitaNovoLogradouro');

            if (result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar vizinhos de ${cpf}.`, fonte);
            else {
                return resultFound(result, fonte, rank, grupo);
            }
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelos vizinhos de ${cpf}.`, 'getVizinhoPFReceitaNovoLogradouro'); });
}