import { Router } from 'express';

import {
    GuardRotasChecaHash,
    GuardRotasOperacoes,
    ControleAcessoGuard,
    GuardRotasLogado
} from '../services/auth/authguard.service';

import {
    getEnderecoSimplificadoCNPJ,
    getEnderecoSimplificadoCPF,
    insertEnderecoSimplificadoCPF
} from '../controllers/enderecos.controller';

import { mapeamentoItensAcesso as mpi } from '../services/auth/controle.acesso';
import { mapeamentoSecoesAcesso as mps } from '../services/auth/controle.acesso';

import { cache } from '../services/apicache.service';

const enderecos: Router = Router();

/**
 * Protege a Rota para usuários que tenham um token válido 
 */
enderecos.use(GuardRotasLogado, GuardRotasChecaHash);

/**
 * Protege a Rota para usuários que tenham permissão para requisitar o item 'Endereço' da seção 'Pesquisa'
 */
enderecos.get('/simplificado/cpf/:cpf', ControleAcessoGuard({ secao: mps.pesquisa, item: mpi.pesquisa.endereco }), cache.connect(), getEnderecoSimplificadoCPF);
enderecos.get('/simplificado/cnpj/:cnpj', ControleAcessoGuard({ secao: mps.pesquisa, item: mpi.pesquisa.endereco }), cache.connect(), getEnderecoSimplificadoCNPJ);
//telefones.get('/simplificado/nome/:nome', getTelefoneSimplificadoNome);
//telefones.get('/simplificado/razaosocial/:razaosocial', getTelefoneSimplificadoRazaoSocial);

/**
 * Protege a Rota para usuários que tenham permissão para postar o item 'Cadastro Endereço' da seção 'Operações'
 */
enderecos.use(ControleAcessoGuard({ secao: mps.operacoes, item: mpi.operacoes.cadastroendereco }));
enderecos.post('/', insertEnderecoSimplificadoCPF);

export default enderecos;
