import { Router } from 'express';

import { apidoc_spec } from '../apidocs';

import {
    verificaLogin,
    verificaLoginExterno
} from '../controllers/usuarios.controller';

const index: Router = Router();

/**
 * @swagger
 * /:
 *   summary: Adds a new user
 *   get:
 *     summary: "Add a new pet to the store"
 *     tags:
 *       - "index"
 *     description: Retorna uma mensagem com o nome do sistema.
 *     produces:
 *       - text/plain
 *     responses:
 *       200:
 *         description: login
 */
index.get('/', function (req, res, next) {
    res.status(200).send('SIAP');
});

/**
 * @swagger
 * /api.json:
 *   get:
 *     summary: "Add a new pet to the store"
 *     tags:
 *       - "index"
 *     description: Retorna um objeto JSON com a documentação das rotas do sistema.
 *     produces:
 *       - application/json
 *     responses:
 *       200:
 *         description: api.json
 */

index.get('/api.json', function (req, res) {
    res.setHeader('Content-Type', 'application/json');
    res.status(200).send(apidoc_spec);
})

/**
 * @swagger
 * /login:
 *   post:
 *     summary: Gera o token para acesso ao sistema.
 *     tags:
 *       - "index"
 *     produces:
 *       - application/json
 *     parameters:
 *     - in: body
 *       name: body
 *       description: Objeto com informações de login e senha.
 *       required: true
 *       schema:
 *         $ref: "#/definitions/LoginForm"
 *     responses:
 *       200:
 *         description: Operação realizada com sucesso.
 *         schema:
 *           $ref: "#/definitions/Resposta"
 */
index.post('/login', verificaLogin);

/**
 * @swagger
 * /externo:
 *   post:
 *     tags:
 *       - "index"
 *     summary: Realiza o login via o token para acesso ao sistema.
 *     description: Rota usada para realizar um login/cadastro automático do usuário.
 *     produces:
 *       - application/json
 *     parameters:
 *     - in: body
 *       name: body
 *       description: Objeto com informações de login e senha.
 *       required: true
 *       schema:
 *         $ref: "#/definitions/LoginForm"
 *     responses:
 *       200:
 *         description: Operação realizada com sucesso.
 *         schema:
 *           $ref: "#/definitions/Resposta"
 */
index.post('/externo', verificaLoginExterno);

export default index;