import { Router } from 'express';

import { 
    GuardRotasPerfilPrivado,
    GuardRotasChecaHash,
    GuardRotasLogado,
    ControleAcessoGuard
} from '../services/auth/authguard.service';

import { 
    getMandadoSimplificadoCPF,
    getMandadoDetalhadoCPF,
    getMandadoSimplificadoRG,
    getMandadoSimplificadoNome
} from '../controllers/mandados.controller';

import { mapeamentoItensAcesso as mpi } from '../services/auth/controle.acesso';
import { mapeamentoSecoesAcesso as mps } from '../services/auth/controle.acesso';

import { cache } from '../services/apicache.service';

const mandados: Router = Router();

/**
 * Protege a Rota para usuários que tenham um token válido e que tenham permissão 
 * para requisitar o item 'Mandados' da seção 'Pesquisa'
 */
mandados.use(GuardRotasLogado, GuardRotasChecaHash);
mandados.use(ControleAcessoGuard({ secao: mps.pesquisa, item: mpi.pesquisa.mandado }));

/**
 * Realiza o cache das rotas abaixo
 */
mandados.use(cache.connect());

mandados.get('/detalhado/cpf/:cpf', getMandadoDetalhadoCPF);

mandados.get('/simplificado/cpf/:cpf', getMandadoSimplificadoCPF);
mandados.get('/simplificado/nome/:nome', getMandadoSimplificadoNome);
mandados.get('/simplificado/rg/:rg', getMandadoSimplificadoRG);

export default mandados;
