import { Router } from 'express';

import {
    GuardRotasChecaHash,
    GuardRotasLogado,
    ControleAcessoGuard
} from '../services/auth/authguard.service';

import { getSocioEmpresaSimplificadoCNPJ } from '../controllers/socios.controller';

import { mapeamentoItensAcesso as mpi } from '../services/auth/controle.acesso';
import { mapeamentoSecoesAcesso as mps } from '../services/auth/controle.acesso';

import { cache } from '../services/apicache.service';

const socios: Router = Router();

/**
 * Protege a Rota para usuários que tenham um token válido e que tenham permissão
 * para requisitar o item 'Sócio' da seção 'Pesquisa'
 */
socios.use(GuardRotasLogado, GuardRotasChecaHash);
socios.use(ControleAcessoGuard({ secao: mps.pesquisa, item: mpi.pesquisa.socio }));

/**
 * Realiza o cache das rotas abaixo
 */
socios.use(cache.connect());

socios.get('/simplificado/cnpj/:cnpj', getSocioEmpresaSimplificadoCNPJ);

export default socios;
