import * as express from 'express';
import * as morgan from 'morgan';
import * as bodyParser from 'body-parser';
import * as helmet from 'helmet';
import * as compression from 'compression';
const swaggerJSDoc = require('swagger-jsdoc');

/**
 * ROTAS
 */
import indexRoutes      from './routes/index.routes';
import benchmarkRoutes  from './routes/benchmark.routes';

/**
 * RECURSOS
 */
import preUsuariosRoutes    from './routes/preusuarios.routes';
import usuariosRoutes       from './routes/usuarios.routes';
import pessoasRoutes        from './routes/pessoas.routes';
import empresasRoutes       from './routes/empresas.routes';
import socioRoutes          from './routes/socios.routes';
import presosRoutes         from './routes/presos.routes';
import logsRoutes           from './routes/logs.routes';
import telefonesRoutes      from './routes/telefones.routes';
import veiculosRoutes       from './routes/veiculos.routes';
import mandadosRoutes       from './routes/mandados.routes';
import enderecosRoutes      from './routes/enderecos.routes';
import obitosRoutes         from './routes/obitos.routes';
import tipologiasRoutes     from './routes/tipologias.routes';
import folhapagamentoRoutes from './routes/folhapagamento.routes';
import empenhosRoutes       from './routes/empenhos.routes';
import licitacoesRoutes     from './routes/licitacoes.routes';
import geocoordenadasRoutes from './routes/geocoordenadas.routes';
import condenacoesRoutes    from './routes/condenacoes.routes';

/**
 *  APPS
 */
import integraRoutes         from './routes/apps/integra.routes';
import mapaconsumoRoutes     from './routes/apps/mapaconsumo.routes';
import cacafantasmasRoutes   from './routes/apps/cacafantasmas.routes';
import tiporankRoutes        from './routes/apps/tiporank.routes';
import dnaRoutes             from './routes/apps/dna.routes';
import nepotismoRoutes       from './routes/apps/nepotismo.routes';
import relacionamentosRoutes from './routes/apps/relacionamentos.routes';
import riscomppbRoutes       from './routes/apps/riscomppb.routes';

import {API_CONFIG} from './config';
import { traceMem } from './memreport';

// Cria uma instância do servidor Express
export const server: express.Express = express();

// Gera o log das requisições
if (server.get('env') !== 'test'){
    server.use(morgan(':req[x-real-ip] - :remote-addr - :remote-user [:date[clf]] ":method :url HTTP/:http-version" :status :res[content-length] - :response-time ms'));
}

server.use(helmet())

// A compressao dos dados ficou a cargo do nginx + brotli/gzip
// server.use(compression())
server.use(bodyParser.json());
server.use(bodyParser.urlencoded({extended: false}));

server.use(function (req, res, next) {
    res.setHeader('Access-Control-Allow-Origin', '*');
    res.setHeader('Access-Control-Allow-Headers', 'Origin, X-Requested-With, Content-Type, Accept, Authorization, hs');
    res.setHeader('Access-Control-Allow-Methods', 'POST, GET, PATCH, DELETE, OPTIONS');
    next();
});

// Rotas
server.use('/', indexRoutes);

server.use('/preusuarios', preUsuariosRoutes);
server.use('/usuarios', usuariosRoutes);

server.use('/benchmark', benchmarkRoutes);
/**
 * RECURSOS
 */
server.use('/pessoas', pessoasRoutes);
server.use('/empresas', empresasRoutes);
server.use('/logs', logsRoutes);
server.use('/presos', presosRoutes);
server.use('/socios', socioRoutes);
server.use('/telefones', telefonesRoutes);
server.use('/veiculos', veiculosRoutes);
server.use('/mandados', mandadosRoutes);
server.use('/obitos', obitosRoutes);
server.use('/enderecos', enderecosRoutes);
server.use('/tipologias', tipologiasRoutes);
server.use('/folhapagamento', folhapagamentoRoutes);
server.use('/empenhos', empenhosRoutes);
server.use('/geocoordenadas', geocoordenadasRoutes);
server.use('/condenacoes', condenacoesRoutes);
server.use('/licitacoes', licitacoesRoutes);

/**
 * APPS
 */
server.use('/dna', dnaRoutes);
server.use('/tiporank', tiporankRoutes);
server.use('/riscomppb', riscomppbRoutes);
server.use('/mapaconsumo', mapaconsumoRoutes);
server.use('/nepotismo', nepotismoRoutes);
server.use('/cacafantasmas', cacafantasmasRoutes);
server.use('/relacionamentos', relacionamentosRoutes);
server.use('/integra', integraRoutes);

API_CONFIG.CFG_ENV = server.get('env');

// Tratamento de erros
server.use((error: any, req, res, next) => {
    console.error(`Error Handler: ${error}\n`);

    res.status(500).json({
        message: error.message,
        error: {}
    });
    return null;
});

if (process.env.SIAP_TRACE_MEM === 'true') {
    traceMem();
}