import * as moment from 'moment';
import {ConnectionPool, ConnectionError} from 'mssql';

const user   = (process.env.SIAP_BD_USER)   ? process.env.SIAP_BD_USER   : 'siapdev';
const pw     = (process.env.SIAP_BD_PW)     ? process.env.SIAP_BD_PW     : 'siapdev.';
const server = (process.env.SIAP_BD_SERVER) ? process.env.SIAP_BD_SERVER : '10.128.24.10';

var dbConfig = {
    user:     user,
    password: pw,
    server:   server,
    database: 'sispesquisa',
    requestTimeout : 120000,
    //requestTimeout : 500,
    connectionTimeout: 15000,
    options: {
        encrypt: false
    },
    pool: {
        max: 10,
        min: 5,
        // acquireTimeoutMillis: 0,
        evictionRunIntervalMillis: 5000,
        idleTimeoutMillis: 10000
    }
};

export class DatabaseService {
    readonly connection: ConnectionPool;

    contadorErros: number;
    contadorTentativasReconexao: number;

    constructor(dbConfig) {
        this.contadorErros               = 0;
        this.contadorTentativasReconexao = 0;

        this.connection = new ConnectionPool(dbConfig);
        this.connection.on('error', err => {
            const hora = moment().format('YYYY-MM-DDTHH:mm:ss');

            console.error(`${hora} - Erro: SQL SERVER`);
            if (err.originalError) {
                console.error(`Codigo: ${(err.originalError.code) ? err.originalError.code : ''}`);
                console.error(`Mensagem: ${(err.originalError.message) ? err.originalError.message : '' }`, '\n');
            } else {
                console.error(err);
                console.error('\n');
            }

            this.contadorErros+=1;
        })
    }

    async connect(): Promise < ConnectionPool > {
        if(!this.connection.connected && !this.connection.connecting) {
            await this.connection.connect();
        }

        return this.connection;
    }

    getConn(): ConnectionPool {
        if (!this.connection.connected) {
            return null;
        } else {
            return this.connection;
        }
    }

    isConnected(): boolean {
        return this.connection.connected;
    }

    async disconnect(): Promise < boolean > {
        try {
            await this.connection.close();
            return true;
        } catch(e) {
            console.error('Error while disconnecting from database:', e);
            return false;
        }
    }
}

export let connection = new DatabaseService(dbConfig);