import * as _ from 'underscore';
import * as ss from 'string-similarity';
import * as rp from 'request-promise-native';
import * as jwt from 'jsonwebtoken';
import * as moment from 'moment';
var heapdump = require('heapdump');
var fs = require('fs');

import * as logModel from './models/log.model';
import { Pessoa } from './models/schemas';
import { Log } from './schemas/log.schema';

import {
    API_CODES,
    API_MSGS,
    API_CONFIG,
    LOG_CODES
} from './config';
import { BenchmarkService } from './services/benchmark.service';

// Remove as Stop Words do nome da pessoa e espaços em branco
export let trataRequisicaNome = function(nomeCompleto: string){
    // Filtra caracteres especiais do texto
    nomeCompleto = nomeCompleto.replace(/[^\w\s]/gi, '');

    const stopWords = ["DA", "DAS", "DE", "DES", "DO", "DOS", "E"];
    return nomeCompleto.split(" ").filter(nome => stopWords.indexOf(nome.toUpperCase()) === -1 && nome !== "");
};

export let trataErro = function(statusCode, msg){
    return { status: statusCode, msg: msg }
}

export let toTextSearch = function(listaNomes: Array<string>){

    let nomeFormatado = "";
    for(let i = 0 ; i < listaNomes.length-1 ; i++){
        nomeFormatado += listaNomes[i] + " AND ";
    }
    nomeFormatado += listaNomes[listaNomes.length-1];

    return nomeFormatado;
}

export let trataResultado = function(resultado: Object){

    return Object.keys(resultado).reduce((obj, key) => {
            obj[key] = typeof resultado[key] === 'string' ? resultado[key].trim() : resultado[key];
            return obj;
    }, {});
};

export let resultNotFound = function(message: string, fonte:string){
    return {
        status: API_CODES.CODE_RECURSO_NAO_ENCONTRADO,
        resultado: {
            fonte: fonte,
            dados: message
        }
    };
}

export let resultFound = function (result, fonte, rank=1, grupo='') {
    return {
        status: API_CODES.CODE_RECURSO_ENCONTRADO,
        resultado: {
            fonte: fonte,
            rank:  rank,
            grupo: grupo,
            dados: result.recordset.map(resultado => trataResultado(resultado))
        }
    };
}

export let resultFoundRaw = function (result, fonte, rank=1, grupo='') {
    return {
        status: API_CODES.CODE_RECURSO_ENCONTRADO,
        resultado: {
            fonte: fonte,
            rank:  rank,
            grupo: grupo,
            dados: result
        }
    };
}


export let logErroBuscaBD = function (error, messagem:string, nomeFuncao:string = null){
    console.error('\n');

    if (nomeFuncao){
        const hora = moment().format('YYYY-MM-DDTHH:mm:ss');
        console.error(`${hora} - Erro: ${nomeFuncao}`);
    }

    console.error(messagem);
    if (error.message) { console.error(error.message); }

    return {
        status: API_CODES.CODE_ERRO_500,
        resultado: {
            fonte: nomeFuncao,
            dados: messagem
        }
    }
}

export let logErroBuscaWS = function (error, messagem:string, nomeFuncao:string = null){
    console.error('\n');

    if (nomeFuncao){
        const hora = moment().format('YYYY-MM-DDTHH:mm:ss');
        console.error(`${hora} - Erro: ${nomeFuncao}`);
    }

    console.error(messagem);
    if (error.message) { console.error(error.message); }

    return {
        status: API_CODES.CODE_ERRO_500,
        resultado: {
            fonte: nomeFuncao,
            dados: messagem
        }
    }
}

export let resultIPCFound = function (arrayResultados, fonte, rank=1, grupo='') {
    return {
        status: API_CODES.CODE_RECURSO_ENCONTRADO,
        resultado: {
            fonte: fonte,
            rank: rank,
            grupo: grupo,
            dados: arrayResultados.map(resultado => {
                return {
                    codigo:         resultado.Codigo,
                    nome:           resultado.Nome,
                    rg:             resultado.RG,
                    dataExpedicao:  resultado.DataExpedicao,
                    nomeMae:        resultado.NomeMae,
                    nomePai:        resultado.NomePai,
                    naturalidade:   resultado.Naturalidade,
                    UFNaturalidade: resultado.UFNaturalidade,
                    dataNascimento: resultado.DataNascimento,
                    cpf:            resultado.CPF.replace(/[.-]/g, ""),
                    logradouro:     resultado.Logradouro,
                    numero:         resultado.Numero,
                    bairro:         resultado.Bairro,
                    cidade:         resultado.Cidade,
                    estado:         resultado.Estado,
                    cep:            resultado.CEP.replace(/[.-]/g, ""),
                    telefone:       resultado.TelefoneConvencional
                }
            })
        }
    };
}

export let filtraNaoEncontrados = function (arrayResultados: Array<any>){
     // Filtra apenas os resultados encontrados
    let res = _.filter(arrayResultados, r => r.status === API_CODES.CODE_RECURSO_ENCONTRADO);

    // Se não houver nenhum resultado
    if (res.length === 0){
        return [{status: API_CODES.CODE_RECURSO_NAO_ENCONTRADO, resultado: {dados: []} }];
    }

    return res;
}

let regraVeiculoDuplicado = (r) => r.renavam;
let regraPessoaDuplicada = (r) => r.cpf;
let regraEmpresaDuplicada = (r) => r.cnpj;
let getEndereco = (r) => r.logradouro + ' ' + r.numero + ' ' + r.cep;
let getTelefone = (r) => (r.ddd + ' ' + r.telefone).replace(/[^0-9]/g,'');
let filtraParecidos =  (lista, regraComparacao, paramSemelhanca=0.8) => {
    var listaUnicos = [];
    for (var i = 0; i < lista.length; i++){
        var atual = regraComparacao(lista[i]);

        // Se existe algo na lista
        if (listaUnicos.length){
            var achouIgual = false;
            for (var j = 0 ; j < listaUnicos.length && !achouIgual; j++){

                var comp = regraComparacao(listaUnicos[j]);
                var v = ss.compareTwoStrings(atual,  comp);

                if (v > paramSemelhanca)
                    achouIgual = true;
            }
            if (!achouIgual)
                listaUnicos.push(lista[i]);
        } else
            listaUnicos.push(lista[i]);
    }

    return listaUnicos;
}

export let removeDuplicados = function (array) {
    return _.uniq(array, regraPessoaDuplicada);
}
export let agrupaEFiltraDuplicados = function (arrayResultados, filtraArray = ['pessoa', 'empresa', 'telefone', 'endereco', 'veiculo']){
    if (arrayResultados[0].status === API_CODES.CODE_RECURSO_NAO_ENCONTRADO){
        return {status: API_CODES.CODE_RECURSO_NAO_ENCONTRADO, msg: API_MSGS.MSG_RECURSO_NAO_ENCONTRADO, dados: []};
    }

    // Agrupa os resultados por tipo de grupo
    let grupos = _.groupBy(arrayResultados, r => r.resultado.grupo);

    let r = {status: "OK", dados: []};

    r.dados = _.map(grupos, (value, grupo) => {
        // Quanto menor o valor de rank, maior a prioridade
        let res = _.sortBy(value, r => r.resultado.rank);

        // Agrupa os resultados ordenados
        res = _.reduce(res, (lista, resultadoAtual) => {
            // Adiciona o rank aos dados
            // resultadoAtual.resultado.dados = resultadoAtual.resultado.dados.map(r => {r.rank = resultadoAtual.resultado.rank; return r;});
            return lista ? lista.concat(resultadoAtual.resultado.dados) : resultadoAtual.resultado.dados;
        }, []);

        if (grupo === "pessoa" && filtraArray.indexOf('pessoa') !== -1)
            res = _.uniq(res, regraPessoaDuplicada);
        else if (grupo === "empresa" && filtraArray.indexOf('empresa') !== -1)
            res = _.uniq(res, regraEmpresaDuplicada);
        else if (grupo === "endereco" && filtraArray.indexOf('endereco') !== -1)
            res = filtraParecidos(res, getEndereco, 0.95);
        else if (grupo === "telefone" && filtraArray.indexOf('telefone') !== -1)
            res = filtraParecidos(res, getTelefone, 0.95);
        else if (grupo === "veiculo" && filtraArray.indexOf('veiculo') !== -1)
            res = _.uniq(res, regraVeiculoDuplicado);

        // gambiarra pra {value: res}
        let t = {};
        t[grupo] = res;
        return t;
    });

    return r;
}

export let agrupaOrdena = function (arrayResultados){
    // Agrupa os resultados por tipo de grupo
    let grupos = _.groupBy(arrayResultados, r => r.resultado.grupo);

    let r = {status: API_CODES.CODE_RECURSO_ENCONTRADO, dados: []};

    r.dados = _.map(grupos, (value, key) => {
        // Quanto menor o valor de rank, maior a prioridade
        let res = _.sortBy(value, r => r.resultado.rank);

        // Agrupa os resultados ordenados
        res = _.reduce(res, (lista, resultadoAtual) => {
            return lista ? lista.concat(resultadoAtual.resultado.dados) : resultadoAtual.resultado.dados;
        }, []);

        // gambiarra pra {value: res}
        let t = {};
        t[key] = res;
        return t;
    });

    return r;
}

// {
//         "status": "OK",
//         "dados": [
//             {
//                 "nome_grupo": [
//                     {
//                         "key_0": "value_0",
//                         "key_1": "value_1"
//                     }
//                 ]
//             }
//         ]
// }

// { status: string, dados: Array<any>}
export let desagrupa = function (obj ) {
    if (obj.dados.length){
        obj.dados = obj.dados.reduce((acc, grupoAtual) => {
            let nomegrupo = _.keys(grupoAtual)[0];
            return (acc) ? acc.concat(grupoAtual[nomegrupo]) : grupoAtual[nomegrupo];
        }, []);
    }

    return obj;
}

export let limitaNumeroResultados = function (obj ) {
    if (obj && obj.dados.length){
        obj.dados = obj.dados.slice(0, API_CONFIG.SIAP_MAX_RESULTS);
    }

    return obj;
}

export let print =  function (data, header=null){
    if (header){
        console.log(header);
    }
    console.log(JSON.stringify(data, null, 4));

    return data;
}

// Transforma um array de arrays em array
export let flat = function (array){
    return [].concat.apply([], array);
}

const mapa={"â":"a","Â":"A","à":"a","À":"A","á":"a","Á":"A","ã":"a","Ã":"A","ê":"e","Ê":"E","è":"e","È":"E","é":"e","É":"E","î":"i","Î":"I","ì":"i","Ì":"I","í":"i","Í":"I","õ":"o","Õ":"O","ô":"o","Ô":"O","ò":"o","Ò":"O","ó":"o","Ó":"O","ü":"u","Ü":"U","û":"u","Û":"U","ú":"u","Ú":"U","ù":"u","Ù":"U"};
export let removerAcentos = function (s){
    if (!s) {
        return '';
    }

    return s.replace(/[\W\[\] ]/g,function(a){return mapa[a]||a})
};


export let fixVeiculoCPF_CNPJ = function (result){
    return result.recordset.map(registro => {
        registro.cpf = registro.cpf.trim();

        if (registro.cpf.length === 11){
            return registro;
        } else {
            registro.cnpj = registro.cpf;
            delete registro.cpf;

            return registro;
        }
    });
}

export let fixObitoCPF = function (result){
    return result.recordset.map(registro => {
        let temp = String(registro.obito_cpf);
        let complemento = '0'.repeat(11-temp.length);

        registro.obito_cpf = complemento + temp;
        return registro;
    });
}

let validaForcaSenha = function (senha) {
    return /^(?=.*?[A-Z])(?=.*?[a-z])(?=.*?[0-9])(?=.*?[#?!@$%^&*-]).{8,}$/.test(senha);
}

export let validaTrocaSenha = function (dados) {
    if (dados.senhanova !== dados.senhanova2) {
        return trataErro(API_CODES.CODE_PASSWORD_INVALIDO, 'Os valores da senha nova precisam ser iguais.');
    } else if (dados.senhaantiga === dados.senhanova) {
        return trataErro(API_CODES.CODE_PASSWORD_INVALIDO, 'A senha nova não pode ser igual a senha atual.');
    } else if ( !validaForcaSenha(dados.senhanova) ) {
        return trataErro(API_CODES.CODE_PASSWORD_INVALIDO, 'A senha nova deve conter ao menos 8 caracteres, os quais devem ser pelo menos: um caractere maiúsculo, um minúsculo, um número e um caractere especial.');
    } else {
        return trataErro(API_CODES.CODE_SUCESSO, 'Senha válida');
    }
}

export let validaRecaptcha = function (response) {
    return rp({
                method: 'POST',
                uri: 'https://www.google.com/recaptcha/api/siteverify',
                form: {secret: API_CONFIG.SIAP_GOOGLE_RECAPTCHA_SECRET_KEY, response: response},
                timeout: 2000,
                json: true
            })
            .then(resultado => {
                return (resultado.success) ? true : false;
            })
            .catch( error => { return logErroBuscaBD(error, `Erro na validação do recaptcha.`, 'validaRecaptcha')} );
}

export let getId_Token = function(authHeader) {
    const token = authHeader.split(" ")[1];
    const decoded = <any>jwt.verify(token, API_CONFIG.SIAP_SENHA_TOKEN, { algorithms: ['HS256'] });
    return decoded.user.id;
}

let traduzAtributoBNMP = {
    'Alcunha': 'vulgo',
    'Data de nascimento': 'dataNascimento',
    'Sexo': 'sexo',
    'Nacionalidade': 'nacionalidade',
    'Nome do Genitor': 'nomePai',
    'Nome da Genitora': 'nomeMae',
    'Naturalidade': 'naturalidade',
    'Carteira de identidade': 'rg',
    'Cadastro de pessoa física': 'cpf',
    'Profissão': 'profissao',
}
export let trataResultadoBNMP = function(data) {
    return data.map(mandado => {
        const {dataMandado, nomeParte, numeroMandado, orgao, situacao, id, detalhes} = mandado;
        const formatado = detalhes.reduce((agrupado, dado) => {
            const ar = dado.split(':');
            const atributo = ar[0];
            let valor = ar[1];

            if (traduzAtributoBNMP[atributo] === 'dataNascimento') { valor = valor.split('/').reverse().join('-');}

            agrupado[traduzAtributoBNMP[atributo]] = (typeof valor === 'string')? valor.trim() : valor ;

            return agrupado;
        }, {});

        return Object.assign({
            id: id,
            dataMandado: dataMandado,
            nome: nomeParte,
            numeroMandado: '0'.repeat((30 - numeroMandado.length)) + numeroMandado,
            orgao: orgao,
            situacao: situacao
        }, formatado);
    });
}


export let modelAddWhereTipoComparacao = function(nomeAtributo, valorAtributo, tipoComparacao){
    if (tipoComparacao === 'fulltext'){
        return `WHERE CONTAINS(${nomeAtributo}, ${valorAtributo})`;
    } else if (tipoComparacao === 'igual' || tipoComparacao === 'exata' || tipoComparacao === 'equal') {
        return `WHERE ${nomeAtributo}=${valorAtributo}`;
    } else {
        console.error(`ERROR: modelAddWhereTipoComparacao - Parâmetro 'tipoComparacao' inválido.`)
        return '';
    }
}

export let modelAddWhereDataNascimento = function (nomeAtributo, valorAtributo, dataNascimento, comparacao = '<') {
    if (dataNascimento) {
        return ` AND ${nomeAtributo} ${comparacao} ${valorAtributo}`;
        // return ` AND
        //                 (
        //                     isdate(${nomeAtributo}) = 0
        //                     OR
        //                     CAST(CAST(${nomeAtributo} as varchar) as date) ${comparacao} ${valorAtributo}
        //                 )`;
    } else {
        return '';
    }
}

export let modelAddWherePrimeiroNome = function (nomeAtributo, valorAtributo, travaPrimeiroNome) {
    if (travaPrimeiroNome) {
        return ` AND ${nomeAtributo} LIKE ${valorAtributo}`;
    } else {
        return '';
    }
}

export let registraNaoEncontrados = function (registros, recurso, valor) {
    if (registros[0].status === API_CODES.CODE_RECURSO_NAO_ENCONTRADO && API_CONFIG.CFG_ENV == 'production') {
        logModel.salvaLog(new Log({ ip: 'internal', usuario: 'sysadmin', tipo: LOG_CODES.TIPO_REGISTRO_NAO_ENCONTRADO, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${recurso}: ${valor}`, user_agent: 'internal', processo: null }))
    }
    return registros;
}



export let geraListaNomes = function (nomeCompleto): Array<string> {
    var nomes = trataRequisicaNome(nomeCompleto).join(' ').split(" ");
    var lista = [];

    lista.push(nomeCompleto);

    var primeiroNome = nomes[0];
    nomes.slice(1).forEach(sobrenome => {
        lista.push(primeiroNome + ' ' + sobrenome);
    });

    // if (nomes[1] && nomes.length > 2) {
    //     var segundoNome = nomes[1];
    //     nomes.slice(2).forEach(sobrenome => {
    //         lista.push(segundoNome + ' ' + sobrenome);
    //     });
    // }

    return lista;
};

export let trataResultadoFacebook = function (registros, nomeReal) {

    // O limiar de semelhança entre os nomes encontrados, menor que 0.5 será
    // desconsiderado.
    const thresholdSemelhanca = 0.7;
    const listaNomes          = geraListaNomes(nomeReal);

    return registros.map(d => {

        let maxSemelhanca = 0;
        const resultado = listaNomes.map(nome => {
            const obj = ss.compareTwoStrings(d.titulo, nome);
            if (obj > maxSemelhanca) {
                maxSemelhanca = obj;
            }

            // Checa se existe uma das combinacoes de nomes geradas dentro da descricao
            // do registro.
            return d.descricao.toLowerCase().indexOf(nome.toLowerCase()) !== -1;
        }).filter(d => d !== false);

        d.obj = maxSemelhanca;

        if (maxSemelhanca > thresholdSemelhanca || resultado.length > 0) {
            return d;
        } else {
            return null;
        }
    }).filter(d => d !== null);
}

export let guardValidacaoApiCache = (req, res) => res.statusCode === 200 && req.method === 'GET';

export let trataResultadosCruzamentoSisobi = function (dados) {
    return dados.map(registro => {
        const tmp_nome         = removerAcentos(registro.nome.trim());
        const tmp_nomeFalecido = removerAcentos(registro.nomeFalecido.trim());

        const semelhanca = ss.compareTwoStrings(tmp_nome, tmp_nomeFalecido);

        if (semelhanca > 0.9) {
            const {nomeFalecido, ...dado } = registro;
            return dado;
        } else {
            return null;
        }
    }).filter(d => d !== null);
}

// Recebe um array de dados e insere um atributo tipologia de nome *atributo*
export let insereAtributoTipologia = function (dados, nomeTipologia) {
    return dados.map(d => {
        d.tipologia = nomeTipologia;
        return d;
    })
}

// Recebe um array de resultados de um PromiseAll, se houver alguma promise
// que tenha falhado, ele retorna um erro
export let retornaErroSeEncontraErro = function (dados) {
    let erro = dados.filter(d => d.status === 'ESERVER').length;
    if (erro) {
        throw "ESERVER";
    }

    return dados;
}


export let hashFotosSispesquisa = function (entrada) {
    const chave = '12345678910';

    let saida = '';
    for (var i = 0; i < entrada.length; i++) {
        saida += (parseInt(entrada[i], 16) ^ parseInt(chave[i % chave.length], 16)).toString(16);
    }

    return saida;
}


/**
 * Força o node a rodar o Garbage Collector e tira um snapshot da Heap.
*/
export let generateHeapDumpAndStats = function () {
    //1. Force garbage collection every time this function is called
    try {
        global.gc();
    } catch (e) {
        console.log("E necessario utilizar o argumento '--expose-gc' para gerar os dumps da heap.");
        return;
        // process.exit();
    }

    //2. Output Heap stats
    var heapUsed = process.memoryUsage().heapUsed;
    console.log("A aplicacao esta utilizando " + heapUsed + " bytes da heap.")

    var dir = './heapdump';

    if (!fs.existsSync(dir)){
        fs.mkdirSync(dir);
    }

    //3. Get Heap dump
    heapdump.writeSnapshot('./heapdump/' + Date.now() + '.heapsnapshot');
}

export let printTempoExecucao = function (tempoInicial: BenchmarkService, nomeFuncao){
    if (API_CONFIG.CFG_ENV !== 'production') {
        console.log(`LOG: ${nomeFuncao} - terminou em ${tempoInicial.elapsed()} ms`);
    }
}

export let checaCPF = function (cpf: string) {
    if (!cpf) { return null; }

    cpf = cpf.replace(/[^0-9]/g, '');
    if (cpf.length !== 11) { return null; }
    if (!validarCPF(cpf)) { return null; }

    return cpf;
}

let validarCPF = function (cpf: string) {
    if (!cpf) { return null; }

    cpf = cpf.replace(/[^\d]+/g, '');
    // Elimina CPFs invalidos conhecidos
    if (cpf.length !== 11 ||
        cpf === '00000000000' ||
        cpf === '11111111111' ||
        cpf === '22222222222' ||
        cpf === '33333333333' ||
        cpf === '44444444444' ||
        cpf === '55555555555' ||
        cpf === '66666666666' ||
        cpf === '77777777777' ||
        cpf === '88888888888' ||
        cpf === '99999999999') { return null; }

    // Valida 1o digito
    let add = 0;
    for (let i = 0; i < 9; i++) {
        add += parseInt(cpf.charAt(i), 10) * (10 - i);
    }
    let rev = 11 - (add % 11);
    if (rev === 10 || rev === 11) { rev = 0; }
    if (rev !== parseInt(cpf.charAt(9), 10)) { return null; }

    // Valida 2o digito
    add = 0;
    for (let i = 0; i < 10; i++) {
        add += parseInt(cpf.charAt(i), 10) * (11 - i);
    }
    rev = 11 - (add % 11);
    if (rev === 10 || rev === 11) { rev = 0; }
    if (rev !== parseInt(cpf.charAt(10), 10)) { return null; }

    return true;
}

export let checaCNPJ = function (cnpj: string) {
    if (!cnpj) { return null; }

    cnpj = cnpj.replace(/[^0-9]/g, '');

    if (cnpj.length !== 14) { return null; }
    if (!validarCNPJ(cnpj)) { return null; }

    return cnpj;
}

let validarCNPJ = function (cnpj) {
    if (!cnpj) { return null; }
    cnpj = cnpj.replace(/[^\d]+/g, '');

    if (cnpj.length !== 14) { return null; }

    // Elimina CNPJs invalidos conhecidos
    if (cnpj === '00000000000000' ||
        cnpj === '11111111111111' ||
        cnpj === '22222222222222' ||
        cnpj === '33333333333333' ||
        cnpj === '44444444444444' ||
        cnpj === '55555555555555' ||
        cnpj === '66666666666666' ||
        cnpj === '77777777777777' ||
        cnpj === '88888888888888' ||
        cnpj === '99999999999999') { return null; }

    // Valida DVs
    let tamanho = cnpj.length - 2;
    let numeros = cnpj.substring(0, tamanho);
    let digitos = cnpj.substring(tamanho);
    let soma = 0;
    let pos = tamanho - 7;
    for (let i = tamanho; i >= 1; i--) {
        soma += numeros.charAt(tamanho - i) * pos--;
        if (pos < 2) { pos = 9; }
    }

    let resultado = soma % 11 < 2 ? 0 : 11 - soma % 11;
    if (resultado !== parseInt(digitos.charAt(0), 10)) { return null; }

    tamanho = tamanho + 1;
    numeros = cnpj.substring(0, tamanho);
    soma = 0;
    pos = tamanho - 7;
    for (let i = tamanho; i >= 1; i--) {
        soma += numeros.charAt(tamanho - i) * pos--;
        if (pos < 2) { pos = 9; }
    }

    resultado = soma % 11 < 2 ? 0 : 11 - soma % 11;
    if (resultado !== parseInt(digitos.charAt(1), 10)) { return null; }

    return true;
}

/**
 * Recebe o logradouro, filtra palavras como 'av', 'rua', etc e converte
 * para uma lista de palavras.
 * @param logradouro
 */
export let filtraLogradouro = function (logradouro: string) {
    if (!logradouro) { return []; }

    // Filtra caracteres especiais do texto
    logradouro = logradouro.replace(/[^\w\s]/gi, '');

    const stopWords = ["AV", "AV.", "AVENIDA", "R", "R.", "RUA", "DA", "DAS", "DE", "DES", "DO", "DOS", "E"];
    const lista = logradouro.toUpperCase().split(' ');

    return lista.filter(l => stopWords.indexOf(l) === -1 && l !== "");
}

export let agrupaControleAcessos = function (lista) {
    let grupos = _.groupBy(lista, l => l.ID_SECAO);
    const attr = Object.keys(grupos)[0];

    const r = {};
    r[attr] = grupos[attr].map(r => r.ID_ITEM);

    return r;
}