import {server as serverExpress} from './serverSetup';
import * as http from 'http';
import * as moment from 'moment';

import {connection as pool} from './services/db.service';
import {cache} from './services/apicache.service';

declare const module: any;

let tentativaReconexaoBD = 0;

/**
 * Cria o servidor HTTP responsável por servir as requisições ao REST.
 */
export let inicializaApp = function(){
    const port = normalizePort(process.env.SIAP_PORT || '3000');
    serverExpress.set('port', port);

    const server = http.createServer(serverExpress);

    server.listen(port);
    server.on('error', onError);
    server.on('listening', onListening);

    if (module.hot) {
        module.hot.accept();
        module.hot.dispose(() => server.close());
    }


    function normalizePort(val: any): number | string | boolean {
        let port = parseInt(val, 10);

        if (isNaN(port)) { return val; }
        if (port >= 0) { return port; }

        return false;
    }

    function onError(error) {
        if (error.syscall !== 'listen') {
            throw error;
        }

        let bind = typeof port === 'string'
            ? 'Pipe ' + port
            : 'Port ' + port

        switch (error.code) {
            case 'EACCES':
                console.error(bind + ' necessita de permissao, falta de privilegios.');
                process.exit(1);
                break;
            case 'EADDRINUSE':
                console.error(bind + ' ja esta em uso.');
                process.exit(1);
                break;
            default:
                throw error;
        }
    }

    function onListening() {
        let addr = server.address();
        let bind = typeof addr === 'string'
            ? 'pipe ' + addr
            : 'porta ' + addr.port;

        console.log('Servindo na ' + bind);
        console.log('Ambiente - ', serverExpress.get('env'));
    }

    return server;
}

/**
 * Função responsável por inicializar uma instância de SIAP.
 * O sistema só é inicializado quando a conexão ao banco de dados for bem sucedida.
 */
export let inicializaSistema = function () {
    pool.connect()
        .then(conn => {
            tentativaReconexaoBD = 0;

            console.log('BD: Conexao com BD efetuada com sucesso.\n');
        })
        .then(() => {
            inicializaApp();
        })
        .catch(err => {
            // Se a conexão com o banco não for bem sucedida, o sistema apresenta
            // uma mensagem de erro e aguarda 5s para tentar novamente.

            const hora = moment().format('YYYY-MM-DDTHH:mm:ss');

            console.error(`${hora} - Erro: Conexao com o banco falhou`);
            if (err.originalError) {
                console.error(`Codigo: ${(err.originalError.code) ? err.originalError.code : ''}`);
                console.error(`Mensagem: ${(err.originalError.message) ? err.originalError.message : '' }`, '\n');
            } else {
                console.error(err);
                console.error('\n');
            }

            tentativaReconexaoBD += 1;
            setTimeout(() => { inicializaSistema(); }, Math.min(tentativaReconexaoBD * 5000, 180000));
        })
}

/**
 *
 * SCRIPT PARA INICIALIZACAO DO SIAP
 *
 */

inicializaSistema();