import * as http from 'http';
import * as moment from 'moment';

import {server as serverExpress} from './serverSetup';

import { connection as pool } from './services/db.service';

let tentativaReconexaoBD = 0;

/**
 *
 * GERA O CLUSTER COM NUMCPUS/2 INSTANCIAS
 *
 */
var cluster = require('cluster');
var numCPUs = require('os').cpus().length;
    numCPUs = (numCPUs > 1) ? numCPUs/2 : numCPUs;

function normalizePort(val: any): number|string|boolean {
  let port = parseInt(val, 10);

  if (isNaN(port)) { return val; }
  if (port >= 0) { return port; }

  return false;
}

/**
 * Cria o servidor HTTP responsável por servir as requisições ao REST.
 */
let inicializaApp = function () {
    const port = normalizePort(process.env.SIAP_PORT || '3000');
    serverExpress.set('port', port);

    const server = http.createServer(serverExpress);

    server.listen(port);
    server.on('error', function onError(error: any) {
        if (error.syscall !== 'listen') {
            throw error;
        }

        let bind = typeof port === 'string'
            ? 'Pipe ' + port
            : 'Port ' + port

        switch (error.code) {
            case 'EACCES':
                console.error(bind + ' necessita de permissao, falta de privilegios.');
                process.exit(1);
                break;
            case 'EADDRINUSE':
                console.error(bind + ' ja esta em uso.');
                process.exit(1);
                break;
            default:
                throw error;
        }
    });

    server.on('listening', function onListening() {
        let id = Number(process.env.id);
        let addr = server.address();
        let bind = (typeof addr === 'string')
            ? 'pipe ' + addr
            : 'porta ' + addr.port;

        console.log(`Worker ${id} servindo na ${bind}` + ' Ambiente - ', serverExpress.get('env'));
    });
}

/**
 * Função responsável por inicializar uma instância de SIAP.
 * O sistema só é inicializado quando a conexão ao banco de dados for bem sucedida.
 */
let inicializaSistema = function () {
    pool.connect()
        .then(conn => {
            console.log('BD: Conexao com BD efetuada com sucesso.\n');
        })
        .then(() => {
            inicializaApp();
        })
        .catch(err => {
            // Se a conexão com o banco não for bem sucedida, o sistema apresenta
            // uma mensagem de erro e aguarda 5s para tentar novamente.

            const hora = moment().format('YYYY-MM-DDTHH:mm:ss');

            console.error(`${hora} - Erro: Conexao com o banco falhou`);
            if (err.originalError) {
                console.error(`Codigo: ${(err.originalError.code) ? err.originalError.code : ''}`);
                console.error(`Mensagem: ${(err.originalError.message) ? err.originalError.message : '' }`, '\n');
            } else {
                console.error(err);
                console.error('\n');
            }

            tentativaReconexaoBD += 1;

            setTimeout(() => { inicializaSistema(); }, Math.min(tentativaReconexaoBD * 5000, 180000));
        })
}


/**
 *
 * SCRIPT PARA INICIALIZACAO DO SIAP
 *
 */
if (cluster.isMaster) {
    // Se for o Master cria as instâncias do siap-server
    for (var i = 0; i < numCPUs; i++) {
        cluster.fork({ id: i });
    }
} else {
    // Se for um Slave inicializa a instância criada do siap-server
    inicializaSistema();
}
