import * as tipologiaModel from '../../models/tipologia.model';
import * as empresaModel from '../../models/empresa.model';
import * as socioModel from '../../models/socio.model';
import * as enderecoModel from '../../models/endereco.model';
import * as dnaModel from '../../models/dna.model';
import * as utilsModel from '../../models/utils/utils.model';

import {
    filtraNaoEncontrados,
    flat,
    print,
} from '../../utils';

export let getInformacoesEmpresa = function (cnpj: string) {
    cnpj = cnpj.replace(/[^0-9]/g, '');

    return Promise.all([
        empresaModel.getEmpresaDetalhadoCNPJ_ReceitaNovo_PessoaJuridica(cnpj),

        socioModel.getSocioPFEmpresaSimplificadoCNPJ_ReceitaNovo(cnpj),
        socioModel.getSocioPJEmpresaSimplificadoCNPJ_ReceitaNovo(cnpj),

        enderecoModel.getEnderecoCNPJ_ReceitaNovo_PessoaJuridica(cnpj).then(end => utilsModel.getGeoCoordenadas(end)),
        enderecoModel.getEnderecoCNPJ_ReceitaNovo_PessoaJuridica(cnpj),

        dnaModel.getInformacoesSociosCNPJ(cnpj),
        dnaModel.getDoacaoEleitoralCNPJ(cnpj),
        dnaModel.getValoresMunicipioCNPJ_Sagres_Municipal(cnpj),

        dnaModel.getTotalEmpenhadoPagoCNPJ_Sagres_Municipal(cnpj),
        dnaModel.getTotalEmpenhadoPagoCNPJ_Sagres_Estadual(cnpj),
        
        dnaModel.getModalidadesCNPJ_Sagres_Municipal(cnpj),
        dnaModel.getModalidadesCNPJ_Sagres_Estadual(cnpj),

        dnaModel.getTopEmpenhosPagosCNPJ_Sagres_Municipal(cnpj),
        dnaModel.getTopEmpenhosPagosCNPJ_Sagres_Estadual(cnpj),

        dnaModel.getValoresModalidadesCNPJ_Sagres_Municipal(cnpj),
        dnaModel.getValoresModalidadesCNPJ_Sagres_Estadual(cnpj),

        dnaModel.getContadoresCNPJ(cnpj),
        dnaModel.getEmpresasContadoresReceberamEstadoCNPJ(cnpj),
        dnaModel.getEmpresasContadoresParticiparamCNPJ(cnpj),
        
        dnaModel.getEmpresasMesmoTelefoneCNPJ(cnpj),
        dnaModel.getEmpresasMesmoEmailCNPJ(cnpj),

        // dnaModel.getDividaAtivaCNPJ(cnpj),
        // dnaModel.getValorTotalDividaAtivaCNPJ(cnpj),
        // dnaModel.geQtdAutoInfracaoDividaAtivaCNPJ(cnpj),
        // dnaModel.getQtdProcessoJudicialDividaAtivaCNPJ(cnpj),

        dnaModel.getAtividadeEconomicaCNPJ(cnpj),

        tipologiaModel.getTipologiaSimplificadoCNPJ_Tipologias(cnpj),
    ])
        .then(r => flat(r))
        .then(tipologias => filtraNaoEncontrados(tipologias))
}