import * as mssql from 'mssql';

import { Pessoa } from '../schemas';
import { BenchmarkService } from './../../services/benchmark.service';
import { connection } from '../../services/db.service';
import { MODEL_PRIORITY } from './../../config';

import {
    trataResultado,
    resultNotFound,
    resultFound,
    logErroBuscaBD,
    printTempoExecucao
} from '../../utils';

const fonte   = MODEL_PRIORITY['sagres_estadual.empenhos'].fonte;
const rank    = MODEL_PRIORITY['sagres_estadual.empenhos'].rank;
const grupo   = MODEL_PRIORITY['sagres_estadual.empenhos'].grupo;

export let getValorTotalDividaAtivaCNPJ = function (cnpj: string){

    const tempoInicial = new BenchmarkService();
    return connection.getConn().request()
        .input('cnpj', mssql.Char(14), cnpj)
        .query(`
            SELECT
                SUM(VALOR_DIVIDA_ATIVA) AS valorTotal
            FROM DNA.dbo.DIVIDA_ATIVA D
            WHERE CPF_CNPJ_DIVIDA=@CNPJ
                AND FASE_DIVIDA_ATIVA <> 'ACAO EXTINTA'
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getValorTotalDividaAtivaCNPJ');

            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar dividas ativas de ${cnpj}.`, fonte);
            else
                return resultFound(result, fonte, rank, 'dividaativa_valortotal');
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelas dividas ativas de ${cnpj}.`, 'getValorTotalDividaAtivaCNPJ'); });
};

export let geQtdAutoInfracaoDividaAtivaCNPJ = function (cnpj: string){

    const tempoInicial = new BenchmarkService();
    return connection.getConn().request()
        .input('cnpj', mssql.Char(14), cnpj)
        .query(`
            SELECT
                COUNT(*) AS  qtdAutoInfracao
            FROM DNA.dbo.DIVIDA_ATIVA D
            WHERE CPF_CNPJ_DIVIDA=@CNPJ
                AND FASE_DIVIDA_ATIVA <> 'ACAO EXTINTA'
                AND NUM_AUTO_INFRACAO <> ''
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'geQtdAutoInfracaoDividaAtivaCNPJ');

            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar dividas ativas de ${cnpj}.`, fonte);
            else
                return resultFound(result, fonte, rank, 'dividaativa_qtdautoinfracao');
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelas dividas ativas de ${cnpj}.`, 'geQtdAutoInfracaoDividaAtivaCNPJ'); });
};

export let getQtdProcessoJudicialDividaAtivaCNPJ = function (cnpj: string){

    const tempoInicial = new BenchmarkService();
    return connection.getConn().request()
        .input('cnpj', mssql.Char(14), cnpj)
        .query(`
            SELECT COUNT(*) AS qtdProcessoJudicial
            FROM DNA.dbo.DIVIDA_ATIVA D
            WHERE CPF_CNPJ_DIVIDA=@CNPJ
                AND FASE_DIVIDA_ATIVA <> 'ACAO EXTINTA'
                AND NUM_PROCESSO_JUDICIAL <> ''
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getQtdProcessoJudicialDividaAtivaCNPJ');

            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar dividas ativas de ${cnpj}.`, fonte);
            else
                return resultFound(result, fonte, rank, 'dividaativa_qtdprocessojud');
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelas dividas ativas de ${cnpj}.`, 'getQtdProcessoJudicialDividaAtivaCNPJ'); });
};

export let getDividaAtivaCNPJ = function (cnpj: string){

    const tempoInicial = new BenchmarkService();
    return connection.getConn().request()
        .input('cnpj', mssql.Char(14), cnpj)
        .query(`
            SELECT CPF_CNPJ_DIVIDA AS cnpj
                ,FASE_DIVIDA_ATIVA AS faseDivida
                ,NATUREZA_DEBITO AS naturezaDebito
                ,DATA_INSCRICAO_DIVIDA_ATIVA AS dataInscricao
                ,VALOR_DIVIDA_ATIVA as valor

            FROM DNA.dbo.DIVIDA_ATIVA D
            WHERE CPF_CNPJ_DIVIDA=@CNPJ
                AND FASE_DIVIDA_ATIVA <> 'ACAO EXTINTA'
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getDividaAtivaCNPJ');

            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar dividas ativas de ${cnpj}.`, fonte);
            else
                return resultFound(result, fonte, rank, 'dividaativa');
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelas dividas ativas de ${cnpj}.`, 'getDividaAtivaCNPJ'); });
};
