import * as mssql from 'mssql';

import { BenchmarkService } from './../../services/benchmark.service';
import { connection } from '../../services/db.service';
import { MODEL_PRIORITY, API_CONFIG } from './../../config';

import {
    resultNotFound,
    resultFound,
    logErroBuscaBD,
    printTempoExecucao
} from '../../utils';

const fonte   = MODEL_PRIORITY['sagres_estadual.empenhos'].fonte;
const rank    = MODEL_PRIORITY['sagres_estadual.empenhos'].rank;
const grupo   = MODEL_PRIORITY['sagres_estadual.empenhos'].grupo;

export let getInformacoesSociosCNPJ = function (cnpj: string){

    const tempoInicial = new BenchmarkService();
    return connection.getConn().request()
        .input('cnpj', mssql.Char(14), cnpj)
        .query(`
            ;WITH SOCIOS(CNPJ, CPF)
            AS
            (
                SELECT CNPJ, RIGHT(CPFSOCIO,11)
                FROM ReceitaNovo.dbo.Socio
                WHERE CNPJ = @CNPJ

                UNION

                SELECT PJ.NR_CGC, PF.NR_CPF
                FROM CNE.dbo.VINCULO V
                    INNER JOIN CNE.dbo.CONDICAO C ON (C.CO_CONDICAO = V.CO_CONDICAO)
                    INNER JOIN CNE.dbo.PESSOA P1 ON (V.SQ_PESSOA_VINCULANTE = P1.SQ_PESSOA AND V.CO_JUNTA_COMERCIAL = P1.CO_JUNTA_COMERCIAL)  --PJ
                    INNER JOIN CNE.dbo.PESSOA_JURIDICA PJ ON (PJ.SQ_PESSOA = P1.SQ_PESSOA AND PJ.CO_JUNTA_COMERCIAL = P1.CO_JUNTA_COMERCIAL)
                    INNER JOIN CNE.dbo.PESSOA P2 ON (V.SQ_PESSOA_VINCULADA = P2.SQ_PESSOA AND V.CO_JUNTA_COMERCIAL = P2.CO_JUNTA_COMERCIAL)  --PF
                    INNER JOIN CNE.dbo.PESSOA_FISICA PF ON (PF.SQ_PESSOA = P2.SQ_PESSOA AND PF.CO_JUNTA_COMERCIAL = P2.CO_JUNTA_COMERCIAL)
                WHERE (C.NO_VINCULO LIKE 'SOCIO%' OR C.NO_VINCULO LIKE 'SÓCIO%')
                    AND PJ.NR_CGC = @CNPJ
            )
            ,
            SOCIOS_COM_TITULO(CNPJ, CPF, TITULOELEITOR)
            AS
            (
                SELECT S.CNPJ, S.CPF, cast(PF.TituloEleitor as varchar(26)) as TituloEleitor
                FROM SOCIOS S
                    LEFT OUTER JOIN ReceitaNovo.dbo.PessoaFisica PF ON (PF.CPF = S.CPF)
            )
            SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                CNPJ as cnpj,
                CPF as cpf,
                ISNULL(CU.FLAG,0) AS cadastroUnico,
                ISNULL(ANALFABETOS.FLAG,0) AS analfabeto,
                ISNULL(FUNC_PUB_ESTADUAL.FLAG,0) AS servidorEstadual,
                ISNULL(FUNC_PUB_MUNICIPAL.FLAG,0) AS servidorMunicipal,
                ISNULL(FUNC_PUB_FEDERAL.FLAG,0) AS servidorFederal,
                ISNULL(FILIADO_PARTIDO.FLAG,0) AS filiadoPartido,
                ISNULL(DOADOR_PF.FLAG,0) AS doadorEleitoral

            FROM SOCIOS_COM_TITULO S
            OUTER APPLY(
                SELECT TOP 1 1 AS FLAG
                FROM Cadastro_Unico.dbo.bloco_00 B
                WHERE B.CPF = S.CPF

                UNION

                SELECT TOP 1 1
                FROM Cadastro_Unico.dbo.bloco_00 B
                WHERE B.TIT_ELEITOR = S.TITULOELEITOR
            ) CU
            OUTER APPLY(
                SELECT TOP 1 1 AS FLAG
                FROM SISPESQUISA.dbo.Analfabetos_tre_PB_2017 A
                WHERE A.CPF = S.CPF

                UNION

                SELECT TOP 1 1
                FROM SISPESQUISA.dbo.Analfabetos_tre_PB_2017 A
                WHERE A.TITULO = S.TITULOELEITOR
            ) ANALFABETOS
            OUTER APPLY(

                SELECT TOP 1 1 AS FLAG
                FROM SAGRES_ESTADUAL.dbo.VW_FOLHA F
                WHERE F.nu_CPF COLLATE Latin1_General_CI_AS = S.CPF

            ) FUNC_PUB_ESTADUAL
            OUTER APPLY(
                SELECT TOP 1 1 AS FLAG
                FROM SAGRES_MUNICIPAL.dbo.V_FolhaPagamento S2
                WHERE S2.CD_CPF = S.CPF
            ) FUNC_PUB_MUNICIPAL
            OUTER APPLY(
                SELECT TOP 1 1 AS FLAG
                FROM SISPESQUISA.dbo.SERVIDORES_FEDERAIS_NORDESTE SF
                WHERE SF.CPF_SERVIDOR = S.CPF
            ) FUNC_PUB_FEDERAL
            OUTER APPLY(
                SELECT TOP 1 1 AS FLAG
                FROM SISPESQUISA.dbo.filiados_pb FIL
                WHERE FIL.cpf = S.CPF
            ) AS FILIADO_PARTIDO
            OUTER APPLY(
                SELECT TOP 1 1 AS FLAG
                FROM Eleitoral.dbo.TSE_RECEITA_PARTIDO_2014 T
                WHERE T.CPF_CNPJ_DOADOR = S.CPF

                UNION

                SELECT TOP 1 1 AS FLAG
                FROM Eleitoral.dbo.TSE_RECEITA_PARTIDO_2016 T
                WHERE T.CPF_CNPJ_DOADOR = S.CPF

                UNION

                SELECT TOP 1 1 AS FLAG
                FROM Eleitoral.dbo.TSE_RECEITA_CANDIDATO_2014 T
                WHERE T.CPF_CNPJ_DOADOR = S.CPF

                UNION

                SELECT TOP 1 1 AS FLAG
                FROM Eleitoral.dbo.TSE_RECEITA_CANDIDATO_2016 T
                WHERE T.CPF_CNPJ_DOADOR = S.CPF
            ) DOADOR_PF
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getInformacoesSociosCNPJ');

            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar as informacoes dos socios do ${cnpj}.`, fonte);
            else
                return resultFound(result, fonte, rank, 'infosocios');
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelas informacoes dos socios de ${cnpj}.`, 'getInformacoesSociosCNPJ'); });
};

export let getDoacaoEleitoralCNPJ = function (cnpj: string){

    const tempoInicial = new BenchmarkService();
    return connection.getConn().request()
        .input('cnpj', mssql.Char(14), cnpj)
        .query(`
            ;WITH DOACOES(VALORES)
            AS
            (
                SELECT VALOR_RECEITA
                FROM Eleitoral.dbo.TSE_RECEITA_PARTIDO_2014
                WHERE CPF_CNPJ_DOADOR = @CNPJ

                UNION ALL

                SELECT VALOR_RECEITA
                FROM Eleitoral.dbo.TSE_RECEITA_PARTIDO_2016
                WHERE CPF_CNPJ_DOADOR = @CNPJ

                UNION ALL

                SELECT VALOR_RECEITA
                FROM Eleitoral.dbo.TSE_RECEITA_CANDIDATO_2014
                WHERE CPF_CNPJ_DOADOR = @CNPJ

                UNION ALL

                SELECT VALOR_RECEITA
                FROM Eleitoral.dbo.TSE_RECEITA_CANDIDATO_2016
                WHERE CPF_CNPJ_DOADOR = @CNPJ
            )

            SELECT SUM(VALORES) as doacao
            FROM DOACOES
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getDoacaoEleitoralCNPJ');

            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar as doacoes de ${cnpj}.`, fonte);
            else
                return resultFound(result, fonte, rank, 'doacaoeleitoral');
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelas doacoes de ${cnpj}.`, 'getDoacaoEleitoralCNPJ'); });
};

export let getContadoresCNPJ = function (cnpj: string){

    const tempoInicial = new BenchmarkService();
    return connection.getConn().request()
        .input('cnpj', mssql.Char(14), cnpj)
        .query(`
            ;WITH CONTADORES(CNPJ, CPF)
            AS
            (
                SELECT CNPJ, CPFContador AS CPF
                FROM Receita_Full.Receita.Contador C
                WHERE CNPJ = @CNPJ

                UNION

                SELECT PJ.CNPJ, C.CPF_CNPJ
                FROM DNA.dbo.CONTADORES C
                    INNER JOIN DNA.dbo.PESSOA_JURIDICA PJ ON (C.INSCRICAO_ESTADUAL = PJ.INSCRICAO_ESTADUAL)
                WHERE PJ.CNPJ = @CNPJ

            )
            SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                C.CNPJ as cnpj, C.CPF as cpf, PF.NOME AS nome
            FROM CONTADORES C
                INNER JOIN ReceitaNovo.dbo.PessoaFisica PF ON (C.CPF = PF.CPF)
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getContadoresCNPJ');

            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar os contadores de ${cnpj}.`, fonte);
            else
                return resultFound(result, fonte, rank, 'contadores');
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelos contadores de ${cnpj}.`, 'getContadoresCNPJ'); });
};

export let getEmpresasContadoresCNPJ = function (cnpj: string){

    const tempoInicial = new BenchmarkService();
    return connection.getConn().request()
        .input('cnpj', mssql.Char(14), cnpj)
        .query(`
            ;WITH CONTADORES(CNPJ, CPF)
            AS
            (
                SELECT CNPJ, CPFContador AS CPF
                FROM Receita_Full.Receita.Contador C
                WHERE CNPJ = @CNPJ

                UNION

                SELECT PJ.CNPJ, C.CPF_CNPJ
                FROM DNA.dbo.CONTADORES C
                    INNER JOIN DNA.dbo.PESSOA_JURIDICA PJ ON (C.INSCRICAO_ESTADUAL = PJ.INSCRICAO_ESTADUAL)
                WHERE PJ.CNPJ = @CNPJ
            )
            ,OUTRAS_EMPRESAS(CNPJ, CPF)
            AS
            (
                SELECT CON.CNPJ, C.CPF
                FROM Receita_Full.Receita.Contador CON
                    INNER JOIN CONTADORES C ON (CON.CPFContador = C.CPF)

                UNION

                SELECT PJ.CNPJ, CON.CPF_CNPJ
                FROM DNA.dbo.CONTADORES CON
                    INNER JOIN CONTADORES C ON (CON.CPF_CNPJ = C.CPF)
                    INNER JOIN DNA.dbo.PESSOA_JURIDICA PJ ON (CON.INSCRICAO_ESTADUAL = PJ.INSCRICAO_ESTADUAL)
            )
            SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                C.CNPJ as cnpj, PJ.RazaoSocial as razaoSocial, PJ.NomeFantasia as nomeFantasia, PJ.Municipio as municipio, PJ.Uf as uf
            FROM OUTRAS_EMPRESAS C
                INNER JOIN ReceitaNovo.dbo.PessoaJuridica PJ ON (C.CNPJ = PJ.CNPJ)
            ORDER BY C.CNPJ
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getEmpresasContadoresCNPJ');

            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar as empresas dos contadores de ${cnpj}.`, fonte);
            else
                return resultFound(result, fonte, rank, 'empresascontadores');
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelas empresas dos contadores de ${cnpj}.`, 'getEmpresasContadoresCNPJ'); });
};

export let getEmpresasContadoresReceberamEstadoCNPJ = function (cnpj: string){

    const tempoInicial = new BenchmarkService();
    return connection.getConn().request()
        .input('cnpj', mssql.Char(14), cnpj)
        .query(`
            ;WITH CONTADORES(CNPJ, CPF)
            AS
            (
                SELECT CNPJ, CPFContador AS CPF
                FROM Receita_Full.Receita.Contador C
                WHERE CNPJ = @CNPJ

                UNION

                SELECT PJ.CNPJ, C.CPF_CNPJ
                FROM DNA.dbo.CONTADORES C
                    INNER JOIN DNA.dbo.PESSOA_JURIDICA PJ ON (C.INSCRICAO_ESTADUAL = PJ.INSCRICAO_ESTADUAL)
                WHERE PJ.CNPJ = @CNPJ

            )
            ,OUTRAS_EMPRESAS_CONTADOR(CNPJ, CPF)
            AS
            (
                SELECT CON.CNPJ, C.CPF
                FROM Receita_Full.Receita.Contador CON
                    INNER JOIN CONTADORES C ON (CON.CPFContador = C.CPF)
                WHERE CON.CNPJ <> @CNPJ

                UNION

                SELECT PJ.CNPJ, CON.CPF_CNPJ
                FROM DNA.dbo.CONTADORES CON
                    INNER JOIN CONTADORES C ON (CON.CPF_CNPJ = C.CPF)
                    INNER JOIN DNA.dbo.PESSOA_JURIDICA PJ ON (CON.INSCRICAO_ESTADUAL = PJ.INSCRICAO_ESTADUAL)
                WHERE CON.CPF_CNPJ <> @CNPJ
            )
            ,OUTRAS_EMPRESAS_CONTADOR_RECEBERAM(CNPJ)
            AS
            (
                SELECT DISTINCT EC.CNPJ
                FROM OUTRAS_EMPRESAS_CONTADOR EC
                    INNER JOIN SAGRES_MUNICIPAL.dbo.Empenhos E ON (EC.CNPJ = E.cd_credor)
                    INNER JOIN SAGRES_MUNICIPAL.dbo.PAGAMENTOS P ON (E.cd_Ugestora = P.cd_Ugestora) AND (E.dt_Ano = P.dt_Ano) AND (E.cd_UnidOrcamentaria = P.cd_UnidOrcamentaria) AND (E.nu_Empenho = P.nu_Empenho)

                UNION

                SELECT DISTINCT EC.CNPJ
                FROM OUTRAS_EMPRESAS_CONTADOR EC
                    INNER JOIN SAGRES_ESTADUAL.dbo.Empenhos E ON (EC.CNPJ = E.cd_credor COLLATE Latin1_General_CI_AS)
                    INNER JOIN SAGRES_ESTADUAL.dbo.PAGAMENTOS P ON (E.cd_Ugestora = P.cd_Ugestora) AND (E.dt_Ano = P.dt_Ano) AND (E.cd_UnidOrcamentaria = P.cd_UnidOrcamentaria) AND (E.nu_Empenho = P.nu_Empenho)
            )

            SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                E.CNPJ as cnpj, PJ.RazaoSocial as razaoSocial, PJ.NomeFantasia as nomeFantasia, PJ.Municipio as municipio, PJ.Uf as uf
            FROM OUTRAS_EMPRESAS_CONTADOR_RECEBERAM E
                INNER JOIN ReceitaNovo.dbo.PessoaJuridica PJ ON (E.CNPJ = PJ.CNPJ)
            ORDER BY E.CNPJ
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getEmpresasContadoresReceberamEstadoCNPJ');

            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar as empresas que receberam dos contadores de ${cnpj}.`, fonte);
            else
                return resultFound(result, fonte, rank, 'empresascontadoresreceberam');
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelas empresas que receberam dos contadores de ${cnpj}.`, 'getEmpresasContadoresReceberamEstadoCNPJ'); });
};

export let getEmpresasContadoresParticiparamCNPJ = function (cnpj: string){

    const tempoInicial = new BenchmarkService();
    return connection.getConn().request()
        .input('cnpj', mssql.Char(14), cnpj)
        .query(`
            ;WITH CONTADORES(CNPJ, CPF)
            AS
            (
                SELECT CNPJ, CPFContador AS CPF
                FROM Receita_Full.Receita.Contador C
                WHERE CNPJ = @CNPJ

                UNION

                SELECT PJ.CNPJ, C.CPF_CNPJ
                FROM DNA.dbo.CONTADORES C
                    INNER JOIN DNA.dbo.PESSOA_JURIDICA PJ ON (C.INSCRICAO_ESTADUAL = PJ.INSCRICAO_ESTADUAL)
                WHERE PJ.CNPJ = @CNPJ

            )
            ,OUTRAS_EMPRESAS_CONTADOR(CNPJ, CPF)
            AS
            (
                SELECT CON.CNPJ, C.CPF
                FROM Receita_Full.Receita.Contador CON
                    INNER JOIN CONTADORES C ON (CON.CPFContador = C.CPF)

                UNION

                SELECT PJ.CNPJ, CON.CPF_CNPJ
                FROM DNA.dbo.CONTADORES CON
                    INNER JOIN CONTADORES C ON (CON.CPF_CNPJ = C.CPF)
                    INNER JOIN DNA.dbo.PESSOA_JURIDICA PJ ON (CON.INSCRICAO_ESTADUAL = PJ.INSCRICAO_ESTADUAL)
            )
            ,LICITACOES_PARTICIPADAS(CD_UGESTORA, ANO, NU_LICITACAO, TP_LICITACAO)
            AS
            (
                SELECT cd_UGestora, dt_Ano, nu_Licitacao, tp_Licitacao
                FROM SAGRES_MUNICIPAL.dbo.Participantes
                WHERE nu_CPFCNPJ=@CNPJ
            )
            ,EMPRESAS_PARTICIPARAM_JUNTO(CNPJ)
            AS
            (
                SELECT P.nu_CPFCNPJ
                FROM SAGRES_MUNICIPAL.dbo.Participantes P
                    INNER JOIN LICITACOES_PARTICIPADAS L ON (P.cd_UGestora = L.CD_UGESTORA AND P.nu_Licitacao = L.NU_LICITACAO AND P.tp_Licitacao = L.TP_LICITACAO AND P.dt_Ano = L.ANO)
                WHERE P.nu_CPFCNPJ <> '00000000000000' AND P.nu_CPFCNPJ <> @CNPJ
            )
            ,OUTRAS_EMPRESAS_CONTADOR_PARTICIPARAM_COMIGO(CNPJ)
            AS
            (
                SELECT DISTINCT E.CNPJ
                FROM OUTRAS_EMPRESAS_CONTADOR E
                    INNER JOIN EMPRESAS_PARTICIPARAM_JUNTO EPJ ON (E.CNPJ = EPJ.CNPJ)
            )

            SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                E.CNPJ as cnpj, PJ.RazaoSocial as razaoSocial, PJ.NomeFantasia as nomeFantasia, PJ.Municipio as municipio, PJ.Uf as uf
            FROM OUTRAS_EMPRESAS_CONTADOR_PARTICIPARAM_COMIGO E
                INNER JOIN ReceitaNovo.dbo.PessoaJuridica PJ ON (E.CNPJ = PJ.CNPJ)
            ORDER BY E.CNPJ
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getEmpresasContadoresParticiparamCNPJ');

            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar as empresas que participaram em licitacoes dos contadores de ${cnpj}.`, fonte);
            else
                return resultFound(result, fonte, rank, 'empresascontadoresparticiparam');
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelas empresas que participaram em licitacoes dos contadores de ${cnpj}.`, 'getEmpresasContadoresParticiparamCNPJ'); });
};

export let getEmpresasMesmoEmailCNPJ = function (cnpj: string){

    const tempoInicial = new BenchmarkService();
    return connection.getConn().request()
        .input('cnpj', mssql.Char(14), cnpj)
        .query(`
            ;WITH EMAIL(CNPJ, EMAIL)
            AS
            (
                SELECT CNPJ, [EMAIL]
                FROM [DNA].[dbo].[PESSOA_JURIDICA]
                WHERE CNPJ = @CNPJ

                --UNION

                --SELECT [CNPJ], LTRIM(RTRIM([CorreioEletronico])) as email
                --FROM [Receita_Full].[Receita].[PJ]
                --WHERE CNPJ= @CNPJ
                --	AND (CorreioEletronico <> '                                                                                                                   ')
            )
            , EMPRESAS_COM_MESMO_EMAIL(CNPJ, EMAIL)
            AS
            (
                SELECT PJ.CNPJ, PJ.EMAIL
                FROM [DNA].[dbo].[PESSOA_JURIDICA] PJ
                    INNER JOIN EMAIL E ON (PJ.EMAIL = E.EMAIL)
                WHERE PJ.CNPJ <> @CNPJ

                --UNION

                --SELECT PJ.[CNPJ], LTRIM(RTRIM([CorreioEletronico])) as email
                --FROM [Receita_Full].[Receita].[PJ]
                --	INNER JOIN EMAIL E ON (PJ.CorreioEletronico = E.EMAIL)
                --WHERE PJ.CNPJ <> @CNPJ
            )

            SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                E.CNPJ as cnpj, PJ.RazaoSocial as razaoSocial, PJ.NomeFantasia as nomeFantasia,
                PJ.Municipio as municipio, PJ.Uf as uf, E.EMAIL as email
            FROM EMPRESAS_COM_MESMO_EMAIL E
                INNER JOIN ReceitaNovo.dbo.PessoaJuridica PJ ON (E.CNPJ = PJ.CNPJ)
            ORDER BY E.CNPJ
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getEmpresasMesmoEmailCNPJ');

            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar as empresas com mesmo email de ${cnpj}.`, fonte);
            else
                return resultFound(result, fonte, rank, 'empresasmesmoemail');
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelas empresas com mesmo email de ${cnpj}.`, 'getEmpresasMesmoEmailCNPJ'); });
};

export let getEmpresasMesmoTelefoneCNPJ = function (cnpj: string){

    const tempoInicial = new BenchmarkService();
    return connection.getConn().request()
        .input('cnpj', mssql.Char(14), cnpj)
        .query(`
            ;WITH TELEFONE(CNPJ, TELEFONE)
            AS
            (
                SELECT CNPJ, TELEFONE
                FROM [DNA].[dbo].[PESSOA_JURIDICA]
                WHERE CNPJ = @CNPJ
            )
            , EMPRESAS_COM_MESMO_TELEFONE(CNPJ, TELEFONE)
            AS
            (
                SELECT PJ.CNPJ, PJ.TELEFONE
                FROM [DNA].[dbo].[PESSOA_JURIDICA] PJ
                    INNER JOIN TELEFONE T ON (PJ.TELEFONE = T.TELEFONE)
                WHERE PJ.CNPJ <> @CNPJ
            )

            SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                E.CNPJ as cnpj, PJ.RazaoSocial as razaoSocial, PJ.NomeFantasia as nomeFantasia,
                PJ.Municipio as municipio, PJ.Uf as uf, E.TELEFONE as telefone
            FROM EMPRESAS_COM_MESMO_TELEFONE E
                INNER JOIN ReceitaNovo.dbo.PessoaJuridica PJ ON (E.CNPJ = PJ.CNPJ)
            ORDER BY E.CNPJ
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getEmpresasMesmoTelefoneCNPJ');

            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar as empresas com mesmo telefone de ${cnpj}.`, fonte);
            else
                return resultFound(result, fonte, rank, 'empresasmesmotelefone');
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelas empresas com mesmo telefone de ${cnpj}.`, 'getEmpresasMesmoTelefoneCNPJ'); });
};

export let getAtividadeEconomicaCNPJ = function (cnpj: string) {

    const tempoInicial = new BenchmarkService();

    return connection.getConn().request()
        .input('cnpj', mssql.Char(14), cnpj)
        .query(`
            ;WITH ATIVIDADE(CNAE, DESCRICAO)
            AS
            (
                SELECT ATUACAO.CO_ATIVIDADE, A.DS_ATIVIDADE
                FROM CNE.dbo.ATUACAO
                    INNER JOIN CNE.dbo.PESSOA_JURIDICA PJ ON (ATUACAO.CO_JUNTA_COMERCIAL = PJ.CO_JUNTA_COMERCIAL AND ATUACAO.SQ_PESSOA = PJ.SQ_PESSOA AND ATUACAO.CO_SEQUENCIAL = PJ.CO_SEQUENCIAL AND ATUACAO.NO_ESQUEMA_ORIGEM = PJ.NO_ESQUEMA_ORIGEM)
                    INNER JOIN CNE.dbo.ATIVIDADE_ECONOMICA A ON (ATUACAO.CO_ATIVIDADE = A.CO_ATIVIDADE)
                WHERE PJ.NR_CGC= @CNPJ

                UNION

                SELECT PJ.CnaeFiscal, A.DS_ATIVIDADE
                FROM ReceitaNovo.dbo.PessoaJuridica PJ
                    INNER JOIN CNE.dbo.ATIVIDADE_ECONOMICA A ON (PJ.CnaeFiscal = A.CO_ATIVIDADE)
                WHERE PJ.CNPJ = @CNPJ
            )

            SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                CNAE as cnae, DESCRICAO as descricao
            FROM ATIVIDADE
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getAtividadeEconomicaCNPJ');

            if (result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar as atividades economicas de de ${cnpj}.`, fonte);
            else
                return resultFound(result, fonte, rank, 'atividadeeconomica');
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelas atividades economicas de ${cnpj}.`, 'getAtividadeEconomicaCNPJ'); });
};