import * as mssql from 'mssql';

import { connection }  from '../../services/db.service';
import { MODEL_PRIORITY, API_CONFIG } from '../../config';
import {
    trataResultado,
    resultNotFound,
    resultFound,
    logErroBuscaBD
} from '../../utils';

import { SIGLA_FONTE } from './../_fontes';

const fonte_dado = SIGLA_FONTE['dna.pj'];

const fonte = MODEL_PRIORITY['dna.pj'].fonte;
const rank  = MODEL_PRIORITY['dna.pj'].rank;
const grupo = MODEL_PRIORITY['dna.pj'].grupo;

export let getEnderecoCNPJ_DNA_PJ = function (cnpj: string){

    return connection.getConn().request()
        .input('CNPJ', mssql.Char(14), cnpj)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                    CNPJ as cnpj,
                    LOGRADOURO_FEDERAL as logradouro,
                    NUMERO_FEDERAL as numero,
                    COMPLEMENTO_FEDERAL as complemento,
                    BAIRRO_FEDERAL as bairro,
                    CEP_FEDERAL as cep,
                    UF_FEDERAL as uf,
                    Municipio as municipio,
                    '${fonte_dado}' as fonte

                FROM DNA.dbo.PESSOA_JURIDICA
                WHERE CNPJ=@CNPJ
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o endereço de ${cnpj}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pelo endereço de ${cnpj}.`, 'getEnderecoCNPJ_DNA_PJ')});
};
